/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1998 
 * All rights reserved
 * 
 */
/*
 * HISTORY
 * $Log: kernelImage.h,v $
 * Revision 1.1.2.5  1998/12/22  03:12:44  tibor
 * 	doc changes add compressed section info and CRC
 * 	[1998/12/22  01:34:35  tibor]
 *
 * Revision 1.1.2.4  1998/12/02  01:02:05  tibor
 * 	changed type defs to refer to section instead of kernel.  Added some flag defs
 * 	[1998/12/02  01:01:49  tibor]
 * 
 * Revision 1.1.2.3  1998/11/28  21:39:17  wayne
 * 	Flesh out with skeleton #defines and the like; more work needed
 * 	[1998/11/28  21:11:58  wayne]
 * 
 * Revision 1.1.2.2  1998/11/25  17:48:42  mervin
 * 	Added #ifndef _KERNELIMAGE_H_ wrapper to avoid
 * 	multiple inclusions.
 * 	[1998/11/25  17:48:30  mervin]
 * 
 * Revision 1.1.2.1  1998/11/25  02:41:52  tibor
 * 	initial release
 * 	[1998/11/24  22:11:22  tibor]
 * 
 * $EndLog$
 */


#ifndef _KERNELIMAGE_H_
#define _KERNELIMAGE_H_

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
 *  Kernel image description, generated by boot kernel and genfw for use in
 *  accessing and downloading kernel (standalone program) images by the MP
 */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
 *  Description of a single section (e.g., code, data, rodata, bss, etc)
 */
typedef struct {
    U8   sec_type;		/* section type (text, data, bss ..) */
                                /*  (see SECTION_DEF_TYPE_XXXX below) */
    U8   flag;			/* any added info relevant to section */
                                /*  (see SECTION_DEF_FLAG_XXXX below) */
    U16  reserved_1;
    U32 *obj;			/* section data (not valid for BSS/SBSS) */
    U32  len;			/* length of section data in bytes */
    U32  load_addr;		/* address of where to load on target */
                                /* in BAF (Bender Address Form): */
                                /*  bit 31 = 1 for FP, else CP target */
                                /*  bit 27 = 1 for scr pad, else sdram*/
    U32  compAddr;		/* compressed section image offset (flash) */
    U32  compLen;		/* compressed section image length */
    U32  crc;			/* for text, rodata, data, computed CRC */
    U32  reserved_2;
} section_def_t;

/* following order not be changed, it is in tigon load order */
#define SECTION_DEF_TYPE_UNKN    0
#define SECTION_DEF_TYPE_TEXT    1
#define SECTION_DEF_TYPE_DATA    2
#define SECTION_DEF_TYPE_RODATA  3
#define SECTION_DEF_TYPE_BSS     4
#define SECTION_DEF_TYPE_SBSS    5
#define SECTION_DEF_TYPE_MAX	 SECTION_DEF_TYPE_SBSS

#define SECTION_DEF_FLAG_MPONLY  0x01	/* section only used by MP */
#define SECTION_DEF_FLAG_SPONLY  0x02	/* section only used by SP */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
 *  Description of the kernel, consisting of several sections and a start loc
 */
typedef struct {
    U32  start_addr;			/* where to start execution (BAF) */
    U32  reserved_1;
    section_def_t *sec_list;		/* array of sections defining kernel */
    U32  num_sections;			/* number of sections in array */
} kernel_def_t;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#endif /* _KERNELIMAGE_H_ */
