/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996 
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: led.c $
 * Revision copper_fhwang/1 1999/10/12 19:32:55 fhwang
 * 	to support new led (4 leds)
 * 
 * Revision 1.1.2.1  1997/11/15  18:38:38  hayes
 * 	initial checkin
 * 	[1997/11/15  18:37:31  hayes]
 *
 * $EndLog$
 */

#if defined(_SP_CONF_H_)
#include "sp_regs.h"
#endif /* defined(_SP_CONF_H_) */

/*
 * Set the LEDs by toggling a serial line to a serial to parallel 
 * shift register.
 *
 * This is designed to use the National 74ACT164
 * LED_DATA is used for the data and LED_LINK is used for
 * clock.  The data must be presented before the rising edge of
 * each clock.
 *
 * See led.h for definition of the bits.
 *
 * This implementation ignores the UART bits, since they are not used
 * on the NIC or the SPs.  If the UART needs to be used in conjunction
 * with this code, we must mask off the UART bits whenever we write the
 * Misc Local Control register to avoid inadvertantly clearing our UART
 * state.
 */

#include "led.h"

#include "link_op.h"
#include "mii.h"
#include "zconf.h"
#include "np.h"
#include "link.h"
#include "link_if.h"

#include "assert.h"
#include "trace.h"
#include "timer.h"

U8 led4_data[LED_PATNS][LED_DATA_LEN] = {
	/* led all off */
	{1, 1, 1, 1},
	/* led 10 link on data off */
	{1, 1, 1, 0},
	/* led 10 link on data on */
	{0, 1, 1, 0},
	/* led 100 link on data off */
	{1, 1, 0, 1},
	/* led 100 link on data on */
	{0, 1, 0, 1},
	/* led 1000 link on data off */
	{1, 0, 1, 1},
	/* led 1000 link on data on */
	{0, 0, 1, 1},
	/* led all on */
	{0, 0, 0, 0},
};

void
set_led(U8 bits)
{
    int i;

    for (i = 0; i<8; i++) {
	if ((bits >> i) & 1) {
	    /* turn bit on */
	    TRP->gen_control.misc_local_control |= SER_LED_DATA;
	    /* clock up */
	    TRP->gen_control.misc_local_control |= SER_LED_CLK;
	    /* clock down */
	    TRP->gen_control.misc_local_control &= ~SER_LED_CLK;
	} else {
	    /* turn bit off */
	    TRP->gen_control.misc_local_control &= ~SER_LED_DATA;
	    /* clock up */
	    TRP->gen_control.misc_local_control |= SER_LED_CLK;
	    /* clock down */
	    TRP->gen_control.misc_local_control &= ~SER_LED_CLK;
	}
    }
}



void set_led4(U32 led_patn)
{
    U8 *data;
    U32 i;
    
    switch (led_patn) {
    case LED_ALL_OFF:
    case LED_10_ON_DT_OFF:
    case LED_10_ON_DT_ON:
    case LED_100_ON_DT_OFF:
    case LED_100_ON_DT_ON:
    case LED_1000_ON_DT_OFF:
    case LED_1000_ON_DT_ON:
    case LED_ALL_ON:
	data = &(led4_data[led_patn][0]);
	for (i = 0; i < LED_DATA_LEN; i++) {
	    if (data[i]) {
		/* turn bit on */
		TRP->gen_control.misc_local_control |= SER_LED_DATA;
		/* clock up */
		TRP->gen_control.misc_local_control |= SER_LED_CLK;
		/* clock down */
		TRP->gen_control.misc_local_control &= ~SER_LED_CLK;
	    } else {
		/* turn bit off */
		TRP->gen_control.misc_local_control &= ~SER_LED_DATA;
		/* clock up */
		TRP->gen_control.misc_local_control |= SER_LED_CLK;
		/* clock down */
		TRP->gen_control.misc_local_control &= ~SER_LED_CLK;
	    }
	    /*
	    LINK_UTRACE("set4_led", 0xd0a00, data[i], i, led_patn,
			TRP->gen_control.misc_local_control);
			*/
	}
	break;
    default:
	break;
    }
}
