/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996, 1997 
 * All rights reserved
 * 
 */
/*
 * HISTORY
 * $Log: link_if.c $
 * Revision copper_fhwang/32 2000/06/17 00:50:34 fhwang
 * 	to allow new setting to be set in the hardwe
 * 
 * Revision copper_fhwang/30 2000/06/03 02:25:23 fhwang
 * 	to sync with ge_latest
 * 
 * Revision copper_fhwang/22 2000/04/13 01:48:12 fhwang
 * 	for broadcom errata on no HCD link
 * 
 * Revision copper_fhwang/18 2000/03/02 19:23:08 fhwang
 * 	fix setting flow control problem
 * 
 * Revision copper_fhwang/15 2000/02/19 00:22:29 fhwang
 * 	to see register 18 value
 * 
 * Revision copper_fhwang/13 2000/02/01 01:07:55 fhwang
 * 	to print c0326 and c0323 message only when link is up
 * 
 * Revision copper_fhwang/8 1999/11/19 02:25:48 fhwang
 * 	move the code inside the bcm5400 case block
 * 
 * Revision copper_fhwang/6 1999/11/14 02:29:11 fhwang
 * 	to shut down serdes when it's disabled
 * 
 * Revision copper_fhwang/4 1999/09/03 02:54:03 fhwang
 * 	set bit 10 of register 0x18 to 1 as recommended
 * 
 * Revision nic_api_12_4_latest_shuang/2 1999/08/31 23:00:22 shuang
 * 	eliminate compiler warning on Tigon-I
 * 
 * Revision copper_fhwang/2 1999/08/27 02:21:10 fhwang
 * 	add GMII interface for tx1000
 * 
 * Revision 1.1.2.28  1999/01/27  19:09:41  hayes
 * 	incorporate autosense code
 * 	[1999/01/27  19:07:54  hayes]
 *
 * Revision 1.1.17.2  1998/11/07  20:59:56  hayes
 * 	link autosense mode now works
 * 
 * Revision 1.1.2.27  1998/07/01  00:31:00  taylor
 * 	Fixed defects 1286 and 1288
 * 	[1998/07/01  00:30:08  taylor]
 * 
 * Revision 1.1.2.26  1998/05/27  01:19:35  fhwang
 * 	OR MII_AUTONEG_ADV_802_3 with advertise value before set the register.
 * 	[1998/05/27  01:03:55  fhwang]
 * 
 * Revision 1.1.2.25  1998/03/18  03:43:59  hayes
 * 	restructure mac mgt interface
 * 	[1998/03/18  03:37:56  hayes]
 * 
 * Revision 1.1.2.24  1998/03/09  17:32:06  hayes
 * 	add support for MII LOOPBACK mode
 * 	[1998/03/09  01:00:01  hayes]
 * 
 * Revision 1.1.2.23  1998/02/27  02:42:14  hayes
 * 	fix link preferences and restructure link mgt interface
 * 	[1998/02/27  02:41:28  hayes]
 * 
 * Revision 1.1.2.22  1998/02/24  00:09:09  kyung
 * 	1. disabling a port didn't set the link state correctly
 * 	2. when fast phy is active, gig port cap change caused
 * 		mac_tx_state regist
 * 	[1998/02/24  00:08:58  kyung]
 * 
 * Revision 1.1.11.1  1998/02/18  19:20:51  kyung
 * 	sync up with ge_latest
 * 	[1998/02/18  19:20:26  kyung]
 * 
 * Revision 1.1.2.20  1998/02/07  00:17:21  taylor
 * 	Rev5: dont overwrite tag type in mac_len reg
 * 	[1998/02/07  00:16:58  taylor]
 * 
 * Revision 1.1.2.19  1998/02/06  22:19:45  hayes
 * 	add support for link preferences
 * 	[1998/02/06  21:33:35  hayes]
 * 
 * Revision 1.1.2.18  1998/01/22  22:51:13  hayes
 * 	fix different phy fctl bug
 * 	[1998/01/22  22:47:23  hayes]
 * 
 * Revision 1.1.2.17  1998/01/09  18:26:06  hayes
 * 	Merged with changes from 1.1.2.16
 * 	[1998/01/09  18:26:02  hayes]
 * 
 * 	fix fctl negotiation bug
 * 	[1998/01/09  18:23:45  hayes]
 * 
 * Revision 1.1.2.16  1998/01/08  22:32:50  tibor
 * 	converted DISABLE_MAC_RX to function
 * 	[1998/01/08  22:21:44  tibor]
 * 
 * Revision 1.1.2.15  1997/12/12  19:45:40  hayes
 * 	fix phy disable
 * 	[1997/12/12  19:45:30  hayes]
 * 
 * Revision 1.1.2.14  1997/12/12  05:34:31  hayes
 * 	set_mac_link() and get_mac_link() now take the desired phy as a parameter
 * 	[1997/12/12  04:03:25  hayes]
 * 
 * Revision 1.1.2.13  1997/12/08  23:10:57  hayes
 * 	call link_down_notify() on link disable
 * 	[1997/12/08  23:10:48  hayes]
 * 
 * Revision 1.1.2.12  1997/11/26  01:08:46  hayes
 * 	improve external serdes bringup
 * 	[1997/11/26  01:07:22  hayes]
 * 
 * Revision 1.1.2.11  1997/11/24  01:00:19  taylor
 * 	Rev5: move data structures into scratchpad
 * 	[1997/11/23  21:42:52  taylor]
 * 
 * Revision 1.1.2.10  1997/11/22  21:39:42  hayes
 * 	changes to support new phy mgt interface for both 110 and 180
 * 	[1997/11/22  21:39:31  hayes]
 * 
 * Revision 1.1.2.9  1997/11/21  23:54:15  hayes
 * 	MII link down works
 * 	[1997/11/21  23:53:52  hayes]
 * 
 * 	MII interface works
 * 	[1997/11/21  22:45:27  hayes]
 * 
 * Revision 1.1.2.8  1997/11/21  04:48:10  hayes
 * 	release for NIC 2.1 pre-Beta
 * 	[1997/11/21  03:19:13  hayes]
 * 
 * 	internal and external SERDES co-exist peacefully
 * 	[1997/11/20  01:28:03  hayes]
 * 
 * Revision 1.1.2.7  1997/11/10  02:55:07  hayes
 * 	adjuct for better link bringup
 * 	[1997/11/10  02:54:52  hayes]
 * 
 * Revision 1.1.2.6  1997/10/27  22:59:51  hayes
 * 	make Tigon 2 support work for internal SERDES
 * 	[1997/10/27  22:56:07  hayes]
 * 
 * Revision 1.1.2.5  1997/10/11  19:02:50  hayes
 * 	updated rerorting of ALTEON, NIC and JUMBO LNK_XXX bits
 * 	[1997/10/11  19:02:25  hayes]
 * 
 * Revision 1.1.2.4  1997/09/17  03:26:26  hayes
 * 	Make get_mac_link() and set_mac_link() aware of active phys vs attached phys.
 * 	Change restart condition to active_phy_fail = TRUE.
 * 	[1997/09/15  22:20:44  hayes]
 * 
 * Revision 1.1.2.3  1997/09/10  03:55:09  hayes
 * 	start code preparation to handle multiple phys per mac
 * 	remove references to LNK_XX_FLOW_CTL_N
 * 	[1997/09/10  03:52:05  hayes]
 * 
 * Revision 1.1.2.2  1997/09/09  02:57:32  hayes
 * 	Add support for using various MII devices
 * 	Add device support for Level One LXT970 in addition
 * 	to ICS 1890
 * 	[1997/09/09  02:55:48  hayes]
 * 
 * Revision 1.1.2.1  1997/08/29  22:11:52  hayes
 * 	initial checkin
 * 
 * Revision 1.1.1.2  1997/08/29  22:10:16  hayes
 * 	initial checkin
 * 
 * $EndLog$
 */

#include "alt_const.h"
#include "link_op.h"
#include "mii.h"
#include "zconf.h"
#include "np.h"
#include "link.h"
#include "link_if.h"

#if defined(_SP_CONF_H_)
#include "sp_regs.h"
#endif /* defined(_SP_CONF_H_) */

#include "timer.h"
#include "mac.h"
#include "assert.h"
#include "trace.h"

extern U32 active_phy_fail_timeout;
link_mgt_state_t mac_mgt_state[NUM_MACS];
extern U32 good_phy;

void
set_mac_link(U32 index, U32 phy, U32 link_set)
{
    mii_pair_t *miip;
    U16 zero = 0;
    U16 loc_reg_control;

    LINK_TRACE(TRACE_TYPE_LNK_PHY, "stmacLNK", 0x99001, 
	       index, phy, active_phy, link_set);


    switch (phy) {
	    
	case ATTACHED_PHY_IF_MII:

	    /* for BRDCOM device, it is 3 speeds available, if we negotiate
	     * to 1000, we want to switch to GMII interface 
	     */
	    if (!((attached_phy & ATTACHED_PHY_IF_GMII) == 
		  ATTACHED_PHY_IF_GMII))
	        /* remove impossible options */
	        link_set &= ~LNK_1000MB;

	    /* save it away */
	    mac_mgt_state[index].link_set_fast = link_set;
	    
	    /* enable or disable */
	    if (link_set & LNK_ENABLE) {

		/* set or negotiate */
		miip = &mii_set[(link_set & LNK_DPX_MASK) >> 
			LNK_DPX_SHIFT]
		    [(link_set & LNK_SPD_MASK) >> LNK_SPD_SHIFT];
		
		if (link_set & LNK_TX_FLOW_CTL_Y) {
		    mac_mgt_state[index].tx_fctl_fast = TG_MAC_TX_STATE_ENA_FC;
		} else {
		    mac_mgt_state[index].tx_fctl_fast = 0;
		}

		if (link_set & LNK_RX_FLOW_CTL_Y) {
		    mac_mgt_state[index].rx_fctl_fast = TG_MAC_RX_STATE_ENA_FC;
		} else {
		    mac_mgt_state[index].rx_fctl_fast = 0;
		}

		loc_reg_control = miip->reg_control;

		if (mii_device_attached != GMII_DEVICE_BCM5400) {
		    if (link_set & LNK_LOOPBACK)
			loc_reg_control |= MII_CONTROL_LOOPBACK;
		    
		    /* now set the PHY */
		    /* speed and duplex are the same for mii and gmii */
		
		    miip->reg_autoneg_adv |= MII_AUTONEG_ADV_802_3;
		    write_mii_mgt(PHYADDR, MII_REG_AUTONEG_AD, 
				  &miip->reg_autoneg_adv);
		    write_mii_mgt(PHYADDR, MII_REG_CONTROL, &loc_reg_control);
		}

		/*
		 * we've changed the link, so we must bring it back up.
		 * notifying a link down will make this happen.
		 * But only if this is the currently active phy...
		 */
		if (active_phy == phy) {
		    link_down_notify();
		    LINK_TRACE(TRACE_TYPE_LNK_PHY, "stmacLNK", 0x99002, 
			       index, phy, active_phy, link_set);
		}

		/* If this is true, the setting is now changed, we need to put
		   new setting to hardware */
		if (BRINGUP_LINK_STATE != BRINGUP_LINK_STATE_STABILIZE_START) {
		    active_phy_fail = 1;
		    reset_phy = 1;
		    first_time_set = 1;
		}
		
		/*
		 * if we've got the priority bit set, force a restart of
		 * this PHY...
		 */
		if (link_set & LNK_PREF) {
		    link_down_notify();
		    /*
		     * insure that this phy gets tried next...
		     */
		    if (mii_device_attached != GMII_DEVICE_BCM5400) {
		        active_phy_fail = 1;
			active_phy = ATTACHED_PHY_IF_MII << 1;
		    }
		}

	    } else {

		/* disable the PHY */
		write_mii_mgt(PHYADDR, MII_REG_AUTONEG_AD, &zero);
		write_mii_mgt(PHYADDR, MII_REG_CONTROL, &zero);

		/*
		 * we've changed the link, so we must bring it back up.
		 * notifying a link down will make this happen.
		 */
		reset_phy = 1;
		mac_mgt_state[index].link_state_fast &= ~LNK_ENABLE;
		if (active_phy == phy) {
		    link_down_notify();
		    LINK_TRACE(TRACE_TYPE_LNK_PHY, "stmacLNK", 0x99003, 
			       index, phy, active_phy, link_set);
		}

		/* If this is true, the setting is now changed, we need to put
		   new setting to hardware */
		if (BRINGUP_LINK_STATE != BRINGUP_LINK_STATE_STABILIZE_START) {
		    active_phy_fail = 1;
		    reset_phy = 1;
		    first_time_set = 1;
		}
		
	    }

	    break;	/* ATTACHED_PHY_IF_MII */
	    
	case ATTACHED_PHY_IF_GMII:
	    /* see if we're being asked to negotiate this */
	    if (link_set & LNK_NEGOTIATE) {
		gig_negotiate = 1;
	    } else {
		gig_negotiate = 0;
	    }

	    /* save it away */
	    mac_mgt_state[index].link_set_gig = link_set;

	    /* are we disabled */
	    if (!(link_set & LNK_ENABLE)) {
		if (gig_negotiate) {
		    set_mii_reg(GMII_REG_AUTONEG_AD, 
				OR_OP, 
				BASE_PAGE_REM_FAULT_OFFLINE);

		    BRINGUP_LINK_STATE = BRINGUP_LINK_STATE_RESET_START;
		}

		mac_mgt_state[index].link_state_gig &= ~LNK_ENABLE;
		link_down_notify();
		return;
	    }

	    if (gig_negotiate) {

		if ((link_set & 
		    (LNK_TX_FLOW_CTL_Y | LNK_RX_FLOW_CTL_Y)) == 
		    (LNK_TX_FLOW_CTL_Y | LNK_RX_FLOW_CTL_Y)) {
		    gig_negotiate_options |= BASE_PAGE_PAUSE_SYM_BCM;
		    mac_mgt_state[index].tx_fctl_gig = TG_MAC_TX_STATE_ENA_FC;
		    mac_mgt_state[index].rx_fctl_gig = TG_MAC_RX_STATE_ENA_FC;
		} else if (link_set & LNK_TX_FLOW_CTL_Y) {
		    gig_negotiate_options |= BASE_PAGE_PAUSE_ASYM_REMOTE_BCM;
		    mac_mgt_state[index].tx_fctl_gig = TG_MAC_TX_STATE_ENA_FC;
		} else if (link_set & LNK_RX_FLOW_CTL_Y) {
		    gig_negotiate_options |= BASE_PAGE_PAUSE_ASYM_LOCAL_BCM;
		    mac_mgt_state[index].rx_fctl_gig = TG_MAC_RX_STATE_ENA_FC;
		} else {
		    gig_negotiate_options |= BASE_PAGE_PAUSE_NONE_BCM;
		}

	    }
	    else {

		if (link_set & LNK_TX_FLOW_CTL_Y) {
		    mac_mgt_state[index].tx_fctl_gig = TG_MAC_TX_STATE_ENA_FC;
		} else {
		    mac_mgt_state[index].tx_fctl_gig = 0;
		}

		if (link_set & LNK_RX_FLOW_CTL_Y) {
		    mac_mgt_state[index].rx_fctl_gig = TG_MAC_RX_STATE_ENA_FC;
		} else {
		    mac_mgt_state[index].rx_fctl_gig = 0;
		}
	    }

	    link_down_notify();
	    
	    /* If this is true, the setting is now changed, we need to put
	       new setting to hardware */
	    if (BRINGUP_LINK_STATE != BRINGUP_LINK_STATE_STABILIZE_START) {
		active_phy_fail = 1;
		reset_phy = 1;
		first_time_set = 1;
	    }

	    break;

	case ATTACHED_PHY_IF_SERDES_INT:
	    /* fall through */

	case ATTACHED_PHY_IF_SERDES_EXT:
	   
	    /* remove impossible options */
	    link_set &= ~(LNK_10MB | LNK_100MB | LNK_HALF_DUPLEX);
	    /* add required options */
	    link_set |= LNK_FULL_DUPLEX;

	    /* save it away */
	    mac_mgt_state[index].link_set_gig = link_set;

	    /* see if we're being asked to negotiate this */
	    if ((link_set & LNK_NEGOTIATE) && !link_sense_mode) {
		gig_negotiate = 1;
	    } else {
		gig_negotiate = 0;
	    }

	    /* are we disabled */
	    if (!(link_set & LNK_ENABLE)) {
#if (TIGON_REV & TIGON_REV5)
		trp->mac_control.mac_serdes_config =
		    TG_MAC_SERDES_RESET |
		    TG_MAC_SERDES_PDOWNT |
		    TG_MAC_SERDES_PDOWNR |
		    TG_MAC_SERDES_PDOWNB;
		good_phy = 0;
#endif
		
		if (gig_negotiate) {
		    gig_negotiate_options = 
			CONFIG_BASE_REM_FAULT_OFFLINE | CONFIG_BASE_FD;
		    BRINGUP_LINK_STATE = BRINGUP_LINK_STATE_RESET_START;
		}

		mac_mgt_state[index].link_state_gig &= ~LNK_ENABLE;
		if (active_phy != ATTACHED_PHY_IF_MII)
		    link_down_notify();
		return;
	    }

	    if (gig_negotiate) {
		gig_negotiate_options = CONFIG_BASE_FD;

		if ((link_set & 
			(LNK_TX_FLOW_CTL_Y | LNK_RX_FLOW_CTL_Y)) == 
		    (LNK_TX_FLOW_CTL_Y | LNK_RX_FLOW_CTL_Y)) {
		    gig_negotiate_options |= CONFIG_BASE_PAUSE_SYM;
		} else if (mac_mgt_state[index].link_set_gig & 
		    LNK_TX_FLOW_CTL_Y) {
		    gig_negotiate_options |= CONFIG_BASE_PAUSE_ASYM_REMOTE;
		} else if (mac_mgt_state[index].link_set_gig & 
		    LNK_RX_FLOW_CTL_Y) {
		    gig_negotiate_options |= CONFIG_BASE_PAUSE_ASYM_LOCAL;
		} else {
		    gig_negotiate_options |= CONFIG_BASE_PAUSE_NONE;
		}
	    } else {
		if (link_set & LNK_TX_FLOW_CTL_Y) {
		    mac_mgt_state[index].tx_fctl_gig = TG_MAC_TX_STATE_ENA_FC;
		} else {
		    mac_mgt_state[index].tx_fctl_gig = 0;
		}

		if (link_set & LNK_RX_FLOW_CTL_Y) {
		    mac_mgt_state[index].rx_fctl_gig = TG_MAC_RX_STATE_ENA_FC;
		} else {
		    mac_mgt_state[index].rx_fctl_gig = 0;
		}
	    }

	    /*
	     * reset the link 
	     */
	    if (active_phy != ATTACHED_PHY_IF_MII) {
		link_down_notify();
	    }

	    break;	/* ATTACHED_PHY_IF_SERDES_EXT */
	    
	default:
	    /* XXX error */
    }
}

#define DELAY_CNT 10
U32 delay_cnt=0;

U32
get_mac_link(U32 index, U32 phy)
{
    U16 mii_reg_vendor = 0;
    U16 mii_reg_status = 0;
#if (TIGON_REV & TIGON_REV5)
    U16 mii_status_sum = 0;
#endif    
    U32 speed = 0;
    U32 duplex = 0;
    U32 link_state = 0;

    switch (phy) {
	case ATTACHED_PHY_IF_MII:

	if (mac_mgt_state[index].link_set_fast & LNK_ENABLE)
	    link_state = LNK_ENABLE;
	else {
	    /* disabled, just return the request */
	    link_state = mac_mgt_state[index].link_set_fast;
	    link_state &= ~LNK_UP;
	    mac_mgt_state[index].link_state_fast = link_state;
	    return(link_state);
	}

	/* read once to clear latch... */
	read_mii_mgt(PHYADDR, MII_REG_STATUS, &mii_reg_status);
	/* and again for new status */
	read_mii_mgt(PHYADDR, MII_REG_STATUS, &mii_reg_status);

	if (mii_reg_status & MII_STATUS_LINK_UP) {

	    switch (mii_device_attached) {
	    case MII_DEVICE_ICS_1892_0:
	    case MII_DEVICE_ICS_1890_2:
		/* read once to clear latch... */
		read_mii_mgt(1, MII_REG_ICS_QUICK_STATUS, 
			     &mii_reg_vendor);
		/* and again for new status */
		read_mii_mgt(1, MII_REG_ICS_QUICK_STATUS, 
			     &mii_reg_vendor);
		
		if (mii_reg_vendor & MII_ICS_QUICKPOLL_FD) {
		    duplex = LNK_FULL_DUPLEX;
		} else {
		    duplex = LNK_HALF_DUPLEX;
		}
		
		if (mii_reg_vendor & MII_ICS_QUICKPOLL_100) {
		    speed = LNK_100MB;
		} else {
		    speed = LNK_10MB;
		}
		link_state |= speed | duplex | LNK_UP;
		break;
		
	    case MII_DEVICE_LXT_970:
		/* read once to clear latch... */
		read_mii_mgt(1, MII_REG_LEVL1_CHIPSTAT, 
			     &mii_reg_vendor);
		/* and again for new status */
		read_mii_mgt(1, MII_REG_LEVL1_CHIPSTAT, 
			     &mii_reg_vendor);
		
		if (mii_reg_vendor & MII_LEVL1_CHIPSTAT_FD) {
		    duplex = LNK_FULL_DUPLEX;
		} else {
		    duplex = LNK_HALF_DUPLEX;
		}
		
		if (mii_reg_vendor & MII_LEVL1_CHIPSTAT_100) {
		    speed = LNK_100MB;
		} else {
		    speed = LNK_10MB;
		}
		link_state |= speed | duplex | LNK_UP;
		break;
		
#if (TIGON_REV & TIGON_REV5)
	    case GMII_DEVICE_BCM5400:
		/* read once to clear latch... */
		read_mii_mgt(PHYADDR, GMII_REG_STATUS_SUMRY,
			     &mii_status_sum);
		/* and again for new status */
		read_mii_mgt(PHYADDR, GMII_REG_STATUS_SUMRY,
			     &mii_status_sum);
		
		switch(mii_status_sum & COS_1000T_FD) {
		case COS_1000T_FD:
		    speed = LNK_1000MB;
		    duplex = LNK_FULL_DUPLEX;
		    break;
		case COS_1000T_HD:
		    speed = LNK_1000MB;
		    duplex = LNK_HALF_DUPLEX;
		    break;
		case COS_100TX_FD:
		    speed = LNK_100MB;
		    duplex = LNK_FULL_DUPLEX;
		    break;
		case COS_100TX_HD:
		    speed = LNK_100MB;
		    duplex = LNK_HALF_DUPLEX;
		    break;
		case COS_10T_FD:
		    speed = LNK_10MB;
		    duplex = LNK_FULL_DUPLEX;
		    break;
		case COS_10T_HD:
		    speed = LNK_10MB;
		    duplex = LNK_HALF_DUPLEX;
		    break;
		case COS_100T4:
		default:
		    break;
		}
		
		if (speed == LNK_1000MB) {
		    link_state = (speed | duplex) & ~LNK_UP;
		    /* quickly change to GMII interface, 
		     * since it takes 4 sec
		     * to switch to different interface 
		     */
		    active_phy_fail = 1;
		    active_phy = ATTACHED_PHY_IF_GMII;
		    reset_phy = 1; /* to reset the phy */
		    first_time_set = 1;
		    active_phy_fail_timeout = trp->gen_control.timer;
		    
		} else if (speed == LNK_10MB) {

		    if (set_10mb == 0) {
			link_state = (speed | duplex) & ~LNK_UP;
			set_mii_reg(MII_REG_CONTROL, WR_OP, 
				    MII_CONTROL_RESET);
			WAIT_US(50);
			set_10mb = 1;
			active_phy_fail = 1;
			active_phy = ATTACHED_PHY_IF_MII;
			reset_phy = 1; /* to reset the phy */
			first_time_set = 1;
			active_phy_fail_timeout = trp->gen_control.timer;
			LINK_UTRACE("glnkmii", 0xc0540,
				mii_status_sum, active_phy, 
				speed, link_state);
		    } else {
			/* it may get link up and found out the MAC is not in
			   correct speed, then bring down the link to renegotiate,
			   this will cause extra link down event,  A delay is to
			   prevent it */
			if (delay_cnt < DELAY_CNT) {
			    delay_cnt++;
			    link_state = (speed | duplex) & ~LNK_UP;
			    break;
			} else {
			    delay_cnt = 0;
			}

			/* turn power on */
			set_mii_reg(0x18, WR_OP, 0x0404);
			
			mii_reg_status = 0x0002;
			write_mii_mgt(BCM5201_ADR, 0x1e,
				      &mii_reg_status);
			
			mii_reg_status = 0x0;
			write_mii_mgt(BCM5201_ADR, 0,
				      &mii_reg_status);

			/* normal mode */
			set_mii_reg(0x18, AND_OP, 0x0004);
			link_state |= speed | duplex | LNK_UP;
			LINK_UTRACE("glnkmii", 0xc0530,
				mii_status_sum, active_phy, 
				speed, link_state);
		    }
		} else {
		    /* it may get link up and found out the MAC is not in
		       correct speed, then bring down the link to renegotiate,
		       this will cause extra link down event,  A delay is to
		       prevent it */
		    if (delay_cnt < DELAY_CNT) {
			delay_cnt++;
			link_state = (speed | duplex) & ~LNK_UP;
			break;
		    } else {
			delay_cnt = 0;
		    }
		    link_state |= speed | duplex | LNK_UP;
		    LINK_UTRACE("glnkmii", 0xc0326,
				mii_status_sum, active_phy, 
				speed, link_state);
		}
		break;
#endif  /* (TIGON_REV & TIGON_REV5) */
		
	    case GMII_DEVICE_LV1_CHEETAH:
		/* read once to clear latch... */
		break;
		
	    case MII_DEVICE_UNKNOWN:
	    case MII_DEVICE_NONE:
	    default:
		/* XXX */
	    }
	} else {
	    link_state = mac_mgt_state[index].link_set_fast;
	    link_state &= ~LNK_UP;
	}
	
	if (link_ready) {
	    if (gig_negotiate) {
		link_state |= LNK_NEGOTIATE;
	    }
	    
	    if (mac_mgt_state[index].tx_fctl_fast) {
		link_state |= LNK_TX_FLOW_CTL_Y;
	    } else {
		link_state &= ~LNK_TX_FLOW_CTL_Y;
	    }
	    
	    if (mac_mgt_state[index].rx_fctl_fast) {
		link_state |= LNK_RX_FLOW_CTL_Y;
	    } else {
		link_state &= ~LNK_RX_FLOW_CTL_Y;
	    }
	}
	
	mac_mgt_state[index].link_state_fast = link_state;
	return(link_state);
	break;
	
    case ATTACHED_PHY_IF_GMII:
#if (TIGON_REV & TIGON_REV5)
	if (mac_mgt_state[index].link_set_gig & LNK_ENABLE)
	    link_state = LNK_ENABLE;
	else {
	    /* disabled, just return the request */
	    link_state = mac_mgt_state[index].link_set_gig;
	    link_state &= ~LNK_UP;
	    mac_mgt_state[index].link_state_gig = link_state;
	    return(link_state);
	}

	/* read once to clear latch... */
	read_mii_mgt(PHYADDR, GMII_REG_STATUS, &mii_reg_status);
	/* and again for new status */
	read_mii_mgt(PHYADDR, GMII_REG_STATUS, &mii_reg_status);
	
	
	if (mii_reg_status & MII_STATUS_LINK_UP) {
	    
	    switch (gmii_device_attached) {
	    case GMII_DEVICE_BCM5400:
		/* read once to clear latch... */
		read_mii_mgt(PHYADDR, GMII_REG_STATUS_SUMRY,
			     &mii_status_sum);
		/* and again for new status */
		read_mii_mgt(PHYADDR, GMII_REG_STATUS_SUMRY,
			     &mii_status_sum);
		
		switch(mii_status_sum & COS_1000T_FD) {
		case COS_1000T_FD:
		    speed = LNK_1000MB;
		    duplex = LNK_FULL_DUPLEX;
		    break;
		case COS_1000T_HD:
		    speed = LNK_1000MB;
		    duplex = LNK_HALF_DUPLEX;
		    break;
		case COS_100TX_FD:
		    speed = LNK_100MB;
		    duplex = LNK_FULL_DUPLEX;
		    break;
		case COS_100TX_HD:
		    speed = LNK_100MB;
		    duplex = LNK_HALF_DUPLEX;
		    break;
		case COS_10T_FD:
		    speed = LNK_10MB;
		    duplex = LNK_FULL_DUPLEX;
		    break;
		case COS_10T_HD:
		    speed = LNK_10MB;
		    duplex = LNK_HALF_DUPLEX;
		    break;
		case COS_100T4:
		default:
		    break;
		}
		
		if (speed != LNK_1000MB) {
		    link_state = (speed | duplex) & ~LNK_UP;
		    /* quickly change to MII interface, since it takes 8 sec
		     * to switch to different interface 
		     */
		    active_phy_fail = 1;
		    active_phy = ATTACHED_PHY_IF_MII;
		    reset_phy = 1; /* to reset the phy */
		    first_time_set = 1;
		    active_phy_fail_timeout = trp->gen_control.timer;
		    LINK_UTRACE("getmclnk",
				0xc0328,
				mii_status_sum, active_phy, 
				speed, link_state);
		    set_mii_reg(MII_REG_CONTROL, WR_OP, MII_CONTROL_RESET);
		    if (speed == LNK_10MB) {
			set_10mb = 1;
		    }
		} else {
		    
		    link_state |= (speed | duplex | LNK_UP);
		    set_10mb = 0;
		    LINK_UTRACE("glnkgmii", 0xc0300, 
				link_state,
				mac_mgt_state[index].link_state_gig,
				mac_mgt_state[index].link_set_gig,
				mii_status_sum);
	
		}		    
		
		break;
		
	    case GMII_DEVICE_LV1_CHEETAH:
		/* read once to clear latch... */
		break;
		
	    default:
		/* XXX */
	    }
	} else {
	    link_state = mac_mgt_state[index].link_state_gig;
	    link_state &= ~LNK_UP;
	}
	
	if (link_ready) {
	    if (gig_negotiate) {
		link_state |= LNK_NEGOTIATE;
	    }
	    
	    if (mac_mgt_state[index].tx_fctl_gig) {
		link_state |= LNK_TX_FLOW_CTL_Y;
	    } else {
		link_state &= ~LNK_TX_FLOW_CTL_Y;
	    }
	    
	    if (mac_mgt_state[index].rx_fctl_gig) {
		link_state |= LNK_RX_FLOW_CTL_Y;
	    } else {
		link_state &= ~LNK_RX_FLOW_CTL_Y;
	    }
	}	        
	mac_mgt_state[index].link_state_gig = link_state;
	
	return(link_state);
#endif  /* (TIGON_REV & TIGON_REV5) */
	break;

	case ATTACHED_PHY_IF_SERDES_INT:
	    /* fall through */

	case ATTACHED_PHY_IF_SERDES_EXT:

	    if (mac_mgt_state[index].link_set_gig & LNK_ENABLE)
		link_state = LNK_ENABLE;
	    else {
		/* disabled, just return the request */
	        link_state = mac_mgt_state[index].link_set_gig;
		link_state &= ~LNK_UP;
	        mac_mgt_state[index].link_state_gig = link_state;
		return(link_state);
	    }

	    link_state |= LNK_1000MB | LNK_FULL_DUPLEX;

	    if (link_ready) {

	        link_state |= LNK_UP;

		if (link_sense_mode)
		    link_state |= LNK_SENSE_NO_NEG;

		if (gig_negotiate) {
		    link_state |= LNK_NEGOTIATE;
		}

		if (mac_mgt_state[index].tx_fctl_gig) {
		    link_state |= LNK_TX_FLOW_CTL_Y;
		} else {
		    link_state &= ~LNK_TX_FLOW_CTL_Y;
		}

		if (mac_mgt_state[index].rx_fctl_gig) {
		    link_state |= LNK_RX_FLOW_CTL_Y;
		} else {
		    link_state &= ~LNK_RX_FLOW_CTL_Y;
		}

		/*
		 * is the link partner another Alteon device?
		 */
		if (link_partner_alteon)
		    link_state |= LNK_ALTEON;

		/*
		 * can the remote end handle large frames
		 */
		if (link_partner_jumbo)
		    link_state |= LNK_JUMBO;

		/*
		 * Is the remote end a NIC?
		 */
		if (link_partner_nic)
		    link_state |= LNK_NIC;

	    } else {
	        link_state = mac_mgt_state[index].link_set_gig;
		link_state &= ~LNK_UP;
	    }
	    mac_mgt_state[index].link_state_gig = link_state;
	    return(link_state);

	    break;

	default:
	    /* XXX error */
    }
    return(link_state);
}

void
set_mac_active(U32 index)
{
    U32 tx_state = trp->mac_control.mac_tx_state;
    U32 rx_state = trp->mac_control.mac_rx_state;

    U32 link_set;
    U32 link_state;

#if (TIGON_REV & TIGON_REV5)
    U16 resolv;
    U16 loc_i, rem_i;       /* PS2, PS1 as indices */
#endif /* (TIGON_REV & TIGON_REV5) */

    switch (active_phy) {
	    
	case ATTACHED_PHY_IF_MII:

	    link_set = mac_mgt_state[index].link_set_fast;
	    link_state = mac_mgt_state[index].link_state_fast;

	    /* enable or disable */
	    if (link_set & LNK_ENABLE) {
		/* enable the MAC */
		tx_state |= TG_MAC_TX_STATE_ENABLE;

		/* MAC duplex setting must match PHY state */
	   	if (link_state & LNK_FULL_DUPLEX) {
		    tx_state |= TG_MAC_TX_STATE_ENA_FULL_DUPLEX;

		    if (mac_mgt_state[index].tx_fctl_fast) {
			tx_state |= TG_MAC_TX_STATE_ENA_FC;
			link_state |= LNK_TX_FLOW_CTL_Y;
		    } else {
			tx_state &= ~TG_MAC_TX_STATE_ENA_FC;
			link_state &= ~LNK_TX_FLOW_CTL_Y;
		    }

		    if (mac_mgt_state[index].rx_fctl_fast) {
			rx_state |= TG_MAC_RX_STATE_ENA_FC;
			link_state |= LNK_RX_FLOW_CTL_Y;
		    } else {
			rx_state &= ~TG_MAC_RX_STATE_ENA_FC;
			link_state &= ~LNK_RX_FLOW_CTL_Y;
		    }

		} else {
		    /* MAC duplex state must match PHY duplex state */
		    /* Also disable FC for half duplex operation */

		    tx_state &= 
			~(TG_MAC_TX_STATE_ENA_FULL_DUPLEX |
			  TG_MAC_TX_STATE_ENA_FC);

		    rx_state &= ~TG_MAC_RX_STATE_ENA_FC;
		    mac_mgt_state[index].tx_fctl_fast = 0;
		    mac_mgt_state[index].rx_fctl_fast = 0;

		    link_state &= 
			~(LNK_TX_FLOW_CTL_Y | LNK_RX_FLOW_CTL_Y);

		}
		/*
		 * disable mac_rx_state reg so we can modify control
		 * bits
		 */
		DISABLE_MAC_RX();

		trp->mac_control.mac_tx_state = tx_state;
		trp->mac_control.mac_rx_state = rx_state;

		ENABLE_MAC_RX();

	    } else {
		/* disable the MAC */
		trp->mac_control.mac_tx_state &= 
		    ~TG_MAC_TX_STATE_ENABLE;
		DISABLE_MAC_RX();

                link_state &= ~LNK_ENABLE;
		link_down_notify();
	    }
	    mac_mgt_state[index].link_state_fast = link_state;

	    break;	/* ATTACHED_PHY_IF_MII */
	    
	case ATTACHED_PHY_IF_GMII:
#if (TIGON_REV & TIGON_REV5)

	    if (gig_negotiate) {
		read_mii_mgt(PHYADDR, GMII_REG_AUTONEG_AD, &loc_i);
		read_mii_mgt(PHYADDR, GMII_REG_AUTONEG_LINK_PA, &rem_i);

		loc_i = (loc_i & BASE_PAGE_PAUSE_MASK) >> 10;
		rem_i = (rem_i & BASE_PAGE_PAUSE_MASK) >> 10;
		resolv = fc_pri_local_resolution[rem_i][loc_i];

	       /*
		*we've resolved what to do, now set up our MAC appropriately...
		*/
		switch(resolv) {
		case LOC_FC_NONE:
		  trp->mac_control.mac_tx_state &= ~TG_MAC_TX_STATE_ENA_FC;
		  trp->mac_control.mac_rx_state &= ~TG_MAC_RX_STATE_ENA_FC;
		  mac_mgt_state[TIGON_MAC_INDEX].tx_fctl_gig = 0;
		  mac_mgt_state[TIGON_MAC_INDEX].rx_fctl_gig = 0;
		  break;
		  
		case LOC_FC_TX:
		  trp->mac_control.mac_tx_state |= TG_MAC_TX_STATE_ENA_FC;
		  trp->mac_control.mac_rx_state &= ~TG_MAC_RX_STATE_ENA_FC;
		  mac_mgt_state[TIGON_MAC_INDEX].tx_fctl_gig = 
		      TG_MAC_TX_STATE_ENA_FC;
		  mac_mgt_state[TIGON_MAC_INDEX].rx_fctl_gig = 0;
		  break;
		  
		case LOC_FC_RX:
		  trp->mac_control.mac_tx_state &= ~TG_MAC_TX_STATE_ENA_FC;
		  trp->mac_control.mac_rx_state |= TG_MAC_RX_STATE_ENA_FC;
		  mac_mgt_state[TIGON_MAC_INDEX].tx_fctl_gig = 0;
		  mac_mgt_state[TIGON_MAC_INDEX].rx_fctl_gig = 
		      TG_MAC_RX_STATE_ENA_FC;
		  break;
		  
		case LOC_FC_BOTH:
		  trp->mac_control.mac_tx_state |= TG_MAC_TX_STATE_ENA_FC;
		  trp->mac_control.mac_rx_state |= TG_MAC_RX_STATE_ENA_FC;
		  mac_mgt_state[TIGON_MAC_INDEX].tx_fctl_gig = 
		      TG_MAC_TX_STATE_ENA_FC;
		  mac_mgt_state[TIGON_MAC_INDEX].rx_fctl_gig = 
		      TG_MAC_RX_STATE_ENA_FC;
		  break;
 
		default:
		  LINK_UTRACE("setmacact", 0x1ffff, rem_i, loc_i,
			      resolv, 0);
		  break;
		}
		trp->mac_control.mac_tx_state |= TG_MAC_TX_STATE_ENABLE;
		LINK_TRACE(TRACE_TYPE_LNK_PHY, "setmcact", 0xc0401, 
			   trp->mac_control.mac_tx_state,
			   trp->mac_control.mac_rx_state,
			   mac_mgt_state[TIGON_MAC_INDEX].link_state_gig,
			   mac_mgt_state[TIGON_MAC_INDEX].link_state_gig);
		break;
	    }
	    /* else fall through */
#endif  /* (TIGON_REV & TIGON_REV5) */

	    break;

	case ATTACHED_PHY_IF_SERDES_INT:
	    /* fall through */

	case ATTACHED_PHY_IF_SERDES_EXT:
	   
	    link_set = mac_mgt_state[index].link_set_gig;
	    link_state = mac_mgt_state[index].link_state_gig;

	    /* are we disabled */
	    if (!(link_set & LNK_ENABLE)) {
		if (!gig_negotiate) {
		    trp->mac_control.mac_tx_state |= TG_MAC_TX_STATE_ENABLE;
		    DISABLE_MAC_RX();
		}
                link_state &= ~LNK_ENABLE;
		link_down_notify();
		mac_mgt_state[index].link_state_gig = link_state;
		return;	/* early return */
	    }

	    if (!gig_negotiate || link_sense_mode) {

                tx_state |= TG_MAC_TX_STATE_ENA_FULL_DUPLEX;

		if (link_set & LNK_TX_FLOW_CTL_Y) {
		    mac_mgt_state[index].tx_fctl_gig = TG_MAC_TX_STATE_ENA_FC;
                    tx_state |= TG_MAC_TX_STATE_ENA_FC;
		} else {
		    mac_mgt_state[index].tx_fctl_gig = 0;
                    tx_state &= ~TG_MAC_TX_STATE_ENA_FC;
		}

		if (link_set & LNK_RX_FLOW_CTL_Y) {
		    mac_mgt_state[index].rx_fctl_gig = TG_MAC_RX_STATE_ENA_FC;
                    rx_state |= TG_MAC_RX_STATE_ENA_FC;
		} else {
		    mac_mgt_state[index].rx_fctl_gig = 0;
                    rx_state &= ~TG_MAC_RX_STATE_ENA_FC;
		}

		/*
		 * disable mac_rx_state before setting any control bits...
		 */
		DISABLE_MAC_RX();

		trp->mac_control.mac_tx_state = tx_state;
		trp->mac_control.mac_rx_state = rx_state;

		ENABLE_MAC_RX();
	    }
	    mac_mgt_state[index].link_state_gig = link_state;

	    break;	/* ATTACHED_PHY_IF_SERDES_EXT */
	    
	default:
	    /* XXX error */
    }
}


