/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996 
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: mii.c $
 * Revision copper_fhwang/36 2000/06/03 02:25:23 fhwang
 * 	to sync with ge_latest
 * 
 * Revision copper_fhwang/29 2000/04/13 01:48:12 fhwang
 * 	for broadcom errata on no HCD link
 * 
 * Revision copper_fhwang/25 2000/03/28 01:42:08 fhwang
 * 	to handle errata only for rev C chip
 * 
 * Revision copper_fhwang/22 2000/03/07 22:25:35 fhwang
 * 	remove some trace message
 * 
 * Revision copper_fhwang/19 2000/03/01 02:39:04 fhwang
 * 	add external loopback support for tx1000
 * 
 * Revision copper_fhwang/15 2000/02/19 00:08:39 fhwang
 * 	change gmii register setting for broadcom errata and ege rate setting
 * 
 * Revision copper_fhwang/13 2000/01/25 01:58:52 fhwang
 * 	set speed/duplex as manual mode for loopback
 * 
 * Revision copper_fhwang/12 2000/01/24 21:45:12 fhwang
 * 	to allow 1000 speed for loopback
 * 
 * Revision copper_fhwang/8 1999/10/12 19:35:59 fhwang
 * 	to support hw misc_0 & misc_2 swap
 * 
 * Revision copper_fhwang/6 1999/09/04 19:26:58 fhwang
 * 	remove 1000 speed from manual mode
 * 
 * Revision copper_fhwang/5 1999/09/03 02:54:03 fhwang
 * 	set bit 10 of register 0x18 to 1 as recommended
 * 
 * Revision copper_fhwang/2 1999/08/27 02:21:10 fhwang
 * 	add GMII interface for tx1000
 * 
 * Revision 1.1.2.9  1998/05/27  01:19:32  fhwang
 * 	remove 100 and FD when enable autonegotiate mode, and add restart_auto mode
 * 	[1998/05/27  00:55:28  fhwang]
 *
 * Revision 1.1.2.8  1998/01/03  02:16:45  kyung
 * 	fast enet board support - added by J.H.
 * 	[1998/01/03  02:16:30  kyung]
 * 
 * Revision 1.1.2.7  1997/10/21  21:59:37  tibor
 * 	converted macros to subr
 * 	[1997/10/21  21:57:17  tibor]
 * 
 * Revision 1.1.2.6  1997/10/10  20:38:05  hayes
 * 	cleaned up tracing
 * 	[1997/10/09  21:39:35  hayes]
 * 
 * Revision 1.1.2.5  1997/10/06  23:53:34  hayes
 * 	more adjustments to ICS 1890 initialization
 * 	[1997/10/06  23:53:17  hayes]
 * 
 * Revision 1.1.2.4  1997/09/17  03:26:30  hayes
 * 	Reset override register after using it.
 * 	Changed trace strategy.
 * 	[1997/09/15  22:23:20  hayes]
 * 
 * Revision 1.1.2.3  1997/09/09  02:57:33  hayes
 * 	Add support for various MII devices
 * 	Add device support for Level One LXT 970 in
 * 	addition to ICS 1890
 * 	[1997/09/09  02:57:13  hayes]
 * 
 * 	completed link code restructuring
 * 	[1997/08/29  22:10:59  hayes]
 * 
 * Revision 1.1.2.2  1997/08/29  22:11:56  hayes
 * 	completed link code restructuring
 * 
 * Revision 1.1.2.1  1997/08/26  21:08:59  hayes
 * 	initial checkin
 * 	[1997/08/26  21:01:29  hayes]
 * 
 * $EndLog$
 */

#include "link.h"
#include "link_if.h"
#include "link_op.h"
#include "mii.h"
#include "zconf.h"
#include "np.h"
#include "mac.h"
#include "timer.h"
#include "trace.h"

#if defined(_SP_CONF_H_)
#include "sp_regs.h"
#endif /* defined(_SP_CONF_H_) */

/*
 * phy reset state variables
 */
U32 mii_reset_state = RESET_PHY_MII_RESET;
U32 mii_reset_timer;
U32 mii_device_attached;

U32 gmii_reset_state = RESET_PHY_GMII_RESET;
U32 gmii_reset_timer;
U32 gmii_device_attached;
extern U16 mii_phy_id[2];

mii_pair_t mii_set[4][4] = {    /* duplex, speed */
    {           /* no duplex - error */
        {0, 0}, /* x, x */
        {0, 0}, /* x, 10 */
        {0, 0}, /* x, 100 */
        {0, 0}  /* x, 10/100 */
    },
    {   /* full duplex */
        {0, 0},         /* FD, x */
        {MII_CONTROL_SPEED_10 | MII_CONTROL_FULL_DUPLEX, 0}, /* FD, 10 */
        {MII_CONTROL_SPEED_100 | MII_CONTROL_FULL_DUPLEX, 0},/* FD, 100 */
        {MII_CONTROL_AUTONEG_ENA | MII_CONTROL_RESTART_AUTONEG,
         MII_AUTONEG_ADV_10_FD | MII_AUTONEG_ADV_100_FD}/* FD, 10/100 */
    },
    {   /* half duplex */
        {0, 0},          /* HD, x */
        {MII_CONTROL_SPEED_10 | MII_CONTROL_HALF_DUPLEX, 0}, /* HD, 10 */
        {MII_CONTROL_SPEED_100 | MII_CONTROL_HALF_DUPLEX, 0},/* HD, 100 */
        {MII_CONTROL_AUTONEG_ENA | MII_CONTROL_RESTART_AUTONEG,
         MII_AUTONEG_ADV_10_HD | MII_AUTONEG_ADV_100_HD} /* HD, 10/100 */
    },
    {   /* full/half duplex */
        {0, 0},         /* FD/HD, x */
        {MII_CONTROL_AUTONEG_ENA | MII_CONTROL_RESTART_AUTONEG,
         MII_AUTONEG_ADV_10_FD | MII_AUTONEG_ADV_10_HD},/* FD/HD, 10 */
        {MII_CONTROL_AUTONEG_ENA | MII_CONTROL_RESTART_AUTONEG,
         MII_AUTONEG_ADV_100_FD | MII_AUTONEG_ADV_100_HD},/* FD/HD, 100 */
        {MII_CONTROL_AUTONEG_ENA | MII_CONTROL_RESTART_AUTONEG,
         MII_AUTONEG_ADV_10_FD | MII_AUTONEG_ADV_100_FD |
         MII_AUTONEG_ADV_10_HD | MII_AUTONEG_ADV_100_HD}/* FD/HD, 10/100 */
    }
};
 

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
 * Function: read_mii_mgt
 *
 * Params:
 *         U32 device   -
 *         U32 reg      -
 *         U16 *data    -
 *
 * Return: None
 *
 * Side effects:
 *
 * Description:
 *
 * MII mgt frame format
 *
 * preamble - 32 bits of 1
 * start of frame - 01
 * operation - 10 (read), 01 (write)
 * phyaddr - which mac entity to talk to - 5 bits.
 * register addr - which MII reg to talk to - 5 bits.
 * ta - turn around - 2 bits
 * data - 16 bits
 */
 
void
read_mii_mgt(U32 device, U32 reg, U16 *data)
{
    U32 i, bit;
 
    *data = 0;
 
    /*  preamble */
    for (i=0; i<32; i++) {
        write_mii_bit(1);
    }
 
    /* SOF */
    write_mii_bit(0);
    write_mii_bit(1);
 
    /* operation - read */
    write_mii_bit(1);
    write_mii_bit(0);
 
    /* phyaddr */
    for (i=0x10; i; i>>=1) {
        bit = device & i;
        if (bit) {
            write_mii_bit(1);
        } else {
            write_mii_bit(0);
        }
    }
 
    /* register addr */
    for (i=0x10; i; i>>=1) {
        bit = reg & i;
        if (bit) {
            write_mii_bit(1);
        } else {
            write_mii_bit(0);
        }
    }
 
    /* TA */
    write_mii_z();
    write_mii_z();
 
    /* read reg */
    for (i=0x8000; i; i>>=1) {
        if (read_mii_bit()) {
            *data |= i;
        }
    }
    write_mii_z();
    return;
}
 
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
 * Function: write_mii_mgt
 *
 * Params:
 *         U32 device   -
 *         U32 reg      -
 *         U16 *data    -
 *
 * Return: None
 *
 * Side effects:
 *
 * Description:
 *
 */
void
write_mii_mgt(U32 device, U32 reg, U16 *data)
{
U32 i;
U32 bit;
 
    /*  preamble */
    for (i=0; i<32; i++) {
        write_mii_bit(1);
    }
 
    /* SOF */
    write_mii_bit(0);
    write_mii_bit(1);
 
    /* operation - write */
    write_mii_bit(0);
    write_mii_bit(1);
 
    /* phyaddr */
    for (i=0x10; i; i>>=1) {
        bit = device & i;
        if (bit) {
            write_mii_bit(1);
        } else {
            write_mii_bit(0);
        }
    }
 
    /* register addr */
    for (i=0x10; i; i>>=1) {
        bit = reg & i;
        if (bit) {
            write_mii_bit(1);
        } else {
            write_mii_bit(0);
        }
    }
 
    /* TA */
    write_mii_bit(1);
    write_mii_bit(0);
 
    /* data */
    for (i=0x8000; i; i>>=1) {
        bit = *data & i;
        if (bit) {
            write_mii_bit(1);
        } else {
            write_mii_bit(0);
        }
    }
    write_mii_z();
    return;
 
}

/* need to save across invocations... */
U32 mii_reset_state_last = 0;
U32 gmii_reset_state_last = 0;

void
reset_mii_phy(void)
{
    U16 mii_data;

    if (mii_reset_state != mii_reset_state_last) {
	LINK_TRACE(TRACE_TYPE_LNK_PHY, "rst_mii", 0xd0101, 
	    mii_reset_state, 0, 0, 0);
	mii_reset_state_last = mii_reset_state;
    }

#if (TIGON_REV & TIGON_REV5)
    if (mii_device_attached == GMII_DEVICE_BCM5400) {
	LINK_TRACE(TRACE_TYPE_LNK_PHY, "rst_gmii", 0xd0210, 
	    gmii_reset_state, mii_reset_state, mii_device_attached, 0);
        reset_gmii_phy();
	return;
    }
#endif  /* (TIGON_REV & TIGON_REV5) */

    switch (mii_reset_state) {
        case RESET_PHY_MII_RESET:

            write_mii_z();
            mii_data = MII_CONTROL_RESET;
            write_mii_mgt(PHYADDR, MII_REG_CONTROL, &mii_data);
            mii_reset_state = RESET_PHY_MII_RESET_WAIT;
            break;
 
        case RESET_PHY_MII_RESET_WAIT:
            /* wait for the reset bit to clear */
            read_mii_mgt(PHYADDR, MII_REG_CONTROL, &mii_data);
            if (!(mii_data & MII_CONTROL_RESET))
                mii_reset_state = RESET_PHY_MII_REG_CONTROL;
            break;
 
        case RESET_PHY_MII_REG_CONTROL:
            /*
             * set control register to enable autonegotiation,
             */
	    /* start auto negotiation */
	    mii_data = MII_CONTROL_RESTART_AUTONEG |
		MII_CONTROL_AUTONEG_ENA;
	    write_mii_mgt(PHYADDR, MII_REG_CONTROL, &mii_data);
            mii_reset_timer = trp->gen_control.timer;
            mii_reset_state = RESET_PHY_MII_REG_CONTROL_WAIT;
            break;
 
        case RESET_PHY_MII_REG_CONTROL_WAIT:
            /* wait 100uS */
            if (((mii_reset_timer + MII_TIMER) -
                trp->gen_control.timer) >= MII_TIMER)

		switch (mii_device_attached) {
		    case MII_DEVICE_ICS_1892_0:
		    case MII_DEVICE_ICS_1890_2:
			mii_reset_state = RESET_PHY_MII_ICS_REG18;
			break;

		    case MII_DEVICE_LXT_970:
			mii_reset_state = RESET_PHY_MII_LEVL1_REG19;
			break;

		    case MII_DEVICE_UNKNOWN:
			mii_reset_state = RESET_PHY_MII_READY;
			break;

		    case MII_DEVICE_NONE:
		    default:
			/* XXX */
			mii_reset_state = RESET_PHY_MII_READY;
		}
            break;

	/*
	 * XXX The following states are specific to the ICS 1890
	 * device
	 */
        case RESET_PHY_MII_ICS_REG18:
            /*
             * XXX this is ICS 1890 specific
             * set the 10Base-T operations register to
             * inhibit jabber isolation, disable loopback,
             * and disable SQE test
             */
            mii_data = 0x0034;
            write_mii_mgt(PHYADDR, 18, &mii_data);
            mii_reset_timer = trp->gen_control.timer;
            mii_reset_state = RESET_PHY_MII_ICS_REG18_WAIT;
            break;
 
        case RESET_PHY_MII_ICS_REG18_WAIT:
            /* wait 100uS */
            if (((mii_reset_timer + MII_TIMER) -
                trp->gen_control.timer) >= MII_TIMER) {
		if (mii_device_attached == MII_DEVICE_ICS_1892_0) {
		    /* For ICS 1892 PHY, we don't need to do
		     * the workarounds that we have to do for 1890
		     * PHY.
		     */
		    mii_reset_state = RESET_PHY_MII_READY;
		}    
	        else
                    mii_reset_state = RESET_PHY_MII_ICS_REG19;
	    }
            break;
 
        case RESET_PHY_MII_ICS_REG19:
            /*
             * XXX this is ICS 1890 specific
             * set the extended control register to
             * autonegotiate with qualified 10Base-T data and
             * protect 10Base-T data from 100Base-TX data.
             */
            mii_data = 0x0006;
            write_mii_mgt(PHYADDR, 19, &mii_data);
            mii_reset_timer = trp->gen_control.timer;
            mii_reset_state = RESET_PHY_MII_ICS_REG19_WAIT;
            break;
 
        case RESET_PHY_MII_ICS_REG19_WAIT:
            /* wait 100uS */
            if (((mii_reset_timer + MII_TIMER) -
                trp->gen_control.timer) >= MII_TIMER)
		mii_reset_state = RESET_PHY_MII_ICS_REG24;
            break;
        case RESET_PHY_MII_ICS_REG24:
	    /*
	     * There is a bug in the ICS 1890 where a test mux is left enabled.
	     * Disabling this will result in a savings of ~20ma in power.
	     * To disable, pin 20 must be grounded (this will be in switch rev d
	     * and later implementations) and then bit 0 of register 24
	     * (register overwrite bit) must be set, followed by setting
	     * bits 8, 9, and 10 of register 23.  Do not disturb the other
	     * bits in the registers.
	     */
            read_mii_mgt(PHYADDR, 24, &mii_data);
            mii_data |= 0x0001;        /* set bit 0 */
            write_mii_mgt(PHYADDR, 24, &mii_data);
            mii_reset_timer = trp->gen_control.timer;
            mii_reset_state = RESET_PHY_MII_ICS_REG24_WAIT;
            break;
 
        case RESET_PHY_MII_ICS_REG24_WAIT:
            /* wait 100uS */
            if (((mii_reset_timer + MII_TIMER) -
                trp->gen_control.timer) >= MII_TIMER)
                mii_reset_state = RESET_PHY_MII_ICS_REG23;
            break;
 
        case RESET_PHY_MII_ICS_REG23:
            read_mii_mgt(PHYADDR, 23, &mii_data);
            mii_data |= 0x0700; /* set bits 8, 9 and 10 */
            write_mii_mgt(PHYADDR, 23, &mii_data);
            mii_reset_timer = trp->gen_control.timer;
            mii_reset_state = RESET_PHY_MII_ICS_REG23_WAIT;
            break;

        case RESET_PHY_MII_ICS_REG23_WAIT:
            /* wait 100uS */
            if (((mii_reset_timer + MII_TIMER) -
                trp->gen_control.timer) >= MII_TIMER)
                mii_reset_state = RESET_PHY_MII_ICS_REG20;
            break;

	/*
	 * the following sets bit 4 in register 20, which allows
	 * the ICS part to enable the acceptance of undelimited
	 * packets.  See ICS 1890 PR1A and PR1B release note, 
	 * July 8, 1997 for more details
	 */
        case RESET_PHY_MII_ICS_REG20:
            read_mii_mgt(PHYADDR, 20, &mii_data);
            mii_data |= 0x0010; /* set bit 4 */
            write_mii_mgt(PHYADDR, 20, &mii_data);
            mii_reset_timer = trp->gen_control.timer;
            mii_reset_state = RESET_PHY_MII_ICS_REG20_WAIT;
            break;
 
        case RESET_PHY_MII_ICS_REG20_WAIT:
            /* wait 100uS */
            if (((mii_reset_timer + MII_TIMER) -
                trp->gen_control.timer) >= MII_TIMER)
                mii_reset_state = RESET_PHY_MII_ICS_REG24_2;
            break;

        case RESET_PHY_MII_ICS_REG24_2:
	    /*
	     * disable register overwrite bit...
	     */
            read_mii_mgt(PHYADDR, 24, &mii_data);
            mii_data &= ~0x0001;        /* clear bit 0 */
            write_mii_mgt(PHYADDR, 24, &mii_data);
            mii_reset_timer = trp->gen_control.timer;
            mii_reset_state = RESET_PHY_MII_ICS_REG24_2_WAIT;
            break;
 
        case RESET_PHY_MII_ICS_REG24_2_WAIT:
            /* wait 100uS */
            if (((mii_reset_timer + MII_TIMER) -
                trp->gen_control.timer) >= MII_TIMER)
                mii_reset_state = RESET_PHY_MII_READY;
            break;
 
	/*
	 * XXX End of ICS 1890 device specific initialization
	 */

	/*
	 * XXX Start of Level 1 LXT 970 device specific initialization
	 */

        case RESET_PHY_MII_LEVL1_REG19:
	    /*
	     * Set the the LXT 970 configuration register for normal operation,
	     * DTE mode, MDIO interrupt disabled, loopback disabled, SQE
	     * disabled, jabber enabled, LEDC off, normal TX clk cycle,
	     * decode, enable 100TP interface
	     */
            mii_data = 0x0840;
            write_mii_mgt(PHYADDR, 19, &mii_data);
            mii_reset_timer = trp->gen_control.timer;
            mii_reset_state = RESET_PHY_MII_LEVL1_REG19_WAIT;
            break;
 
        case RESET_PHY_MII_LEVL1_REG19_WAIT: 
           /* wait 100uS */
            if (((mii_reset_timer + MII_TIMER) -
                trp->gen_control.timer) >= MII_TIMER)
                mii_reset_state = RESET_PHY_MII_READY;
            break;
	/*
	 * XXX End of Level 1 LXT 970 device specific initialization
	 */

        case RESET_PHY_MII_READY:
	    break;

        default:
    }
}

U32 set_10mb = 0;
U32 first_time_set = 1;
#if (TIGON_REV & TIGON_REV5)
void
reset_gmii_phy(void)
{
    U16 gmii_data, gmii_data1, gmii_data2, gmii_data3, gmii_data4;
    U16 gmii_data13;
    U32 link_set = 
        (active_phy == ATTACHED_PHY_IF_GMII)?
        mac_mgt_state[TIGON_MAC_INDEX].link_set_gig:
        mac_mgt_state[TIGON_MAC_INDEX].link_set_fast;

    if (gmii_reset_state != gmii_reset_state_last) {
	/*
	LINK_TRACE(TRACE_TYPE_LNK_PHY, "rst_gmii", 0xd0201,
	*/
	LINK_UTRACE("rst_gmii", 0xd0201, 
	    gmii_reset_state, first_time_set, link_set, active_phy);
	gmii_reset_state_last = gmii_reset_state;
    }

    switch (gmii_reset_state) {
    case RESET_PHY_GMII_RESET:
	set_mii_reg(MII_REG_CONTROL, WR_OP, MII_CONTROL_RESET);

	/* to enable link ready change signal */
	gmii_data = GMII_THREE_LED_MODE;
	set_mii_reg(GMII_REG_EXTND_CONTROL, AND_OP, gmii_data);
	
	gmii_reset_state = RESET_PHY_GMII_RESET_WAIT;
	break;
	
    case RESET_PHY_GMII_RESET_WAIT:
	/* wait for the reset bit to clear */
	read_mii_mgt(PHYADDR, GMII_REG_CONTROL, &gmii_data);
	if (!(gmii_data & GMII_CONTROL_RESET))
	    gmii_reset_state = RESET_PHY_GMII_REG_CONTROL;
	gmii_reset_state = RESET_PHY_GMII_REG_CONTROL;
	break;
	
    case RESET_PHY_GMII_REG_CONTROL:
	/*
	 * set control register to enable autonegotiation,
	 */
	
	/* to enable link ready change signal */
	gmii_data = GMII_THREE_LED_MODE;
	set_mii_reg(GMII_REG_EXTND_CONTROL, AND_OP, gmii_data);

	if (set_10mb == 1) {
	    
	    read_mii_mgt(PHYADDR, 0x18, &gmii_data);
	    read_mii_mgt(PHYADDR, 0x18, &gmii_data);
	    
	    set_mii_reg(0x18, WR_OP, 0x0404); /* turn power on */
	    
	    read_mii_mgt(PHYADDR, 0x18, &gmii_data1);
	    read_mii_mgt(PHYADDR, 0x18, &gmii_data1);
	    
	    read_mii_mgt(BCM5201_ADR, 0x1e, &gmii_data2);
	    read_mii_mgt(BCM5201_ADR, 0x1e, &gmii_data2);
	    
	    gmii_data4 = 0x0002;
	    write_mii_mgt(BCM5201_ADR, 0x1e, &gmii_data4);
	    
	    read_mii_mgt(BCM5201_ADR, 0x1e, &gmii_data3);
	    read_mii_mgt(BCM5201_ADR, 0x1e, &gmii_data3);
	    
	    gmii_data13 = 0x0;
	    write_mii_mgt(BCM5201_ADR, 0, &gmii_data13);
	    set_mii_reg(0x18, AND_OP, 0x0004);
	    
	}
	
	if (first_time_set == 1) {
	    first_time_set = 0;

	    /* according broadcom errata for rev C,
	     * 1. to raise bit10 in register 9 to 1,
	     * to set master preference mode
	     * 2. to disable power management, enable it after link-up
	     */
	    if ((mii_phy_id[1] >= B5400_REV_C0) &&
	        (mii_phy_id[1] <= B5400_REV_C5)) {
		    
		set_mii_reg(0x09, OR_OP, 0x400); /* master preference */
		set_mii_reg(0x18, OR_OP, 0x20); /* disable power management */
	    }

	    set_mii_reg(0x18, OR_OP, 0x2000); /* set edge rate */
	    
	    if (!(link_set & LNK_ENABLE))
		return;
	    
	    /* see if we're being asked to negotiate this */
	    if (link_set & LNK_NEGOTIATE) {
		gig_negotiate = 1;
	    } else {
		gig_negotiate = 0;
	    }
	    switch(gmii_device_attached) {
	    case GMII_DEVICE_BCM5400:
		if (gig_negotiate) {
		    
		    /* program register 9 for 1000 advertisement */
		    gmii_data = 0;
		    if (link_set & LNK_1000MB) {
			if (link_set & LNK_FULL_DUPLEX)
			    gmii_data |= GMII_ADV_1000_FD;
			if (link_set & LNK_HALF_DUPLEX)
			    gmii_data |= GMII_ADV_1000_HD;
			set_mii_reg(GMII_REG_1000T_CONTROL, WR_OP, gmii_data);
		    } else {
			set_mii_reg(GMII_REG_1000T_CONTROL, WR_OP, gmii_data);
		    }
		    
		    
		    /* program register 4 for 10/100 advertisement */
		    gmii_data1 = GMII_AUTONEG_ADV_802_3;
		    if (link_set & LNK_FULL_DUPLEX) {
			if (link_set & LNK_100MB)
			    gmii_data1 |= GMII_AUTONEG_ADV_100_FD;
			if (link_set & LNK_10MB)
			    gmii_data1 |= GMII_AUTONEG_ADV_10_FD;
		    }
		    
		    if (link_set & LNK_HALF_DUPLEX) {
			if (link_set & LNK_100MB)
			    gmii_data1 |= GMII_AUTONEG_ADV_100_HD;
			if (link_set & LNK_10MB)
			    gmii_data1 |= GMII_AUTONEG_ADV_10_HD;
		    }
		    /* set PS1 PS2 bit */
		    if ((link_set & (LNK_TX_FLOW_CTL_Y | LNK_RX_FLOW_CTL_Y))== 
			(LNK_TX_FLOW_CTL_Y | LNK_RX_FLOW_CTL_Y)) {
			gmii_data1 |= BASE_PAGE_PAUSE_SYM_BCM;
		    } else if (link_set & LNK_TX_FLOW_CTL_Y) {
			gmii_data1 |= BASE_PAGE_PAUSE_ASYM_REMOTE_BCM;
		    } else if (link_set & LNK_RX_FLOW_CTL_Y) {
			gmii_data1 |= BASE_PAGE_PAUSE_ASYM_LOCAL_BCM;
		    } else {
			gmii_data1 |= BASE_PAGE_PAUSE_NONE_BCM;
		    }
		    set_mii_reg(GMII_REG_AUTONEG_AD, WR_OP, gmii_data1);
		    
		    
		    /* program register 0 to start auto negotiation */
		    gmii_data2 = 0;
		    
		    if ((link_set & LNK_1000MB) &&
			((link_set & LNK_LOOPBACK) ||
			(link_set & LNK_EXT_LOOPBACK))) {
			/* for loopback to work, it should be programmed as
			   manual mode */
			/* external loopback only for 1000MB */
			set_loopback(link_set);
		    } else if (link_set & LNK_LOOPBACK) {
			/* not 1000 MB */
			set_loopback(link_set);
		    } else {
			gmii_data2 |= (GMII_CONTROL_RESTART_AUTONEG | 
				       GMII_CONTROL_AUTONEG_ENA);
			set_mii_reg(GMII_REG_CONTROL, WR_OP, gmii_data2);
		    }
		    
		} else { /* if (gig_negotiate) */
		    gmii_data = 0;

		    if ((link_set & LNK_1000MB) &&
			((link_set & LNK_LOOPBACK) ||
			(link_set & LNK_EXT_LOOPBACK))) {
			/* external loopback only for 1000MB */
			set_loopback(link_set);
		    } else if (link_set & LNK_LOOPBACK) {
			/* not 1000 MB */
			set_loopback(link_set);

		    } else {
			
			if (link_set & LNK_HALF_DUPLEX)
			    gmii_data |= GMII_CONTROL_HALF_DUPLEX;
			else  /* set to default FULL DUPLEX mode */
			    gmii_data |= GMII_CONTROL_FULL_DUPLEX;

			if (link_set & LNK_10MB)
			    gmii_data |= GMII_CONTROL_SPEED_10;
			else if (link_set & LNK_100MB)
			    gmii_data |= GMII_CONTROL_SPEED_100;
			else /* set to default 100 speed,no 1000 for manual */
			    gmii_data |= GMII_CONTROL_SPEED_100;
			
			set_mii_reg(GMII_REG_CONTROL, WR_OP, gmii_data);
		    }
		    
		}
		break;
	    case GMII_DEVICE_LV1_CHEETAH:
		break;
	    default:
		break;
	    }
	}
	
	gmii_reset_state = RESET_PHY_GMII_REG_CONTROL_WAIT;
	gmii_reset_timer = trp->gen_control.timer;
	break;
	
    case RESET_PHY_GMII_REG_CONTROL_WAIT:
	/* wait 100 us */
	if (((gmii_reset_timer + MII_TIMER) - 
	     trp->gen_control.timer) >= MII_TIMER) {
	    
	    if (gmii_device_attached == GMII_DEVICE_BCM5400) {
	    }
	    else if (gmii_device_attached == GMII_DEVICE_LV1_CHEETAH) {
	    }
	    else {
	    }
	    /* clear attention mask */
	    trp->mac_control.mac_rx_state |= TG_MAC_RX_ATTN_MASK;
	    gmii_reset_state = RESET_PHY_GMII_REG_WAIT1;
	}

	break;
    case RESET_PHY_GMII_REG_WAIT1:
	gmii_reset_state = RESET_PHY_GMII_READY;
	mii_reset_state = RESET_PHY_MII_READY;
	break;
	
    case RESET_PHY_GMII_READY:
	break;
	
    default:
	break;
    }
}

#if 0 /* used only for debug */
void phy_status(char *msg, U16 bitptn, U16 reg1, U16 reg2, U16 reg3, U16 reg4)
{
    U16 gmii_data, gmii_data1, gmii_data2, gmii_data3;
		    
    if (bitptn & 0x1000) 
	read_mii_mgt(PHYADDR, reg1, &gmii_data);
    read_mii_mgt(PHYADDR, reg1, &gmii_data);
    
    if (bitptn & 0x100) 
	read_mii_mgt(PHYADDR, reg2, &gmii_data1);
    read_mii_mgt(PHYADDR, reg2, &gmii_data1);
    
    if (bitptn & 0x10) 
	read_mii_mgt(PHYADDR, reg3, &gmii_data2);
    read_mii_mgt(PHYADDR, reg3, &gmii_data2);
    
    if (bitptn & 0x1) 
	read_mii_mgt(PHYADDR, reg4, &gmii_data3);
    read_mii_mgt(PHYADDR, reg4, &gmii_data3);

    LINK_UTRACE(msg, 0xf0000, gmii_data, gmii_data1, 
		gmii_data2, gmii_data3 );
		    
}
#endif /* if 0 */

void set_loopback(U32 link_set)
{
    U16 gmii_data = 0;


    if (link_set & LNK_LOOPBACK) {
	gmii_data |= GMII_CONTROL_LOOPBACK;
	if (link_set & LNK_HALF_DUPLEX)
	    gmii_data |= GMII_CONTROL_HALF_DUPLEX;
	else  /* set to default FULL DUPLEX mode */
	    gmii_data |= GMII_CONTROL_FULL_DUPLEX;
	
	if (link_set & LNK_10MB)
	    gmii_data |= GMII_CONTROL_SPEED_10;
	else if (link_set & LNK_100MB)
	    gmii_data |= GMII_CONTROL_SPEED_100;
	else /* set to default 1000 speed, ok in loopback */
	    gmii_data |= GMII_CONTROL_SPEED_1000;
	
	set_mii_reg(GMII_REG_CONTROL, WR_OP, gmii_data);
    } else if (link_set & LNK_EXT_LOOPBACK) {
	set_mii_reg(0, WR_OP, 0x40);
	set_mii_reg(0x18, WR_OP, 0x0c00);
	set_mii_reg(0x17, WR_OP, 0x0400);
	set_mii_reg(0x16, WR_OP, 0x0006);
	set_mii_reg(0x17, WR_OP, 0x2500);
	set_mii_reg(0x16, WR_OP, 0x0006);
	set_mii_reg(0x17, WR_OP, 0x4600);
	set_mii_reg(0x16, WR_OP, 0x0006);
	set_mii_reg(0x17, WR_OP, 0x6700);
	set_mii_reg(0x16, WR_OP, 0x0006);
	set_mii_reg(0x1f, WR_OP, 0x0010);
	set_mii_reg(9, WR_OP, 0x1500);
	set_mii_reg(9, WR_OP, 0x1d00);
    }

    LINK_UTRACE("setloop", 0xe0100, 
	       link_set, gmii_data, active_phy, 0);
}

#endif  /* (TIGON_REV & TIGON_REV5) */
  

/*********************************************************************************
*
* Function:	read_mii_bit
*
* Purpose:	receive bit from phy
*		The MDC (managment data clock) is defined in section 22.2.2.11 of
*		the 802.3u spec.
*
* Parms:	none
*
* Error:	none
*
* Returns:	bit
*
*********************************************************************************/
U32
read_mii_bit(void)
{
    U32 bit;
    
    /* read */
    bit = TRP->gen_control.misc_local_control & TG_MLC_MII_DATA_IN;

    /* clock up */ 
    TRP->gen_control.misc_local_control |= TG_MLC_MII_CLK_OUT; 
    
	
    /* clock down */
    TRP->gen_control.misc_local_control &= ~TG_MLC_MII_CLK_OUT;
	
    return bit;
}


/*********************************************************************************
*
* Function:	write_mii_bit
*
* Purpose:	transmit bit from mii to phy
*		The MDC (managment data clock) is defined in section 22.2.2.11 of
*		the 802.3u spec.
*
* Parms:	bit - bit to send
*
* Error:	none
*
* Returns:	nothing
*
*********************************************************************************/
void
write_mii_bit(U32 bit)
{
    /* data */ 
    TRP->gen_control.misc_local_control = 
	(TRP->gen_control.misc_local_control & ~TG_MLC_MII_MASK) | 
	TG_MLC_MII_DATA_OUT_ENA | 
	(bit << 18);

    /* change clock up & down order */
    /* clock down */ 
    TRP->gen_control.misc_local_control &= ~TG_MLC_MII_CLK_OUT;
	 
    /* clock up */ 
    TRP->gen_control.misc_local_control |= TG_MLC_MII_CLK_OUT;
}


#if (TIGON_REV & TIGON_REV5)
/*********************************************************************************
*
* Function:	reset_mii_led
*
* Purpose:	to reset mii & led in the initial stage.
*
* Parms:	bit - bit to send
*
* Error:	none
*
* Returns:	nothing
*
*********************************************************************************/
void
reset_mii_led()
{
    /* WRITE 0 */
    TRP->gen_control.misc_local_control &= ~TG_MLC_MISC_OUT_2;
	 
    /* WRITE 1 */
    TRP->gen_control.misc_local_control |= TG_MLC_MISC_OUT_2;
}
#endif  /* (TIGON_REV & TIGON_REV5) */


/*********************************************************************************
*
* Function:	write_mii_z
*
* Purpose:	transmit 0 bit from mii to phy
*		The MDC (managment data clock) is defined in section 22.2.2.11 of
*		the 802.3u spec.
*
* Parms:	none
*
* Error:	none
*
* Returns:	nothing
*
*********************************************************************************/
void
write_mii_z(void)
{
    /* drop out enable */ 
    TRP->gen_control.misc_local_control = 
	(TRP->gen_control.misc_local_control & ~TG_MLC_MII_MASK);
    /* clock up */ 
    TRP->gen_control.misc_local_control |= TG_MLC_MII_CLK_OUT; 
    /* clock down */ 
    TRP->gen_control.misc_local_control &= ~TG_MLC_MII_CLK_OUT; 
}

void set_mii_reg(U16 addr, U16 op, U16 data)
{
    U16 gmii_data=data;
    
    if (op == WR_OP)
	write_mii_mgt(PHYADDR, addr, &gmii_data);
    else {
	read_mii_mgt(PHYADDR, addr, &gmii_data);
	if (op == OR_OP) 
	    gmii_data |= data;
	else if (op == AND_OP)
	    gmii_data &= ~data;
	write_mii_mgt(PHYADDR, addr, &gmii_data);
    }
}

