/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: cksum.c,v $
 * Revision 1.1.2.9  1999/01/14  00:03:48  shuang
 * 	cksum for ether crc may offset at odd byte.
 * 	[1999/01/13  23:57:15  shuang]
 *
 * Revision 1.1.2.8  1998/12/08  02:36:20  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:27:33  shuang]
 * 
 * Revision 1.1.2.7  1998/09/30  18:49:43  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:39:20  shuang]
 * 
 * Revision 1.1.2.6  1998/06/23  01:29:27  shuang
 * 	added dynamic nic mem support.
 * 	[1998/06/23  01:21:19  shuang]
 * 
 * Revision 1.1.2.5  1998/04/25  03:07:42  shuang
 * 	     add IP_FRAG_SUPPORT debug flag, add crc_cksum function
 * 	     [1998/04/25  03:06:46  shuang]
 * 
 * Revision 1.1.2.4  1998/04/24  00:00:29  hayes
 * 	     added const to rcsid definitions to eliminate compiler warnings
 * 	     [1998/04/23  23:58:47  hayes]
 * 
 * Revision 1.1.2.3  1998/04/11  05:15:00  shuang
 * 	     move fast timer to cpu B, add receive coalesed ticks
 * 	     [1998/04/11  04:57:41  shuang]
 * 
 * Revision 1.1.2.2  1998/03/26  01:34:19  shuang
 * 	     cleanup
 * 	     [1998/03/26  01:24:42  shuang]
 * 
 * Revision 1.1.2.1  1998/01/14  23:25:30  ted
 * 	     rearrange directories
 * 	     [1998/01/14  23:17:25  ted]
 * 
 * Revision 1.1.2.1  1997/11/19  19:22:57  ted
 * 	     Support Tigon and Tigon2 in the same driver
 * 	     [1997/11/19  19:18:13  ted]
 * 
 * Revision 1.1.2.4  1996/11/20  20:04:14  ted
 * 	     New include file structuring.
 * 	     [1996/11/20  19:49:04  ted]
 * 
 * Revision 1.1.2.3  1996/11/15  04:24:00  hayes
 * 	     convert to new header file structure.
 * 	     [1996/11/15  04:11:56  hayes]
 * 
 * Revision 1.1.2.2  1996/11/05  21:54:13  hayes
 * 	     Final pre-ODE merge
 * 	     [1996/11/05  21:51:13  hayes]
 * 
 * Revision 1.1.2.1  1996/11/01  18:24:45  ted
 * 	     change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	     [1996/10/29  22:05:40  ted]
 * 
 * $EndLog$
 */
/*
 * FILE cksum.c
 *
 * COPYRIGHT (c) Essential Communication Corp. 1995
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/cksum.c,v $
 * $Revision: 1.1.2.9 $ $Locker:  $
 * $Date: 1999/01/14 00:03:48 $ $Author: shuang $ $State: Exp $
 */

#ifndef __lint
static const char rcsid[] = "$Header: /projects/rcs/sw/ge/./nic/fw2/common/cksum.c,v 1.1.2.9 1999/01/14 00:03:48 shuang Exp $";
#endif /* __lint */


#include "alt_def.h"
#include "nic_conf.h"
#include "nic_api.h"
#include "tg.h"

#include "nic.h"
#include "proto.h"
#include "assert.h"
#include "trace.h"
#include "dma.h"
#include "ring.h"
#include "mac.h"
#include "timer.h"
#include "profile.h"
#include "cksum.h"


#define WRAP(b)         WRAP_BUF(b, buf_end, buf_size)

/*
 * return a 16-bit cksum
 */
U16
cksum16(U16 *buf, U32 nwords, U32 buf_end, U32 buf_size)
{
    U32 sum;
    U32 n;

    WRAP(buf);

    /* prevent wrap checking within loop */
    sum = 0;
    n = (U16 *)buf_end - buf;   /* how many words to buf end */

    if (nwords > n) {           /* wrap around */
        sum = sum16(buf, n);
        sum += sum16((U16 *)(buf_end - buf_size), nwords - n);
    } else {
        sum = sum16(buf, nwords);
    }
    sum = (sum >> 16) + (sum & 0xffff);
    sum = (sum >> 16) + (sum & 0xffff);

    return ((U16)sum);
}

/* null routine to assist with scratchpad setup */
void cksum16_end(void) {}

/*
 * return cksum of 4 CRC bytes
 */
U16
crc_cksum(U8 *buf)
{
    register U32 sum;

    if ((U32)buf & 1) { /* start at odd address */
	if ((U32)(buf + 4) < NIC_RX_BUF_END) {
	    sum = ((buf[1] + buf[3]) << 8) + (buf[0] + buf[2]);
	} else {
	    RX_WRAP_BUF(buf);
	    sum = *buf++;
	    RX_WRAP_BUF(buf);
	    sum += (*buf++ << 8);
	    RX_WRAP_BUF(buf);
	    sum += *buf++;
	    RX_WRAP_BUF(buf);
	    sum += (*buf++ << 8);
	}
    } else { /* start at even address */
	if ((U32)(buf + 4) < NIC_RX_BUF_END) {
	    sum = ((buf[0] + buf[2]) << 8) + (buf[1] + buf[3]);
	} else {
	    RX_WRAP_BUF(buf);
	    sum = (*buf++ << 8);
	    RX_WRAP_BUF(buf);
	    sum += *buf++;
	    RX_WRAP_BUF(buf);
	    sum += (*buf++ << 8);
	    RX_WRAP_BUF(buf);
	    sum += *buf;
	}
    }
    sum = (sum >> 16) + (sum & 0xffff);
    sum = (sum >> 16) + (sum & 0xffff);

    NIC_TRACE(TRACE_TYPE_RECV, "crcSm", 0x49100, 
	buf,
	sum,
	0,
	0);

    return ((U16)sum);
}


#if (IP_SANITY)

/*
 * return 1's complement of tcp/udp cksum
 */
U16
tcp_udp_cksum(U32 buf, U32 buf_end, U32 buf_size)
{
    U32 sum;
    U32 len;
    U32 hlen;
    U16 *bp;
    ip_t *ip;
    
    WRAP(buf);
    ip = (ip_t *)buf;

    /* calculate pseudo header */
    bp = &ip->ipLen;
    WRAP(bp);
    sum = *bp;

    /* subtract ip header len */
    hlen = ((ip->ipVerHdrLen & IP_HLEN_MASK) << 2);
    sum -= hlen;

    len = sum; /* tcp/udp data length */

    NIC_TRACE(TRACE_TYPE_SEND|TRACE_TYPE_RECV, "tcpUdpSm", 0x49200, 
	buf,
	buf_end,
	buf_size,
	len);

    /* add protocol type */
    bp = (U16 *)&ip->ipProtocol;
    WRAP(bp);
    sum += *(U8 *)bp;

    /* add source IP addr */
    bp = (U16 *)&ip->ipSourceAddr;
    WRAP(bp);
    sum += *bp++;
    WRAP(bp);
    sum += *bp++;

    /* add dest IP addr */
    WRAP(bp);
    sum += *bp++;
    WRAP(bp);
    sum += *bp++;

    /* add rest */
    sum += (*nicfp->cksum16_stub)
	    ((U16 *)((U32)buf + hlen), (len >> 1), buf_end, buf_size);

    /* add the last byte for an odd case */
    if (len & 1) {
        bp = (U16 *)((U32)buf + hlen + len - 1); /* last odd byte */
        WRAP(bp);
        sum += (*(U8 *)bp << 8);
    }

    /* add in carry */
    sum = (sum >> 16) + (sum & 0xffff);
    sum = (sum >> 16) + (sum & 0xffff);

    NIC_TRACE(TRACE_TYPE_SEND|TRACE_TYPE_RECV, "tcpUdpSm", 0x49300, 
	buf,
	sum,
	hlen,
	len);
    
    return ((U16)(~sum)); /* 1's complement */
}

void
rx_ip_sanity(U32 frame_start, U32 frame_len)
{
    /* first 32 bytes are contiguous */

    eth_t *eth = (eth_t *)(frame_start);
    ip_t *ip = (ip_t *)((U32)frame_start + ETH_HLEN);

    NIC_TRACE(TRACE_TYPE_SEND|TRACE_TYPE_RECV, "ipHdrSm", 0x49400,
	frame_start,
	frame_len,
	eth->ethFrameType,
	ip->ipLen);

    if (eth->ethFrameType != ETH_IP)
        return;

    if ((frame_len > 64) && ((frame_len - ETH_HLEN) != ip->ipLen)) {
        NIC_UTRACE("?IpLen", 0x49500,
	    frame_start,
	    frame_len,
	    (frame_len - ETH_HLEN),
	    ip->ipLen);
        ASSERT(0);
    }
#if (0)
    if ((ip->ipProtocol == IP_PROTO_TCP) || (ip->ipProtocol == IP_PROTO_UDP)) {

        U32 cksum = tcp_udp_cksum((U32)ip, NIC_RX_BUF_END, NIC_RX_BUF_SIZE);
        if (cksum != 0 && cksum != 0xffff) {
	    NIC_UTRACE("tcpUdpCk", 0x49600,
		cksum,
		ip,
		ip->ipProtocol,
		0);
            ASSERT(0);
        }
    }
#endif /* XXX */
    return;
}

#endif /* (IP_SANITY) */

