/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: command.c $
 * Revision nic_api_12_4_latest_shuang/7 2000/02/23 00:14:05 shuang
 * 	keep nicSendBDs stats from being cleared by TG_CMD_CLEAR_STATS cmd.
 * 
 * Revision nic_api_12_4_latest_shuang/4 2000/02/12 18:50:05 shuang
 * 	fix incorrect stats nicRecvBDs and nicJumboRecvBDs.
 * 	honor TG_CMD_CODE_STACK_UP and TG_CMD_CODE_STACK_DOWN commands.
 * 
 * Revision nic_api_12_4_latest_shuang/2 1999/11/13 22:03:57 shuang
 * 	fix ext add/del mcast addr command
 * 
 * Revision 1.1.2.28  1999/01/20  19:49:49  shuang
 * 	avoid race condition on clearing any mbox event
 * 	[1999/01/20  19:48:06  shuang]
 *
 * Revision 1.1.2.27  1999/01/16  02:55:34  shuang
 * 	use of mini ring and jumbo ring is optional
 * 	[1999/01/16  02:52:56  shuang]
 * 
 * Revision 1.1.2.26  1998/12/08  02:36:22  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:27:37  shuang]
 * 
 * Revision 1.1.2.25  1998/09/30  18:49:46  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:39:28  shuang]
 * 
 * Revision 1.1.2.24  1998/08/27  23:22:43  shuang
 * 	add new mcast and mbox cmds, intr masks, obsolete recv ret cons idx
 * 	[1998/08/27  23:20:28  shuang]
 * 
 * Revision 1.1.2.23  1998/08/03  16:14:22  hayes
 * 	complete firmware update for NT
 * 	[1998/08/03  16:11:14  hayes]
 * 
 * 	checkin so merge w/ fc can happen
 * 	[1998/07/29  23:15:22  hayes]
 * 
 * 	host based send ring now works off RCB flag
 * 	[1998/07/29  18:30:52  hayes]
 * 
 * 	host based send ring works
 * 	[1998/07/28  03:57:14  hayes]
 * 
 * Revision 1.1.2.22  1998/07/29  22:31:00  shuang
 * 	add feature - flow control support
 * 	[1998/07/29  21:59:35  shuang]
 * 
 * Revision 1.1.2.21  1998/06/26  02:47:11  shuang
 * 	need to clear hw macstats area also for clearing stats command.
 * 	[1998/06/26  02:46:26  shuang]
 * 
 * Revision 1.1.2.20  1998/06/11  23:30:20  shuang
 * 	Added comment for jump table generated from switch-case statement.
 * 	[1998/06/11  23:19:17  shuang]
 * 
 * Revision 1.1.2.19  1998/05/14  01:25:21  shuang
 * 	Fix incorrect using TG_MAC_TX_ATTN_MASK to mask off Rx attention bits.
 * 	[1998/05/14  01:20:09  shuang]
 * 
 * Revision 1.1.2.18  1998/04/25  01:42:25  shuang
 * 	added const to rcsid to shut off compiler warning
 * 	[1998/04/25  01:05:11  shuang]
 * 
 * Revision 1.1.2.17  1998/04/15  17:39:08  ted
 * 	Fix refresh stats command
 * 	[1998/04/15  17:31:12  ted]
 * 
 * Revision 1.1.2.16  1998/04/11  05:15:01  shuang
 * 	move fast timer to cpu B, add receive coalesed ticks
 * 	[1998/04/11  04:57:45  shuang]
 * 
 * Revision 1.1.2.15  1998/04/01  07:32:19  shuang
 * 	add features: pass (CRC, bad, partial) frame, timer tick to 5us...
 * 	[1998/04/01  07:27:20  shuang]
 * 
 * Revision 1.1.2.14  1998/03/27  18:24:58  shuang
 * 	Avoid race condition on clearing mailbox command event
 * 	[1998/03/27  18:21:06  shuang]
 * 
 * Revision 1.1.2.13  1998/03/26  01:34:21  shuang
 * 	cleanup
 * 	[1998/03/26  01:24:49  shuang]
 * 
 * Revision 1.1.2.11  1998/03/04  22:28:35  hayes
 * 	     enable scratchpad usage, uncluding stats refreesh
 * 	     [1998/03/04  22:19:47  hayes]
 * 
 * Revision 1.1.2.10  1998/02/24  23:55:45  shuang
 * 	     Create mcast.h mcast.c for multicast functions
 * 	     [1998/02/24  23:51:21  shuang]
 * 
 * Revision 1.1.2.9  1998/02/24  17:41:45  shuang
 * 	     Modulize multicast functions
 * 	     [1998/02/24  17:32:39  shuang]
 * 
 * Revision 1.1.2.8  1998/02/21  21:07:37  hayes
 * 	     protect RX state reg from inadvertantly clearing attn bits
 * 	     [1998/02/21  20:44:15  hayes]
 * 
 * Revision 1.1.2.7  1998/02/19  01:29:44  suresh
 * 	     add dummy function to ease scratchpad setup
 * 	     [1998/02/19  01:14:54  suresh]
 * 
 * Revision 1.1.2.6  1998/02/11  19:11:19  ted
 * 	     Hack fix for stats reset, this needs redone when scratchpad is put back in
 * 	     [1998/02/11  18:38:45  ted]
 * 
 * 	     intermediate checkin for merge
 * 	     [1998/02/10  00:28:27  ted]
 * 
 * Revision 1.1.2.5  1998/02/09  17:18:04  hayes
 * 	     convert Q_DMA_XXX macros to functions
 * 	     [1998/02/09  17:16:51  hayes]
 * 
 * Revision 1.1.2.4  1998/02/03  21:51:29  ted
 * 	     Use new codes for TG_EVENT_MULTICAST_LIST_UPDATED events to match API spec
 * 	     [1998/02/03  21:50:39  ted]
 * 
 * Revision 1.1.2.3  1998/01/29  19:37:17  ted
 * 	     Fix broken syntax
 * 	     [1998/01/29  19:23:29  ted]
 * 
 * Revision 1.1.2.2  1998/01/29  18:40:17  ted
 * 	     Add multicast mode support and move some things into the fast nic structure
 * 	     [1998/01/29  18:28:47  ted]
 * 
 * Revision 1.1.2.1  1998/01/14  23:25:38  ted
 * 	     rearrange directories
 * 	     [1998/01/14  23:17:34  ted]
 * 
 * Revision 1.1.2.8  1998/01/06  02:46:13  ted
 * 	     remove noisy trace, Fix multicast on Tigon 2
 * 	     [1998/01/06  02:42:41  ted]
 * 
 * Revision 1.1.2.7  1998/01/05  01:31:46  ted
 * 	     Close window of opportunity in TX processing
 * 	     [1998/01/05  00:56:04  ted]
 * 
 * Revision 1.1.2.6  1997/12/19  18:09:47  ted
 * 	     Fix multicast delete bug
 * 	     [1997/12/19  18:09:26  ted]
 * 
 * Revision 1.1.2.5  1997/12/12  05:34:36  hayes
 * 	     setting phy characteristics of inactive link no longer causes link up/down
 * 	     [1997/12/12  04:01:31  hayes]
 * 
 * Revision 1.1.2.4  1997/11/30  00:57:31  hayes
 * 	     hack to insure set_mac_link() works until multiple phy's are supported in mgt.
 * 	     [1997/11/30  00:55:45  hayes]
 * 
 * Revision 1.1.2.3  1997/11/25  22:07:09  ted
 * 	     Fix up statistics
 * 	     [1997/11/25  22:00:05  ted]
 * 
 * Revision 1.1.2.2  1997/11/21  23:54:19  hayes
 * 	     MII link down works
 * 	     [1997/11/21  23:54:00  hayes]
 * 
 * 	     MII interface works
 * 	     [1997/11/21  22:45:35  hayes]
 * 
 * Revision 1.1.2.1  1997/11/19  19:23:02  ted
 * 	     Support Tigon and Tigon2 in the same driver
 * 	     [1997/11/19  19:18:37  ted]
 * 
 * Revision 1.15.7.22  1997/11/14  19:16:15  hayes
 * 	     first checkin after mongo merge...
 * 	     [1997/11/14  19:15:33  hayes]
 * 
 * 	     checkin after stats restructuring
 * 	     [1997/11/12  19:01:07  hayes]
 * 
 * 	     intermediate checkin
 * 	     [1997/11/12  01:30:33  hayes]
 * 
 * Revision 1.15.7.21  1997/11/11  19:05:52  hayes
 * 	     restructuring for scratchpad use
 * 	     [1997/11/11  19:04:10  hayes]
 * 
 * Revision 1.15.7.20  1997/11/10  00:27:24  hayes
 * 	     update mcast filtering for Tigon2- this still needs to be tested
 * 	     [1997/11/10  00:21:31  hayes]
 * 
 * Revision 1.15.7.19  1997/10/21  22:55:58  hayes
 * 	     more REV5 fixes
 * 	     [1997/10/21  22:55:06  hayes]
 * 
 * Revision 1.15.7.18  1997/10/21  01:35:29  skapur
 * 	     Merge of release 2.0 and ge_latest branch
 * 	     [1997/10/21  01:30:47  skapur]
 * 
 * Revision 1.15.7.17  1997/09/20  19:44:44  hayes
 * 	     fix up for TIGON_REV5
 * 	     [1997/09/20  19:44:22  hayes]
 * 
 * Revision 1.15.7.16  1997/09/16  21:46:24  ted
 * 	     Change from MIB-II V1 (RFC 1213) to MIB-II V2 (RFC 1573)
 * 	     [1997/09/16  21:41:50  ted]
 * 
 * Revision 1.15.7.15  1997/09/16  02:27:45  skapur
 * 	     Merged with changes from 1.15.7.14
 * 	     [1997/09/16  02:27:36  skapur]
 * 
 * 	     Added stats for Jumbo Grams; Cleaned up the code and cleand up taces
 * 	     [1997/09/16  02:21:26  skapur]
 * 
 * Revision 1.15.7.14  1997/09/11  18:05:36  ted
 * 	     Mixing vlans, jumbos, and redundancy works, plus small fixes from release 1.1 and better NIC tracing
 * 	     [1997/09/11  17:59:21  ted]
 * 
 * Revision 1.15.7.13  1997/08/29  22:12:04  hayes
 * 	     completed link code restructuring
 * 
 * Revision 1.15.26.2  1997/08/29  22:09:11  hayes
 * 	     completed link code restructuring
 * 
 * Revision 1.15.7.12  1997/08/23  17:40:02  skapur
 * 	     Solaris jumbogram support
 * 	     [1997/08/23  17:37:45  skapur]
 * 
 * Revision 1.15.7.11  1997/08/13  18:45:50  ted
 * 	     Fix compile bug when tracing enabled
 * 	     [1997/08/13  18:37:25  ted]
 * 
 * Revision 1.15.7.10  1997/07/25  22:58:54  ted
 * 	     Merge fixes from ge_0_22 and add link negotiation support
 * 	     [1997/07/25  22:31:58  ted]
 * 
 * 	     Architectural performance change
 * 	     [1997/07/24  21:44:59  ted]
 * 
 * Revision 1.15.7.9  1997/07/09  19:32:34  skapur
 * 	     Adding multicast filtering support
 * 	     [1997/07/09  19:31:48  skapur]
 * 
 * Revision 1.15.7.8  1997/07/05  01:12:45  ted
 * 	     Copy new MAC addr into statistics area when it's been updated.  Fixes bug 345.
 * 	     [1997/07/05  01:12:32  ted]
 * 
 * Revision 1.15.7.7  1997/05/22  22:25:25  suresh
 * 	     Statistics tweak - remove nicCmdsGetStats from tg_stats
 * 	     Also bump up nicCmdsClearStats counter in the fw when stats cleared
 * 	     via "altstat -C". Adjust altstat.c to print nicCmdsClearStats counter and
 * 	     not print nicCmdsGetStats.
 * 	     [1997/05/22  22:24:46  suresh]
 * 
 * Revision 1.15.7.6  1997/05/08  22:57:55  skapur
 * 	     Changed the ifType from 1(other) to 6 (ethernet)
 * 	     [1997/05/08  22:57:27  skapur]
 * 
 * Revision 1.15.7.5  1997/05/07  23:50:23  skapur
 * 	     Some interface related Stats (rfc 1213) needs to be re-initialized after the stats are cleared by the host
 * 	     [1997/05/07  23:48:42  skapur]
 * 
 * Revision 1.15.7.4  1997/05/03  04:58:41  skapur
 * 	     Cleaned up stats code
 * 	     [1997/05/03  04:57:58  skapur]
 * 
 * Revision 1.15.7.3  1997/04/11  23:36:08  ted
 * 	     Misc. cleanup and added test code for SBUS HW Hack
 * 	     [1997/04/11  23:13:12  ted]
 * 
 * Revision 1.15.7.2  1997/03/24  08:34:16  ted
 * 	     New event ring handling, producer lives in host memory, misc. other more modest changes
 * 	     [1997/03/24  08:27:37  ted]
 * 
 * Revision 1.15.7.1  1997/03/21  04:15:31  skapur
 * 	     Performance enhancement for Recv and Send; lots changes; almost complete overhaul of the entire firmware
 * 	     [1997/03/21  04:07:57  skapur]
 * 
 * Revision 1.15.2.6  1996/12/18  03:20:50  skapur
 * 	     Added support for 2 new commands, i.e. TG_CMD_CLEAR_STATS and
 * 	     TG_CMD_SET_MULTICAST_MODE
 * 	     [1996/12/18  03:16:13  skapur]
 * 
 * Revision 1.15.2.5  1996/12/05  20:07:02  hayes
 * 	     Added hardware promiscuous mode enable and disable support.
 * 	     Added hardware set mac addr support.
 * 	     [1996/12/05  20:01:47  hayes]
 * 
 * Revision 1.15.2.4  1996/11/20  20:04:15  ted
 * 	     Redo include file structuring.
 * 	     [1996/11/20  19:50:00  ted]
 * 
 * Revision 1.15.2.3  1996/11/15  04:24:01  hayes
 * 	     convert to new header file structure.
 * 	     [1996/11/15  04:11:57  hayes]
 * 
 * Revision 1.15.2.2  1996/11/05  21:54:14  hayes
 * 	     Final pre-ODE merge
 * 	     [1996/11/05  21:51:16  hayes]
 * 
 * Revision 1.15.2.1  1996/11/01  18:24:46  ted
 * 	     change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	     [1996/10/29  22:05:47  ted]
 * 
 * $EndLog$
 */
/*
 * FILE command.c
 *
 * COPYRIGHT (c) Essential Communication Corp. 1995
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/command.c,v $
 * $Revision: 1.1.2.28 $ $Locker:  $
 * $Date: 1999/01/20 19:49:49 $ $Author: shuang $ $State: Exp $
 */

#ifndef __lint
static const char rcsid[] = "$Header: /projects/rcs/sw/ge/./nic/fw2/common/command.c,v 1.1.2.28 1999/01/20 19:49:49 shuang Exp $";
#endif /* __lint */


#include "alt_def.h"
#include "nic_conf.h"
#include "nic_api.h"
#include "tg.h"

#include "nic.h"
#include "proto.h"
#include "trace.h"
#include "timer.h"
#include "dma.h"
#include "ring.h"
#include "link_if.h"    /* for tg_link_set */
#include "link_op.h"    /* for tg_link_set */
#include "profile.h"
#include "assert.h"


/* mbox event */
#define MBOX_EVENT_PCI_INTR		(1 << 0)
#define MBOX_EVENT_COMMAND		(1 << 1)
#define MBOX_EVENT_SEND_PROD		(1 << 2)
#define MBOX_EVENT_RECV_PROD		(1 << 3)
#define MBOX_EVENT_RECV_JUMBO_PROD	(1 << 4)
#define MBOX_EVENT_RECV_MINI_PROD	(1 << 5)
#define MBOX_EVENT_UNKNOWNS		~(MBOX_EVENT_PCI_INTR | \
					  MBOX_EVENT_COMMAND | \
					  MBOX_EVENT_SEND_PROD | \
					  MBOX_EVENT_RECV_PROD | \
					  MBOX_EVENT_RECV_JUMBO_PROD | \
					  MBOX_EVENT_RECV_MINI_PROD | \
					  0)
  

/*
 * limits for TG_CMD_SET_PARTIAL_RECV_COUNT
 * ie, 2-byte length field minus 4 bytes of CRC
 */
#define MAX_PARTIAL_RECV_COUNT	(0xffff - 4)

/* globals */
U32 mh_command_stub;


/*
 * process a command.
 * command producer index is found in the second mailbox register
 * all the do while loops below are to avoid the race condition on
 * clearing the mailbox event
 */
void
mh_command(void)
{
    struct tg_command cmd;
    U32 mbox_event;
    U32 cmd_cons;

    mbox_event = trp->gen_control.mailbox_event;

    /* note the priority */
    if (mbox_event & MBOX_EVENT_SEND_PROD) {
	do {
	    nicfp->send_prod_index = tsp->mbox_send_prod_index;
	    trp->gen_control.mailbox_event = MBOX_EVENT_SEND_PROD; /* clear */
	    nicfp->stats.nicRingSetSendProdIndex++;
	} while (!(trp->gen_control.mailbox_event & MBOX_EVENT_SEND_PROD) &&
		 (tsp->mbox_send_prod_index != nicfp->send_prod_index));

	set_event_reg(nicfp->send_cmd_event);

        NIC_TRACE((TRACE_TYPE_SEND | TRACE_TYPE_COMMAND), "#cmdSndP", 0x30100,
	    nicfp->send_prod_index,
	    nicfp->send_prod_index_queued,
	    nicfp->send_prod_index_avail,
	    nicfp->send_cons_index_queued);
        RETURN(TG_FW_EVENT_NUM_MAILBOX);
    }

    if (mbox_event & MBOX_EVENT_RECV_PROD) {
	do {
	    nicfp->recv_prod_index = tsp->mbox_recv_prod_index;
	    trp->gen_control.mailbox_event = MBOX_EVENT_RECV_PROD; /* clear */
	    nicfp->stats.nicCmdsSetRecvProdIndex++;
	} while (!(trp->gen_control.mailbox_event & MBOX_EVENT_RECV_PROD) &&
		 (tsp->mbox_recv_prod_index != nicfp->recv_prod_index));

	set_event_reg(TG_FW_EVENT_RECV_BD);

        NIC_TRACE((TRACE_TYPE_RECV_BD|TRACE_TYPE_COMMAND), "#cmdRcvP", 0x30200,
	    nicfp->recv_prod_index,
	    nicfp->recv_prod_index_queued,
	    nicfp->recv_prod_index_avail,
	    nicfp->recv_cons_index);
        RETURN(TG_FW_EVENT_NUM_MAILBOX);
    }

    if (mbox_event & MBOX_EVENT_RECV_MINI_PROD) {
	do {
	    nicfp->recv_mini_prod_index = tsp->mbox_recv_mini_prod_index;
	    trp->gen_control.mailbox_event = MBOX_EVENT_RECV_MINI_PROD;/*clear*/
	    nicfp->stats.nicCmdsSetRecvMiniProdIndex++;
	} while (!(trp->gen_control.mailbox_event & MBOX_EVENT_RECV_MINI_PROD)
	    && (tsp->mbox_recv_mini_prod_index != nicfp->recv_mini_prod_index));

	if (!is_global_flags(GF_RCB_MINI_RING_DISABLED))
	    set_global_flags(GF_CMD_MINI_FRAMES_ENABLED);
	set_event_reg(TG_FW_EVENT_RECV_MINI_BD);

        NIC_TRACE((TRACE_TYPE_RECV_BD|TRACE_TYPE_COMMAND), "#cmdMinP", 0x30300,
	    nicfp->recv_mini_prod_index,
	    nicfp->recv_mini_prod_index_queued,
	    nicfp->recv_mini_prod_index_avail,
	    nicfp->recv_mini_cons_index);
        RETURN(TG_FW_EVENT_NUM_MAILBOX);
    }

    if (mbox_event & MBOX_EVENT_RECV_JUMBO_PROD) {
	do {
	    nicfp->recv_jumbo_prod_index = tsp->mbox_recv_jumbo_prod_index;
	    trp->gen_control.mailbox_event = MBOX_EVENT_RECV_JUMBO_PROD; /*clr*/
	    nicfp->stats.nicCmdsSetRecvJumboProdIndex++;
	} while (!(trp->gen_control.mailbox_event & MBOX_EVENT_RECV_JUMBO_PROD)
	  && (tsp->mbox_recv_jumbo_prod_index != nicfp->recv_jumbo_prod_index));

	if (!is_global_flags(GF_RCB_JUMBO_RING_DISABLED))
	    set_global_flags(GF_CMD_JUMBO_FRAMES_ENABLED);
	set_event_reg(TG_FW_EVENT_RECV_JUMBO_BD);

        NIC_TRACE((TRACE_TYPE_RECV_BD|TRACE_TYPE_COMMAND), "#cmdJumP", 0x30400,
	    nicfp->recv_jumbo_prod_index,
	    nicfp->recv_jumbo_prod_index_queued,
	    nicfp->recv_jumbo_prod_index_avail,
	    nicfp->recv_jumbo_cons_index);
        RETURN(TG_FW_EVENT_NUM_MAILBOX);
    }

    if (mbox_event & MBOX_EVENT_PCI_INTR) {
        trp->gen_control.mailbox_event = MBOX_EVENT_PCI_INTR; /* clear */
        RETURN(TG_FW_EVENT_NUM_MAILBOX);
    }

    /* return if not a valid command */
    if (!(mbox_event & MBOX_EVENT_COMMAND)) {
	NIC_UTRACE("?HcmdMbx", 0x30500,
	    mbox_event,
	    0,
	    0,
	    0);

        trp->gen_control.mailbox_event = MBOX_EVENT_UNKNOWNS; /* clear */
        RETURN(TG_FW_EVENT_NUM_MAILBOX);
    }

    /* dequeue a command from the host */
    cmd_cons = tsp->gen_com.command_cons_index;
    do {
	if (cmd_cons != tsp->mbox_command_prod_index) {
	    cmd = tsp->gen_com.command_ring[cmd_cons]; /* struct copy */

	    NIC_TRACE(TRACE_TYPE_COMMAND, "cmdDeq", 0x30600,
		tsp->mbox_command_prod_index,
		cmd_cons,
		cmd.TG_COMMAND_w0,
		0);

	    cmd_cons = ((cmd_cons + 1) & (COMMAND_RING_ENTRIES - 1));
	    tsp->gen_com.command_cons_index = cmd_cons;

	    switch (cmd.TG_COMMAND_cmd) {
	    case TG_CMD_SET_RECV_PRODUCER_INDEX:

		/* Recv is our second highest priority command */
		nicfp->recv_prod_index = 
		    (cmd.TG_COMMAND_w0 & TG_COMMAND_INDEX_MASK);
		nicfp->stats.nicCmdsSetRecvProdIndex++;
		set_event_reg(TG_FW_EVENT_RECV_BD);

	   NIC_TRACE((TRACE_TYPE_RECV_BD|TRACE_TYPE_COMMAND),"OcmdRcvP",0x30700,
		nicfp->recv_prod_index,
		nicfp->recv_prod_index_queued,
		nicfp->recv_prod_index_avail,
		nicfp->recv_cons_index);
		break;

	    case TG_CMD_SET_RECV_JUMBO_PRODUCER_INDEX:
		nicfp->recv_jumbo_prod_index = 
		    (cmd.TG_COMMAND_w0 & TG_COMMAND_INDEX_MASK);
		if (!is_global_flags(GF_RCB_JUMBO_RING_DISABLED))
		    set_global_flags(GF_CMD_JUMBO_FRAMES_ENABLED);
		nicfp->stats.nicCmdsSetRecvJumboProdIndex++;
		set_event_reg(TG_FW_EVENT_RECV_JUMBO_BD);

	   NIC_TRACE((TRACE_TYPE_RECV_BD|TRACE_TYPE_COMMAND),"OcmdJumP",0x30800,
		nicfp->recv_jumbo_prod_index,
		nicfp->recv_jumbo_prod_index_queued,
		nicfp->recv_jumbo_prod_index_avail,
		nicfp->recv_jumbo_cons_index);
		break;

	    default:
		cmd_cons = do_mailbox_slow(cmd, cmd_cons);
		break;
	    }
	}

	/* clear mbox event if no more commands */
	if (cmd_cons == tsp->mbox_command_prod_index) {
	    trp->gen_control.mailbox_event = MBOX_EVENT_COMMAND; /* clear */

	    NIC_TRACE(TRACE_TYPE_COMMAND, "cmdDone", 0x30900,
		tsp->mbox_command_prod_index,
		cmd_cons,
		cmd.TG_COMMAND_w0,
		0);
	}
    } while (!(trp->gen_control.mailbox_event & MBOX_EVENT_COMMAND) &&
	     (cmd_cons != tsp->mbox_command_prod_index));

    RETURN(TG_FW_EVENT_NUM_MAILBOX);
}

/* null routine to assist with scratchpad setup */
void mh_command_end(void) {}

U32
do_mailbox_slow(struct tg_command cmd, U32 cmd_cons)
{
    struct tg_event event;
    tg_macaddr_t mc_addr;
    U32 tm;

    /* NOTE: a jump table is generated from the following switch statement */
    switch (cmd.TG_COMMAND_cmd) {
    case TG_CMD_HOST_STATE:
        switch (cmd.TG_COMMAND_code) {
        case TG_CMD_CODE_STACK_UP:
            nicfp->host_state |= TG_HOST_STATE_STACK_UP;
            nicfp->stats.ifAdminStatus = IF_ADMIN_STATUS_UP;
            nicfp->stats.ifOperStatus = IF_OPER_STATUS_UP;
            set_event_handler(ev_handler_stub,
			      TG_FW_EVENT_NUM_MAC_RX_COMP,
			      h_mac_rx_comp_stub);

            /* Following is a approximation to get the number without div */
            tm = trp->gen_control.timer;
            nicfp->stats.ifLastChange = 
                (tm >> 17) + (tm >> 18) - (tm >> 19) + (tm >> 20) - (tm >>21);
            break;

        case TG_CMD_CODE_STACK_DOWN:
            nicfp->host_state &= ~TG_HOST_STATE_STACK_UP;
            nicfp->stats.ifAdminStatus = IF_ADMIN_STATUS_DOWN;
            nicfp->stats.ifOperStatus = IF_OPER_STATUS_DOWN;
            set_event_handler(ev_handler_stub,
			      TG_FW_EVENT_NUM_MAC_RX_COMP,
			      (U32)h_mac_rx_comp_nohost);

            /* Following is a approximation to get the number without div */
            tm = trp->gen_control.timer;
            nicfp->stats.ifLastChange = 
                (tm >> 17) + (tm >> 18) - (tm >> 19) + (tm >> 20) - (tm >>21);
            break;

        default:
            NIC_UTRACE("?cmdHstS", 0x31000,
		cmd.TG_COMMAND_w0,
		0,
		0,
		0);
	    break;
        }
        nicfp->stats.nicCmdsHostState++;
        break;

    case TG_CMD_ADD_MULTICAST_ADDR:
	mc_addr = tsp->gen_com.mcast_xfer_buf; /* struct copy */
        add_mcast(&mc_addr);
        break;

    case TG_CMD_EXT_ADD_MULTICAST_ADDR:
	/* get mcast address from command slots */
	*(U32 *)&mc_addr = tsp->gen_com.command_ring[cmd_cons].TG_COMMAND_w0;
	cmd_cons = (cmd_cons + 1) & (COMMAND_RING_ENTRIES - 1);

	*(U32 *)&mc_addr.octet[2] =
	    tsp->gen_com.command_ring[cmd_cons].TG_COMMAND_w0;
	cmd_cons = (cmd_cons + 1) & (COMMAND_RING_ENTRIES - 1);

	tsp->gen_com.command_cons_index = cmd_cons;
        add_mcast(&mc_addr);
        break;

    case TG_CMD_DEL_MULTICAST_ADDR:
	mc_addr = tsp->gen_com.mcast_xfer_buf; /* struct copy */
        del_mcast(&mc_addr);
	break;
 
    case TG_CMD_EXT_DEL_MULTICAST_ADDR:
	/* get mcast address from command slots */
	*(U32 *)&mc_addr = tsp->gen_com.command_ring[cmd_cons].TG_COMMAND_w0;
	cmd_cons = (cmd_cons + 1) & (COMMAND_RING_ENTRIES - 1);

	*(U32 *)&mc_addr.octet[2] =
	    tsp->gen_com.command_ring[cmd_cons].TG_COMMAND_w0;
	cmd_cons = (cmd_cons + 1) & (COMMAND_RING_ENTRIES - 1);

	tsp->gen_com.command_cons_index = cmd_cons;
        del_mcast(&mc_addr);
        break;

    case TG_CMD_SET_MULTICAST_MODE:
        switch (cmd.TG_COMMAND_code) {
        case TG_CMD_CODE_MULTICAST_ENABLE:
            /* save away the bit state of the hardware multicast mask */
            if (!is_global_flags(GF_CMD_MCAST_MODE)) {
                nicp->mc_filter_saved[0] = trp->mac_control.mac_rx_mcast_filt1;
                nicp->mc_filter_saved[1] = trp->mac_control.mac_rx_mcast_filt2;
                nicp->mc_filter_saved[2] = trp->mac_control.mac_rx_mcast_filt3;
                nicp->mc_filter_saved[3] = trp->mac_control.mac_rx_mcast_filt4;
                trp->mac_control.mac_rx_mcast_filt1 = 0xffffffff;
                trp->mac_control.mac_rx_mcast_filt2 = 0xffffffff;
                trp->mac_control.mac_rx_mcast_filt3 = 0xffffffff;
                trp->mac_control.mac_rx_mcast_filt4 = 0xffffffff;
            }
	    set_global_flags(GF_CMD_MCAST_MODE);
            break;
 
        case TG_CMD_CODE_MULTICAST_DISABLE:
            /* restore the bit state of the hardware multicast mask */
            if (is_global_flags(GF_CMD_MCAST_MODE)) {
                trp->mac_control.mac_rx_mcast_filt1 = nicp->mc_filter_saved[0];
                trp->mac_control.mac_rx_mcast_filt2 = nicp->mc_filter_saved[1];
                trp->mac_control.mac_rx_mcast_filt3 = nicp->mc_filter_saved[2];
                trp->mac_control.mac_rx_mcast_filt4 = nicp->mc_filter_saved[3];
            }
	    clear_global_flags(GF_CMD_MCAST_MODE);
            break;

        default:
            NIC_UTRACE("?cmdMcMd", 0x31100,
		cmd.TG_COMMAND_w0,
		0,
		0,
		0);
	    break;
        }
        nicfp->stats.nicCmdsSetMulticastMode++;
        break;

    case TG_CMD_SET_PROMISC_MODE:
        switch (cmd.TG_COMMAND_code) {
        case TG_CMD_CODE_PROMISC_ENABLE:
            trp->mac_control.mac_rx_state =
                (trp->mac_control.mac_rx_state & ~TG_MAC_RX_ATTN_MASK) |
                TG_MAC_RX_STATE_ENA_PROMISC;
            nicfp->stats.ifPromiscuousMode = 1; /* True(1) */
	    set_global_flags(GF_CMD_PROMISC_MODE);
            break;

        case TG_CMD_CODE_PROMISC_DISABLE:
            trp->mac_control.mac_rx_state &=
		~(TG_MAC_RX_ATTN_MASK | TG_MAC_RX_STATE_ENA_PROMISC);
            nicfp->stats.ifPromiscuousMode = 2; /* False(2) */
	    clear_global_flags(GF_CMD_PROMISC_MODE);
            break;

        default:
            NIC_UTRACE("?cmdProm", 0x31200,
		cmd.TG_COMMAND_w0,
		0,
		0,
		0);
	    break;
        }
        nicfp->stats.nicCmdsSetPromiscMode++;
        break;

    case TG_CMD_SET_MAC_ADDR:
        trp->mac_control.mac_src_addr = tsp->gen_com.mac_addr; 
        nicfp->stats.ifPhysAddress = tsp->gen_com.mac_addr; 
        nicfp->stats.nicCmdsSetMACAddr++;
        break;

    case TG_CMD_LINK_NEGOTIATION:
        switch (cmd.TG_COMMAND_code) {
        case TG_CMD_CODE_NEGOTIATE_BOTH:
	    set_mac_link(TIGON_MAC_INDEX, ATTACHED_PHY_IF_SERDES_EXT,
			 tsp->gen_com.tuneParms.link_negotiation);
	    set_mac_link(TIGON_MAC_INDEX, ATTACHED_PHY_IF_MII,
			 tsp->gen_com.tuneParms.fast_link_negotiation);
            break;

        case TG_CMD_CODE_NEGOTIATE_GIGABIT:
	    set_mac_link(TIGON_MAC_INDEX, ATTACHED_PHY_IF_SERDES_EXT,
			 tsp->gen_com.tuneParms.link_negotiation);
            break;

	case TG_CMD_CODE_NEGOTIATE_10_100:
	    set_mac_link(TIGON_MAC_INDEX, ATTACHED_PHY_IF_MII,
			 tsp->gen_com.tuneParms.fast_link_negotiation);
	    break;

        default:
            NIC_UTRACE("?cmdLink", 0x31300,
		cmd.TG_COMMAND_w0,
		0,
		0,
		0);
	    break;
        }
        nicfp->stats.nicCmdsLinkNegotiate++;
        break;

    case TG_CMD_CLEAR_STATS:
        clear_stats();
        break;

    case TG_CMD_REFRESH_STATS:
        /*
         * This command is used to support Stop/Resume feature
         * The idea is, host is going to give the NIC the previous
         * set of stats and the nic is suppose to take it start to
         * work with these, as if it was the previous values..
         * Therefor, simply copy the stats from the gen_info.stats2
         * area to gen_info.stats area.
         */
        /* Following loop is to ensure that we MUST put this request onto DMA Q
         * This loop is not dangerous. If we are able to make it to here,
         * this means that we are not really swamped and putting on the
         * DMA Q will happen in the first shot. But on the other hand, for
         * some reason, if we miss the first time, this loop ensures that
         * we MUST enque this request
         */
	NIC_TRACE(TRACE_TYPE_STATS, "Qsts2nic", 0x31400,
	    0,
	    0,
	    0,
	    0);

	nicfp->refresh_stats_active = TRUE;
	for (;;) {
            if ((*nicfp->q_dma_to_nic_stub)
		    (nicfp->stats2_ptr,
		    (U32)&nicp->gen_info.stats,
		    sizeof(struct tg_stats), 
		    TIGON_TYPE_REFRESH_STATS,
		    0,
		    nicfp->dma_to_nic_bd_state)) {
		break;
	    }
	}
        break;

    case TG_CMD_RESET_JUMBO_RING:
        /*
         * This command is issued by the Host to tell NIC to stop using the
         * Jumbo ring even if a Jumbo Frame is received.
         */
        nicfp->recv_jumbo_prod_index = 0;
        nicfp->recv_jumbo_prod_index_queued = 0;
        nicfp->recv_jumbo_prod_index_avail = 0;
        nicfp->recv_jumbo_cons_index = 0;
	nicfp->stats.nicTotalRecvBDs -= nicfp->stats.nicJumboRecvBDs;
        nicfp->stats.nicJumboRecvBDs = 0;
	clear_global_flags(GF_CMD_JUMBO_FRAMES_ENABLED);

        /* Send an event back to host */
        event.TG_EVENT_w0 = (TG_EVENT_RESET_JUMBO_RING << TG_EVENT_EVENT_SHIFT);
        must_enq_event(&event);
        nicfp->stats.nicCmdsResetJumboRing++;
        nicfp->stats.nicEventsResetJumboRing++;
        break;

    case TG_CMD_UPDATE_GENCOM_STATS:
        /* Reset the basic default values for Stats */
        nicfp->stats.ifIndex = tsp->gen_com.ifIndex;
        nicfp->stats.ifMtu = tsp->gen_com.ifMTU;
        nicfp->stats.ifPhysAddress = tsp->gen_com.mac_addr;

        /* Reset the value of Promiscuous mode */
        if (trp->mac_control.mac_rx_state & TG_MAC_RX_STATE_ENA_PROMISC)
            nicfp->stats.ifPromiscuousMode = 1; /* True(1) */
        else
            nicfp->stats.ifPromiscuousMode = 2; /* False(2) */
        nicfp->stats.nicCmdsUpdateGencommStats++;
        break;

#if FW_PROFILE
    case TG_CMD_CLEAR_PROFILE:
        bzero((U32 *)&nicfp->stats.nicProfile, (sizeof(U32) * 32));
        nicfp->stats.nicCmdsClearProfile++;
        break;
#endif /* FW_PROFILE */

    case TG_CMD_SET_PARTIAL_RECV_COUNT:
      {
	U32 count;

	count = cmd.TG_COMMAND_w0 &
		(TG_COMMAND_CODE_MASK | TG_COMMAND_INDEX_MASK);

	/* command only valid for non-cksum mode */
	if (!is_global_flags(GF_RCB_RX_TCP_UDP_CKSUM | GF_RCB_RX_IP_CKSUM) &&
	    (count <= MAX_PARTIAL_RECV_COUNT)) {

	    nicfp->rx_partial_count = count;
	    break;
	}
	goto cmd_err;
      }

    default:
    cmd_err:
        NIC_UTRACE("?cmdErr", 0x31500,
	    cmd.TG_COMMAND_w0,
	    0,
	    0,
	    0);

        event.TG_EVENT_w0 =
	    (TG_EVENT_ERROR << TG_EVENT_EVENT_SHIFT) |
            (TG_EVENT_CODE_ERR_INVALID_CMD << TG_EVENT_CODE_SHIFT) |
            (cmd.TG_COMMAND_cmd);
        enq_event(&event);
        nicfp->stats.nicCmdsUnknown++;
        nicfp->stats.nicEventsError++;
	break;
    }

    return (cmd_cons);
}

/**********************************************************************/

#define CRC32_POLYNOMIAL        0xEDB88320
 
/* datalen is number of bytes */
U32
gen_crc (U8 *databuf, U32 datalen)
{
    register U32 idx, bit, data, crc = 0xFFFFFFFF;

    for (idx = 0; idx < datalen; idx++) {
        data = *databuf++;
        for (bit = 0; bit < 8; bit++, data >>= 1) {
            crc = (crc >> 1) ^ (((crc ^ data) & 1)? CRC32_POLYNOMIAL: 0);
        }
    }
    return (crc);
}

void
clear_stats(void)
{
    static U8 alteonStr[] = "Alteon AceNIC V";
    U32 tempRxBD;
    U32 tempJumboRxBD;
    U32 tempMiniRxBD;
    U32 tempTotalRxBD;
    U32 tempClearStats;
    U32 tempOperStats;
    U32 tempAdminStats;
    U32 tempSendBDs;
    U32 rx_state, tx_state;
    U32 i;

    /* save mac state */
    rx_state = trp->mac_control.mac_rx_state;
    tx_state = trp->mac_control.mac_tx_state;

    /* stop mac after next packet before clear hw macstats area */
    trp->mac_control.mac_rx_state = 
	(trp->mac_control.mac_rx_state & ~TG_MAC_RX_ATTN_MASK) |
	TG_MAC_RX_STATE_STOP_NEXT;
    trp->mac_control.mac_tx_state = 
	(trp->mac_control.mac_tx_state & ~TG_MAC_TX_ATTN_MASK) |
	TG_MAC_TX_STATE_STOP_NEXT;

    /* Save RxBDs temporarily */
    tempSendBDs = nicfp->stats.nicSendBDs;
    tempRxBD = nicfp->stats.nicRecvBDs;
    tempJumboRxBD = nicfp->stats.nicJumboRecvBDs;
    tempMiniRxBD = nicfp->stats.nicMiniRecvBDs;
    tempTotalRxBD = nicfp->stats.nicTotalRecvBDs;
    tempClearStats = ++nicfp->stats.nicCmdsClearStats;
    tempOperStats = nicfp->stats.ifOperStatus;
    tempAdminStats = nicfp->stats.ifAdminStatus;

    /* clear */
    bzero((U32 *)&nicfp->stats, sizeof(struct tg_stats));
#if (USE_U32_STAT_CNT)
    nicfp->ifHCInOctets			= 0;
    nicfp->ifHCOutOctets		= 0;
    nicfp->ifHCOutPkts[UCAST_PKT_INDEX]	= 0;
    nicfp->ifHCOutPkts[MCAST_PKT_INDEX]	= 0;
    nicfp->ifHCOutPkts[BCAST_PKT_INDEX]	= 0;
#endif /* (USE_U32_STAT_CNT) */

    /* Restore the value back */
    nicfp->stats.nicSendBDs = tempSendBDs;
    nicfp->stats.nicRecvBDs = tempRxBD;
    nicfp->stats.nicJumboRecvBDs = tempJumboRxBD;
    nicfp->stats.nicMiniRecvBDs = tempMiniRxBD;
    nicfp->stats.nicTotalRecvBDs = tempTotalRxBD;
    nicfp->stats.nicCmdsClearStats = tempClearStats;
    nicfp->stats.ifOperStatus = tempOperStats; 
    nicfp->stats.ifAdminStatus = tempAdminStats; 

    /* Set up the basic default values for Stats */
    nicfp->stats.ifIndex = tsp->gen_com.ifIndex;
    nicfp->stats.ifMtu = tsp->gen_com.ifMTU;
    nicfp->stats.ifType = 6;  /* Type = Ethernet(6) for now */
    nicfp->stats.ifPhysAddress = tsp->gen_com.mac_addr;
    nicfp->stats.ifSpeed = 1000000000;  /* 1 Gig bits/sec */
    nicfp->stats.ifHighSpeed = 1000;  /* 1000 Mbits/sec */
    nicfp->stats.ifLinkUpDownTrapEnable = 2; /* Always Disabled(2) */
    nicfp->stats.ifConnectorPresent = 1; /* Always True (1) */

    /* Reset the value of Promiscuous mode */
    if (trp->mac_control.mac_rx_state & TG_MAC_RX_STATE_ENA_PROMISC)
        nicfp->stats.ifPromiscuousMode = 1; /* True(1) */
    else
        nicfp->stats.ifPromiscuousMode = 2; /* False(2) */

    for (i = 0; i < 15; i++)
         nicfp->stats.ifDescr[i] = alteonStr[i];

    nicfp->stats.ifDescr[i++] = (U8)0x30 +
      ((trp->gen_control.misc_host_control & TG_MHC_TIGON_VERSION_MASK) >> 28);
    nicfp->stats.ifDescr[i++] = '\0'; /* Null terminate the str */

    /* clear hw macstats area after stop mac */
    bzero((U32 *)&tsp->macstats, sizeof(struct mac_stats));

    /* restart mac */
    if (!(rx_state & TG_MAC_RX_STATE_STOP_NEXT)) {
	trp->mac_control.mac_rx_state &=
	    ~(TG_MAC_RX_ATTN_MASK | TG_MAC_RX_STATE_STOP_NEXT);
    }
    if (!(tx_state & TG_MAC_TX_STATE_STOP_NEXT)) {
	trp->mac_control.mac_tx_state &=
	    ~(TG_MAC_TX_ATTN_MASK | TG_MAC_TX_STATE_STOP_NEXT);
    }

    return;
}
