/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: data.c $
 * Revision copper_fhwang/6 2000/06/20 23:10:35 fhwang
 * 	upgrade to 12_4_13
 * 
 * Revision copper_fhwang/4 2000/04/13 19:43:48 fhwang
 * 	upgrade to 12.4.12
 * 
 * Revision copper_fhwang/2 2000/04/08 00:38:02 fhwang
 * 	change fw version to 0x12041100
 * 
 * Revision nic_api_12_4_latest_shuang/13 2000/03/27 23:19:36 shuang
 * 	fix: Tx hang on jumbo frames checksum offload
 * 
 * Revision nic_api_12_4_latest_shuang/11 2000/02/25 19:28:54 shuang
 * 	fix work around HW MAC HD collision/retransmission for cksum
 * 
 * Revision nic_api_12_4_latest_shuang/9 2000/02/12 18:50:05 shuang
 * 	fix incorrect stats nicRecvBDs and nicJumboRecvBDs.
 * 	honor TG_CMD_CODE_STACK_UP and TG_CMD_CODE_STACK_DOWN commands.
 * 
 * Revision nic_api_12_4_latest_shuang/6 2000/02/04 18:54:16 shuang
 * 	bump up revision
 * 
 * Revision nic_api_12_4_latest_shuang/4 2000/02/01 00:48:57 shuang
 * 	clear tx/rx base registers prior to the 1st rd dma.
 * 
 * Revision nic_api_12_4_latest_shuang/2 1999/08/27 01:38:33 shuang
 * 	jumbo frame length will be 0x8870
 * 
 * Revision 1.1.2.12  1999/01/20  19:49:51  shuang
 * 	avoid race condition on clearing any mbox event
 * 	[1999/01/20  19:48:10  shuang]
 *
 * Revision 1.1.2.11  1999/01/16  02:55:36  shuang
 * 	use of mini ring and jumbo ring is optional
 * 	[1999/01/16  02:53:00  shuang]
 * 
 * Revision 1.1.2.10  1998/12/21  00:36:17  shuang
 * 	fix - return last send BD
 * 	[1998/12/21  00:36:00  shuang]
 * 
 * Revision 1.1.2.9  1998/12/18  01:50:12  shuang
 * 	cannot group dma completion for sendCons, recvRetProd, evtProd
 * 	[1998/12/18  01:46:41  shuang]
 * 
 * Revision 1.1.2.8  1998/12/12  00:35:59  shuang
 * 	add debug code to check recv ring
 * 	[1998/12/12  00:35:02  shuang]
 * 
 * Revision 1.1.2.7  1998/12/10  02:55:34  shuang
 * 	Prevent from starving tx side. Fix stats counters.
 * 	[1998/12/10  02:54:30  shuang]
 * 
 * Revision 1.1.2.6  1998/12/09  18:15:37  shuang
 * 	revise fw init, version log
 * 	[1998/12/09  17:53:32  shuang]
 * 
 * Revision 1.1.2.5  1998/09/30  18:49:48  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:39:32  shuang]
 * 
 * Revision 1.1.2.4  1998/06/23  01:29:30  shuang
 * 	added dynamic nic mem support.
 * 	[1998/06/23  01:21:26  shuang]
 * 
 * Revision 1.1.2.3  1998/04/24  00:00:30  hayes
 * 	added const to rcsid definitions to eliminate compiler warnings
 * 	[1998/04/23  23:58:48  hayes]
 * 
 * Revision 1.1.2.2  1998/02/19  01:29:46  suresh
 * 	Allow use of second CPU and remove junk define
 * 	[1998/02/19  01:15:32  suresh]
 * 
 * Revision 1.1.2.1  1998/01/14  23:25:39  ted
 * 	rearrange directories
 * 	[1998/01/14  23:17:36  ted]
 * 
 * Revision 1.1.2.1  1997/11/19  19:23:04  ted
 * 	Support Tigon and Tigon2 in the same driver
 * 	[1997/11/19  19:18:41  ted]
 * 
 * Revision 1.10.6.3  1997/11/14  19:16:17  hayes
 * 	first checkin after mongo merge...
 * 	[1997/11/14  19:11:16  hayes]
 * 
 * Revision 1.10.6.2  1997/02/11  02:50:11  ted
 * 	Fix initialization of nic_stack_start for register optimized code.
 * 	[1997/02/11  02:50:02  ted]
 * 
 * Revision 1.10.6.1  1997/02/07  23:20:03  skapur
 * 	replacing TGP/NIC/TDP  with reg variables
 * 	[1997/02/07  23:19:31  skapur]
 * 
 * Revision 1.10.2.4  1996/11/20  20:04:16  ted
 * 	Redo include file structuring.
 * 	[1996/11/20  19:50:35  ted]
 * 
 * Revision 1.10.2.3  1996/11/15  04:24:02  hayes
 * 	convert to new header file structure.
 * 	[1996/11/15  04:11:57  hayes]
 * 
 * Revision 1.10.2.2  1996/11/05  21:54:15  hayes
 * 	Final pre-ODE merge
 * 	[1996/11/05  21:51:19  hayes]
 * 
 * Revision 1.10.2.1  1996/11/01  18:24:47  ted
 * 	Change rr_ to tg_
 * 	[1996/10/29  21:38:21  ted]
 * 
 * $EndLog$
 */

/*
 * FILE  data.c
 *
 * COPYRIGHT (c) Essential Communication Corp. 1995
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/data.c,v $
 * $Revision: 1.1.2.12 $ $Locker:  $
 * $Date: 1999/01/20 19:49:51 $ $Author: shuang $ $State: Exp $
 */

#ifndef __lint
static const char rcsid[] = "$Header: /projects/rcs/sw/ge/./nic/fw2/common/data.c,v 1.1.2.12 1999/01/20 19:49:51 shuang Exp $";
#endif /* __lint */



#include "alt_def.h"
#include "nic_conf.h"
#include "nic_api.h"
#include "tg.h"

#include "nic.h"
#include "compile.h"


/* temporary stacks used before initialization */
U32 stack_start   = (U32)(TG_DATA_END - 4);
U32 stack_start_b = (U32)(TG_DATA_END - 4 - 128);

/* Tracing variable */
U32 TraceInited = FALSE;
/* these get initialized in init_trace() */
U32 *curtb;
U32 *endtb;
U32 tbuf[ALT_NTRACE_SIZE * ALT_TRACE_ELEM_SIZE];

/* these are for identifying the build */
static const char alt_fw_version[]		= FW_VERSION;
static const char alt_fw_compile_time[]		= FW_COMPILE_TIME;
static const char alt_fw_compile_by[]		= FW_COMPILE_BY;
static const char alt_fw_compile_host[]		= FW_COMPILE_HOST;
static const char alt_fw_compile_domain[]	= FW_COMPILE_DOMAIN;
static const char alt_fw_compiler[]		= FW_COMPILER;
static const char alt_fw_build_opts[]		= "";

/*
 * need bump this up manually for each ckeckin
 * VVVVVVrr: 
 *	VVVVVV - version_no, ie, api VV.VV.VV
 *	rr     - revision_no,
 */
const unsigned int_fw_version			= 0x12041300;
