/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: dma.c $
 * Revision api12_3_ted/2 1999/08/12 21:07:40 ted
 * 	Add assembler routines
 * 
 * Revision nic_ge_latest_shuang/2 1999/03/23 18:59:52 shuang
 * 	work around Tigon chip 4Gb cross over bug
 * 
 * Revision 1.1.2.24  1998/12/21  00:33:09  shuang
 * 	fix - return last send BD
 * 	[1998/12/21  00:08:46  shuang]
 *
 * Revision 1.1.2.23  1998/12/18  01:50:14  shuang
 * 	cannot group dma completion for sendCons, recvRetProd, evtProd
 * 	[1998/12/18  01:46:47  shuang]
 * 
 * Revision 1.1.2.22  1998/12/12  00:35:50  shuang
 * 	add debug code to check recv ring
 * 	[1998/12/12  00:32:51  shuang]
 * 
 * Revision 1.1.2.21  1998/12/09  18:15:39  shuang
 * 	revise fw init, version log
 * 	[1998/12/08  22:40:10  shuang]
 * 
 * Revision 1.1.2.20  1998/12/08  02:36:24  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:27:42  shuang]
 * 
 * Revision 1.1.2.19  1998/10/27  03:11:12  hayes
 * 	change ASSERT() to PANIC() for DMA attentions
 * 	[1998/10/27  03:11:01  hayes]
 * 
 * Revision 1.1.2.18  1998/09/30  18:49:50  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:39:39  shuang]
 * 
 * Revision 1.1.2.17  1998/08/03  17:24:37  shuang
 * 	use TG_CFG_MODE_SBUS flag instead of TG_CFG_MODE_1_DMA_ACTIVE
 * 	[1998/08/03  17:23:10  shuang]
 * 
 * Revision 1.1.2.16  1998/08/01  00:09:59  shuang
 * 	use TG_CFG_MODE_1_DMA_ACTIVE flag instead of TG_CFG_MODE_SBUS
 * 	for sbus detection
 * 	[1998/08/01  00:08:18  shuang]
 * 
 * Revision 1.1.2.15  1998/07/31  22:07:57  shuang
 * 	walk around pci master abort problem on sbus
 * 	[1998/07/31  22:07:08  shuang]
 * 
 * Revision 1.1.2.14  1998/07/29  00:07:23  shuang
 * 	add feature - send coallescing BD update only
 * 	[1998/07/29  00:02:36  shuang]
 * 
 * 	add feature - send coallescing BD now
 * 	[1998/07/25  02:30:24  shuang]
 * 
 * Revision 1.1.2.13  1998/06/23  01:29:32  shuang
 * 	added dynamic nic mem support.
 * 	[1998/06/23  01:21:32  shuang]
 * 
 * Revision 1.1.2.12  1998/06/08  23:55:13  ted
 * 	     Snapshot NT generification phase 1
 * 	     [1998/06/04  19:52:02  ted]
 * 
 * Revision 1.1.2.11  1998/04/24  00:00:31  hayes
 * 	     added const to rcsid definitions to eliminate compiler warnings
 * 	     [1998/04/23  23:58:49  hayes]
 * 
 * Revision 1.1.2.10  1998/04/11  05:15:04  shuang
 * 	     move fast timer to cpu B, add receive coalesed ticks
 * 	     [1998/04/11  04:57:52  shuang]
 * 
 * Revision 1.1.2.9  1998/04/06  17:38:29  hayes
 * 	     code generation optimizations
 * 	     [1998/04/06  17:38:05  hayes]
 * 
 * Revision 1.1.2.8  1998/04/01  22:21:22  hayes
 * 	     change ref_ring prod/cons to head/tail
 * 	     [1998/04/01  00:59:44  hayes]
 * 
 * 	     checkpoint
 * 	     [1998/03/31  19:56:51  hayes]
 * 
 * 	     checkpoint- dma completion avoidance works
 * 	     [1998/03/30  07:00:27  hayes]
 * 
 * 	     wr DMA completion avoidance works
 * 	     [1998/03/29  04:20:46  hayes]
 * 
 * 	     rd DMA event avoidance works
 * 	     [1998/03/29  03:11:40  hayes]
 * 
 * Revision 1.1.2.7  1998/03/26  01:34:22  shuang
 * 	     cleanup
 * 	     [1998/03/26  01:24:56  shuang]
 * 
 * Revision 1.1.2.6  1998/02/27  00:25:42  ted
 * 	     Increase performance on async memory Tigon Rev 5 cards
 * 	     [1998/02/27  00:25:32  ted]
 * 
 * Revision 1.1.2.5  1998/02/19  01:29:49  suresh
 * 	     Checksum offload for Rev 6 support
 * 	     [1998/02/19  01:17:13  suresh]
 * 
 * 	     intermediate checksum offload state
 * 	     [1998/02/15  01:26:18  suresh]
 * 
 * Revision 1.1.2.4  1998/02/11  19:11:20  ted
 * 	     Fix up functions to allow DMA support for V6 Tigon and V5/V6 asynchronous memory Tigon
 * 	     [1998/02/11  18:39:43  ted]
 * 
 * Revision 1.1.2.3  1998/02/09  17:18:05  hayes
 * 	     convert Q_DMA_XXX macros to functions
 * 	     [1998/02/09  17:16:53  hayes]
 * 
 * Revision 1.1.2.2  1998/01/20  04:39:20  hayes
 * 	     add tigon Rev6 support
 * 	     [1998/01/20  04:37:53  hayes]
 * 
 * Revision 1.1.2.1  1998/01/14  23:25:41  ted
 * 	     rearrange directories
 * 	     [1998/01/14  23:17:40  ted]
 * 
 * Revision 1.1.2.3  1997/12/29  19:02:05  hayes
 * 	     remove old, incorrect comments
 * 	     [1997/12/29  18:35:13  hayes]
 * 
 * Revision 1.1.2.2  1997/11/21  04:48:14  hayes
 * 	     release for NIC 3.1 pre-Beta
 * 	     [1997/11/21  04:45:31  hayes]
 * 
 * Revision 1.1.2.1  1997/11/19  19:23:05  ted
 * 	     Support Tigon and Tigon2 in the same driver
 * 	     [1997/11/19  19:19:02  ted]
 * 
 * Revision 1.12.6.9  1997/11/14  19:16:19  hayes
 * 	     first checkin after mongo merge...
 * 	     [1997/11/14  19:11:21  hayes]
 * 
 * 	     checkin after stats restructuring
 * 	     [1997/11/12  19:01:12  hayes]
 * 
 * Revision 1.12.6.8  1997/09/16  02:35:22  skapur
 * 	     Merged with changes from 1.12.6.7
 * 	     [1997/09/16  02:35:09  skapur]
 * 
 * 	     Took out ifdefed code and cleaned out the traces
 * 	     [1997/09/16  02:32:22  skapur]
 * 
 * Revision 1.12.6.7  1997/09/11  18:05:38  ted
 * 	     Mixing vlans, jumbos, and redundancy works, plus small fixes from release 1.1 and better NIC tracing
 * 	     [1997/09/11  17:59:26  ted]
 * 
 * Revision 1.12.6.6  1997/08/13  18:45:56  ted
 * 	     Enhance tracing in the case of read DMA attention
 * 	     [1997/08/13  18:38:45  ted]
 * 
 * 	     Enhance tracing in case of write DMA attention
 * 	     [1997/08/03  21:31:41  ted]
 * 
 * Revision 1.12.6.5  1997/04/11  23:36:11  ted
 * 	     Added code to deal with Read and Write DMA Master Aborts
 * 	     [1997/04/11  23:14:54  ted]
 * 
 * Revision 1.12.6.4  1997/03/28  02:46:56  skapur
 * 	     Cleaned out unnecessay commend out code
 * 	     [1997/03/28  01:58:09  skapur]
 * 
 * Revision 1.12.6.3  1997/03/24  08:34:18  ted
 * 	     New event ring handling, producer lives in host memory, misc. other more modest changes
 * 	     [1997/03/24  08:27:41  ted]
 * 
 * Revision 1.12.6.2  1997/03/17  08:47:39  skapur
 * 	     integration changes...
 * 	     [1997/03/16  03:01:41  skapur]
 * 
 * Revision 1.12.6.1  1997/02/23  00:23:30  ted
 * 	     DMA Assist state changed to have only 1 channel Active; Once HW is fixed, remove this hack
 * 	     [1997/02/23  00:06:04  ted]
 * 
 * Revision 1.12.2.5  1996/11/29  17:43:18  hayes
 * 	     Cleanup init_dma().
 * 	     [1996/11/29  17:36:49  hayes]
 * 
 * Revision 1.12.2.4  1996/11/20  20:04:17  ted
 * 	     Redo include file structuring.
 * 	     [1996/11/20  19:50:47  ted]
 * 
 * Revision 1.12.2.3  1996/11/15  04:24:02  hayes
 * 	     convert to new header file structure.
 * 	     [1996/11/15  04:11:58  hayes]
 * 
 * Revision 1.12.2.2  1996/11/05  21:54:16  hayes
 * 	     Final pre-ODE merge
 * 	     [1996/11/05  21:51:26  hayes]
 * 
 * Revision 1.12.2.1  1996/11/01  18:24:50  ted
 * 	     change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	     [1996/10/29  22:05:50  ted]
 * 
 * $EndLog$
 */
/*
 * FILE dma.c
 *
 * COPYRIGHT (c) Essential Communication Corp. 1995
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/dma.c,v $
 * $Revision: 1.1.2.24 $ $Locker:  $
 * $Date: 1998/12/21 00:33:09 $ $Author: shuang $ $State: Exp $
 */

#ifndef __lint
static const char rcsid[] = "$Header: /projects/rcs/sw/ge/./nic/fw2/common/dma.c,v 1.1.2.24 1998/12/21 00:33:09 shuang Exp $";

#endif /* __lint */



#include "alt_def.h"
#include "nic_conf.h"
#include "nic_api.h"
#include "tg.h"

#include "nic.h"
#include "trace.h"
#include "proto.h"
#include "assert.h"
#include "ring.h"
#include "profile.h"

/* should be defined in tg.h */
#define TG_DMA_STATE_CROSS_ADDR_ERROR	(0x08000000)

/*
 * Set up the DMA assist rings and enable the DMA assist.  From now on,
 * all DMA must go through the assist logic.  Using the other DMA
 * method will confuse the DMA assist logic.
 */
void
init_dma_assist(void)
{
    /* zero the assist rings */
    bzero((U32 *)tdp->rd_dma_ring,
          sizeof(struct tg_dma_descr) * TG_DMA_ASSIST_DESCRS * 2);

    /* chaining must be done prior to initializing the assist pointers */
    trp->host_dma_assist.assist_state = TG_DMA_ASSIST_CHAIN_RINGS;

    trp->host_dma_assist.read_chan_producer = 0;
    trp->host_dma_assist.read_chan_consumer = 0;
    trp->host_dma_assist.read_chan_ref      = 0;

    trp->host_dma_assist.read_chan_lo_pri_producer = 0;
    trp->host_dma_assist.read_chan_lo_pri_consumer = 0;
    trp->host_dma_assist.read_chan_lo_pri_ref      = 0;

    trp->host_dma_assist.write_chan_producer = 0;
    trp->host_dma_assist.write_chan_consumer = 0;
    trp->host_dma_assist.write_chan_ref      = 0;

    trp->host_dma_assist.write_chan_lo_pri_producer = 0;
    trp->host_dma_assist.write_chan_lo_pri_consumer = 0;
    trp->host_dma_assist.write_chan_lo_pri_ref      = 0;

    nicfp->rd_dma_ref_prod = nicp->rd_dma_ref_ring;
    nicfp->rd_dma_ref_cons = nicp->rd_dma_ref_ring;
    nicfp->wr_dma_ref_prod = nicp->wr_dma_ref_ring;
    nicfp->wr_dma_ref_cons = nicp->wr_dma_ref_ring;

    if (!nicp->sw_dma_assist) {
        trp->host_dma_assist.assist_state |= TG_DMA_ASSIST_ENABLE; 

	NIC_UTRACE("swDmaOff", 0x10100,
	    trp->host_dma_assist.assist_state,
	    0,
	    0,
	    0);

	if (tsp->gen_com.mode_status & TG_CFG_MODE_1_DMA_ACTIVE) {
	    trp->host_dma_assist.assist_state |= TG_DMA_ASSIST_1_DMA_ACTIVE;

	    NIC_UTRACE("1DmaOn", 0x10200,
		trp->host_dma_assist.assist_state,
		0,
		0,
		0);
	}
    } else {
	NIC_UTRACE("swDmaOn", 0x10300,
	    trp->host_dma_assist.assist_state,
	    0,
	    0,
	    0);
    }
    return;
}

U32
q_dma_to_host(tg_hostaddr_t host_addr, U32 nic_addr, U32 len, 
              U32 type, U32 index, U32 state) 
{
    struct tg_dma_descr *dd_prod, *ndd_prod, *dd_cons;
    dma_ref_ring_t *ref_prod;
    U32 ref_type;

    dd_prod = trp->host_dma_assist.write_chan_next;
    if (!dd_prod ||
	((ndd_prod = get_next_wr_dma_descr(dd_prod)) ==
	     trp->host_dma_assist.write_chan_ref) ||
 	(ndd_prod == (dd_cons = trp->host_dma_assist.write_chan_consumer))) {

        nicfp->stats.nicDmaWriteRingFull++; 

	NIC_TRACE(TRACE_TYPE_DMA, "QwDmaF", 0x10400,
	    trp->host_dma_assist.write_chan_producer,
	    trp->host_dma_assist.write_chan_consumer,
	    trp->host_dma_assist.write_chan_ref,
	    nicfp->stats.nicDmaWriteRingFull);
        return (ERROR);
    }

    dd_prod->host_addr = host_addr; 
    dd_prod->nic_addr  = nic_addr; 
    dd_prod->len       = len; 
    dd_prod->type      = type; 
    dd_prod->index     = index; 
    dd_prod->state     = state; 
    dd_prod->cksum     = (U32)dd_cons; /* only to debug dma overrun */

    trp->host_dma_assist.write_chan_producer = ndd_prod;
    nicfp->wr_dma_free_cnt--;

    /* wr dma completion avoidance logic */
    ref_prod = nicfp->wr_dma_ref_prod;
    ref_type = ref_prod->type;
    if (((ref_type == type) || /* self */
	    ((ref_type & WR_DMA_GROUP_2) && (type & WR_DMA_GROUP_2))) &&
	(ref_prod->count < WR_DMA_MAX_DELTA)) { /* hit */
		
	ref_prod->count++;

	if (get_next_wr_dma_ref(nicfp->wr_dma_ref_cons) == ref_prod) {
	    /*
	     * only one pending dma completion
	     * to prevent dma from taking premature dma completion
	     * bump the ref pointer to keep it one behind the prod pointer
	     */ 
	    trp->host_dma_assist.write_chan_ref++;
	}

	NIC_TRACEV_IF(TRACE_TYPE_DMA, "QwDmaHit", 0x10500,
		     type, TIGON_TYPE_RX,
	    trp->host_dma_assist.write_chan_producer,
	    trp->host_dma_assist.write_chan_consumer,
	    trp->host_dma_assist.write_chan_ref,
	    (type << 16) | (ref_prod->count));

    } else { /* miss */
	/* set up new type */
	nicfp->wr_dma_ref_prod = ref_prod = get_next_wr_dma_ref(ref_prod);
	ASSERT((ref_prod != nicfp->wr_dma_ref_cons) &&
	       (ref_prod->type == TIGON_TYPE_NULL));
	ref_prod->type = type;
	ref_prod->count = 1;
    }

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QwDmaIdx", 0x10600,
		 type, TIGON_TYPE_RX,
	(dd_prod - tdp->wr_dma_ring),
        trp->host_dma_assist.write_chan_producer,
        trp->host_dma_assist.write_chan_consumer,
        trp->host_dma_assist.write_chan_ref);

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QwDmaD0", 0x10700,
		 type, TIGON_TYPE_RX,
	*(U32 *)dd_prod,	/* host addr */
	*((U32 *)dd_prod + 1),	/* host addr */
	*((U32 *)dd_prod + 2),	/* nic addr */
	*((U32 *)dd_prod + 3));	/* reserved & len */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QwDmaD1", 0x10800,
		 type, TIGON_TYPE_RX,
	*((U32 *)dd_prod + 4),	/* state */
	*((U32 *)dd_prod + 5),	/* reserved & cksum addr */
	*((U32 *)dd_prod + 6),	/* type */
	*((U32 *)dd_prod + 7));	/* index */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QwDmaAvd", 0x10900,
		 type, TIGON_TYPE_RX,
        nicfp->wr_dma_ref_prod,
        nicfp->wr_dma_ref_cons,
	ref_prod->type,
	ref_prod->count);

    return (OK);
}

/* null routine to assist with scratchpad setup */
void q_dma_to_host_end(void) {}

#if !USE_ASM
U32
q_dma_to_host_cksum(tg_hostaddr_t host_addr, U32 nic_addr, U32 len, 
                    U32 type, U32 index, U32 state)
{ 
    struct tg_dma_descr *dd_prod, *ndd_prod;

    dd_prod = trp->host_dma_assist.write_chan_producer;
    if ((ndd_prod = get_next_wr_dma_descr(dd_prod)) == 
	trp->host_dma_assist.write_chan_ref) {

        nicfp->stats.nicDmaWriteRingFull++; 

	NIC_TRACE(TRACE_TYPE_DMA, "QwDmaF", 0x11000,
	    trp->host_dma_assist.write_chan_producer,
	    trp->host_dma_assist.write_chan_consumer,
	    trp->host_dma_assist.write_chan_ref,
	    nicfp->stats.nicDmaWriteRingFull);
        return (ERROR);
    }
    ASSERT(ndd_prod != trp->host_dma_assist.write_chan_consumer);

    dd_prod->host_addr = host_addr; 
    dd_prod->nic_addr  = nic_addr; 
    dd_prod->len       = len; 
    dd_prod->type      = type; 
    dd_prod->index     = index; 
    dd_prod->cksum     = (U32)((U8*)&dd_prod->cksum + 2); 
    dd_prod->state     = state; 

    trp->host_dma_assist.write_chan_producer = ndd_prod; 
    nicfp->wr_dma_free_cnt--;

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QwDmaIdx", 0x11200,
		 type, TIGON_TYPE_RX,
	(dd_prod - tdp->wr_dma_ring),
        trp->host_dma_assist.write_chan_producer,
        trp->host_dma_assist.write_chan_consumer,
        trp->host_dma_assist.write_chan_ref);

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QwDmaD0", 0x11300,
		 type, TIGON_TYPE_RX,
	*(U32 *)dd_prod,	/* host addr */
	*((U32 *)dd_prod + 1),	/* host addr */
	*((U32 *)dd_prod + 2),	/* nic addr */
	*((U32 *)dd_prod + 3));	/* reserved & len */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QwDmaD1", 0x11400,
		 type, TIGON_TYPE_RX,
	*((U32 *)dd_prod + 4),	/* state */
	*((U32 *)dd_prod + 5),	/* reserved & cksum addr */
	*((U32 *)dd_prod + 6),	/* type */
	*((U32 *)dd_prod + 7));	/* index */

    return (OK);
}

/* null routine to assist with scratchpad setup */
void q_dma_to_host_cksum_end(void) {}
#endif /* !USE_ASM */

U32
q_dma_to_nic(tg_hostaddr_t host_addr, U32 nic_addr, U32 len, 
             U32 type, U32 index, U32 state) 
{
    struct tg_dma_descr *dd_prod, *ndd_prod, *dd_cons;
    dma_ref_ring_t *ref_prod;
    U32 ref_type;

    dd_prod = trp->host_dma_assist.read_chan_next; 
    if (!dd_prod ||
	((ndd_prod = get_next_rd_dma_descr(dd_prod)) ==
	     trp->host_dma_assist.read_chan_ref) ||
 	(ndd_prod == (dd_cons = trp->host_dma_assist.read_chan_consumer))) {

        nicfp->stats.nicDmaReadRingFull++; 

	NIC_TRACE(TRACE_TYPE_DMA, "QrDmaF", 0x11500, 
	    trp->host_dma_assist.read_chan_producer,
	    trp->host_dma_assist.read_chan_consumer,
	    trp->host_dma_assist.read_chan_ref,
	    nicfp->stats.nicDmaReadRingFull);
        return (ERROR);
    }

    dd_prod->host_addr = host_addr; 
    dd_prod->nic_addr  = nic_addr; 
    dd_prod->len       = len;
    dd_prod->type      = type;
    dd_prod->index     = index;
    dd_prod->state     = state;
    dd_prod->cksum     = (U32)dd_cons; /* only to debug dma overrun */

    trp->host_dma_assist.read_chan_producer = ndd_prod; 

    /* rd dma completion avoidance logic */
    ref_prod = nicfp->rd_dma_ref_prod;
    ref_type = ref_prod->type;
    if ((ref_type & RD_DMA_GROUP_1) && (type & RD_DMA_GROUP_1) &&
	(ref_prod->count < RD_DMA_MAX_DELTA)) { /* hit */
		
	ref_prod->count++;

	if (get_next_rd_dma_ref(nicfp->rd_dma_ref_cons) == ref_prod) {
	    /*
	     * only one pending dma completion
	     * to prevent dma from taking premature dma completion
	     * bump the ref pointer to keep it one behind the prod pointer
	     */ 
	    trp->host_dma_assist.read_chan_ref++;
	}

	NIC_TRACEV_IF(TRACE_TYPE_DMA, "QrDmaHit", 0x11600,
		      type, TIGON_TYPE_TX,
	    trp->host_dma_assist.read_chan_producer,
	    trp->host_dma_assist.read_chan_consumer,
	    trp->host_dma_assist.read_chan_ref,
	    (type << 16) | (ref_prod->count));

    } else { /* miss */
	/* set up new type */
	nicfp->rd_dma_ref_prod = ref_prod = get_next_rd_dma_ref(ref_prod);
	ASSERT((ref_prod != nicfp->rd_dma_ref_cons) &&
	       (ref_prod->type == TIGON_TYPE_NULL));
	ref_prod->type = type;
	ref_prod->count = 1;
    }

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QrDmaIdx", 0x11700, 
		 type, TIGON_TYPE_TX,
	(dd_prod - tdp->rd_dma_ring),
        trp->host_dma_assist.read_chan_producer,
        trp->host_dma_assist.read_chan_consumer,
        trp->host_dma_assist.read_chan_ref);

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QrDmaD0", 0x11800,
		 type, TIGON_TYPE_TX,
	*(U32 *)dd_prod,	/* host addr */
	*((U32 *)dd_prod + 1),	/* host addr */
	*((U32 *)dd_prod + 2),	/* nic addr */
	*((U32 *)dd_prod + 3));	/* reserved & len */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QrDmaD1", 0x11900,
		 type, TIGON_TYPE_TX,
	*((U32 *)dd_prod + 4),	/* state */
	*((U32 *)dd_prod + 5),	/* reserved & cksum addr */
	*((U32 *)dd_prod + 6),	/* type */
	*((U32 *)dd_prod + 7));	/* index */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QrDmaAvd", 0x12000,
		 type, TIGON_TYPE_TX,
        nicfp->rd_dma_ref_prod,
        nicfp->rd_dma_ref_cons,
        ref_prod->type,
        ref_prod->count);

    return (OK);
}

/* null routine to assist with scratchpad setup */
void q_dma_to_nic_end(void) {}

#if !USE_ASM
U32
q_dma_to_nic_cksum(tg_hostaddr_t host_addr, U32 nic_addr, U32 len, 
                   U32 type, U32 index, U32 state) 
{ 
    struct tg_dma_descr *dd_prod, *ndd_prod; 

    dd_prod = trp->host_dma_assist.read_chan_producer; 
    if ((ndd_prod = get_next_rd_dma_descr(dd_prod)) ==
	trp->host_dma_assist.read_chan_ref) { 

        nicfp->stats.nicDmaReadRingFull++; 

	NIC_TRACE(TRACE_TYPE_DMA, "QrDmaF", 0x12100, 
	    trp->host_dma_assist.read_chan_producer,
	    trp->host_dma_assist.read_chan_consumer,
	    trp->host_dma_assist.read_chan_ref,
	    nicfp->stats.nicDmaReadRingFull);
        return (ERROR);
    }
    ASSERT(ndd_prod != trp->host_dma_assist.read_chan_consumer);

    dd_prod->host_addr = host_addr;
    dd_prod->nic_addr  = nic_addr;
    dd_prod->len       = len;
    dd_prod->type      = type;
    dd_prod->index     = index;
    dd_prod->cksum     = (U32)((U8*)&dd_prod->cksum + 2); 
    dd_prod->state     = state;

    trp->host_dma_assist.read_chan_producer = ndd_prod; 

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QrDmaIdx", 0x12200, 
		 type, TIGON_TYPE_TX,
	(dd_prod - tdp->rd_dma_ring),
        trp->host_dma_assist.read_chan_producer,
        trp->host_dma_assist.read_chan_consumer,
        trp->host_dma_assist.read_chan_ref);

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QrDmaD0", 0x12300,
		 type, TIGON_TYPE_TX,
	*(U32 *)dd_prod,	/* host addr */
	*((U32 *)dd_prod + 1),	/* host addr */
	*((U32 *)dd_prod + 2),	/* nic addr */
	*((U32 *)dd_prod + 3));	/* reserved & len */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "QrDmaD1", 0x12400,
		 type, TIGON_TYPE_TX,
	*((U32 *)dd_prod + 4),	/* state */
	*((U32 *)dd_prod + 5),	/* reserved & cksum addr */
	*((U32 *)dd_prod + 6),	/* type */
	*((U32 *)dd_prod + 7));	/* index */

    return (OK);
}

/* null routine to assist with scratchpad setup */
void q_dma_to_nic_cksum_end(void) {}
#endif /* !USE_ASM */

void
h_dma_rd_attn(void)
{
    struct tg_dma_descr *dd_cons;
    U32 dma_rd_state;
    U32 assist_state;

    dd_cons = trp->host_dma_assist.read_chan_consumer;
    dma_rd_state = trp->host_dma.dma_rd_state;
    assist_state = trp->host_dma_assist.assist_state;

    NIC_UTRACE("#rDmaATN", 0x12500,
	assist_state,
	dma_rd_state,
	trp->host_dma_assist.read_chan_producer,
	dd_cons);

    NIC_UTRACE("rDmaATN0", 0x12510,
	*(U32 *)dd_cons,	/* host addr */
	*((U32 *)dd_cons + 1),	/* host addr */
	*((U32 *)dd_cons + 2),	/* nic addr */
	*((U32 *)dd_cons + 3));	/* reserved & len */

    NIC_UTRACE("rDmaATN1", 0x12520,
	*((U32 *)dd_cons + 4),	/* state */
	*((U32 *)dd_cons + 5),	/* reserved & cksum addr */
	*((U32 *)dd_cons + 6),	/* type */
	*((U32 *)dd_cons + 7));	/* index */

    /* work around Tigon chip bugs */
    if (((dma_rd_state & TG_DMA_STATE_ERROR_MASK) == 
	    TG_DMA_STATE_CROSS_ADDR_ERROR) &&
	((trp->host_dma.dma_rd_len & 0xffff) == 0)) {

	/* 4GB cross over */
	trp->host_dma_assist.assist_state = assist_state | TG_DMA_ASSIST_PAUSE;
	trp->host_dma.dma_rd_state = dma_rd_state | TG_DMA_STATE_RESET;
	trp->host_dma_assist.read_chan_consumer = dd_cons;
	trp->host_dma_assist.assist_state = assist_state;
	NIC_UTRACE("rDma4Gb", 0x2F030, 0, 0, 0, 0);

    } else if (is_global_flags(GF_MOD_SBUS) &&
	       (dma_rd_state & TG_DMA_STATE_PCI_MASTER_ABORT)) {

	/* fix potential hang problem of dma channal between SBUS and X-Chip */
	trp->host_dma_assist.assist_state = assist_state | TG_DMA_ASSIST_PAUSE;
	trp->host_dma.dma_rd_state = dma_rd_state | TG_DMA_STATE_RESET;
	trp->host_dma_assist.read_chan_consumer = dd_cons;
	trp->host_dma_assist.assist_state = assist_state;
	nicfp->stats.nicDmaRdMasterAborts++;

    } else {
	PANIC();
    }
    RETURN(TG_FW_EVENT_NUM_DMA_RD_ATTN);
}

void
h_dma_wr_attn(void)
{
    struct tg_dma_descr *dd_cons;
    U32 dma_wr_state;
    U32 assist_state;

    dd_cons = trp->host_dma_assist.write_chan_consumer;
    dma_wr_state = trp->host_dma.dma_wr_state;
    assist_state = trp->host_dma_assist.assist_state;

    NIC_UTRACE("#wDmaATN", 0x12600,
	assist_state,
	dma_wr_state,
	trp->host_dma_assist.write_chan_producer,
	dd_cons);

    NIC_UTRACE("wDmaATN0", 0x12610,
	*(U32 *)dd_cons,	/* host addr */
	*((U32 *)dd_cons + 1),	/* host addr */
	*((U32 *)dd_cons + 2),	/* nic addr */
	*((U32 *)dd_cons + 3));	/* reserved & len */

    NIC_UTRACE("wDmaATN1", 0x12620,
	*((U32 *)dd_cons + 4),	/* state */
	*((U32 *)dd_cons + 5),	/* reserved & cksum addr */
	*((U32 *)dd_cons + 6),	/* type */
	*((U32 *)dd_cons + 7));	/* index */

    /* work around Tigon chip bugs */
    if (((dma_wr_state & TG_DMA_STATE_ERROR_MASK) == 
	    TG_DMA_STATE_CROSS_ADDR_ERROR) &&
	((trp->host_dma.dma_wr_len & 0xffff) == 0)) {

	/* 4GB cross over */
	trp->host_dma_assist.assist_state = assist_state | TG_DMA_ASSIST_PAUSE;
	trp->host_dma.dma_wr_state = dma_wr_state | TG_DMA_STATE_RESET;
	trp->host_dma_assist.write_chan_consumer = dd_cons;
	trp->host_dma_assist.assist_state = assist_state;
	NIC_UTRACE("wDma4Gb", 0x1F030, 0, 0, 0, 0);

    } else if (is_global_flags(GF_MOD_SBUS) &&
	       (dma_wr_state & TG_DMA_STATE_PCI_MASTER_ABORT)) {

	/* fix potential hang problem of dma channal between SBUS and X-Chip */
	trp->host_dma_assist.assist_state = assist_state | TG_DMA_ASSIST_PAUSE;
	trp->host_dma.dma_wr_state = dma_wr_state | TG_DMA_STATE_RESET;
	trp->host_dma_assist.write_chan_consumer = dd_cons;
	trp->host_dma_assist.assist_state = assist_state;
	nicfp->stats.nicDmaWrMasterAborts++;

    } else {
	PANIC();
    }
    RETURN(TG_FW_EVENT_NUM_DMA_WR_ATTN);
}

tgDmaDescr_t preload_wr_descr;  /* locate in scratchpad */
tgDmaDescr_t preload_rd_descr;  /* locate in scratchpad */
tgDmaDescr_t *preload_wr_descr_p;
tgDmaDescr_t *preload_rd_descr_p;

void
sw_dma_assist(void)
{
    register U32 dma_assist_reg;

    register U32 preload_wr = 0;
    register U32 preload_rd = 0;
 
#define WR_PROD trp->host_dma_assist.write_chan_producer
#define WR_CONS trp->host_dma_assist.write_chan_consumer
#define WR_REF  trp->host_dma_assist.write_chan_ref

#define RD_PROD trp->host_dma_assist.read_chan_producer
#define RD_CONS trp->host_dma_assist.read_chan_consumer
#define RD_REF  trp->host_dma_assist.read_chan_ref

    register tgDmaDescr_t *wr_cons;
    register tgDmaDescr_t *rd_cons;

    register tgDmaDescr_t *preload_wr_cons = 0;
    register tgDmaDescr_t *preload_rd_cons = 0;

    register U32 wr_started = 0;
    register U32 rd_started = 0;

    register U16 *wr_cksum_addr = 0;
    register U16 *rd_cksum_addr = 0;

    wr_cons = WR_CONS;
    rd_cons = RD_CONS;

    /* WARNING - this assumes a merged DMA ring */
    while(1) {
        dma_assist_reg = trp->host_dma_assist.assist_state;
        if (!preload_wr) {
            if (wr_started) {
                if (WR_PROD != preload_wr_cons) {
                    preload_wr_descr_p->host_addr = preload_wr_cons->host_addr;
                    preload_wr_descr_p->nic_addr = preload_wr_cons->nic_addr;
                    preload_wr_descr_p->len = preload_wr_cons->len;
                    preload_wr_descr_p->state = preload_wr_cons->state;
                    preload_wr_descr_p->cksum = preload_wr_cons->cksum;

                    preload_wr_cons = preload_wr_cons + 1;

                    preload_wr = 1;
                    preload_wr_cons =
			(preload_wr_cons >
			&tdp->wr_dma_ring[TG_DMA_ASSIST_DESCRS - 1])?
			    &tdp->wr_dma_ring[0]: 
			    preload_wr_cons;
                }
            } else {
                if (dma_assist_reg & TG_DMA_ASSIST_WR_NOT_EMPTY) {
                    preload_wr_descr_p->host_addr = wr_cons->host_addr;
                    preload_wr_descr_p->nic_addr = wr_cons->nic_addr;
                    preload_wr_descr_p->len = wr_cons->len;
                    preload_wr_descr_p->state = wr_cons->state;
                    preload_wr_descr_p->cksum = wr_cons->cksum;

                    preload_wr_cons = wr_cons + 1;

                    preload_wr = 1;
                    preload_wr_cons =
			(preload_wr_cons > 
			&tdp->wr_dma_ring[TG_DMA_ASSIST_DESCRS - 1])?
			    &tdp->wr_dma_ring[0]:
			    preload_wr_cons;
                }
            }
        }

        if (!preload_rd) {
            if (rd_started) {
                if (RD_PROD != preload_rd_cons) {
                    preload_rd_descr_p->host_addr = preload_rd_cons->host_addr;
                    preload_rd_descr_p->nic_addr = preload_rd_cons->nic_addr;
                    preload_rd_descr_p->len = preload_rd_cons->len;
                    preload_rd_descr_p->state = preload_rd_cons->state;
                    preload_rd_descr_p->cksum = preload_rd_cons->cksum;

                    preload_rd_cons = preload_rd_cons + 1;

                    preload_rd = 1;
                    preload_rd_cons =
			(preload_rd_cons > 
			&tdp->rd_dma_ring[TG_DMA_ASSIST_DESCRS - 1])?
			    &tdp->rd_dma_ring[0]:
			    preload_rd_cons;
                }
            } else {
                if (dma_assist_reg & TG_DMA_ASSIST_RD_NOT_EMPTY) {
                    preload_rd_descr_p->host_addr = rd_cons->host_addr;
                    preload_rd_descr_p->nic_addr = rd_cons->nic_addr;
                    preload_rd_descr_p->len = rd_cons->len;
                    preload_rd_descr_p->state = rd_cons->state;
                    preload_rd_descr_p->cksum = rd_cons->cksum;

                    preload_rd_cons = rd_cons + 1;

                    preload_rd = 1;
                    preload_rd_cons =
			(preload_rd_cons > 
			&tdp->rd_dma_ring[TG_DMA_ASSIST_DESCRS - 1])?
			    &tdp->rd_dma_ring[0]:
			    preload_rd_cons;
                }
            }
        }

        if (!(dma_assist_reg & TG_DMA_ASSIST_WR_BUSY)) {
                
            if (wr_started) {
                if (wr_started & TG_DMA_STATE_SAVE_CKSUM) {
                    *wr_cksum_addr = trp->host_dma.dma_wr_tcpip_cksum;
                }

                WR_CONS = wr_cons + 1;
                wr_cons = WR_CONS;
                wr_started = 0;
            }

            if (preload_wr) {
                /* check upper word and don't write if not necessary */
                trp->host_dma.dma_wr_host_addr = preload_wr_descr_p->host_addr;
                trp->host_dma.dma_wr_loc_addr = preload_wr_descr_p->nic_addr;
                trp->host_dma.dma_wr_len = preload_wr_descr_p->len;
                trp->host_dma.dma_wr_state = preload_wr_descr_p->state;

                wr_started = preload_wr_descr_p->state;
                wr_cksum_addr = (U16 *)preload_wr_descr_p->cksum;
                preload_wr = 0;
            }
        }
        if (!(dma_assist_reg & TG_DMA_ASSIST_RD_BUSY)) {

            if (rd_started) {
                if (rd_started & TG_DMA_STATE_SAVE_CKSUM) {
                    *rd_cksum_addr = trp->host_dma.dma_rd_tcpip_cksum;
                }

                /* increment TX BUF producer */
                if (rd_started & TG_DMA_STATE_UPDATE_TX_BUF_PROD)
                    trp->local_mem_conf.tx_buf_producer = 
                        trp->host_dma.dma_rd_loc_addr;

                RD_CONS = rd_cons + 1;
                rd_cons = RD_CONS;
                rd_started = 0;
            }

            if (preload_rd) {
                /* check upper word and don't write if not necessary */

                trp->host_dma.dma_rd_host_addr = preload_rd_descr_p->host_addr;
                trp->host_dma.dma_rd_loc_addr = preload_rd_descr_p->nic_addr;
                trp->host_dma.dma_rd_len = preload_rd_descr_p->len;
                trp->host_dma.dma_rd_state = preload_rd_descr_p->state;

                rd_started = preload_rd_descr_p->state;
                rd_cksum_addr = (U16 *)preload_rd_descr_p->cksum;
                preload_rd = 0;
            }
        }
    }
    /* not reached */

#undef WR_PROD
#undef WR_CONS
#undef WR_REF
#undef RD_PROD
#undef RD_CONS
#undef RD_REF
}

/* null routine to assist with scratchpad setup */
void sw_dma_assist_end(void) {}

void
sw_dma_assist_plus_timer(void)
{
    register U32 dma_assist_reg;

    register U32 preload_wr = 0;
    register U32 preload_rd = 0;
 
#define WR_PROD trp->host_dma_assist.write_chan_producer
#define WR_CONS trp->host_dma_assist.write_chan_consumer
#define WR_REF  trp->host_dma_assist.write_chan_ref

#define RD_PROD trp->host_dma_assist.read_chan_producer
#define RD_CONS trp->host_dma_assist.read_chan_consumer
#define RD_REF  trp->host_dma_assist.read_chan_ref

    register tgDmaDescr_t *wr_cons;
    register tgDmaDescr_t *rd_cons;

    register tgDmaDescr_t *preload_wr_cons = 0;
    register tgDmaDescr_t *preload_rd_cons = 0;

    register U32 wr_started = 0;
    register U32 rd_started = 0;

    register U16 *wr_cksum_addr = 0;
    register U16 *rd_cksum_addr = 0;

    wr_cons = WR_CONS;
    rd_cons = RD_CONS;

    /* WARNING - this assumes a merged DMA ring */
    while(1) {
        dma_assist_reg = trp->host_dma_assist.assist_state;
        if (!preload_wr) {
            if (wr_started) {
                if (WR_PROD != preload_wr_cons) {
                    preload_wr_descr_p->host_addr = preload_wr_cons->host_addr;
                    preload_wr_descr_p->nic_addr = preload_wr_cons->nic_addr;
                    preload_wr_descr_p->len = preload_wr_cons->len;
                    preload_wr_descr_p->state = preload_wr_cons->state;
                    preload_wr_descr_p->cksum = preload_wr_cons->cksum;

                    preload_wr_cons = preload_wr_cons + 1;

                    preload_wr = 1;
                    preload_wr_cons =
			(preload_wr_cons > 
			&tdp->wr_dma_ring[TG_DMA_ASSIST_DESCRS - 1])?
			    &tdp->wr_dma_ring[0]:
			    preload_wr_cons;
                }
            } else {
                if (dma_assist_reg & TG_DMA_ASSIST_WR_NOT_EMPTY) {
                    preload_wr_descr_p->host_addr = wr_cons->host_addr;
                    preload_wr_descr_p->nic_addr = wr_cons->nic_addr;
                    preload_wr_descr_p->len = wr_cons->len;
                    preload_wr_descr_p->state = wr_cons->state;
                    preload_wr_descr_p->cksum = wr_cons->cksum;

                    preload_wr_cons = wr_cons + 1;

                    preload_wr = 1;
                    preload_wr_cons =
			(preload_wr_cons > 
			&tdp->wr_dma_ring[TG_DMA_ASSIST_DESCRS - 1])?
			    &tdp->wr_dma_ring[0]:
			    preload_wr_cons;
                }
            }
        }

        if (!preload_rd) {
            if (rd_started) {
                if (RD_PROD != preload_rd_cons) {
                    preload_rd_descr_p->host_addr = preload_rd_cons->host_addr;
                    preload_rd_descr_p->nic_addr = preload_rd_cons->nic_addr;
                    preload_rd_descr_p->len = preload_rd_cons->len;
                    preload_rd_descr_p->state = preload_rd_cons->state;
                    preload_rd_descr_p->cksum = preload_rd_cons->cksum;

                    preload_rd_cons = preload_rd_cons + 1;

                    preload_rd = 1;
                    preload_rd_cons =
			(preload_rd_cons > 
			&tdp->rd_dma_ring[TG_DMA_ASSIST_DESCRS - 1])?
			    &tdp->rd_dma_ring[0]:
			    preload_rd_cons;
                }
            } else {
                if (dma_assist_reg & TG_DMA_ASSIST_RD_NOT_EMPTY) {
                    preload_rd_descr_p->host_addr = rd_cons->host_addr;
                    preload_rd_descr_p->nic_addr = rd_cons->nic_addr;
                    preload_rd_descr_p->len = rd_cons->len;
                    preload_rd_descr_p->state = rd_cons->state;
                    preload_rd_descr_p->cksum = rd_cons->cksum;

                    preload_rd_cons = rd_cons + 1;

                    preload_rd = 1;
                    preload_rd_cons =
			(preload_rd_cons > 
			&tdp->rd_dma_ring[TG_DMA_ASSIST_DESCRS - 1])?
			    &tdp->rd_dma_ring[0]:
			    preload_rd_cons;
                }
            }
        }

        if (!(dma_assist_reg & TG_DMA_ASSIST_WR_BUSY)) {
                
            if (wr_started) {
                if (wr_started & TG_DMA_STATE_SAVE_CKSUM) {
                    *wr_cksum_addr = trp->host_dma.dma_wr_tcpip_cksum;
                }

                WR_CONS = wr_cons + 1;
                wr_cons = WR_CONS;
                wr_started = 0;
            }

            if (preload_wr) {
                /* check upper word and don't write if not necessary */
                trp->host_dma.dma_wr_host_addr = preload_wr_descr_p->host_addr;
                trp->host_dma.dma_wr_loc_addr = preload_wr_descr_p->nic_addr;
                trp->host_dma.dma_wr_len = preload_wr_descr_p->len;
                trp->host_dma.dma_wr_state = preload_wr_descr_p->state;

                wr_started = preload_wr_descr_p->state;
                wr_cksum_addr = (U16 *)preload_wr_descr_p->cksum;
                preload_wr = 0;
            }
        }
        if (!(dma_assist_reg & TG_DMA_ASSIST_RD_BUSY)) {

            if (rd_started) {
                if (rd_started & TG_DMA_STATE_SAVE_CKSUM) {
                    *rd_cksum_addr = trp->host_dma.dma_rd_tcpip_cksum;
                }

                /* increment TX BUF producer */
                if (rd_started & TG_DMA_STATE_UPDATE_TX_BUF_PROD)
                    trp->local_mem_conf.tx_buf_producer = 
                        trp->host_dma.dma_rd_loc_addr;

                RD_CONS = rd_cons + 1;
                rd_cons = RD_CONS;
                rd_started = 0;
            }

            if (preload_rd) {
                /* check upper word and don't write if not necessary */

                trp->host_dma.dma_rd_host_addr = preload_rd_descr_p->host_addr;
                trp->host_dma.dma_rd_loc_addr = preload_rd_descr_p->nic_addr;
                trp->host_dma.dma_rd_len = preload_rd_descr_p->len;
                trp->host_dma.dma_rd_state = preload_rd_descr_p->state;

                rd_started = preload_rd_descr_p->state;
                rd_cksum_addr = (U16 *)preload_rd_descr_p->cksum;
                preload_rd = 0;
            }
        }

        /* high frequency timer loop */
        if (trp->gen_control.event_b & TG_FW_EVENT_TIMER) {

	    register U32 event;
	    U32 send_coal_ticks;
	    U32 recv_coal_ticks;

            /* reset timer */
            trp->gen_control.timer_ref_b = 
                trp->gen_control.timer + TG_TICK_FAST;

            event = 0;  

	    send_coal_ticks = tsp->gen_com.tuneParms.send_coal_ticks;
	    if (send_coal_ticks &&
		    ((nicp->send_coal_tick_count += TG_TICK_FAST) >=
		    send_coal_ticks)) {
		nicp->send_coal_tick_count = 0;
		nicp->send_cons_index_to_host_pending = TRUE;
		event |= TG_FW_EVENT_DO_TX_CONS;
            } else if (nicp->send_cons_index_to_host_pending ||
		       nicp->send_cons_index_to_host_pending_no_intr) {
                event |= TG_FW_EVENT_DO_TX_CONS;
            }

	    recv_coal_ticks = tsp->gen_com.tuneParms.recv_coal_ticks;
            if (recv_coal_ticks &&
		    ((nicp->recv_coal_tick_count += TG_TICK_FAST) >=
		    recv_coal_ticks)) {
		nicp->recv_coal_tick_count = 0;
                event |= TG_FW_EVENT_DO_RX_PROD;
            } else if (nicp->recv_ret_prod_index_to_host_pending) {
                event |= TG_FW_EVENT_DO_RX_PROD;
            }

            if (event)
                set_event_reg(event);
        }
    }
    /* not reached */

#undef WR_PROD
#undef WR_CONS
#undef WR_REF
#undef RD_PROD
#undef RD_CONS
#undef RD_REF
}

/* null routine to assist with scratchpad setup */
void sw_dma_assist_plus_timer_end(void) {}
