/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: dma.h $
 * Revision api_12_4_latest_hayes/2 1999/11/22 22:58:19 hayes
 * 	Change OR of DMA bits to set
 * 
 * Revision api12_3_ted/2 1999/08/12 21:06:47 ted
 * 	Assemblerize header files
 * 
 * Revision 1.1.2.14  1998/12/21  00:33:11  shuang
 * 	fix - return last send BD
 * 	[1998/12/21  00:04:44  shuang]
 *
 * Revision 1.1.2.13  1998/12/08  02:36:26  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:27:46  shuang]
 * 
 * Revision 1.1.2.12  1998/10/14  01:14:23  shuang
 * 	support tx buffer ratio
 * 	[1998/10/14  01:06:46  shuang]
 * 
 * Revision 1.1.2.11  1998/09/30  18:49:52  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:39:43  shuang]
 * 
 * Revision 1.1.2.10  1998/04/11  05:15:05  shuang
 * 	move fast timer to cpu B, add receive coalesed ticks
 * 	[1998/04/11  04:57:56  shuang]
 * 
 * Revision 1.1.2.9  1998/04/01  22:21:22  hayes
 * 	change INC_XXX to GET_NEXT_XXX
 * 	[1998/04/01  01:00:10  hayes]
 * 
 * 	checkpoint
 * 	[1998/03/31  19:56:52  hayes]
 * 
 * 	checkpoint- dma completion avoidance works
 * 	[1998/03/30  07:00:28  hayes]
 * 
 * Revision 1.1.2.8  1998/03/26  01:34:23  shuang
 * 	cleanup
 * 	[1998/03/26  01:25:00  shuang]
 * 
 * Revision 1.1.2.7  1998/02/19  01:29:51  suresh
 * 	     simplification since we don't worry about dummy DMAs any more
 * 	     [1998/02/19  01:17:55  suresh]
 * 
 * 	     intermediate checksum offload state
 * 	     [1998/02/15  01:50:28  suresh]
 * 
 * Revision 1.1.2.6  1998/02/11  19:11:21  ted
 * 	     Undo functionization of q_dma_event_prod
 * 	     [1998/02/11  18:40:24  ted]
 * 
 * Revision 1.1.2.5  1998/02/09  17:18:05  hayes
 * 	     convert Q_DMA_XXX macros to functions
 * 	     [1998/02/09  17:16:55  hayes]
 * 
 * Revision 1.1.2.4  1998/02/05  20:22:19  ted
 * 	     fix stats, was using wrong structure
 * 	     [1998/02/05  20:17:17  ted]
 * 
 * Revision 1.1.2.3  1998/02/04  23:04:46  ted
 * 	     Move the DMA reference entry by entry when transferring receive BDs to the NIC, PR 813
 * 	     [1998/02/04  23:04:04  ted]
 * 
 * Revision 1.1.2.2  1998/01/29  02:32:54  skapur
 * 	     Check in for Rev6 checksum support
 * 	     [1998/01/29  02:27:05  skapur]
 * 
 * Revision 1.1.2.1  1998/01/14  23:25:42  ted
 * 	     rearrange directories
 * 	     [1998/01/14  23:17:41  ted]
 * 
 * Revision 1.1.2.4  1998/01/14  19:56:03  skapur
 * 	     Merged with changes from 1.1.2.3
 * 	     [1998/01/14  19:55:41  skapur]
 * 
 * 	     Checksum off-load support for Tigon2. This is an intermediate checkin; more work needs to be done
 * 	     [1998/01/14  19:26:19  skapur]
 * 
 * Revision 1.1.2.3  1997/12/27  23:18:56  hayes
 * 	     fix previous checkin...
 * 	     [1997/12/27  23:18:47  hayes]
 * 
 * Revision 1.1.2.2  1997/12/27  22:22:01  hayes
 * 	     use more Tigon 2 hardware assist
 * 	     [1997/12/27  22:21:48  hayes]
 * 
 * Revision 1.1.2.1  1997/11/19  19:23:06  ted
 * 	     Support Tigon and Tigon2 in the same driver
 * 	     [1997/11/19  19:19:05  ted]
 * 
 * Revision 1.9.7.13  1997/11/14  19:16:20  hayes
 * 	     first checkin after mongo merge...
 * 	     [1997/11/14  19:11:23  hayes]
 * 
 * 	     checkin after stats restructuring
 * 	     [1997/11/12  19:01:15  hayes]
 * 
 * Revision 1.9.7.12  1997/11/11  19:05:55  hayes
 * 	     restructuring for scratchpad use
 * 	     [1997/11/11  19:04:16  hayes]
 * 
 * Revision 1.9.7.11  1997/10/21  01:35:31  skapur
 * 	     Merge of release 2.0 and ge_latest branch
 * 	     [1997/10/21  01:30:52  skapur]
 * 
 * Revision 1.9.7.10  1997/09/16  02:38:20  skapur
 * 	     Merged with changes from 1.9.7.9
 * 	     [1997/09/16  02:38:12  skapur]
 * 
 * 	     Took out ifdefed code and cleaned out the traces
 * 	     [1997/09/16  02:32:28  skapur]
 * 
 * Revision 1.9.7.9  1997/09/11  18:05:40  ted
 * 	     Mixing vlans, jumbos, and redundancy works, plus small fixes from release 1.1 and better NIC tracing
 * 	     [1997/09/11  17:59:30  ted]
 * 
 * Revision 1.9.7.8  1997/08/13  18:46:00  ted
 * 	     Put index in DMA entry to enhance debuggability
 * 	     [1997/08/13  18:39:36  ted]
 * 
 * Revision 1.9.7.7  1997/07/25  22:58:56  ted
 * 	     Architectural performance change
 * 	     [1997/07/24  21:45:03  ted]
 * 
 * Revision 1.9.7.6  1997/05/21  03:35:38  ted
 * 	     Put index and type back into Q_DMA_TO_NIC for fix on bug 189
 * 	     [1997/05/21  03:02:37  ted]
 * 
 * Revision 1.9.7.5  1997/04/11  23:36:13  ted
 * 	     Modified code for performance enhancements
 * 	     [1997/04/11  23:15:45  ted]
 * 
 * Revision 1.9.7.4  1997/03/28  02:46:58  skapur
 * 	     Added NIC Stats counters to keep track of Full DMA Rings
 * 	     [1997/03/28  01:59:08  skapur]
 * 
 * Revision 1.9.7.3  1997/03/24  08:34:19  ted
 * 	     New event ring handling, producer lives in host memory, misc. other more modest changes
 * 	     [1997/03/24  08:27:44  ted]
 * 
 * Revision 1.9.7.2  1997/03/21  04:15:32  skapur
 * 	     Performance enhancement for Recv and Send; lots changes; almost complete overhaul of the entire firmware
 * 	     [1997/03/21  04:08:02  skapur]
 * 
 * Revision 1.9.7.1  1997/03/17  08:47:40  skapur
 * 	     integration changes...
 * 	     [1997/03/16  03:01:42  skapur]
 * 
 * Revision 1.9.2.4  1996/11/29  17:43:19  hayes
 * 	     Fixed up DMA macros to correspond to updated tg.h and moved data
 * 	     structures in TDP.
 * 	     [1996/11/29  17:41:16  hayes]
 * 
 * Revision 1.9.2.3  1996/11/15  04:24:04  hayes
 * 	     convert to new header file structure.
 * 	     [1996/11/15  04:12:12  hayes]
 * 
 * Revision 1.9.2.2  1996/11/05  21:54:17  hayes
 * 	     Final pre-ODE merge
 * 	     [1996/11/05  21:51:29  hayes]
 * 
 * Revision 1.9.2.1  1996/11/01  18:24:51  ted
 * 	     change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	     [1996/10/29  22:05:54  ted]
 * 
 * $EndLog$
 */
/*
 * FILE dma.h
 *
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/dma.h,v $
 * $Revision: 1.1.2.14 $ $Locker:  $
 * $Date: 1998/12/21 00:33:11 $ $Author: shuang $ $State: Exp $
 */

#ifndef _DMA_H_
#define _DMA_H_

#include "trace.h"

/* grouping of dma avoidance */
#define	WR_DMA_GROUP_1	(TIGON_TYPE_RX_PROD | \
			 TIGON_TYPE_TX_CONS | \
			 TIGON_TYPE_EVENT_PROD)

#define	WR_DMA_GROUP_2	((U16)~WR_DMA_GROUP_1)

#define RD_DMA_GROUP_1	(TIGON_TYPE_SEND_DATA | \
			 TIGON_TYPE_SEND_DATA_LAST)

#define	RD_DMA_GROUP_2	((U16)~RD_DMA_GROUP_1)

/* grouping of BD types to control tracing */
#define	TIGON_TYPE_TX	(TIGON_TYPE_SEND_BD | \
			 TIGON_TYPE_SEND_DATA | \
			 TIGON_TYPE_SEND_DATA_LAST | \
			 TIGON_TYPE_SEND_DATA_LAST_VLAN | \
			 TIGON_TYPE_SEND_DATA_COAL | \
			 TIGON_TYPE_SEND_DATA_LAST_COAL | \
			 TIGON_TYPE_SEND_DATA_LAST_VLAN_COAL | \
			 TIGON_TYPE_RECV_BD | \
			 TIGON_TYPE_RECV_JUMBO_BD | \
			 TIGON_TYPE_RECV_MINI_BD | \
			 TIGON_TYPE_RECV_BD_LAST)

#define	TIGON_TYPE_RX	(TIGON_TYPE_RECV_DATA | \
			 TIGON_TYPE_RECV_DATA_LAST | \
			 TIGON_TYPE_RECV_BD | \
			 TIGON_TYPE_RECV_JUMBO_BD | \
			 TIGON_TYPE_RECV_MINI_BD | \
			 TIGON_TYPE_RECV_BD_LAST)


#if !defined(_IN_ASM_)
__inline__ static tgDmaDescr_t *
get_next_rd_dma_descr(tgDmaDescr_t *ddp)
{
    return ((ddp == &tdp->rd_dma_ring[TG_DMA_ASSIST_DESCRS - 1])?
	    tdp->rd_dma_ring:
	    (ddp + 1));
}

__inline__ static tgDmaDescr_t *
get_next_wr_dma_descr(tgDmaDescr_t *ddp)
{
    return ((ddp == &tdp->wr_dma_ring[TG_DMA_ASSIST_DESCRS - 1])?
	    tdp->wr_dma_ring:
	    (ddp + 1));
}

__inline__ static U32
get_next_dma_descr_index(U32 index)
{
    return ((index == (TG_DMA_ASSIST_DESCRS - 1))?
	    0:
	    (index + 1));
}

__inline__ static dma_ref_ring_t *
get_next_rd_dma_ref(dma_ref_ring_t *drp)
{
    return ((drp == &nicp->rd_dma_ref_ring[DMA_REF_RING_ENTRIES - 1])?
	    nicp->rd_dma_ref_ring:
	    (drp + 1));
}

__inline__ static dma_ref_ring_t *
get_next_wr_dma_ref(dma_ref_ring_t *drp)
{
    return ((drp == &nicp->wr_dma_ref_ring[DMA_REF_RING_ENTRIES - 1])?
	    nicp->wr_dma_ref_ring:
	    (drp + 1));
}

__inline__ static bool
can_q_dma_to_host(S32 n)
{
    struct tg_dma_descr *dd_prod, *dd_ref, *dd_cons;
    S32 m, k;

    if (nicfp->wr_dma_free_cnt > n)
	return (TRUE);

    dd_prod = trp->host_dma_assist.write_chan_next; 
    if (!dd_prod) {
	/* ring full */
	nicfp->stats.nicDmaWriteRingFull++; 
	return (FALSE);
    }

    /* re-calculate free count */
    dd_cons = trp->host_dma_assist.write_chan_consumer;
    m = dd_cons - dd_prod;
    if (m <= 0) {
	m += TG_DMA_ASSIST_DESCRS;
    }

    dd_ref = trp->host_dma_assist.write_chan_ref;
    k = dd_ref - dd_prod;
    if (k <= 0) {
	k += TG_DMA_ASSIST_DESCRS;
    }

    nicfp->wr_dma_free_cnt = MIN(m, k);
    if (nicfp->wr_dma_free_cnt > n) {
	NIC_TRACE(TRACE_TYPE_DMA, "canQwDma", 0x15000,
	    dd_prod,
	    dd_cons,
	    dd_ref,
	    (nicfp->wr_dma_free_cnt << 24) | (m << 16) | (k << 8) | n);
	return (TRUE);
    } else {
	/* ring full */
	nicfp->stats.nicDmaWriteRingFull++; 

	NIC_TRACE(TRACE_TYPE_DMA, "cantQwDm", 0x15100,
	    dd_prod,
	    dd_cons,
	    dd_ref,
	    (nicfp->wr_dma_free_cnt << 24) | (m << 16) | (k << 8) | n);
	return (FALSE);
    }
}

__inline__ static U32
q_dma_to_host_index(tg_hostaddr_t host_addr, U32 len, U32 type, 
                    U32 index, U32 state)
{
    return (*nicfp->q_dma_to_host_stub)
		(host_addr,
		(U32)&trp->host_dma_assist.write_chan_producer->index,
		len,
		type,
		index,
		state);
}

__inline__ static void
dma_to_nic(struct recv_bd *bdp)
{
    NIC_TRACE(TRACE_TYPE_DMA, "QrDmaM", 0x15200,
	 (U32)bdp->BD_host_addr,
	 bdp->BD_nic_addr,
	 bdp->BD_w2,
	 0);

    trp->host_dma.dma_rd_host_addr = bdp->BD_host_addr;
    trp->host_dma.dma_rd_loc_addr  = bdp->BD_nic_addr;
    trp->host_dma.dma_rd_len       = bdp->BD_w2 & BD_LEN_MASK;

    /* start the DMA... */
    trp->host_dma.dma_rd_state     = nicfp->dma_to_nic_bd_state;

    /* we'll receive an event when DMA completes... */
    return;
}
#endif /* !defined(_IN_ASM_) */

#endif /* _DMA_H_ */
