/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: fwmain.c $
 * Revision nic_api_12_4_latest_shuang/8 2000/02/29 18:28:37 shuang
 * 	enable support of 2MB external memory
 * 
 * Revision nic_api_12_4_latest_shuang/6 2000/02/12 18:50:05 shuang
 * 	fix incorrect stats nicRecvBDs and nicJumboRecvBDs.
 * 	honor TG_CMD_CODE_STACK_UP and TG_CMD_CODE_STACK_DOWN commands.
 * 
 * Revision nic_api_12_4_latest_shuang/4 2000/02/01 00:48:58 shuang
 * 	clear tx/rx base registers prior to the 1st rd dma.
 * 
 * Revision nic_api_12_4_latest_shuang/2 1999/12/09 19:51:56 shuang
 * 	work around Tg2 MAC6 bug only if get rx buf full attention.
 * 
 * Revision api_12_4_latest_hayes/2 1999/11/16 00:47:17 hayes
 * 	check completion status of gen_info DMA
 * 
 * Revision copper_fhwang/3 1999/10/12 19:38:22 fhwang
 * 	to support new led (4 leds)
 * 
 * Revision copper_fhwang/2 1999/08/27 23:16:55 fhwang
 * 	add checking GMII for correct led config
 * 
 * Revision sgi_api_12_3_latest_shuang/4 1999/07/14 18:58:50 shuang
 * 	work around MAC6 Tigon2 bug, fix rx buffer full cleanup
 * 
 * Revision nic_api_12_3_latest_shuang/3 1999/05/20 23:41:37 shuang
 * 	minimal rx buf is 128Kb, rx high water mark is half of rx buf
 * 
 * Revision snap_ted/2 1999/03/24 00:09:21 ted
 * 	Allow firmware build on x86 solaris
 * 
 * Revision 1.1.2.45  1999/01/24  00:10:55  shuang
 * 	fix tx only cksum mode setup.
 * 	[1999/01/24  00:00:40  shuang]
 *
 * Revision 1.1.2.44  1999/01/16  02:55:38  shuang
 * 	use of mini ring and jumbo ring is optional
 * 	[1999/01/16  02:53:03  shuang]
 * 
 * Revision 1.1.2.43  1998/12/18  01:50:17  shuang
 * 	save some global vars for debugging purpose
 * 	[1998/12/18  01:49:05  shuang]
 * 
 * Revision 1.1.2.42  1998/12/09  18:15:41  shuang
 * 	revise fw init, version log
 * 	[1998/12/09  17:53:35  shuang]
 * 
 * Revision 1.1.2.41  1998/12/08  02:36:28  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:27:50  shuang]
 * 
 * Revision 1.1.2.40  1998/11/19  01:14:18  shuang
 * 	tcp/udp cksum ok for extra padding frames on rx split jumbo
 * 	[1998/11/19  01:13:30  shuang]
 * 
 * Revision 1.1.2.39  1998/11/12  22:20:33  shuang
 * 	fix init chip's vlan tag type
 * 	[1998/11/12  22:19:26  shuang]
 * 
 * Revision 1.1.2.38  1998/10/14  01:14:25  shuang
 * 	support tx buffer ratio
 * 	[1998/10/14  01:06:49  shuang]
 * 
 * Revision 1.1.2.37  1998/09/30  18:49:55  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:39:50  shuang]
 * 
 * Revision 1.1.2.36  1998/09/17  20:58:08  shuang
 * 	fix enq mac tx hang problem on checksum mode
 * 	[1998/09/17  20:54:27  shuang]
 * 
 * Revision 1.1.2.35  1998/08/27  17:33:06  shuang
 * 	fix defect1370- flow control
 * 	[1998/08/27  17:31:42  shuang]
 * 
 * Revision 1.1.2.34  1998/08/03  16:14:26  hayes
 * 	complete firmware update for NT
 * 	[1998/08/03  16:11:17  hayes]
 * 
 * 	checkin so merge w/ fc can happen
 * 	[1998/07/29  23:15:26  hayes]
 * 
 * 	host based send ring now works off RCB flag
 * 	[1998/07/29  18:30:57  hayes]
 * 
 * 	host based send ring works
 * 	[1998/07/28  03:57:18  hayes]
 * 
 * Revision 1.1.2.33  1998/07/29  22:31:02  shuang
 * 	add feature - flow control support
 * 	[1998/07/29  21:59:39  shuang]
 * 
 * Revision 1.1.2.32  1998/07/09  00:08:16  shuang
 * 	Receive checksum offload won't be supported once PCI is configured
 * 	to assert all PBE on the dma write operations since checksum
 * 	generated by the TIGON is bogus in this case.
 * 	[1998/07/09  00:06:56  shuang]
 * 
 * Revision 1.1.2.31  1998/06/25  21:31:22  shuang
 * 	restore back red zone of stack checking.
 * 	[1998/06/25  21:27:16  shuang]
 * 
 * Revision 1.1.2.30  1998/06/23  23:40:17  ted
 * 	Turn off software assist for 1 DMA at a time mode
 * 	[1998/06/23  23:37:39  ted]
 * 
 * Revision 1.1.2.29  1998/06/23  22:24:41  shuang
 * 	mac buffer size should be multiple of 4KB in dynamic nic memory mode.
 * 	[1998/06/23  22:23:21  shuang]
 * 
 * Revision 1.1.2.28  1998/06/23  01:29:34  shuang
 * 	added dynamic nic mem support + relocated cpu b stack to scratchpad.
 * 	[1998/06/23  01:19:50  shuang]
 * 
 * Revision 1.1.2.27  1998/05/30  01:11:08  shuang
 * 	     Added code to differentiate patches for Rev5 sync/async sram.
 * 	     [1998/05/30  01:10:02  shuang]
 * 
 * Revision 1.1.2.26  1998/05/05  23:50:50  shuang
 * 	     always use sw dma assist from now on
 * 	     [1998/05/05  23:49:40  shuang]
 * 
 * Revision 1.1.2.25  1998/04/30  00:42:57  shuang
 * 	     arranged tx space checking for efficiency
 * 	     [1998/04/30  00:40:36  shuang]
 * 
 * Revision 1.1.2.24  1998/04/27  23:56:20  shuang
 * 	     Fix relocated null_handler into scratch pad
 * 	     [1998/04/27  23:55:35  shuang]
 * 
 * Revision 1.1.2.23  1998/04/27  23:49:56  shuang
 * 	     relocated null_handler into scratch pad
 * 	     [1998/04/27  23:48:04  shuang]
 * 
 * Revision 1.1.2.22  1998/04/24  00:00:33  hayes
 * 	     added const to rcsid definitions to eliminate compiler warnings
 * 	     [1998/04/23  23:58:50  hayes]
 * 
 * Revision 1.1.2.21  1998/04/17  07:28:07  ted
 * 	     Swap led routines, backwards check
 * 	     [1998/04/17  07:27:57  ted]
 * 
 * Revision 1.1.2.20  1998/04/15  17:39:10  ted
 * 	     Put init_mac2() ahead of sending FIRMWARE_OPERATIONAL event to avoid long period of nic inactivity when it should be operational
 * 	     [1998/04/15  17:32:03  ted]
 * 
 * Revision 1.1.2.19  1998/04/11  05:15:06  shuang
 * 	     move fast timer to cpu B, add receive coalesed ticks
 * 	     [1998/04/11  04:58:00  shuang]
 * 
 * Revision 1.1.2.18  1998/04/01  22:21:23  hayes
 * 	          init timer before queueing events
 * 	          [1998/04/01  22:18:29  hayes]
 * 
 * 	          checkpoint
 * 	          [1998/03/31  19:56:54  hayes]
 * 
 * 	          checkpoint- dma completion avoidance works
 * 	          [1998/03/30  07:00:29  hayes]
 * 
 * 	          misc cleanup
 * 	          [1998/03/29  04:22:02  hayes]
 * 
 * 	          intermediate checkin
 * 	          [1998/03/29  00:18:05  hayes]
 * 
 * Revision 1.1.2.17  1998/04/01  07:32:21  shuang
 * 	          add enable scratch pad control - code, vars, stack...
 * 	          [1998/04/01  07:23:40  shuang]
 * 
 * Revision 1.1.2.16  1998/04/01  00:34:57  skapur
 * 	          Adding Vlan Tag Insertion/Deletion support
 * 	          [1998/04/01  00:31:57  skapur]
 * 
 * Revision 1.1.2.15  1998/03/26  01:34:24  shuang
 * 	          cleanup
 * 	          [1998/03/26  01:25:03  shuang]
 * 
 * Revision 1.1.2.14  1998/03/04  04:37:09  hayes
 * 	          make scratch_pad_enable external
 * 	          [1998/03/04  04:36:28  hayes]
 * 
 * 	          add fix for static IDD current draw for Tigon Rev6
 * 	          [1998/03/02  21:33:08  hayes]
 * 
 * Revision 1.1.2.13  1998/02/24  17:41:47  shuang
 * 	          Modulize multicast functions
 * 	          [1998/02/24  17:33:01  shuang]
 * 
 * Revision 1.1.2.12  1998/02/20  00:42:17  ted
 * 	          turn off data cache even for non-sw DMA assist case
 * 	          [1998/02/20  00:42:05  ted]
 * 
 * Revision 1.1.2.11  1998/02/19  01:29:53  suresh
 * 	          Checksum offload for Rev 6 support
 * 	          [1998/02/19  01:18:19  suresh]
 * 
 * 	          intermediate checksum offload state
 * 	          [1998/02/15  01:26:19  suresh]
 * 
 * Revision 1.1.2.10  1998/02/11  19:11:21  ted
 * 	          V5/V6 checksum support, first try
 * 	          [1998/02/11  18:45:38  ted]
 * 
 * 	          intermediate checkin for merge
 * 	          [1998/02/10  00:28:29  ted]
 * 
 * Revision 1.1.2.9  1998/02/10  02:49:11  shuang
 * 	          Fix bug in checking stack overlap and move init_trace() call
 * 	          [1998/02/10  02:40:43  shuang]
 * 
 * Revision 1.1.2.8  1998/02/05  20:22:19  ted
 * 	          add support for dontFragJumbos flag
 * 	          [1998/02/05  20:17:47  ted]
 * 
 * Revision 1.1.2.7  1998/02/04  23:04:48  ted
 * 	          Move the DMA reference entry by entry when transferring receive BDs to the NIC, PR 813
 * 	          [1998/02/04  23:04:06  ted]
 * 
 * Revision 1.1.2.6  1998/01/30  06:45:22  hayes
 * 	          remove re-initialization of misc_local_control
 * 	          [1998/01/30  06:42:47  hayes]
 * 
 * Revision 1.1.2.5  1998/01/29  18:40:20  ted
 * 	          Add multicast mode support and move some things into the fast nic structure
 * 	          [1998/01/29  18:28:50  ted]
 * 
 * Revision 1.1.2.4  1998/01/29  02:32:56  skapur
 * 	          Check in for Rev6 checksum support
 * 	          [1998/01/29  02:27:09  skapur]
 * 
 * 	          merging for Rev6 changes for Checksum support
 * 	          [1998/01/28  23:11:43  skapur]
 * 
 * 	          To merge with John's code
 * 	          [1998/01/20  19:23:20  skapur]
 * 
 * Revision 1.1.2.3  1998/01/28  21:27:55  hayes
 * 	          add stack bounds checking and redzone support
 * 	          [1998/01/28  21:26:51  hayes]
 * 
 * Revision 1.1.2.2  1998/01/20  04:39:21  hayes
 * 	          add tigon Rev6 support
 * 	          [1998/01/20  04:37:55  hayes]
 * 
 * Revision 1.1.2.1  1998/01/14  23:25:49  ted
 * 	          rearrange directories
 * 	          [1998/01/14  23:17:44  ted]
 * 
 * Revision 1.1.2.5  1998/01/14  20:00:21  skapur
 * 	          Merged with changes from 1.1.2.4
 * 	          [1998/01/14  20:00:06  skapur]
 * 
 * 	          Checksum off-load support for Tigon2. This is an intermediate checkin; more work needs to be done
 * 	          [1998/01/14  19:26:34  skapur]
 * 
 * Revision 1.1.2.4  1998/01/05  01:31:47  ted
 * 	          Fix occasional corruption problem (PR 689)
 * 	          [1998/01/05  00:56:33  ted]
 * 
 * Revision 1.1.2.3  1997/12/29  19:02:07  hayes
 * 	          remove obsolete reference to pci.h, remove replicate stuff
 * 	          [1997/12/29  18:59:19  hayes]
 * 
 * Revision 1.1.2.2  1997/11/21  04:48:15  hayes
 * 	          release for NIC 3.1 pre-Beta
 * 	          [1997/11/21  04:45:24  hayes]
 * 
 * Revision 1.1.2.1  1997/11/19  19:23:09  ted
 * 	          Support Tigon and Tigon2 in the same driver
 * 	          [1997/11/19  19:19:15  ted]
 * 
 * Revision 1.22.4.32  1997/11/14  19:16:24  hayes
 * 	          first checkin after mongo merge...
 * 	          [1997/11/14  19:11:28  hayes]
 * 
 * 	          checkin after stats restructuring
 * 	          [1997/11/12  19:01:22  hayes]
 * 
 * 	          intermediate checkin
 * 	          [1997/11/12  01:30:37  hayes]
 * 
 * Revision 1.22.4.31  1997/11/11  19:08:53  hayes
 * 	          remove debugging ASSERT()
 * 	          [1997/11/11  19:08:43  hayes]
 * 
 * Revision 1.22.4.30  1997/11/11  19:05:56  hayes
 * 	          restructuring for scratchpad use
 * 	          [1997/11/11  19:04:18  hayes]
 * 
 * Revision 1.22.4.29  1997/11/03  18:42:24  hayes
 * 	          cleanup comments
 * 	          [1997/11/03  18:40:29  hayes]
 * 
 * Revision 1.22.4.28  1997/10/28  01:51:16  hayes
 * 	          now works for Tigon2-SX
 * 	          [1997/10/27  23:31:12  hayes]
 * 
 * Revision 1.22.4.27  1997/10/21  18:18:09  hayes
 * 	          fix for REV5
 * 	          [1997/10/21  18:17:47  hayes]
 * 
 * Revision 1.22.4.26  1997/10/21  01:35:32  skapur
 * 	          Merge of release 2.0 and ge_latest branch
 * 	          [1997/10/21  01:30:58  skapur]
 * 
 * Revision 1.22.4.25  1997/10/02  23:11:37  hayes
 * 	          Cleanup and remove dead code.
 * 	          Fix initialization of event register for REV5
 * 	          [1997/10/02  21:40:30  hayes]
 * 
 * Revision 1.22.4.24  1997/09/20  19:44:46  hayes
 * 	          fix up for TIGON_REV5
 * 	          [1997/09/20  19:44:25  hayes]
 * 
 * Revision 1.22.4.23  1997/09/16  21:46:26  ted
 * 	          Change from MIB-II V1 (RFC 1213) to MIB-II V2 (RFC 1573)
 * 	          [1997/09/16  21:41:57  ted]
 * 
 * Revision 1.22.4.22  1997/09/16  02:43:15  skapur
 * 	          Merged with changes from 1.22.4.21
 * 	          [1997/09/16  02:43:03  skapur]
 * 
 * 	          Now Zero out the entire NIC structure at initialization; took out unused variables; misc. code clean up
 * 	          [1997/09/16  02:41:13  skapur]
 * 
 * Revision 1.22.4.21  1997/09/11  18:05:42  ted
 * 	          Mixing vlans, jumbos, and redundancy works, plus small fixes from release 1.1 and better NIC tracing
 * 	          [1997/09/11  17:59:37  ted]
 * 
 * Revision 1.22.4.20  1997/08/29  22:12:10  hayes
 * 	          completed link code restructuring
 * 
 * Revision 1.22.25.2  1997/08/29  22:09:17  hayes
 * 	          completed link code restructuring
 * 
 * Revision 1.22.4.19  1997/08/23  17:40:05  skapur
 * 	          Solaris jumbogram support
 * 	          [1997/08/23  17:37:53  skapur]
 * 
 * Revision 1.22.4.18  1997/07/09  19:32:36  skapur
 * 	          Adding multicast filtering support
 * 	          [1997/07/09  19:32:05  skapur]
 * 
 * Revision 1.22.4.17  1997/05/21  03:35:40  ted
 * 	          Initialize data structures for fix for bug 189.
 * 	          Change unsigned to signed to make comparisons easier.
 * 	          [1997/05/21  03:12:43  ted]
 * 
 * Revision 1.22.4.16  1997/05/08  22:57:57  skapur
 * 	          Changed the ifType from 1(other) to 6 (ethernet)
 * 	          [1997/05/08  22:57:23  skapur]
 * 
 * Revision 1.22.4.15  1997/05/07  23:49:54  skapur
 * 	          Reset the HW macstats at the initialization of the driver. HW does not do it automatically, so the FW has to do it
 * 	          [1997/05/07  23:49:39  skapur]
 * 
 * Revision 1.22.4.14  1997/05/03  04:58:42  skapur
 * 	          Cleaned up stats code
 * 	          [1997/05/03  04:58:02  skapur]
 * 
 * Revision 1.22.4.13  1997/04/25  18:48:45  ted
 * 	          New DMA state initialization logic.
 * 	          New intermediate variable used for keeping track of receive bds.
 * 	          [1997/04/25  18:33:35  ted]
 * 
 * Revision 1.22.4.12  1997/04/11  23:36:16  ted
 * 	          Cleanup + SBUS related code
 * 	          [1997/04/11  23:16:31  ted]
 * 
 * Revision 1.22.4.11  1997/03/28  02:47:00  skapur
 * 	          Fixed Replication code; moved some of the bits which were or'ed in the macros into the appropriate initialization variables in fwmain
 * 	          [1997/03/28  02:23:23  skapur]
 * 
 * Revision 1.22.4.10  1997/03/24  08:34:21  ted
 * 	          New event ring handling, producer lives in host memory, misc. other more modest changes
 * 	          [1997/03/24  08:27:51  ted]
 * 
 * Revision 1.22.4.9  1997/03/21  04:15:35  skapur
 * 	          Performance enhancement for Recv and Send; lots changes; almost complete overhaul of the entire firmware
 * 	          [1997/03/21  04:08:13  skapur]
 * 
 * Revision 1.22.4.8  1997/03/17  08:47:42  skapur
 * 	          integration changes...
 * 	          [1997/03/16  03:01:44  skapur]
 * 
 * Revision 1.22.4.7  1997/02/28  20:02:22  ted
 * 	          Remove DMA corruption workaround, fix 3 subtle bugs
 * 	          [1997/02/28  20:01:38  ted]
 * 
 * Revision 1.22.4.6  1997/02/23  00:23:32  ted
 * 	          Added FW14 event; removed TX_COMP SW hack ifZero'ed
 * 	          [1997/02/23  00:08:52  ted]
 * 
 * Revision 1.22.4.5  1997/02/17  05:05:50  ted
 * 	          Add back in TG_TX_COMP hardware buf workaround code
 * 	          [1997/02/17  05:05:16  ted]
 * 
 * Revision 1.22.4.4  1997/02/17  00:44:18  skapur
 * 	          If def'ed out the code for hack for SW TX_COMP event
 * 	          [1997/02/17  00:43:48  skapur]
 * 
 * Revision 1.22.4.3  1997/02/17  00:07:56  ted
 * 	          Working with Rev 1, Chip Express boards (funky internal loopback sync algorith)
 * 	          [1997/02/16  23:44:12  ted]
 * 
 * Revision 1.22.4.2  1997/02/11  04:08:05  ted
 * 	          Turn on SW0 event and enable in event mask to work around
 * 	          mac_rx_comp bug in Rev A tigon.
 * 	          [1997/02/11  02:42:34  ted]
 * 
 * Revision 1.22.4.1  1997/02/07  23:20:05  skapur
 * 	          replacing TGP/NIC/TDP  with reg variables
 * 	          [1997/02/07  23:19:27  skapur]
 * 
 * Revision 1.22.2.6  1996/12/05  20:07:03  hayes
 * 	          Fix SIMULATOR problem.
 * 	          [1996/12/05  20:03:17  hayes]
 * 
 * Revision 1.22.2.5  1996/11/29  17:43:25  hayes
 * 	          Change some #if (SIMULATOR) to #if (NO_HOST) for better testing
 * 	          and simulation.
 * 	          [1996/11/29  17:37:43  hayes]
 * 
 * Revision 1.22.2.4  1996/11/20  20:04:19  ted
 * 	          Redo include file structuring.
 * 	          [1996/11/20  19:51:12  ted]
 * 
 * Revision 1.22.2.3  1996/11/15  04:24:06  hayes
 * 	          convert to new header file structure.
 * 	          [1996/11/15  04:12:00  hayes]
 * 
 * Revision 1.22.2.2  1996/11/05  21:54:19  hayes
 * 	          Final pre-ODE merge
 * 	          [1996/11/05  21:51:36  hayes]
 * 
 * Revision 1.22.2.1  1996/11/01  18:24:54  ted
 * 	          change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	          [1996/10/29  22:06:03  ted]
 * 
 * $EndLog$
 */
/*
 * FILE fwmain.c
 *
 * COPYRIGHT (c) Essential Communication Corp. 1995
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/fwmain.c,v $
 * $Revision: 1.1.2.45 $ $Locker:  $
 * $Date: 1999/01/24 00:10:55 $ $Author: shuang $ $State: Exp $
 */

#ifndef __lint
static const char rcsid[] = "$Header: /projects/rcs/sw/ge/./nic/fw2/common/fwmain.c,v 1.1.2.45 1999/01/24 00:10:55 shuang Exp $";
#endif /* __lint */

#include "alt_def.h"
#include "nic_conf.h"
#include "nic_api.h"
#include "tg.h"

#include "nic.h"
#include "trace.h"
#include "proto.h"
#include "timer.h"
#include "ring.h"
#include "link_op.h"
#include "dma.h"
#include "assert.h"
#include "profile.h"

/* build definitions */
#define USE_SW_DMA_ASSIST       1 /* use sw dma assist */
#define	USE_FAST_STACK_CPU_B	1 /* use scratch pad as stack for cpu b */
#define INIT_SCRATCH_PAD_STACK	0 /* init scratch pad stack to known pattern */
#define	USE_RX_CKSUM		0 /* use cksum mode on rx (for debug only) */

/* reserve how much stack on scratch pad */
#define SCRATCH_PAD_STACK_SIZE  (512)

/* relocate to scratchpad */
#define	TO_SCRATCHPAD(src, len, dstp, sizep) \
	copy2scratchpad("+"#src, (U32)src, (U32)len, (U32*)dstp, (U32*)sizep)

/* globals */
/* for debugging */
U32 gglobal_flags;
tg_shmem_t *gtsp;
tg_regs_t *gtrp;
tg_data_t *gtdp;
nic_t *gnicp;
nic_fast_t *gnicfp;

#if defined(SCRATCH_PAD)
/* how we want to utilize the scratch pad */
U32 scratch_pad_enable_stack = 1;
U32 scratch_pad_enable_vars  = 1;
U32 scratch_pad_enable_code  = 1;
#else /* defined(SCRATCH_PAD) */
/* force to disable all */
U32 scratch_pad_enable_code  = 0;
U32 scratch_pad_enable_stack = 0;
U32 scratch_pad_enable_vars  = 0;
#endif /* defined(SCRATCH_PAD) */

U32 adj_stack_start   = TG_BEG_SCRATCH + SCRATCH_PAD_STACK_SIZE - sizeof(U32);
U32 scratch_pad_sz    = TG_END_SCRATCH - TG_BEG_SCRATCH - sizeof(U32);
U32 scratch_pad_mem   = TG_BEG_SCRATCH;


#if (DYNAMIC_NIC_MEM)
/* help both cpu's setup */
U32 mem_size, tx_buf_start, tx_buf_size, rx_buf_start, rx_buf_size;
U32 rx_buf_attn_hi, rx_buf_attn_lo, rx_buf_xon, rx_buf_xoff;
#endif /* (DYNAMIC_NIC_MEM) */

U32 *ev_handler_stub;
U32 null_handler_stub;
void (*sw_dma_assist_stub)(void);

/* externals */
extern void _disp_loop(void);
extern const U32 int_fw_version;
extern U32 _end; /*implicit; needed for stack bounds checking... */

/* forward declarations */
static void fw_init(void);
static void fw_init_b(void);
static U32 copy2scratchpad(char *, U32, U32, U32*, U32*);
void main_b(void);


#define set_event_disp(event_num, func) \
	set_event_handler(ev_handlerp, event_num, (U32)func)
	
void main_without__main(void);

/*
 * initialization and main loop for firmware
 */
void
main_without__main(void)
{ 
    /* init trp */
    trp = (tg_regs_t *)(TG_BEG_REGS);

    /* cpu stuffs */
    trp->cpu_control.cpu_state |= TG_CPU_PAGE_0_HALT_ENA;
    trp->local_mem_conf.cpu_priority = 3;

    /* init trace buffer */
    init_trace();

    /* write 0 to last word in scratchpad to fix static IDD problem */
    *(U32 *)((U32)TG_END_SCRATCH - sizeof(U32)) = 0;

#if (DYNAMIC_NIC_MEM)
    {
	U32 buf_size;

	/* init run time memory dependent stuffs */
	mem_size = probe_mem_size();
	switch (mem_size) {
#if 0 /* 256k memory NIC is NOT supported */
	case NIC_MEM_256K:
	    rx_buf_attn_hi = RX_BUF_HI_256K;
	    rx_buf_attn_lo = RX_BUF_LO_256K;
	    rx_buf_xon     = RX_BUF_XON_256K;
	    rx_buf_xoff    = RX_BUF_XOFF_256K;
	    break;
#endif
	case NIC_MEM_512K:
	    #if 0 /* dynamic assigned later */
	    rx_buf_attn_hi = RX_BUF_HI_512K;
	    #endif
	    rx_buf_attn_lo = RX_BUF_LO_512K;
	    rx_buf_xon     = RX_BUF_XON_512K;
	    rx_buf_xoff    = RX_BUF_XOFF_512K;
	    break;

	default:
	    NIC_UTRACE("badMemSz", 0x00100,
		mem_size,
		0,
		0,
		0);
	    ASSERT(0);
	    mem_size = NIC_MEM_2M;
	    /* FALL THRU */

	case NIC_MEM_2M:
	case NIC_MEM_1M:
	    #if 0 /* dynamic assigned later */
	    rx_buf_attn_hi = RX_BUF_HI_1M;
	    #endif
	    rx_buf_attn_lo = RX_BUF_LO_1M;
	    rx_buf_xon     = RX_BUF_XON_1M;
	    rx_buf_xoff    = RX_BUF_XOFF_1M;
	    break;
	}

	/* allocate tx/rx buffers */
	buf_size = 
	    mem_size - (U32)&_end - sizeof(tg_data_t) - sizeof(nic_t);
	if (!scratch_pad_enable_vars)
	    buf_size -= sizeof(nic_fast_t);
	buf_size &= ~0xfff; /* must be multiple of 4KB */
	tx_buf_size = get_tx_buf_size(buf_size);
	rx_buf_size = buf_size - tx_buf_size;

	rx_buf_start = TG_DATA_END - rx_buf_size;
	tx_buf_start = rx_buf_start - tx_buf_size;

	/* assign rx buf high water mark to be half of rx buf */
	rx_buf_attn_hi = rx_buf_size >> 1;
    }
#endif /* (DYNAMIC_NIC_MEM) */

    /* init tdp, nicp */
#if (DYNAMIC_NIC_MEM)
    tdp = (tg_data_t *)
	  (TG_DATA_END - rx_buf_size - tx_buf_size - sizeof(tg_data_t));
#else /* (DYNAMIC_NIC_MEM) */
    tdp = (tg_data_t *)(TG_DATA_END - sizeof(tg_data_t));
#endif /* (DYNAMIC_NIC_MEM) */
    nicp = (struct nic *)((U32)tdp - sizeof(nic_t));

    /* setup real stack values */
    stack_start   = (U32)&tdp->stack  [TG_STACK_WORDS - 1];
    stack_start_b = (U32)&tdp->stack_b[TG_STACK_WORDS - 1];

    /* in case the stack ever moves to scratch */
    trp->cpu_control.adjust_stack = TG_BEG_SCRATCH - (U32)tdp->stack_b;
    trp->cpu_control_b.adjust_stack = 
        TG_BEG_SCRATCH - (U32)tdp->stack_page_align_padding;

    if (scratch_pad_enable_stack) {

        /* make sure our external stack is on page boundary */
	ASSERT(!((U32)tdp->stack & 0xfff));

        /* relocate stack to scratch pad */
        __asm__ volatile
        ("
            .set noreorder
            lw   $sp, adj_stack_start
            move $fp, $sp
            .set reorder
        ");

#if (INIT_SCRATCH_PAD_STACK)
	/* initialize scratch pad stack zone */
        memset_w((U32 *)TG_BEG_SCRATCH, 0xaa55aa55, SCRATCH_PAD_STACK_SIZE>>2);
#endif /* (CK_SCRATCH_PAD_STACK) */

        probe_adj_stack(); /* quick test our stack adjust feature */
        scratch_pad_sz  -= SCRATCH_PAD_STACK_SIZE;
        scratch_pad_mem += SCRATCH_PAD_STACK_SIZE;

    } else {
        /* relocate to real stack */
        __asm__ volatile
        ("
            .set noreorder
            lw   $sp, stack_start
            move $fp, $sp
            .set reorder
        ");
    }

    /* setup nicfp */
    if (scratch_pad_enable_vars) {
        nicfp = (nic_fast_t *)scratch_pad_mem;
        scratch_pad_sz  -= sizeof(nic_fast_t);
        scratch_pad_mem += sizeof(nic_fast_t);
    } else {
        nicfp = (nic_fast_t *)((U32)nicp - sizeof(nic_fast_t));
    }

    if (!scratch_pad_enable_code) {
        scratch_pad_sz = 0; /* disable later code relocation */
    }

    /* firmware initialization */
    fw_init();
}

static void
fw_init(void)
{ 
    register U32 dispp __asm__ ("$16");
    register U32 *ev_handlerp __asm__("$18");

    struct tg_event event, error_event;
    static U8 alteonStr[] = "Alteon AceNIC V";
    U32 scratch_pad_size = scratch_pad_sz;
    U32 scratch_pad_loc = scratch_pad_mem;
    U32 events, high_event;
    U32 rcb_flags, jumbo_rcb_flags, mini_rcb_flags;
    U32 align;
    U32 i;
    bool send_error_event;

    send_error_event = FALSE;
    ev_handler_stub = ev_handlerp = ev_handler;

    NIC_UTRACE("hwVer", 0x00200,
	trp->gen_control.misc_host_control,
	int_fw_version,
	0,
	0);

    /* check hw revision */
    switch (trp->gen_control.misc_host_control & TG_MHC_TIGON_VERSION_MASK) {
    case TG_MHC_LSI_STANDARD_CELL_VERS_B:
	break;

    default:
	send_error_event = TRUE;

	NIC_UTRACE("badHwVer", 0x00300,
	    trp->gen_control.misc_host_control,
	    int_fw_version,
	    0,
	    0);
	ASSERT(0);
    }

    /* clear semaphore */
    trp->cpu_control_b.cpu_state |= TG_CPU_HALT;
    trp->gen_control.semaphore_a = 0;
    if (trp->gen_control.semaphore_a) {
        trp->gen_control.semaphore_a = 0;
    } else {
        trp->gen_control.semaphore_a = 0;
        if (trp->gen_control.semaphore_a) {
            trp->gen_control.semaphore_a = 0;
        } else {
            trp->gen_control.semaphore_b = 0;
        }
    }

    /* zero out nic and nic fast structure */
    bzero((U32 *)nicp, sizeof(struct nic));
    bzero((U32 *)nicfp, sizeof(struct nic_fast));

    /* register myself */
    nicfp->cpu_id = TG_CPU_A;

#if (DYNAMIC_NIC_MEM)
    /* save memory dependent info */
    nicfp->tx_buf         = tx_buf_start;
    nicfp->rx_buf         = rx_buf_start;
    nicfp->tx_buf_red	  = nicfp->rx_buf - TX_BUF_RED_ZONE_SIZE;
    nicfp->tx_buf_size    = tx_buf_size;
    nicfp->rx_buf_size    = rx_buf_size;
    nicfp->rx_buf_attn_hi = rx_buf_attn_hi;
    nicfp->rx_buf_attn_lo = rx_buf_attn_lo;
    nicfp->rx_buf_xon     = rx_buf_xon;
    nicfp->rx_buf_xoff    = rx_buf_xoff;

    NIC_UTRACE("**DAWN_A", 0x00400,
	DYNAMIC_NIC_MEM,
	USE_SW_DMA_ASSIST,
	USE_FAST_STACK_CPU_B,
	INIT_SCRATCH_PAD_STACK);

    NIC_UTRACE("txRxBfSz", 0x00500,
	nicfp->tx_buf_size,
	nicfp->rx_buf_size,
	nicfp->tx_buf,
	nicfp->rx_buf);

    NIC_UTRACE("bfAtnMrk", 0x00600,
	nicfp->rx_buf_attn_hi,
	nicfp->rx_buf_attn_lo,
	nicp,
	nicfp);

#else /* (DYNAMIC_NIC_MEM) */

    NIC_UTRACE("txRxBfSz", 0x00700,
	TX_BUF_SIZE_1M,
	RX_BUF_SIZE_1M,
	(U32)tdp->tx_buf,
	(U32)tdp->rx_buf);

    NIC_UTRACE("bufAtnWM", 0x00800,
	RX_BUF_HI_1M,
	RX_BUF_LO_1M,
	nicp,
	nicfp);

#endif /* (DYNAMIC_NIC_MEM) */

    /* make sure external stack do not overlap bss region */
    {
#if (DYNAMIC_NIC_MEM)
	U32 mem_end = mem_size - 1;
#else /* (DYNAMIC_NIC_MEM) */
	U32 mem_end = NIC_MEM_1M - 1;
#endif /* (DYNAMIC_NIC_MEM) */
#if defined(STACK_REDZONE)
	U32 stk_end = (U32)tdp->redzone;
#else /* defined(STACK_REDZONE) */
	U32 stk_end = (U32)tdp->stack;
#endif /* defined(STACK_REDZONE) */

	if (((U32)&_end & mem_end) > (stk_end & mem_end)) {
	    NIC_UTRACE("redZone1", 0x00900,
		&_end,
		stk_end,
		mem_end,
		0);
	    ASSERT(0);
	}
    }

#if defined(STACK_REDZONE)
    /* init stack redzone with known pattern */
    memset_w(&tdp->redzone[0], 0xdeadbabe, STACK_REDZONE_SIZE);
#endif /* defined(STACK_REDZONE) */

    /* pci configuration */
    NIC_UTRACE("pciConf", 0x01000,
	*(U32 *)trp, 		/* deviceId & vendorId */
	*((U32 *)trp + 1),	/* status & command */
	*((U32 *)trp + 3),	/* bist & hdrType & latencyTimer & cacheLnSz */
	*((U32 *)trp + 15));	/* maxLat & MinGrant & IntrPin & IntrLine */

    /* general configuration */
    NIC_UTRACE("genConf", 0x01100,
	tsp->gen_com.mode_status,
	tsp->gen_com.dma_read_config,
	tsp->gen_com.dma_write_config,
	tsp->gen_com.tx_buf_ratio);

    /*
     * Initialization works like this-
     * We are completely controlled by the host.
     * The host downloads the NIC code (this code!) into the nic
     * and starts it running.
     * It is assumed that the host has initialized
     * the shared memory region and mapped all of the structures
     * that the NIC DMA's to and from the host.
     * Once we complete initialization,
     * we give the host a FIRMWARE_OPERATIONAL event.
     */
 
    /* initialize timer for simulator */
    trp->gen_control.timer = 0;

    /* disable DMA assist and reset to known state */
    trp->host_dma_assist.assist_state = 0;

    /* set up dma state variables */
    if (tsp->gen_com.mode_status & TG_CFG_MODE_BYTE_SWAP_BD) {
        nicfp->dma_to_host_bd_state = 
            tsp->gen_com.dma_write_config |
            TG_DMA_STATE_ACTIVE |
            TG_DMA_STATE_DISABLE_PROD_COMP |
	    0;

        nicfp->dma_to_nic_bd_state = 
            tsp->gen_com.dma_read_config |
            TG_DMA_STATE_ACTIVE;

    } else {
        nicfp->dma_to_host_bd_state = 
            tsp->gen_com.dma_write_config |
            TG_DMA_STATE_ACTIVE |
            TG_DMA_STATE_NO_SWAP | 
            TG_DMA_STATE_DISABLE_PROD_COMP |
	    0;

        nicfp->dma_to_nic_bd_state = 
            tsp->gen_com.dma_read_config |
            TG_DMA_STATE_NO_SWAP |
            TG_DMA_STATE_ACTIVE |
	    0;
    }

    if (tsp->gen_com.mode_status & TG_CFG_MODE_BYTE_SWAP_DATA) {
        nicfp->dma_to_nic_data_state = 
            tsp->gen_com.dma_read_config |
            TG_DMA_STATE_ACTIVE |
	    0;

        nicfp->dma_to_host_data_state = 
            tsp->gen_com.dma_write_config |
            TG_DMA_STATE_ACTIVE |
            TG_DMA_STATE_UPDATE_RX_BUF_CONS |
            TG_DMA_STATE_DISABLE_PROD_COMP |
	    0;

    } else {
        nicfp->dma_to_host_data_state = 
            tsp->gen_com.dma_write_config |
            TG_DMA_STATE_ACTIVE |
            TG_DMA_STATE_NO_SWAP | 
            TG_DMA_STATE_UPDATE_RX_BUF_CONS |
            TG_DMA_STATE_DISABLE_PROD_COMP |
	    0;

        nicfp->dma_to_nic_data_state = 
            tsp->gen_com.dma_read_config |
            TG_DMA_STATE_NO_SWAP | 
            TG_DMA_STATE_ACTIVE |
	    0;
    }

    /* init to known value to work around Tigon2 RDMA and MAC problems
     * after power cycled:
     * 1) the tx buf base and rx buf base registers are un-resettable by
     *    a reset.
     * 2) during a dma read, if the NIC location reaches rx buf base,
     *    it's wrapped to the tx buf base.
     */
    trp->local_mem_conf.tx_buf_base = 0;
    trp->local_mem_conf.rx_buf_base = 0;

    /* reset dma */
    trp->host_dma.dma_wr_state = TG_DMA_STATE_RESET;
    trp->host_dma.dma_rd_state = TG_DMA_STATE_RESET;
    WAIT_US(100);

    /* setup gen_info buffer descr */
    nicp->gen_info_bd.BD_host_addr = tsp->gen_com.gen_info_ptr;
    nicp->gen_info_bd.BD_nic_addr  = (U32)&nicp->gen_info;
    nicp->gen_info_bd.BD_w2        = sizeof(struct tg_gen_info); /* len */

    /* dma gen_info area into NIC */
    dma_to_nic(&nicp->gen_info_bd);
    while (trp->host_dma.dma_rd_state & TG_DMA_STATE_ACTIVE)
        ; /* wait for DMA to complete */

    /* check for error... */
    if (trp->host_dma.dma_rd_state &
        (TG_DMA_STATE_FIFO_LOCAL_ERROR |
        TG_DMA_STATE_FIFO_UNDERRUN |
        TG_DMA_STATE_FIFO_OVERRUN |
        TG_DMA_STATE_HOST_ADDR_OVERFLOW |
        TG_DMA_STATE_PCI_PARITY_ERR |
        TG_DMA_STATE_PCI_MASTER_ABORT |
        TG_DMA_STATE_PCI_TARGET_ABORT)) {
 
        NIC_UTRACE("*dmaRDfl", 0x01200d,
            trp->host_dma.dma_rd_state,
            trp->host_dma.dma_rd_len,
            trp->host_dma.dma_rd_loc_addr,
            (U32)HOSTADDR(trp->host_dma.dma_rd_host_addr));
 
        PANIC();
    }

    NIC_UTRACE("rcbFlags", 0x01200,
	nicp->gen_info.send_rcb.RCB_flags,
	nicp->gen_info.recv_standard_rcb.RCB_flags,
	nicp->gen_info.recv_jumbo_rcb.RCB_flags,
	nicp->gen_info.recv_mini_rcb.RCB_flags);

    /* setup global flags */
    global_flags = 0;

    rcb_flags = nicp->gen_info.send_rcb.RCB_flags;
    if (rcb_flags & RCB_FLAG_TCP_UDP_CKSUM)
	set_global_flags(GF_RCB_TX_TCP_UDP_CKSUM);
    if (rcb_flags & RCB_FLAG_IP_CKSUM)
	set_global_flags(GF_RCB_TX_IP_CKSUM);
    if (rcb_flags & RCB_FLAG_NO_PSEUDO_HDR_CKSUM)
	set_global_flags(GF_RCB_TX_NO_PSEUDO_HDR_CKSUM);
    if (rcb_flags & RCB_FLAG_VLAN_ASSIST)
	set_global_flags(GF_RCB_TX_VLAN_ASSIST);
    if (rcb_flags & RCB_FLAG_COAL_UPDATE_ONLY)
	set_global_flags(GF_RCB_TX_COAL_UPDATE_ONLY);
    if (rcb_flags & RCB_FLAG_HOST_RING)
	set_global_flags(GF_RCB_TX_HOST_RING);
    if (rcb_flags & RCB_FLAG_IEEE_SNAP_CKSUM)
	set_global_flags(GF_RCB_TX_IEEE_SNAP_CKSUM);

#define RCB_FLAG_COMMON		(RCB_FLAG_TCP_UDP_CKSUM | \
				 RCB_FLAG_IP_CKSUM | \
				 RCB_FLAG_NO_PSEUDO_HDR_CKSUM | \
				 RCB_FLAG_VLAN_ASSIST | \
				 RCB_FLAG_IEEE_SNAP_CKSUM)

    mini_rcb_flags = nicp->gen_info.recv_mini_rcb.RCB_flags;
    if (mini_rcb_flags & RCB_FLAG_RING_DISABLED)
	set_global_flags(GF_RCB_MINI_RING_DISABLED);

    jumbo_rcb_flags = nicp->gen_info.recv_jumbo_rcb.RCB_flags;
    if (jumbo_rcb_flags & RCB_FLAG_RING_DISABLED)
	set_global_flags(GF_RCB_JUMBO_RING_DISABLED);
    else if (jumbo_rcb_flags & RCB_FLAG_USE_EXT_RECV_BD)
	set_global_flags(GF_RCB_USE_EXT_RECV_BD);

    rcb_flags = nicp->gen_info.recv_standard_rcb.RCB_flags;
    i = rcb_flags & RCB_FLAG_COMMON;

    /* common RCB flags should be same for all receive RCBs */
    if ((!is_global_flags(GF_RCB_MINI_RING_DISABLED) &&
	    (i != (mini_rcb_flags & RCB_FLAG_COMMON))) ||
	(!is_global_flags(GF_RCB_JUMBO_RING_DISABLED) &&
	    (i != (jumbo_rcb_flags & RCB_FLAG_COMMON)))) {

	send_error_event = TRUE;

	NIC_UTRACE("badRxRcb", 0x01300,
	    mini_rcb_flags,
	    rcb_flags,
	    jumbo_rcb_flags,
	    0);
	ASSERT(0);
    }

    if (rcb_flags & RCB_FLAG_TCP_UDP_CKSUM)
	set_global_flags(GF_RCB_RX_TCP_UDP_CKSUM);
    if (rcb_flags & RCB_FLAG_IP_CKSUM)
	set_global_flags(GF_RCB_RX_IP_CKSUM);
    if (rcb_flags & RCB_FLAG_NO_PSEUDO_HDR_CKSUM)
	set_global_flags(GF_RCB_RX_NO_PSEUDO_HDR_CKSUM);
    if (rcb_flags & RCB_FLAG_VLAN_ASSIST)
	set_global_flags(GF_RCB_RX_VLAN_ASSIST);
    if (rcb_flags & RCB_FLAG_IEEE_SNAP_CKSUM)
	set_global_flags(GF_RCB_RX_IEEE_SNAP_CKSUM);

    if (tsp->gen_com.mode_status & TG_CFG_MODE_DONT_FRAG_JUMBOS)
        set_global_flags(GF_MOD_DONT_FRAG_JUMBOS);
    if (tsp->gen_com.mode_status & TG_CFG_MODE_ALLOW_BAD_FRAMES)
        set_global_flags(GF_MOD_ALLOW_BAD_FRAMES);
    if (tsp->gen_com.mode_status & TG_CFG_MODE_INCLUDE_CRC)
        set_global_flags(GF_MOD_INCLUDE_CRC);
    if (tsp->gen_com.mode_status & TG_CFG_MODE_SBUS)
        set_global_flags(GF_MOD_SBUS);
    if (tsp->gen_com.mode_status & TG_CFG_MODE_USE_OP1_LEDS)
        set_global_flags(GF_MOD_USE_OP1_LEDS);
    if (tsp->gen_com.mode_status & TG_CFG_MODE_DONT_RUPT_SENDS)
        set_global_flags(GF_MOD_DONT_RUPT_SENDS);
    if (tsp->gen_com.mode_status & TG_CFG_MODE_DONT_RUPT_RECVS)
        set_global_flags(GF_MOD_DONT_RUPT_RECVS);
    if (tsp->gen_com.mode_status & TG_CFG_MODE_DONT_RUPT_EVENTS)
        set_global_flags(GF_MOD_DONT_RUPT_EVENTS);

    /* setup nicfp host pointers */
    nicfp->event_producer_ptr	   = nicp->gen_info.event_producer_ptr;
    nicfp->recv_return_producer_ptr= nicp->gen_info.recv_return_producer_ptr;
    nicfp->send_consumer_ptr	   = nicp->gen_info.send_consumer_ptr;
    nicfp->event_ring_ptr	   = nicp->gen_info.event_rcb.ring_addr;
    nicfp->send_ring_ptr	   = nicp->gen_info.send_rcb.ring_addr;
    nicfp->recv_ring_ptr	   = nicp->gen_info.recv_standard_rcb.ring_addr;
    nicfp->recv_jumbo_ring_ptr	   = nicp->gen_info.recv_jumbo_rcb.ring_addr;
    nicfp->recv_mini_ring_ptr	   = nicp->gen_info.recv_mini_rcb.ring_addr;
    nicfp->recv_ret_ring_ptr       = nicp->gen_info.recv_return_rcb.ring_addr;
    nicfp->stats2_ptr              = nicp->gen_info.stats2_ptr;

#if 0
    /* zero stats */
    bzero((U32 *)&nicfp->stats, sizeof(struct tg_stats));
#endif /*0*/

    /* zero HW macstats area in shmem */
    bzero((U32 *)&tsp->macstats, sizeof(struct mac_stats));

    /* Set up the basic default values for Stats
     * Following exists in command.c as well, so if it changes here
     * make sure to chage it there as well 
     */ 
    nicfp->stats.ifIndex	= tsp->gen_com.ifIndex; 
    nicfp->stats.ifMtu		= tsp->gen_com.ifMTU; 
    nicfp->stats.ifPhysAddress	= tsp->gen_com.mac_addr; 
    nicfp->stats.ifType		= 6;  /* Type = Ethernet(6) for now */
    nicfp->stats.ifOperStatus	= IF_OPER_STATUS_DOWN; 
    nicfp->stats.ifAdminStatus	= IF_ADMIN_STATUS_DOWN; 
    nicfp->stats.ifSpeed	= 1000000000;  /* 1 Gig bits/sec */
    nicfp->stats.ifHighSpeed	= 1000;  /* 1000 Mbits/sec */
    nicfp->stats.ifLinkUpDownTrapEnable = 2; /* Always Disabled(2) */
    nicfp->stats.ifConnectorPresent = 1; /* Always True (1) */

    for (i = 0; i < 15; i++)
         nicfp->stats.ifDescr[i] = alteonStr[i];

    nicfp->stats.ifDescr[i++] = (U8) 0x30 + 
       ((trp->gen_control.misc_host_control & TG_MHC_TIGON_VERSION_MASK) >> 28);
    nicfp->stats.ifDescr[i++] = '\0'; /* Null terminate the string */

    NIC_UTRACE("globFlgs", 0x01400,
	global_flags,
	tsp->gen_com.mode_status,
	sizeof(nic_fast_t),
	sizeof(nic_t));
 
    /* Move most common routines into scratchpad. */
    dispp =
	TO_SCRATCHPAD
	(_disp_loop,
	16,
	&scratch_pad_loc,
	&scratch_pad_size);

    if (scratch_pad_enable_code) {
        /* jump table must appear on 256B boundary */
        align = ROUNDUP(scratch_pad_loc, 256);

        scratch_pad_size -= (align - scratch_pad_loc);
        scratch_pad_loc = align;
    }

    ev_handler_stub = ev_handlerp = (U32 *)
	TO_SCRATCHPAD
	(ev_handler,
	sizeof(ev_handler),
	&scratch_pad_loc,
	&scratch_pad_size);

    if (is_global_flags(GF_RCB_TX_TCP_UDP_CKSUM | GF_RCB_TX_IP_CKSUM)) {
        if (trp->gen_control.misc_config & TG_MSC_ENA_SYNC_SRAM_TIMING) {
	    /* use sw DMA assist to bypass cksum bug on synchronous SRAM */
            if (tsp->gen_com.mode_status & TG_CFG_MODE_1_DMA_ACTIVE) {
		/* sw dma assist cannot support 1 dma at a time */
		send_error_event = TRUE;

		NIC_UTRACE("cant1Dma", 0x01500,
		    global_flags,
		    trp->gen_control.misc_host_control,
		    trp->gen_control.misc_config,
		    tsp->gen_com.mode_status);
		ASSERT(0);
            } else {
                nicp->sw_dma_assist = TRUE;
	    }
        }

        nicfp->dma_to_nic_data_state |=
	    TG_DMA_STATE_SAVE_CKSUM |
	    TG_DMA_STATE_ENABLE_CKSUM |
	    0;

        nicfp->q_dma_to_nic_stub =
            (U32 (*)(tg_hostaddr_t, U32, U32, U32, U32, U32))
            TO_SCRATCHPAD
	    (q_dma_to_nic_cksum,
	    (U32)q_dma_to_nic_cksum_end - (U32)q_dma_to_nic_cksum,
	    &scratch_pad_loc,
	    &scratch_pad_size);

        h_send_data_ready_stub = 
            TO_SCRATCHPAD
	    (h_send_data_ready_cksum,
	    (U32)h_send_data_ready_cksum_end - (U32)h_send_data_ready_cksum,
	    &scratch_pad_loc,
	    &scratch_pad_size);
        
        h_dma_rd_assist_stub = 
            TO_SCRATCHPAD
	    (h_dma_rd_assist_cksum,
	    (U32)h_dma_rd_assist_cksum_end - (U32)h_dma_rd_assist_cksum,
	    &scratch_pad_loc,
	    &scratch_pad_size);

        NIC_UTRACE("tCksmOn", 0x01600,
	    global_flags,
	    0,
	    0,
	    0);

    } else {
        nicfp->q_dma_to_nic_stub = 
            (U32 (*)(tg_hostaddr_t, U32, U32, U32, U32, U32))
            TO_SCRATCHPAD
	    (q_dma_to_nic,
	    (U32)q_dma_to_nic_end - (U32)q_dma_to_nic,
	    &scratch_pad_loc,
	    &scratch_pad_size);

        h_send_data_ready_stub = 
            TO_SCRATCHPAD
	    (h_send_data_ready,
	    (U32)h_send_data_ready_end - (U32)h_send_data_ready,
	    &scratch_pad_loc,
	    &scratch_pad_size);

        h_dma_rd_assist_stub = 
            TO_SCRATCHPAD
	    (h_dma_rd_assist,
	    (U32)h_dma_rd_assist_end - (U32)h_dma_rd_assist,
	    &scratch_pad_loc,
	    &scratch_pad_size);

        NIC_UTRACE("tCksmOff", 0x01650,
	    global_flags,
	    0,
	    0,
	    0);
    }

    if (is_global_flags(GF_RCB_TX_HOST_RING)) {
	/* 
	 * we always use 0x2000 as our start address.
	 * this insures we have space for 512 BDs.
	 */
	nicfp->send_ring = (struct send_bd *)0x2000;

	h_send_bd_ready_stub = 
	    TO_SCRATCHPAD
	    (h_send_bd_ready,
	    (U32)h_send_bd_ready_end - (U32)h_send_bd_ready,
	    &scratch_pad_loc,
	    &scratch_pad_size);

	set_event_disp(TG_FW_EVENT_NUM_SEND_BD, h_send_bd_ready_stub);
	nicfp->send_cmd_event = TG_FW_EVENT_SEND_BD;
	nicfp->send_ring_size = nicp->gen_info.send_rcb.RCB_max_len;

	NIC_UTRACE("hstSRing", 0x01660,
	    is_global_flags(GF_RCB_TX_HOST_RING),
	    (U32)nicfp->send_ring,
	    nicfp->send_ring_size,
	    0);

    } else {
	nicfp->send_ring =
	    (struct send_bd *)(U32)HOSTADDR(nicp->gen_info.send_rcb.ring_addr);

	if ((U32)nicfp->send_ring < 0x2000 || (U32)nicfp->send_ring > 0x4000) {
	    send_error_event = TRUE;

	    NIC_UTRACE("badSRing", 0x01700,
		nicfp->send_ring,
		0,
		0,
		0);
	    ASSERT(0);
	}
	set_event_disp(TG_FW_EVENT_NUM_SEND_BD, scream_handler);
	nicfp->send_cmd_event = TG_FW_EVENT_SEND_DATA;
	nicfp->send_ring_size = nicp->gen_info.send_rcb.RCB_max_len;

	NIC_UTRACE("nicSRing", 0x01800,
	    is_global_flags(GF_RCB_TX_HOST_RING),
	    (U32)nicfp->send_ring,
	    nicfp->send_ring_size,
	    0);
    }

    set_event_disp(TG_FW_EVENT_NUM_SEND_DATA, h_send_data_ready_stub);
    set_event_disp(TG_FW_EVENT_NUM_DMA_RD_ASST_HI, h_dma_rd_assist_stub);

#if (USE_RX_CKSUM) /* debug only */
    set_global_flags(GF_RCB_RX_TCP_UDP_CKSUM | GF_RCB_RX_IP_CKSUM);
#endif /* (USE_RX_CKSUM) */

    if (is_global_flags(GF_RCB_RX_TCP_UDP_CKSUM | GF_RCB_RX_IP_CKSUM)) {
	/*
	 * TIGON generates bogus cksum if the PCI is configured 
	 * to assert all PBE on dma write operations
	 */
	if (trp->gen_control.pci_state & TG_PCI_STATE_DMA_WR_ALL_ALIGN) {
	    send_error_event = TRUE;

	    NIC_UTRACE("wDmaAllA", 0x01900,
		global_flags,
		trp->gen_control.misc_host_control,
		trp->gen_control.misc_config,
		trp->gen_control.pci_state);
	    ASSERT(0);
	}

	if (trp->gen_control.misc_config & TG_MSC_ENA_SYNC_SRAM_TIMING) {
	    /* use sw DMA assist to bypass cksum bug on synchronous SRAM */
	    if (tsp->gen_com.mode_status & TG_CFG_MODE_1_DMA_ACTIVE) {
		/* sw dma assist cannot support 1 dma at a time */
		send_error_event = TRUE;

		NIC_UTRACE("cant1Dma", 0x02000,
		    global_flags,
		    trp->gen_control.misc_host_control,
		    trp->gen_control.misc_config,
		    tsp->gen_com.mode_status);
		ASSERT(0);
	    } else {
		nicp->sw_dma_assist = TRUE;
	    }
	}

	nicfp->dma_to_host_data_state |=
	    TG_DMA_STATE_SAVE_CKSUM |
	    TG_DMA_STATE_ENABLE_CKSUM |
	    TG_DMA_STATE_CLEAR_CKSUM |
	    0;

	nicfp->q_dma_to_host_stub = 
	    (U32 (*)(tg_hostaddr_t, U32, U32, U32, U32, U32))
	    TO_SCRATCHPAD
	    (q_dma_to_host_cksum,
	    (U32)q_dma_to_host_cksum_end - (U32)q_dma_to_host_cksum,
	    &scratch_pad_loc,
	    &scratch_pad_size);

	h_mac_rx_comp_stub = 
	    TO_SCRATCHPAD
	    (h_mac_rx_comp_cksum,
	    (U32)h_mac_rx_comp_cksum_end - (U32)h_mac_rx_comp_cksum,
	    &scratch_pad_loc,
	    &scratch_pad_size);

	h_dma_wr_assist_stub = (void (*)(void))
	    TO_SCRATCHPAD
	    (h_dma_wr_assist_cksum,
	    (U32)h_dma_wr_assist_cksum_end - (U32)h_dma_wr_assist_cksum,
	    &scratch_pad_loc,
	    &scratch_pad_size);

	NIC_UTRACE("rCksmOn", 0x02100,
	    global_flags,
	    0,
	    0,
	    0);

    } else {
        nicfp->q_dma_to_host_stub = 
            (U32 (*)(tg_hostaddr_t, U32, U32, U32, U32, U32))
            TO_SCRATCHPAD
	    (q_dma_to_host,
	    (U32)q_dma_to_host_end - (U32)q_dma_to_host,
	    &scratch_pad_loc,
	    &scratch_pad_size);

        h_mac_rx_comp_stub = 
            TO_SCRATCHPAD
	    (h_mac_rx_comp,
	    (U32)h_mac_rx_comp_end - (U32)h_mac_rx_comp,
	    &scratch_pad_loc,
	    &scratch_pad_size);

        h_dma_wr_assist_stub = (void (*)(void))
            TO_SCRATCHPAD
	    (h_dma_wr_assist,
	    (U32)h_dma_wr_assist_end - (U32)h_dma_wr_assist,
	    &scratch_pad_loc,
	    &scratch_pad_size);

	NIC_UTRACE("rCksmOff", 0x02150,
	    global_flags,
	    0,
	    0,
	    0);
    }
    set_event_disp(TG_FW_EVENT_NUM_MAC_RX_COMP, h_mac_rx_comp_stub);
    set_event_disp(TG_FW_EVENT_NUM_DMA_WR_ASST_HI, (U32)h_dma_wr_assist_stub);

#if (USE_SW_DMA_ASSIST)
    if (!(tsp->gen_com.mode_status & TG_CFG_MODE_1_DMA_ACTIVE))
	nicp->sw_dma_assist = TRUE;
#endif /* (USE_SW_DMA_ASSIST) */

    h_recv_bd_ready_stub = 
        TO_SCRATCHPAD
	(h_recv_bd_ready,
	(U32)h_recv_bd_ready_end - (U32)h_recv_bd_ready,
	&scratch_pad_loc,
	&scratch_pad_size);

    set_event_disp(TG_FW_EVENT_NUM_RECV_BD, h_recv_bd_ready_stub);

    if (is_global_flags(GF_RCB_JUMBO_RING_DISABLED)) {
	set_event_disp(TG_FW_EVENT_NUM_RECV_JUMBO_BD, scream_handler);
    } else {
	h_recv_jumbo_bd_ready_stub = 
	    TO_SCRATCHPAD
	    (h_recv_jumbo_bd_ready,
	    (U32)h_recv_jumbo_bd_ready_end - (U32)h_recv_jumbo_bd_ready,
	    &scratch_pad_loc,
	    &scratch_pad_size);

	set_event_disp(TG_FW_EVENT_NUM_RECV_JUMBO_BD, h_recv_jumbo_bd_ready_stub);
    }

    if (is_global_flags(GF_RCB_MINI_RING_DISABLED)) {
	set_event_disp(TG_FW_EVENT_NUM_RECV_MINI_BD, scream_handler);
    } else {
	h_recv_mini_bd_ready_stub = 
	    TO_SCRATCHPAD
	    (h_recv_mini_bd_ready,
	    (U32)h_recv_mini_bd_ready_end - (U32)h_recv_mini_bd_ready,
	    &scratch_pad_loc,
	    &scratch_pad_size);
	set_event_disp(TG_FW_EVENT_NUM_RECV_MINI_BD, h_recv_mini_bd_ready_stub);
    }

    mh_command_stub = 
        TO_SCRATCHPAD
	(mh_command,
	(U32)mh_command_end - (U32)mh_command,
	&scratch_pad_loc,
	&scratch_pad_size);

    set_event_disp(TG_FW_EVENT_NUM_MAILBOX, mh_command_stub);

    h_timer_stub = 
        TO_SCRATCHPAD
	(h_timer,
	(U32)h_timer_end - (U32)h_timer,
	&scratch_pad_loc,
	&scratch_pad_size);

    set_event_disp(TG_FW_EVENT_NUM_TIMER, h_timer_stub);

    h_do_update_tx_cons_stub = 
        TO_SCRATCHPAD
	(h_do_update_tx_cons,
	(U32)h_do_update_tx_cons_end - (U32)h_do_update_tx_cons,
	&scratch_pad_loc,
	&scratch_pad_size);

    set_event_disp(TG_FW_EVENT_NUM_DO_TX_CONS, h_do_update_tx_cons_stub);

    h_do_update_rx_prod_stub = 
        TO_SCRATCHPAD
	(h_do_update_rx_prod,
	(U32)h_do_update_rx_prod_end - (U32)h_do_update_rx_prod,
	&scratch_pad_loc,
	&scratch_pad_size);

    set_event_disp(TG_FW_EVENT_NUM_DO_RX_PROD, h_do_update_rx_prod_stub);

    nicfp->cksum16_stub = (U16 (*)(U16 *, U32, U32, U32))
        TO_SCRATCHPAD
	(cksum16,
	(U32)cksum16_end - (U32)cksum16,
	&scratch_pad_loc,
	&scratch_pad_size);

    nicfp->peek_mac_rx_wa_relocated = (tgMacDescr_t *(*)(void))
        TO_SCRATCHPAD
	(peek_mac_rx_wa,
	(U32)peek_mac_rx_wa_end - (U32)peek_mac_rx_wa,
	&scratch_pad_loc,
	&scratch_pad_size);

    nicfp->peek_mac_rx_stub =
    nicfp->peek_mac_rx_relocated = (tgMacDescr_t *(*)(void))
        TO_SCRATCHPAD
	(peek_mac_rx,
	(U32)peek_mac_rx_end - (U32)peek_mac_rx,
	&scratch_pad_loc,
	&scratch_pad_size);

    nicfp->deq_mac_rx_stub = (void (*)(void))
        TO_SCRATCHPAD
	(deq_mac_rx,
	(U32)deq_mac_rx_end - (U32)deq_mac_rx,
	&scratch_pad_loc,
	&scratch_pad_size);

    h_mac_rx_attn_stub = 
        TO_SCRATCHPAD
	(h_mac_rx_attn,
	(U32)h_mac_rx_attn_end - (U32)h_mac_rx_attn,
	&scratch_pad_loc,
	&scratch_pad_size);

    set_event_disp(TG_FW_EVENT_NUM_MAC_RX_ATTN, h_mac_rx_attn_stub);

    /* set all the HW rings key off the TX buf base and RX buf base */
    init_mac1();
#if (DYNAMIC_NIC_MEM)
    init_mac_buf();
#endif /* (DYNAMIC_NIC_MEM) */

    /* Set up the DMA assist rings and enable the DMA assist.
     * From now on, all DMA must go through the assist logic.
     * Using the other DMA method will confuse the DMA assist logic.
     */
    init_dma_assist();

    /* init shared memory stuffs */
#if CHECKFORERROR 
    tsp->mbox_command_prod_index = 0;
#endif
    tsp->gen_com.event_cons_index = 0;
    tsp->gen_com.command_cons_index = 0;

    /* init non-zero fields of nicp and nicfp */
    nicfp->send_coal_index = nicp->gen_info.send_rcb.RCB_max_len - 1;
    nicfp->recv_std_buf_size = nicp->gen_info.recv_standard_rcb.RCB_max_len;

    if (!is_global_flags(GF_RCB_MINI_RING_DISABLED))
	nicfp->recv_mini_buf_size = nicp->gen_info.recv_mini_rcb.RCB_max_len;

    i = nicp->gen_info.recv_return_rcb.RCB_max_len;
    if (i == 0) /* backward compatible */
	i = RECV_RETURN_RING_ENTRIES;
    nicfp->recv_coal_index = nicfp->recv_ret_ring_size_mask
			   = i - 1;

    /* size of recv return ring should be RECV_RETURN_RING_ENTRIES or half */
    if ((i != RECV_RETURN_RING_ENTRIES) &&
	!(is_global_flags(GF_RCB_MINI_RING_DISABLED) &&
	    (i == (RECV_RETURN_RING_ENTRIES >> 1)))) {

	send_error_event = TRUE;

	NIC_UTRACE("badRetSz", 0x02170,
	    nicp->gen_info.recv_return_rcb.RCB_max_len,
	    global_flags,
	    0,
	    0);
	ASSERT(0);
    }

    nicfp->tx_buf_prod = NIC_TX_BUF;
    nicfp->tx_buf_next = NIC_TX_BUF;
    nicfp->tx_new_frame = TRUE;

    NIC_UTRACE("rxBdBfSz", 0x02200,
	nicfp->recv_std_buf_size,
	nicfp->recv_mini_buf_size,
	0,
	0);
 
    /* init multicast stuffs */
    mcast_init();

    /* initialize event mask */
    event_mask =
        TG_FW_EVENT_DO_INTERRUPT |
        TG_FW_EVENT_TIMER |
        TG_FW_EVENT_DO_RX_PROD |
        TG_FW_EVENT_DO_TX_CONS |
        TG_FW_EVENT_MAC_RX_ATTN |
        TG_FW_EVENT_MAILBOX |
        TG_FW_EVENT_DMA_RD_ATTN |
        TG_FW_EVENT_DMA_WR_ATTN |
        TG_FW_EVENT_DMA_RD_ASST_HI |
        TG_FW_EVENT_DMA_WR_ASST_HI |
        TG_FW_EVENT_RECV_BD |
        TG_FW_EVENT_RECV_JUMBO_BD |
        TG_FW_EVENT_RECV_MINI_BD |
        TG_FW_EVENT_MAC_RX_COMP |
        TG_FW_EVENT_SEND_BD |
        TG_FW_EVENT_SEND_DATA |
        0;

    /* init the event reg */
    trp->gen_control.event =
        TG_FW_EVENT_REMOTE_CPU_ATTN | 
        TG_FW_EVENT_UPDATE_LEDS |
        0;

#if defined(MAILBOX)
    init_mailbox();
#else /* !defined(MAILBOX) */
    trp->gen_control.mailbox_event = 0xffffffff;
#endif /* defined(MAILBOX) */

#if (FW_PROFILE)
    null_handler_stub =
	TO_SCRATCHPAD
	(null_profile_handler,
	(U32)null_profile_handler_end - (U32)null_profile_handler,
	&scratch_pad_loc,
	&scratch_pad_size);

    set_event_disp(-1, (U32)null_handler_stub);
#else /* (FW_PROFILE) */
    null_handler_stub =
	TO_SCRATCHPAD
	(null_handler,
	(U32)null_handler_end - (U32)null_handler,
	&scratch_pad_loc,
	&scratch_pad_size);

    set_event_disp(-1, (U32)null_handler_stub);
#endif /* (FW_PROFILE) */

    /* init timers */
    init_timer();

    /* Initialize CPU B */
    trp->cpu_control_b.cpu_state |= TG_CPU_HALT;
    trp->cpu_control_b.pc         = (U32)_start_b;
    trp->cpu_control_b.cpu_state  = 0;
        
    /* Wait for CPU B to tell us to enter dispatch loop */
    while (! (trp->gen_control.event & TG_FW_EVENT_REMOTE_CPU_ATTN));

    /* Initialize the MAC interface.
     * This probes out the PHY(s) and takes a while
     * so we need to do it before we send the firmware operational event.
     */
    init_mac2();

    /* generate firmware up event */
    event.TG_EVENT_w0 = TG_EVENT_FIRMWARE_OPERATIONAL << TG_EVENT_EVENT_SHIFT;
    enq_event(&event);
    nicfp->stats.nicEventsNICFirmwareOperational++;
    if (send_error_event) {
	error_event.TG_EVENT_w0 =
	    (TG_EVENT_ERROR << TG_EVENT_EVENT_SHIFT) |
	    (TG_EVENT_CODE_ERR_BAD_CONFIG << TG_EVENT_CODE_SHIFT);
        enq_event(&error_event);
        nicfp->stats.nicEventsError++;

        NIC_UTRACE("fwOpFail", 0x02300,
	    error_event.TG_EVENT_w0,
	    0,
	    0,
	    0);
	ASSERT(0);
    }

    /* wait for the operational event to complete to send interrupt */
    while ((trp->host_dma.dma_wr_state & TG_DMA_STATE_ACTIVE) &&
           (tsp->gen_com.maskRupts))
        ; /* wait for (DMA to complete, Host ready) */
    interrupt_host(); /* interrupt right away */

    /* init tx buf spae */
    nicfp->tx_buf_space =
	trp->local_mem_conf.tx_buf_consumer -
        trp->local_mem_conf.tx_buf_producer -
	TX_BUF_RED_ZONE_SIZE; 
    if (nicfp->tx_buf_space <= -TX_BUF_RED_ZONE_SIZE) {
        nicfp->tx_buf_space += NIC_TX_BUF_SIZE;
    }

    nicfp->tx_q_mac_pending_cnt = 1; /* not 0 for fast comparison later */

    /* init chip's vlan tag type */
    if (is_global_flags(GF_RCB_TX_VLAN_ASSIST)) {
        trp->mac_control.mac_len =
	    (trp->mac_control.mac_len & ~TG_MAC_TX_LEN_TAG_TYPE_MASK) |
	    (ETHER_TYPE_WITH_VLAN_TAG << 16);
    }

    /* load the appropriate LED routines based upon our MAC configuration-
     * GIG only -> 2leds, dual PHY cards have 6 leds...
     */
    if (attached_phy & ATTACHED_PHY_IF_GMII) {
        h_update_led_stub = 
            TO_SCRATCHPAD
	    (h_update_led4,
	    (U32)h_update_led4_end - (U32)h_update_led4,
	    &scratch_pad_loc,
	    &scratch_pad_size);
    } else if (attached_phy & ATTACHED_PHY_IF_MII) {
        h_update_led_stub = 
            TO_SCRATCHPAD
	    (h_update_led6,
	    (U32)h_update_led6_end - (U32)h_update_led6,
	    &scratch_pad_loc,
	    &scratch_pad_size);
    } else {
        h_update_led_stub = 
            TO_SCRATCHPAD
	    (h_update_led2,
	    (U32)h_update_led2_end - (U32)h_update_led2,
	    &scratch_pad_loc,
	    &scratch_pad_size);
    }
    set_event_disp(TG_FW_EVENT_NUM_UPDATE_LEDS, h_update_led_stub);

    /* save into global vars for debugging */
    gglobal_flags = global_flags;
    gtsp = tsp; 
    gtrp = trp;
    gtdp = tdp;
    gnicp = nicp;
    gnicfp = nicfp;

    NIC_UTRACE("intState", 0x02400,
	trp->gen_control.pci_state,
	trp->host_dma_assist.assist_state,
	trp->host_dma.dma_wr_state,
	trp->host_dma.dma_rd_state);

    NIC_UTRACE("**initCp", 0x02500,
	trp->gen_control.misc_local_control,
	trp->gen_control.misc_config,
	trp->mac_control.mac_tx_state,
	trp->mac_control.mac_rx_state);

    /* main dispatcher loop */
    __asm__ volatile
    ("
        .set noreorder
        .globl _disp_loop
    _disp_loop:
        lw      %0,0x60($28)     # load event register
        pri     %1,%0,%2         # determine next event
        joff    %1,%3            # jump to next event
        move    $31,%4
        .set reorder
        nop
        halt
        " : "=r" (events), "=r" (high_event)
        : "r" (event_mask), "r" (ev_handlerp), "r" (dispp)
    );
    /* not reached */
}

void
scream_handler(void)
{
    NIC_UTRACE("#scream", 0x02600,
	0,
	0,
	0,
	0);
    ASSERT(0);
}

void
null_profile_handler(void)
{
    RETURN(TG_FW_EVENT_NUM_SW0);
}

/* null routine to assist with scratchpad setup */
void null_profile_handler_end(void) {}

void
null_handler(void)
{
    __asm__("nop");
    __asm__("nop");
    __asm__("nop");
    __asm__("nop");
    __asm__("nop");
    __asm__("nop");
    RETURN(TG_FW_EVENT_NUM_SW0);
}

/* null routine to assist with scratchpad setup */
void null_handler_end(void) {}

/*
 * quick test Tigon chip's cpu adjust stack feature
 */
void
probe_adj_stack(void)
{
    U32 temp[SCRATCH_PAD_STACK_SIZE >> 2]; /* force to wrap stack */
    U32 *stk = &temp[0];

    ASSERT((U32)stk < TG_BEG_SCRATCH);
    *stk = 0xdeadbeef;
    (U32)stk -= (U32)trp->cpu_control.adjust_stack;

    /* following is required since bit23 is masked off
     * by Tigon chip's adjust_stack register (0x150)
     */
    (U32)stk &= (TG_DATA_END - 1);
    if (*stk != 0xdeadbeef) {
        NIC_UTRACE("iStackEr", 0x02700,
	    (U32)trp->cpu_control.adjust_stack,
	    (U32)trp->cpu_control_b.adjust_stack,
	    (U32)stk,
	    *stk);
	ASSERT(0);
    }
}

/*
 * probe installed nic memory: 256K, 512K, 1M, 2M...
 * NOTE: only work for wrapable memory banks
 */
U32
probe_mem_size(void)
{
    U32 seed  = 0xabbababe;
    U32 start = NIC_MEM_256K;
    U32 mem, probe;
    U32 mem_sv, probe_sv;
    U32 data;

    for (mem = start; mem < TG_END_SRAM; mem <<= 1) {
	probe = mem << 1;

	/* save */
	mem_sv   = *(U32 *)mem;
	probe_sv = *(U32 *)probe;

	/* write */
	*(U32 *)probe = 0;
	*(U32 *)mem   = seed;

	/* read */
	data = *(U32 *)probe;

	/* restore */
	*(U32 *)probe = probe_sv;
	*(U32 *)mem   = mem_sv;

	if (data == seed)
	    break;
    }

    NIC_UTRACE("probeMem", 0x02800,
	mem,
	0,
	0,
	0);
    return (mem);
}

U32
get_tx_buf_size(U32 total_buf_sz /* multiple of 4KB */)
{
    U32 tx_buf_ratio;
    U32 tx_buf_sz;
    U32 buf_sz;
    U32 bit;

    tx_buf_ratio = tsp->gen_com.tx_buf_ratio & TX_BUF_RATIO_MASK;

    NIC_TRACE(TRACE_TYPE_SEND, "tBfRatio", 0x02900,
	tx_buf_ratio,
	0,
	0,
	0);

    if (!tx_buf_ratio) {
	/* set to min buf size if 0 */
	return (MIN_TX_MAC_BUF);
    }

    buf_sz = total_buf_sz;
    tx_buf_sz = 0;

    /* loop thru tx_buf_ratio bits */
    for (bit = ((TX_BUF_RATIO_MASK + 1) >> 1); bit; bit >>= 1) {
	buf_sz >>= 1;
	if (tx_buf_ratio & bit) {
	    tx_buf_sz += buf_sz;
	}
    }
    tx_buf_sz &= ~0xfff; /* round to multiple of 4KB */

    if (tx_buf_sz < MIN_TX_MAC_BUF) {
	/* at least MIN_TX_MAC_BUF for tx buf */
	return (MIN_TX_MAC_BUF);
    }

    if ((total_buf_sz - tx_buf_sz) < MIN_RX_MAC_BUF) {
	/* at least MIN_RX_MAC_BUF for rx buf */
	return (total_buf_sz - MIN_RX_MAC_BUF);
    }

    return (tx_buf_sz);
}

U32
copy2scratchpad(char *name, U32 start_addr, U32 len,
		U32 *scratch_pad_loc, U32 *scratch_pad_size)
{
    U32 actual_loc;
 
    len = ROUNDUP(len, 4);
    if (len <= *scratch_pad_size) {
        actual_loc = *scratch_pad_loc;
	#if 0
	*name = '+'; 
	#endif

        NIC_UTRACE(name, 0x03000,
	    start_addr,
	    *scratch_pad_loc,
	    len,
	    *scratch_pad_size);

        wcopy((U32 *)start_addr, (U32 *)*scratch_pad_loc, (len >> 2));
        *scratch_pad_size -= len; /* Word Boundary */
        *scratch_pad_loc  += len; /* Word Boundary */
    } else {
        actual_loc = start_addr;
	*name = '-';

        NIC_UTRACE(name, 0x03100,
	    start_addr,
	    *scratch_pad_loc,
	    len,
	    *scratch_pad_size);
    }
    return (actual_loc);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
 * Entry point for TigonSC CPU B (Tx CPU)
 */
void
main_b(void)
{
    trp = ((tg_regs_t *)(TG_BEG_REGS));
#if (DYNAMIC_NIC_MEM)
    tdp = (tg_data_t *)
          (TG_DATA_END - rx_buf_size - tx_buf_size - sizeof(tg_data_t));
#else /* (DYNAMIC_NIC_MEM) */
    tdp = (tg_data_t *)(TG_DATA_END - sizeof(tg_data_t));
#endif /* (DYNAMIC_NIC_MEM) */
    nicp = (struct nic *)((U32)tdp - sizeof(nic_t));

    /* write 0 to last word in scratchpad to fix static IDD problem */
    *(U32 *)((U32)TG_END_SCRATCH_B - sizeof(U32)) = 0;

    scratch_pad_sz  = TG_END_SCRATCH_B - TG_BEG_SCRATCH - sizeof(U32);
    scratch_pad_mem = TG_BEG_SCRATCH;

#if (USE_FAST_STACK_CPU_B)
    /* relocate stack to scratch pad */
    __asm__ volatile
    ("
	.set noreorder
	lw	$sp, adj_stack_start
	move	$fp, $sp
	.set reorder
    ");

    scratch_pad_sz  -= SCRATCH_PAD_STACK_SIZE;
    scratch_pad_mem += SCRATCH_PAD_STACK_SIZE;
#endif /* (USE_FAST_STACK_CPU_B) */

    /* nic fast structure resides in scratch pad */
    nicfp = (nic_fast_t *)scratch_pad_mem;
    scratch_pad_sz  -= sizeof(nic_fast_t);
    scratch_pad_mem += sizeof(nic_fast_t);

    fw_init_b();
}

void
fw_init_b(void)
{
    U32 scratch_pad_size = scratch_pad_sz;
    U32 scratch_pad_loc  = scratch_pad_mem;

    NIC_UTRACE("**DAWN_B", 0x03200,
	0,
	tdp,
	nicp,
	nicfp);

    init_timer_b();

    trp->cpu_control_b.cpu_state |=
#if (0)
        TG_CPU_DATA_CACHE_ENA |
#endif /* XXX */
        TG_CPU_PAGE_0_HALT_ENA;
 
#if 0 /* only first field cpu_id is used */
    /* zero out nic and nic fast structure */
    bzero((U32 *)nicfp, sizeof(struct nic_fast));
#endif /*0*/

    /* register myself */
    nicfp->cpu_id = TG_CPU_B;

    /* move most common routines into scratch pad */
    if (nicp->sw_dma_assist) {
        sw_dma_assist_stub = (void (*)(void))
            TO_SCRATCHPAD
	    (sw_dma_assist_plus_timer,
	    (U32)sw_dma_assist_plus_timer_end - (U32)sw_dma_assist_plus_timer,
	    &scratch_pad_loc,
	    &scratch_pad_size);

        preload_wr_descr_p = (tgDmaDescr_t *)
            TO_SCRATCHPAD
	    (&preload_wr_descr,
	    sizeof(tgDmaDescr_t),
	    &scratch_pad_loc,
	    &scratch_pad_size);

        preload_rd_descr_p = (tgDmaDescr_t *)
            TO_SCRATCHPAD
	    (&preload_rd_descr,
	    sizeof(tgDmaDescr_t),
	    &scratch_pad_loc,
	    &scratch_pad_size);
    } else {
        sw_dma_assist_stub = (void (*)(void))
            TO_SCRATCHPAD
	    (h_hf_timer,
	    (U32)h_hf_timer_end - (U32)h_hf_timer,
	    &scratch_pad_loc,
	    &scratch_pad_size);
    }

    /* signal CPU A to enter dispatch loop */
    trp->gen_control.event_b = TG_FW_EVENT_SET_REMOTE_CPU_ATTN;

    (*sw_dma_assist_stub)();
}
