/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: mac.c $
 * Revision copper_fhwang/25 2000/06/03 02:24:57 fhwang
 * 	not to set active_phy to call set_mac_link
 * 
 * Revision copper_fhwang/20 2000/01/25 02:20:17 fhwang
 * 	to remove the checking for TG_MAC_RX_STATE_LINK_SYNC bit to prevent ever breaking the link if cable is disconnected
 * 
 * Revision copper_fhwang/16 2000/01/04 01:16:42 fhwang
 * 	remove two lines of code which are in the block of checking LINK_SYNC bit
 * 
 * Revision copper_fhwang/13 1999/12/29 03:43:37 fhwang
 * 	put back checking for TG_MAC_RX_STATE_LINK_SYNC if there is link ready change attn
 * 
 * Revision nic_api_12_4_latest_shuang/4 1999/12/09 19:51:56 shuang
 * 	work around Tg2 MAC6 bug only if get rx buf full attention.
 * 
 * Revision copper_fhwang/8 1999/12/08 20:10:12 fhwang
 * 	update ifSpeed when link is up
 * 
 * Revision copper_fhwang/6 1999/09/03 22:01:03 fhwang
 * 	add GMII support on set_mac_link call
 * 
 * Revision copper_fhwang/3 1999/08/27 01:59:35 fhwang
 * 	set fastlinkstate if 10/100Mb link up
 * 
 * Revision nic_api_12_4_latest_shuang/2 1999/08/27 01:36:22 shuang
 * 	jumbo frame length will be 0x8870
 * 
 * Revision sgi_api_12_3_latest_shuang/5 1999/07/15 16:30:15 shuang
 * 	add WRAP_RX_BUF() for buf_cons_adj
 * 
 * Revision sgi_api_12_3_latest_shuang/3 1999/07/14 18:58:50 shuang
 * 	work around MAC6 Tigon2 bug, fix rx buffer full cleanup
 * 
 * Revision llc_fhwang/2 1999/03/12 00:35:03 fhwang
 * 	call link_down_notify whenever there is link attention.  
 * 	Remove the check for TG_MAC_RX_STATE_LINK_SYNC bit.
 * 
 * Revision 1.1.2.22  1998/12/08  02:36:30  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:27:55  shuang]
 *
 * Revision 1.1.2.21  1998/09/30  18:49:59  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:39:57  shuang]
 * 
 * Revision 1.1.2.20  1998/08/27  17:33:08  shuang
 * 	fix defect1370- flow control
 * 	[1998/08/27  17:31:47  shuang]
 * 
 * Revision 1.1.2.19  1998/07/29  22:31:04  shuang
 * 	add feature - flow control support
 * 	[1998/07/29  21:59:43  shuang]
 * 
 * Revision 1.1.2.18  1998/06/23  01:29:36  shuang
 * 	     added dynamic nic mem support.
 * 	     [1998/06/23  01:21:40  shuang]
 * 
 * Revision 1.1.2.17  1998/05/15  20:14:30  shuang
 * 	          stop mac receiver for a while if run out of rx buffer
 * 	          [1998/05/15  20:13:13  shuang]
 * 
 * Revision 1.1.2.16  1998/05/14  01:25:19  shuang
 * 	               Fix incorrect using TG_MAC_TX_ATTN_MASK to mask off Rx attention bits.
 * 	               [1998/05/14  01:20:38  shuang]
 * 
 * Revision 1.1.2.15  1998/05/12  01:27:48  shuang
 * 	               do link_down_notify on mac rx not synced attn
 * 	               [1998/05/12  01:27:11  shuang]
 * 
 * Revision 1.1.2.14  1998/04/24  00:00:36  hayes
 * 	               added const to rcsid definitions to eliminate compiler warnings
 * 	               [1998/04/23  23:58:52  hayes]
 * 
 * Revision 1.1.2.13  1998/04/11  05:15:07  shuang
 * 	               move fast timer to cpu B, add receive coalesed ticks
 * 	               [1998/04/11  04:58:08  shuang]
 * 
 * Revision 1.1.2.12  1998/03/28  00:53:17  shuang
 * 	               No interrupt avoidance on link state change
 * 	               [1998/03/28  00:50:19  shuang]
 * 
 * Revision 1.1.2.11  1998/03/26  01:34:25  shuang
 * 	               cleanup
 * 	               [1998/03/26  01:25:06  shuang]
 * 
 * Revision 1.1.2.9  1998/02/23  01:22:33  hayes
 * 	               cleanup link UP/DOWN tracing
 * 	               [1998/02/23  01:21:01  hayes]
 * 
 * Revision 1.1.2.8  1998/02/21  21:07:39  hayes
 * 	               Reorder MAC RX attn handling to time critical attns are handled first.
 * 	               Insure that touching the mac_rx_state register doesn't inadvertantly clear
 * 	               and attn bits.
 * 	               Remove obsolete descriptor attn handling code, obsolete in rev 5
 * 	               Close last windows in low rx buffer case.
 * 	               [1998/02/21  20:58:23  hayes]
 * 
 * Revision 1.1.2.7  1998/02/19  01:29:54  suresh
 * 	               Merged with changes from 1.1.2.6
 * 	               [1998/02/19  01:29:17  suresh]
 * 
 * 	               New traces, TDP correction, extra routine to assist scratchpad setup
 * 	               [1998/02/19  01:19:07  suresh]
 * 
 * Revision 1.1.2.6  1998/02/14  21:14:42  hayes
 * 	               fix RX_BUF_ATTN
 * 	               [1998/02/14  21:13:14  hayes]
 * 
 * Revision 1.1.2.5  1998/01/30  06:45:23  hayes
 * 	               always bring link down when we get a sync attention
 * 	               [1998/01/30  06:43:18  hayes]
 * 
 * Revision 1.1.2.4  1998/01/30  01:48:25  hayes
 * 	               update linkState field on link up/down
 * 	               [1998/01/30  01:47:42  hayes]
 * 
 * Revision 1.1.2.3  1998/01/29  18:40:21  ted
 * 	               Add multicast mode support and move some things into the fast nic structure
 * 	               [1998/01/29  18:28:54  ted]
 * 
 * Revision 1.1.2.2  1998/01/21  21:34:21  hayes
 * 	               fix invalid descriptor bug
 * 	               [1998/01/21  21:28:32  hayes]
 * 
 * Revision 1.1.2.1  1998/01/14  23:25:51  ted
 * 	               rearrange directories
 * 	               [1998/01/14  23:17:46  ted]
 * 
 * Revision 1.1.2.10  1998/01/09  03:15:04  ted
 * 	               Change mechanism for backing up the producer following an attention
 * 	               [1998/01/09  00:30:56  ted]
 * 
 * Revision 1.1.2.9  1998/01/09  03:12:35  hayes
 * 	               re-enable promiscuous mode when applicable
 * 	               [1998/01/09  03:12:18  hayes]
 * 
 * Revision 1.1.2.8  1998/01/04  02:26:29  hayes
 * 	               handle transient link conditions
 * 	               [1998/01/04  02:26:09  hayes]
 * 
 * Revision 1.1.2.7  1997/12/28  00:23:09  hayes
 * 	               cleanup LEDs for both 6 and 2 LED cards
 * 	               [1997/12/28  00:22:57  hayes]
 * 
 * Revision 1.1.2.6  1997/12/15  20:02:54  ted
 * 	               Remove TIGON_REV4 support
 * 	               [1997/12/15  19:35:24  ted]
 * 
 * Revision 1.1.2.5  1997/12/12  05:34:38  hayes
 * 	               setting phy characteristics of inactive link no longer causes link up/down
 * 	               [1997/12/12  04:01:33  hayes]
 * 
 * Revision 1.1.2.4  1997/11/30  00:57:34  hayes
 * 	               fixup RX buf and descr attns
 * 	               [1997/11/30  00:56:17  hayes]
 * 
 * Revision 1.1.2.3  1997/11/21  23:54:21  hayes
 * 	               MII link down works
 * 	               [1997/11/21  23:54:02  hayes]
 * 
 * 	               MII interface works
 * 	               [1997/11/21  22:45:38  hayes]
 * 
 * Revision 1.1.2.2  1997/11/21  04:48:16  hayes
 * 	               release for NIC 3.1 pre-Beta
 * 	               [1997/11/21  04:45:27  hayes]
 * 
 * 	               intermediate checkin
 * 	               [1997/11/20  01:28:30  hayes]
 * 
 * Revision 1.3.4.53  1997/11/14  19:16:27  hayes
 * 	               first checkin after mongo merge...
 * 	               [1997/11/14  19:11:31  hayes]
 * 
 * 	               checkin after stats restructuring
 * 	               [1997/11/12  19:01:26  hayes]
 * 
 * Revision 1.3.4.52  1997/11/11  19:05:57  hayes
 * 	               restructuring for scratchpad use
 * 	               [1997/11/11  19:04:20  hayes]
 * 
 * Revision 1.3.4.51  1997/11/10  23:54:37  hayes
 * 	               remove obsolete reference to phy_link_state
 * 	               [1997/11/10  23:52:32  hayes]
 * 
 * Revision 1.3.4.50  1997/11/10  00:27:26  hayes
 * 	               Remove board_state references
 * 	               Cleanup link codee
 * 	               [1997/11/10  00:24:47  hayes]
 * 
 * Revision 1.3.4.49  1997/11/03  22:38:32  hayes
 * 	               fix link bringup/mac_rx_attn()
 * 	               [1997/11/03  22:38:04  hayes]
 * 
 * Revision 1.3.4.48  1997/10/30  02:21:19  hayes
 * 	               get link negotiation working for Tigon2
 * 	               [1997/10/30  02:18:34  hayes]
 * 
 * Revision 1.3.4.47  1997/10/29  03:44:35  hayes
 * 	               add rx buf cleanup
 * 	               [1997/10/29  03:42:27  hayes]
 * 
 * Revision 1.3.4.46  1997/10/28  01:51:18  hayes
 * 	               works for Tigon2-SX
 * 	               [1997/10/27  23:31:50  hayes]
 * 
 * Revision 1.3.4.45  1997/10/21  18:18:04  hayes
 * 	               fix for REV5
 * 	               [1997/10/21  18:17:41  hayes]
 * 
 * Revision 1.3.4.44  1997/10/21  01:35:33  skapur
 * 	               Merge of release 2.0 and ge_latest branch
 * 	               [1997/10/21  01:31:04  skapur]
 * 
 * Revision 1.3.4.43  1997/10/17  00:03:42  hayes
 * 	               update for fixed tg.h and REV5
 * 	               [1997/10/17  00:03:08  hayes]
 * 
 * Revision 1.3.4.42  1997/09/17  03:26:36  hayes
 * 	               finish preparing for multiple phys
 * 	               [1997/09/16  18:24:16  hayes]
 * 
 * Revision 1.3.4.41  1997/09/10  03:55:13  hayes
 * 	               begin code prep for multiple phys per mac
 * 	               [1997/09/10  03:52:56  hayes]
 * 
 * 	               completed link code restructuring
 * 	               [1997/08/29  22:09:21  hayes]
 * 
 * Revision 1.3.4.40  1997/08/29  22:12:18  hayes
 * 	               completed link code restructuring
 * 
 * Revision 1.3.4.39  1997/08/26  21:09:12  hayes
 * 	               link code preparation for SC changes
 * 	               [1997/08/26  21:08:23  hayes]
 * 
 * Revision 1.3.4.38  1997/08/18  21:15:14  hayes
 * 	               disable MAC RX attns after receiving a config
 * 	               [1997/08/18  21:13:16  hayes]
 * 
 * Revision 1.3.4.37  1997/08/02  19:58:23  hayes
 * 	               now set additional_rx_bits_wanted
 * 	               [1997/08/02  19:56:49  hayes]
 * 
 * Revision 1.3.4.36  1997/07/29  03:40:58  ted
 * 	               Add OP1 LED support
 * 	               [1997/07/29  03:40:03  ted]
 * 
 * Revision 1.3.4.35  1997/07/25  22:59:00  ted
 * 	               Merge fixes from ge_0_22 and add link negotiation support
 * 	               [1997/07/25  22:32:03  ted]
 * 
 * Revision 1.3.9.4  1997/07/23  07:38:47  hayes
 * 	               remove promisc mode in code from switch
 * 	               [1997/07/23  07:38:39  hayes]
 * 
 * Revision 1.3.9.3  1997/07/23  02:58:05  hayes
 * 	               bring link code to same level as switch
 * 	               [1997/07/23  02:57:34  hayes]
 * 
 * Revision 1.3.9.2  1997/07/16  00:20:21  ted
 * 	               fix bug in setting FC properly in tg_link_set()
 * 	               [1997/07/15  21:57:23  ted]
 * 
 * 	               disable rx_attns when link drops
 * 	               [1997/07/15  00:39:16  ted]
 * 
 * Revision 1.3.9.1  1997/07/11  02:25:41  ted
 * 	               add link negotiation support
 * 	               [1997/07/11  02:11:09  ted]
 * 
 * Revision 1.3.4.32  1997/05/21  22:59:49  ted
 * 	               Minor statistics tweaks
 * 	               [1997/05/21  22:59:08  ted]
 * 
 * Revision 1.3.4.31  1997/05/21  03:35:41  ted
 * 	               New link configuration code
 * 	               [1997/05/21  03:13:04  ted]
 * 
 * Revision 1.3.6.2  1997/05/17  18:05:08  hayes
 * 	               change LC states to match D2.1 draft.
 * 
 * Revision 1.3.4.30  1997/05/03  04:58:44  skapur
 * 	               Cleaned up stats code
 * 	               [1997/05/03  04:58:07  skapur]
 * 
 * Revision 1.3.4.29  1997/04/25  18:48:47  ted
 * 	               Remove SBUS compile time flags
 * 	               [1997/04/25  18:35:16  ted]
 * 
 * Revision 1.3.4.28  1997/04/17  02:30:52  skapur
 * 	               Major overhaul of the MAC State Machine with NO LINK CONFIGS
 * 	               [1997/04/17  02:30:31  skapur]
 * 
 * Revision 1.3.4.27  1997/04/16  04:17:01  hayes
 * 	               insure that we get our initial link up
 * 	               [1997/04/16  04:16:53  hayes]
 * 
 * Revision 1.3.4.26  1997/04/16  03:55:03  hayes
 * 	               adjust flaps count for non-negotiate mode
 * 	               [1997/04/16  03:54:52  hayes]
 * 
 * Revision 1.3.4.25  1997/04/16  03:42:37  hayes
 * 	               fix no negotiate mode for gig links
 * 	               [1997/04/16  03:42:15  hayes]
 * 
 * Revision 1.3.4.24  1997/04/15  21:03:42  hayes
 * 	               added LINK_FLAP_THRESHHOLD
 * 	               [1997/04/15  21:00:50  hayes]
 * 
 * Revision 1.3.4.23  1997/04/15  08:24:30  hayes
 * 	               minor tweaks...
 * 	               [1997/04/15  08:24:22  hayes]
 * 
 * Revision 1.3.4.22  1997/04/15  05:46:30  hayes
 * 	               updated for new gig autonegotiation state machine
 * 	               [1997/04/15  05:44:03  hayes]
 * 
 * Revision 1.3.4.21  1997/04/12  00:38:37  ted
 * 	               Turn Link LED ON after the clearing the RX Attentions in mac_reset
 * 	               [1997/04/12  00:38:21  ted]
 * 
 * Revision 1.3.4.20  1997/04/11  23:36:18  ted
 * 	               SBUS related code
 * 	               [1997/04/11  23:16:57  ted]
 * 
 * Revision 1.3.4.19  1997/04/10  01:51:27  akhan
 * 	               Took out link configuaration; If it breaks any of your code, please yell at Sukhdev not Ahsan (Sukhdev checking in as Ahsan)
 * 	               [1997/04/10  01:50:37  akhan]
 * 
 * Revision 1.3.4.18  1997/03/28  02:47:01  skapur
 * 	               Commented out TG_MAC_TX_STATE_ENA_CHECK_CRS
 * 	               [1997/03/28  02:01:16  skapur]
 * 
 * Revision 1.3.4.17  1997/03/24  08:34:22  ted
 * 	               New event ring handling, producer lives in host memory, misc. other more modest changes
 * 	               [1997/03/24  08:27:55  ted]
 * 
 * Revision 1.3.4.16  1997/03/17  08:47:43  skapur
 * 	               added remote fault detect and remote assert() support
 * 	               [1997/03/17  08:28:07  skapur]
 * 
 * 	               integration changes...
 * 	               [1997/03/16  03:01:46  skapur]
 * 
 * Revision 1.3.4.15  1997/03/13  19:47:48  hayes
 * 	               Adjust init_mac2() so that the rx_state reg has the RX_ENABLE bit
 * 	               set in accordance with tg_mac_set.
 * 	               [1997/03/13  19:47:37  hayes]
 * 
 * Revision 1.3.4.14  1997/03/13  19:25:29  hayes
 * 	               Change Gbit PHY control from the old MII pins to the LADDR_21/22
 * 	               pins.  Remove always setting the MAC in promiscuous mode since we now have
 * 	               a real MAC, and save and restore promiscuous mode when resetting the MAC.
 * 	               [1997/03/13  19:24:44  hayes]
 * 
 * Revision 1.3.4.13  1997/03/10  19:50:51  hayes
 * 	               reset the Gig mac when recurring link_attns occur, even without seeing
 * 	               the link flapping.  We can get into this situation when the FW is
 * 	               stopped under GDB, and the cable is unplugged and replaces and the
 * 	               LINK_RDY signal stabilizes before the FW is restarted.  We now count
 * 	               the number of consecutive times through link attn w/o any progress
 * 	               and reset the MAC after the threshhold is exceeded.
 * 	               [1997/03/10  19:50:37  hayes]
 * 
 * Revision 1.3.4.12  1997/03/10  19:44:41  hayes
 * 	               RX_ENABLE will stay off if turned off by GDB.  This will make debugging
 * 	               easier.  RX_ENABLE must be re-enabled in GDB to re-enable the link.
 * 	               [1997/03/10  19:44:31  hayes]
 * 
 * Revision 1.3.4.11  1997/03/09  23:25:28  hayes
 * 	               Merged 10/100 fixes from switch code.
 * 	               [1997/03/09  23:25:17  hayes]
 * 
 * Revision 1.3.4.10  1997/03/09  22:55:10  skapur
 * 	               fix for proper gig link bringup
 * 	               [1997/03/09  22:54:57  skapur]
 * 
 * Revision 1.3.4.9  1997/03/03  21:28:50  ted
 * 	               Updated MAC/PHY API to use link.h and new LNK_XXX
 * 	               defines for tg_link_set and tg_link_state.
 * 	               [1997/03/03  21:27:50  ted]
 * 
 * Revision 1.3.4.8  1997/03/03  18:21:37  ted
 * 	               Added writing all writable registers in phy_reset()
 * 	               [1997/03/03  18:21:27  ted]
 * 
 * Revision 1.3.4.7  1997/03/01  01:21:36  ted
 * 	               Merged with changes from 1.3.4.6
 * 	               [1997/03/01  01:21:29  ted]
 * 
 * 	               cleanup
 * 	               [1997/03/01  01:20:46  ted]
 * 
 * 	               autonegotiation now works
 * 	               [1997/03/01  01:15:32  ted]
 * 
 * Revision 1.3.4.2  1997/02/11  04:08:07  ted
 * 	               Intermediate checkin
 * 	               [1997/02/11  04:01:18  ted]
 * 
 * Revision 1.3.4.1  1997/02/11  02:50:14  ted
 * 	               Add synchronization hack to get byte sync to workaround
 * 	               AMCC loopback sync probelms.  The algorithm is:
 * 	               Set tx_config reg to pattern.
 * 	               Check for same pattern in rx_config reg.
 * 	               If not pattern, toggle loopback bit off, pause
 * 	               and then back on.  This will cause the external loopback
 * 	               to synchronize.  repeat until all patterns pass properly,
 * 	               after sampling them multiple times each.
 * 	               [1997/02/11  02:45:14  ted]
 * 
 * 	               checkin before trying experimetal sync code
 * 	               [1997/02/10  04:58:17  ted]
 * 
 * Revision 1.3.2.6  1996/12/05  20:07:04  hayes
 * 	               Fix SIMULATOR specific problems with initialization changes
 * 	               [1996/12/05  20:03:57  hayes]
 * 
 * Revision 1.3.2.5  1996/11/29  17:43:24  hayes
 * 	               Split init_mac() into init_mac1() and init_mac2() since
 * 	               all Tigon related things now key off the TX and RX buf base
 * 	               registers.  These must be set up much earlier in the init
 * 	               processing.  Init_mac2() covers the real MAC initialization and
 * 	               link initialization.
 * 	               [1996/11/29  17:36:04  hayes]
 * 
 * Revision 1.3.2.4  1996/11/20  20:04:21  ted
 * 	               Redo include file structuring.
 * 	               [1996/11/20  19:51:23  ted]
 * 
 * Revision 1.3.2.3  1996/11/15  04:24:07  hayes
 * 	               convert to new header file structure.
 * 	               [1996/11/15  04:12:01  hayes]
 * 
 * Revision 1.3.2.2  1996/11/05  21:54:21  hayes
 * 	               Final pre-ODE merge
 * 	               [1996/11/05  21:51:39  hayes]
 * 
 * Revision 1.3.2.1  1996/11/01  18:24:57  ted
 * 	               Get rid of a compiler warning
 * 	               [1996/10/30  00:07:12  ted]
 * 
 * 	               change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	               [1996/10/29  22:06:10  ted]
 * 
 * Revision 1.3.4.6  1997/02/24  23:58:29  skapur
 * 	               Moved the Code to send 10ms worth of Quiet after the link_flaps
 * 	               [1997/02/24  23:58:14  skapur]
 * 
 * Revision 1.3.4.5  1997/02/23  00:23:33  ted
 * 	               Incorporated 10/100 mac code. John still needs to clean this to match up with 10/100 code
 * 	               [1997/02/23  00:11:54  ted]
 * 
 * Revision 1.3.4.2  1997/02/11  04:08:07  ted
 * 	               Intermediate checkin
 * 	               [1997/02/11  04:01:18  ted]
 * 
 * Revision 1.3.4.1  1997/02/11  02:50:14  ted
 * 	               Add synchronization hack to get byte sync to workaround
 * 	               AMCC loopback sync probelms.  The algorithm is:
 * 	               Set tx_config reg to pattern.
 * 	               Check for same pattern in rx_config reg.
 * 	               If not pattern, toggle loopback bit off, pause
 * 	               and then back on.  This will cause the external loopback
 * 	               to synchronize.  repeat until all patterns pass properly,
 * 	               after sampling them multiple times each.
 * 	               [1997/02/11  02:45:14  ted]
 * 
 * 	               checkin before trying experimetal sync code
 * 	               [1997/02/10  04:58:17  ted]
 * 
 * Revision 1.3.2.6  1996/12/05  20:07:04  hayes
 * 	               Fix SIMULATOR specific problems with initialization changes
 * 	               [1996/12/05  20:03:57  hayes]
 * 
 * Revision 1.3.2.5  1996/11/29  17:43:24  hayes
 * 	               Split init_mac() into init_mac1() and init_mac2() since
 * 	               all Tigon related things now key off the TX and RX buf base
 * 	               registers.  These must be set up much earlier in the init
 * 	               processing.  Init_mac2() covers the real MAC initialization and
 * 	               link initialization.
 * 	               [1996/11/29  17:36:04  hayes]
 * 
 * Revision 1.3.2.4  1996/11/20  20:04:21  ted
 * 	               Redo include file structuring.
 * 	               [1996/11/20  19:51:23  ted]
 * 
 * Revision 1.3.2.3  1996/11/15  04:24:07  hayes
 * 	               convert to new header file structure.
 * 	               [1996/11/15  04:12:01  hayes]
 * 
 * Revision 1.3.2.2  1996/11/05  21:54:21  hayes
 * 	               Final pre-ODE merge
 * 	               [1996/11/05  21:51:39  hayes]
 * 
 * Revision 1.3.2.1  1996/11/01  18:24:57  ted
 * 	               Get rid of a compiler warning
 * 	               [1996/10/30  00:07:12  ted]
 * 
 * 	               change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	               [1996/10/29  22:06:10  ted]
 * 
 * $EndLog$
 */
/*
 * FILE mac.c
 *
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/mac.c,v $
 * $Revision: 1.1.2.22 $ $Locker:  $
 * $Date: 1998/12/08 02:36:30 $ $Author: shuang $ $State: Exp $
 */

#ifndef __lint
static const char rcsid[] = "$Header: /projects/rcs/sw/ge/./nic/fw2/common/mac.c,v 1.1.2.22 1998/12/08 02:36:30 shuang Exp $";
#endif /* __lint */


#include "alt_def.h"
#include "nic_conf.h"
#include "nic_api.h"
#include "tg.h"

#include "nic.h"
#include "trace.h"
#include "assert.h"
#include "proto.h"
#include "timer.h"
#include "dma.h"
#include "ring.h"
#include "mac.h"
#include "link.h"
#include "link_op.h"
#include "link_if.h"
#include "zconf.h"
#include "mii.h"
#include "profile.h"


/* globals */
U32 link_attn_count = 0;
U32 gig_negotiate_options;
U32 gig_negotiate;
U32 link_errs = 0;
U32 h_mac_rx_attn_stub;

void
init_mac_buf(void)
{
    trp->local_mem_conf.rx_buf_base     = NIC_RX_BUF;
    trp->local_mem_conf.rx_buf_producer = NIC_RX_BUF;
    trp->local_mem_conf.rx_buf_consumer = NIC_RX_BUF;

    trp->local_mem_conf.tx_buf_base     = NIC_TX_BUF;
    trp->local_mem_conf.tx_buf_producer = NIC_TX_BUF;
    trp->local_mem_conf.tx_buf_consumer = NIC_TX_BUF;
    return;
}

void
init_mac2(void)
{
    NIC_TRACE(TRACE_TYPE_MAC, "initMac2", 0x80100,
	0,
	0,
	0,
	0);
    
    /* set up out recognized unicast address */
    trp->mac_control.mac_src_addr = tsp->gen_com.mac_addr; /* struct copy */

    /* initialize MAC randomizer to low 32 bits of MAC addr */
    trp->mac_control.mac_random = *((U32 *)&(tsp->gen_com.mac_addr) + 1);

    /* determine if we have a 10/100 PHY and/or a Gbit PHY */
    probe_phy();

    if (attached_phy & ATTACHED_PHY_IF_GMII) {

	/* We want to adv 1000 speed in case link parter is 1g speed */
	set_mac_link(TIGON_MAC_INDEX, ATTACHED_PHY_IF_MII,
		     tsp->gen_com.tuneParms.link_negotiation);

        active_phy = ATTACHED_PHY_IF_GMII;
	set_mac_link(TIGON_MAC_INDEX, ATTACHED_PHY_IF_GMII,
		     tsp->gen_com.tuneParms.link_negotiation);
	
    } else {
	active_phy = ATTACHED_PHY_IF_SERDES_EXT;
	set_mac_link(TIGON_MAC_INDEX, ATTACHED_PHY_IF_SERDES_EXT,
		     tsp->gen_com.tuneParms.link_negotiation);

	if (attached_phy & ATTACHED_PHY_IF_MII) {
	    active_phy = ATTACHED_PHY_IF_MII;
	    set_mac_link(TIGON_MAC_INDEX, ATTACHED_PHY_IF_MII,
			 LNK_10MB | LNK_100MB | LNK_FULL_DUPLEX |
			 LNK_HALF_DUPLEX | LNK_NEGOTIATE | LNK_ENABLE);
	}
    }

    /* Setup threshold values for Descriptors */
    trp->mac_control.mac_thresholds =
        (NIC_RX_BUF_LO   << TG_MAC_RX_THRESH_BUF_SHIFT) |
        (RX_MD_XON       << TG_MAC_RX_THRESH_DESC_XON_SHIFT) |
        (RX_MD_XOFF      << TG_MAC_RX_THRESH_DESC_XOFF_SHIFT) |
        (NIC_RX_BUF_XON  << TG_MAC_RX_THRESH_BUFF_XON_SHIFT) |
        (NIC_RX_BUF_XOFF << TG_MAC_RX_THRESH_BUFF_XOFF_SHIFT);

    NIC_UTRACE("macThres", 0x80200,
	trp->mac_control.mac_thresholds,
	0,
	0,
	0);

    additional_rx_bits_wanted = 0;
    link_ready = FALSE;

    return;
}

void
h_mac_tx_attn(void)
{
    NIC_UTRACE("#tMacATN", 0x80300,
	trp->mac_control.mac_tx_state,
	0,
	0,
	0);

    nicfp->stats.nicMacTxTotalAttns++;

#if 0 /*XXX*/
    switch(trp->mac_control.mac_tx_state & TG_MAC_TX_ATTN_MASK) {
      case TG_MAC_TX_STATE_XOFF_ATTN:
      case TG_MAC_TX_STATE_XON_ATTN:
      case TG_MAC_TX_STATE_WORD_URUN_ERR:
      case TG_MAC_TX_STATE_WORD_ORUN_ERR:
      case TG_MAC_TX_STATE_AVAIL_URUN_ERR:
      case TG_MAC_TX_STATE_AVAIL_ORUN_ERR:
      default:
    }
#endif /*XXX*/

    RETURN(TG_FW_EVENT_NUM_MAC_TX_ATTN);
}

void
h_mac_rx_attn(void)
{
    U32 rx_state, space, buf_prod, buf_cons, buf_cons_adj;
    S32 nframes;
    tgMacDescr_t *md_prod, *md_cons;
    tgDmaDescr_t *dma_prod, *dma_cons;
    bool dma_entry_found;

    nicfp->stats.nicMacRxTotalAttns++;
    rx_state = trp->mac_control.mac_rx_state;

    NIC_UTRACE("#rMacATN", rx_state,
	trp->local_mem_conf.rx_buf_producer,
	trp->local_mem_conf.rx_buf_consumer,
	trp->local_mem_conf.mac_rx_descr_producer,
	trp->local_mem_conf.mac_rx_descr_consumer);

    /* rx state attns are ordered from most time critical to least */
    if (rx_state & TG_MAC_RX_STATE_BUF_ATTN) {
        nicfp->stats.nicMacRxBufAttns++;

	/* check rx buffer space
	 * it may has been a while before we handle it, or may from timer
	 */
	buf_prod = trp->local_mem_conf.rx_buf_producer;
	buf_cons = trp->local_mem_conf.rx_buf_consumer;

	space = buf_cons - buf_prod;
	if (space > NIC_RX_BUF_SIZE) /* wrap */
	    space += NIC_RX_BUF_SIZE;

	if (!space || (space > NIC_RX_BUF_HI)) { /* enough rx buf */

	    /* mark the last mac rx descriptor to work around
	     * and switch to the MAC6 work-around code
	     */ 
	    nicfp->md_prod_mac6_bug = trp->local_mem_conf.mac_rx_descr_producer;
	    nicfp->peek_mac_rx_stub = nicfp->peek_mac_rx_wa_relocated;

	    NIC_TRACE(TRACE_TYPE_V6_MAC6_BUG, "ckMac6b", 0x80400,
		0,
		0,
		0,
		nicfp->md_prod_mac6_bug);

	    /* clear rx buf attn and continue */
	    trp->mac_control.mac_rx_state =
		(trp->mac_control.mac_rx_state &
		 ~(TG_MAC_RX_ATTN_MASK | TG_MAC_RX_STATE_STOP_NEXT)) |
		TG_MAC_RX_STATE_BUF_ATTN;

	    NIC_TRACE(TRACE_TYPE_V6_MAC6_BUG, "rxBufOk", space,
		trp->local_mem_conf.rx_buf_producer,
		trp->local_mem_conf.rx_buf_consumer,
		trp->local_mem_conf.mac_rx_descr_producer,
		trp->local_mem_conf.mac_rx_descr_consumer);

	} else { /* rx buf full */

	    /* stop reception of new frames, however,
	     * mac may continue rx data till reach the end of current frame
	     * current incoming frame may not have a chance to reach the 
	     * frame end due to rx buf full or hit chip bug MAC6,
	     * and results in a hidden pending completion frame
	     * in the mac rx ring and rx buf
	     */
	    trp->mac_control.mac_rx_state = 
		(trp->mac_control.mac_rx_state & ~TG_MAC_RX_ATTN_MASK) |
		TG_MAC_RX_STATE_STOP_NEXT;

	    /* mark the last mac rx descriptor to work around
	     * and switch to the MAC6 work-around code
	     */ 
	    nicfp->md_prod_mac6_bug = trp->local_mem_conf.mac_rx_descr_producer;
	    nicfp->peek_mac_rx_stub = nicfp->peek_mac_rx_wa_relocated;

	    NIC_TRACE(TRACE_TYPE_V6_MAC6_BUG, "ckMac6a", 0x80500,
		0,
		0,
		0,
		nicfp->md_prod_mac6_bug);

	    /* set up adjusted rx buf consumer */
	    buf_cons_adj = trp->local_mem_conf.rx_buf_consumer;

	    /* search thru write dma queue for a pending dma data
	     * the rx buffer consumer will be updated to be the nic
	     * address of a found dma entry once it is started
	     */
	    dma_prod = trp->host_dma_assist.write_chan_producer;
	    dma_cons = trp->host_dma_assist.write_chan_consumer;
	    dma_entry_found = FALSE;

	    if (dma_prod != dma_cons) {
		do {
		    if (dma_cons->type &
		        (TIGON_TYPE_RECV_DATA | TIGON_TYPE_RECV_DATA_LAST)) {

			nicfp->stats.nicMacRxDmaCleanup++;
			buf_cons_adj = dma_cons->nic_addr;
			dma_entry_found = TRUE;
			break;
		    }
		    dma_cons = get_next_wr_dma_descr(dma_cons);
		} while (dma_cons != dma_prod);
	    }

	    if (dma_entry_found || nicfp->rx_frame_split) {

		/* there are pending data dma's, wait till data dma done */
		NIC_TRACE(TRACE_TYPE_V6_MAC6_BUG, "dmaSplit",
			(dma_entry_found? buf_cons_adj: 0),
		    trp->local_mem_conf.rx_buf_producer,
		    trp->local_mem_conf.rx_buf_consumer,
		    trp->local_mem_conf.mac_rx_descr_producer,
		    trp->local_mem_conf.mac_rx_descr_consumer);

	    } else {

		/* search thru mac rx queue to discard frames */
		buf_prod = trp->local_mem_conf.rx_buf_producer;
		md_prod = trp->local_mem_conf.mac_rx_descr_producer;
		(U32)md_prod = ROUNDDN((U32)md_prod, 8);
		md_cons = trp->local_mem_conf.mac_rx_descr_consumer;

		nframes = md_prod - md_cons;
		if (nframes < 0)  /* wrap */
		    nframes += TG_NUM_MAC_RX_DESCRS;
		
		switch (nframes) {
		case 0: 
		    nicfp->stats.nicMacRxZeroFrameCleanup++;

		    /* WARNING: the following adjustment may cause
		     * an invalid mac descriptor entry at md_prod later
		     * if there is a pending incomplete frame
		     */
		    buf_cons_adj = buf_prod;

		    NIC_TRACE(TRACE_TYPE_V6_MAC6_BUG, "clean0", buf_cons_adj,
			trp->local_mem_conf.rx_buf_producer,
			trp->local_mem_conf.rx_buf_consumer,
			trp->local_mem_conf.mac_rx_descr_producer,
			trp->local_mem_conf.mac_rx_descr_consumer);
		    break;

		case 1: 
		    nicfp->stats.nicMacRxOneFrameCleanup++;
		    buf_cons_adj = md_cons->w0;

		    space = buf_prod - buf_cons_adj;
		    if (space > NIC_RX_BUF_SIZE)
			space += NIC_RX_BUF_SIZE;

		    if (space > 9022) {
			/* discard */
			nicfp->stats.ifInDiscards++;
			trp->local_mem_conf.mac_rx_descr_consumer = md_cons + 1;
			trp->local_mem_conf.mac_rx_descr_ref = md_cons + 1;
			
			/* WARNING: the following adjustment may cause
			 * an invalid mac descriptor entry at md_prod later
			 * if there is a pending incomplete frame
			 */
			buf_cons_adj = buf_prod;
		    }

		    NIC_TRACE(TRACE_TYPE_V6_MAC6_BUG, "clean1", buf_cons_adj,
			trp->local_mem_conf.rx_buf_producer,
			trp->local_mem_conf.rx_buf_consumer,
			trp->local_mem_conf.mac_rx_descr_producer,
			trp->local_mem_conf.mac_rx_descr_consumer);
		    break;

		default:
		    nicfp->stats.nicMacRxMultipleFrameCleanup++;
		    do {
			space = md_cons->w0 - buf_prod;
			if (space > NIC_RX_BUF_SIZE) /* wrap */
			    space += NIC_RX_BUF_SIZE;
			
			if (space > NIC_RX_BUF_HI) {
			    buf_cons_adj = md_cons->w0;
			    break;
			}

			/* discard */
			nicfp->stats.ifInDiscards++;
			trp->local_mem_conf.mac_rx_descr_consumer = md_cons + 1;
			md_cons = trp->local_mem_conf.mac_rx_descr_consumer;
			trp->local_mem_conf.mac_rx_descr_ref = md_cons;

		    } while (md_cons != md_prod);

		    if (md_cons == md_prod) {
			/* adjust with the length of last entry */
			md_cons = (md_cons == &tdp->rx_mac_ring[0])?
				    &tdp->rx_mac_ring[TG_NUM_MAC_RX_DESCRS - 1]:
				    (md_cons - 1);
			buf_cons_adj += (md_cons->w1 & TG_MAC_RX_W1_LEN_MASK);
			RX_WRAP_BUF(buf_cons_adj);
		    }

		    NIC_TRACE(TRACE_TYPE_V6_MAC6_BUG, "clean2", buf_cons_adj,
			trp->local_mem_conf.rx_buf_producer,
			trp->local_mem_conf.rx_buf_consumer,
			trp->local_mem_conf.mac_rx_descr_producer,
			trp->local_mem_conf.mac_rx_descr_consumer);
		    break;
		}
	    }

	    /* adjust rx buf consumer
	     * value used may lag behind the real one if there is a new
	     * data dma started just after we went thru the dma queue
	     * for the adjusted rx buf consumer
	     * no harm done in this case, however
	     */
	    trp->local_mem_conf.rx_buf_consumer = buf_cons_adj;

	    /* stop mac rx for a while if rx buf
	     * still not reach the hi water mark
	     */
	    space = buf_cons_adj - trp->local_mem_conf.rx_buf_producer;
	    if (space > NIC_RX_BUF_SIZE) /* wrap */
		space += NIC_RX_BUF_SIZE;

	    if (!space || (space > NIC_RX_BUF_HI)) {

		/* clear rx buf attn and continue */
		trp->mac_control.mac_rx_state =
		    (trp->mac_control.mac_rx_state &
		     ~(TG_MAC_RX_ATTN_MASK | TG_MAC_RX_STATE_STOP_NEXT)) |
		    TG_MAC_RX_STATE_BUF_ATTN;

		NIC_TRACE(TRACE_TYPE_V6_MAC6_BUG, "cleanOk", space,
		    trp->local_mem_conf.rx_buf_producer,
		    trp->local_mem_conf.rx_buf_consumer,
		    trp->local_mem_conf.mac_rx_descr_producer,
		    trp->local_mem_conf.mac_rx_descr_consumer);
	    } else {

		/* set up timer, stop rx and mask out attn for a while */
		nicfp->stats.nicMacRxTimerCleanup++;
		event_mask &= ~TG_FW_EVENT_MAC_RX_ATTN;

		NIC_TRACE(TRACE_TYPE_V6_MAC6_BUG, "cleanTm", space,
		    trp->local_mem_conf.rx_buf_producer,
		    trp->local_mem_conf.rx_buf_consumer,
		    trp->local_mem_conf.mac_rx_descr_producer,
		    trp->local_mem_conf.mac_rx_descr_consumer);
	    }
	}
        RETURN(TG_FW_EVENT_NUM_MAC_RX_ATTN);
    }

    /* this bit is only set when the link state changes.
     * Once it goes down, it will stay down until link is re-established.
     */
    if (rx_state & TG_MAC_RX_STATE_LINK_STATE_ATTN) {

        /* clear attn and fall through */
        nicfp->stats.nicMacRxLinkAttns++;
        trp->mac_control.mac_rx_state = 
            (trp->mac_control.mac_rx_state & ~TG_MAC_RX_ATTN_MASK) |
            TG_MAC_RX_STATE_LINK_STATE_ATTN;
	
	link_down_notify();
	RETURN(TG_FW_EVENT_NUM_MAC_RX_ATTN);
	
    }

    if (rx_state & TG_MAC_RX_STATE_NOT_SYNCED_ATTN) {
        nicfp->stats.nicMacRxSyncAttns++;
        link_down_notify();

        /* clear attn */
        trp->mac_control.mac_rx_state = 
            (trp->mac_control.mac_rx_state & ~TG_MAC_RX_ATTN_MASK ) |
            TG_MAC_RX_STATE_NOT_SYNCED_ATTN;

        RETURN(TG_FW_EVENT_NUM_MAC_RX_ATTN);
    }

    if (rx_state & TG_MAC_RX_STATE_AUTO_NEG_CHNGD) {

#if defined(REMOTE_ASSERT)
        /* remote assert debugging... */
        if (trp->mac_control.mac_rx_config == CONFIG_BASE_REMOTE_ASSERT) {
            NIC_UTRACE("remAssrt", 0x80500,
		trp->mac_control.mac_rx_config, 
                0,
		0,
		0);
            ASSERT(0);
        }
#endif /* defined(REMOTE_ASSERT) */

        nicfp->stats.nicMacRxConfigAttns++;
        ENABLE_MAC_RX();
        autoneg_8023z(rx_state);

        /* clear attn */
        trp->mac_control.mac_rx_state = 
            (trp->mac_control.mac_rx_state & ~TG_MAC_RX_ATTN_MASK) |
            TG_MAC_RX_STATE_AUTO_NEG_CHNGD;

        RETURN(TG_FW_EVENT_NUM_MAC_RX_ATTN);
    }

    if (rx_state & (TG_MAC_RX_STATE_FIFO_OVERRUN |
		    TG_MAC_RX_STATE_XOFF_ATTN |
		    TG_MAC_RX_STATE_XON_ATTN |
		    TG_MAC_RX_STATE_IND_ATTN)) {
	ASSERT(0);
    }

    RETURN(TG_FW_EVENT_NUM_MAC_RX_ATTN);
}

/* null routine to assist with scratchpad setup */
void h_mac_rx_attn_end(void) {}

void
link_down_notify(void)
{
    struct tg_event event;

    /* link_ready manipulation required by get_mac_link() */
    if (link_ready) {
        link_ready = FALSE;

        /* inform the link state machine that the current active phy 
         * has failed
         */
        active_phy_fail = 1;
        NIC_UTRACE("linkDOWN", 0x80600, 
            trp->mac_control.mac_rx_state,
	    0,
	    0,
	    0);
    }

    tsp->gen_com.linkState &= ~LNK_UP;
    event_mask &= ~TG_FW_EVENT_MAC_RX_ATTN;

    if (nicfp->host_state & TG_HOST_STATE_LINK_RDY_ON) {
        nicfp->host_state &= ~TG_HOST_STATE_LINK_RDY_ON;
        event.TG_EVENT_w0 =
            (TG_EVENT_LINK_STATE_CHANGED << TG_EVENT_EVENT_SHIFT) |
            (TG_EVENT_CODE_LINK_DOWN << TG_EVENT_CODE_SHIFT);
        must_enq_interrupt_event(&event);
        nicfp->stats.nicEventsLinkStateChanged++;
    }
    return;
}
 
void
link_up_notify(void)
{
    U32 lnk_state;
    
    /* link_ready manipulation required by get_mac_link() */
    if (!link_ready) {
        link_ready = TRUE;
        NIC_UTRACE("linkUP", 0x80700, 
            trp->mac_control.mac_rx_state,
	    0,
	    0,
	    0);
    }

    /* host notification done in h_timer() */

    if (is_global_flags(GF_CMD_PROMISC_MODE)) {
        trp->mac_control.mac_rx_state = 
            (trp->mac_control.mac_rx_state & ~TG_MAC_RX_ATTN_MASK) |
            TG_MAC_RX_STATE_ENA_PROMISC;
    }

    if (active_phy == ATTACHED_PHY_IF_MII) {
        lnk_state = tsp->gen_com.fastLinkState =
	  get_mac_link(TIGON_MAC_INDEX, active_phy);
    } else {
        lnk_state = tsp->gen_com.linkState =
	  get_mac_link(TIGON_MAC_INDEX, active_phy);
    }
        
    switch(lnk_state & (LNK_10MB | LNK_100MB | LNK_1000MB)) {
    case LNK_1000MB:
	nicp->gen_info.stats.ifSpeed = nicfp->stats.ifSpeed = 1000000000;
	break;
    case LNK_100MB:
	nicp->gen_info.stats.ifSpeed = nicfp->stats.ifSpeed = 100000000;
	break;
    case LNK_10MB:
	nicp->gen_info.stats.ifSpeed = nicfp->stats.ifSpeed = 10000000;
	break;
    default:
        break;
    }
    return;
}
