/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: mac.h,v $
 * Revision 1.1.2.10  1998/12/08  02:36:33  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:28:00  shuang]
 *
 * Revision 1.1.2.9  1998/09/30  18:50:01  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:40:02  shuang]
 * 
 * 	          bring link code to same level as switch
 * 	          [1997/07/23  02:57:37  hayes]
 * 	[1997/07/23  02:58:07  hayes]
 * 
 * 	          remove UTRACE() from some macros
 * 	          [1997/07/15  21:56:51  ted]
 * 	[1997/07/16  00:20:23  ted]
 * 
 * 	          add link negotiation support
 * 	          [1997/07/11  02:11:17  ted]
 * 	[1997/07/11  02:25:44  ted]
 * 
 * Revision 1.1.2.8  1998/06/23  01:29:38  shuang
 * 	added dynamic nic mem support.
 * 	[1998/06/23  01:21:44  shuang]
 * 
 * Revision 1.1.2.7  1998/04/11  05:15:09  shuang
 * 	     move fast timer to cpu B, add receive coalesed ticks
 * 	     [1998/04/11  04:58:12  shuang]
 * 
 * Revision 1.1.2.6  1998/03/26  01:34:26  shuang
 * 	     cleanup
 * 	     [1998/03/26  01:25:10  shuang]
 * 
 * Revision 1.1.2.5  1998/02/21  21:07:40  hayes
 * 	          fix buf attn threshhold values
 * 	          [1998/02/21  20:44:43  hayes]
 * 
 * 	          first checkin after mongo merge...
 * 	          [1997/11/14  19:11:35  hayes]
 * 
 * 	          checkin after stats restructuring
 * 	          [1997/11/12  19:01:30  hayes]
 * 
 * 	          intermediate checkin
 * 	          [1997/11/12  01:30:38  hayes]
 * 
 * 	          Added define for INValid DESCRiptor
 * 	          [1997/11/10  00:25:18  hayes]
 * 
 * 	          add rx buf cleanup
 * 	          [1997/10/29  03:42:30  hayes]
 * 
 * 	          remove PEEK_MAC_RX() and DEQ_MAC_RX() macros
 * 	          [1997/10/27  23:35:33  hayes]
 * 
 * 	          completed link code restructuring
 * 	          [1997/08/29  22:12:25  hayes]
 * 
 * 	          link code preparation for SC changes
 * 	          [1997/08/26  21:04:04  hayes]
 * 
 * 	          Merge fixes from ge_0_22 and add link negotiation support
 * 	          [1997/07/25  22:32:10  ted]
 * 
 * 	          updated for new gig autonegotiation state machine
 * 	          [1997/04/15  05:44:10  hayes]
 * 
 * 	          Added NIC stats counter to keep track pf TX full ring
 * 	          [1997/03/28  02:02:10  skapur]
 * 
 * 	          misc changes
 * 	          [1997/03/17  08:28:26  skapur]
 * 
 * 	          Remove old TG_LNK_XXX defines.  They have been renamed and
 * 	          moved to tg/common/link.h
 * 	          [1997/03/03  21:28:39  ted]
 * 
 * 	          Merged with changes from 1.1.7.2
 * 	          [1997/03/01  02:49:25  ted]
 * 
 * 	          This version of mac.h mathes with mac.c, checked in earlier. Without this header file, mac.c will not compile. Also, did the merge to include ENQ_MAC_TX modifications as well. The check in of this file was earlier ignored and was breaking the build.
 * 	          [1997/03/01  02:34:36  ted]
 * 
 * 	          Remove DMA corruption workaround, fix 3 subtle bugs
 * 	          [1997/02/28  20:01:42  ted]
 * 
 * 	          Coped this file from 10/100; lots of new literals are added for 10/100/Gig link negotiations, etc.;
 * 	          [1997/02/23  00:20:05  ted]
 * 
 * Revision 1.1.2.4  1998/02/19  01:29:55  suresh
 * 	          TDP correction
 * 	          [1998/02/19  01:19:36  suresh]
 * 
 * Revision 1.1.2.3  1998/02/05  20:22:20  ted
 * 	          fix stats, was using wrong structure
 * 	          [1998/02/05  20:18:29  ted]
 * 
 * Revision 1.1.2.2  1998/01/20  04:39:21  hayes
 * 	          add tigon Rev6 support
 * 	          [1998/01/20  04:37:57  hayes]
 * 
 * Revision 1.1.2.1  1998/01/14  23:25:52  ted
 * 	          rearrange directories
 * 	          [1998/01/14  23:17:47  ted]
 * 
 * Revision 1.1.2.6  1998/01/09  03:15:05  ted
 * 	          Fix Rev 5 Tigon hardware bug workaround for timing case that caused not passing up received packets (found by SGI)
 * 	          [1998/01/09  00:31:43  ted]
 * 
 * Revision 1.1.2.5  1997/12/28  01:21:55  hayes
 * 	          Tigon 2 MAC TX optimizations
 * 	          [1997/12/28  01:21:47  hayes]
 * 
 * Revision 1.1.2.4  1997/12/02  02:53:41  hayes
 * 	          fix TRACE=0 compile bug
 * 	          [1997/12/02  02:53:27  hayes]
 * 
 * Revision 1.1.2.3  1997/11/30  00:57:36  hayes
 * 	          remove dead variables
 * 	          [1997/11/30  00:56:39  hayes]
 * 
 * Revision 1.1.2.2  1997/11/21  04:48:18  hayes
 * 	          release for NIC 3.1 pre-Beta
 * 	          [1997/11/21  04:45:29  hayes]
 * 
 * 	          intermediate checkin
 * 	          [1997/11/20  01:28:33  hayes]
 * 
 * Revision 1.1.21.2  1997/08/29  22:09:26  hayes
 * 	          completed link code restructuring
 * 
 * Revision 1.1.2.3  1996/11/15  04:24:09  hayes
 * 	          convert to new header file structure.
 * 	          [1996/11/15  04:12:14  hayes]
 * 
 * Revision 1.1.2.2  1996/11/05  21:54:22  hayes
 * 	          Final pre-ODE merge
 * 	          [1996/11/05  21:51:43  hayes]
 * 
 * Revision 1.1.2.1  1996/11/01  18:24:58  ted
 * 	          change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	          [1996/10/29  22:07:28  ted]
 * 
 * Revision 1.1.2.3  1996/11/15  04:24:09  hayes
 * 	          convert to new header file structure.
 * 	          [1996/11/15  04:12:14  hayes]
 * 
 * Revision 1.1.2.2  1996/11/05  21:54:22  hayes
 * 	          Final pre-ODE merge
 * 	          [1996/11/05  21:51:43  hayes]
 * 
 * Revision 1.1.2.1  1996/11/01  18:24:58  ted
 * 	          change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	          [1996/10/29  22:07:28  ted]
 * 
 * $EndLog$
 */
/*
 * FILE mac.h
 *
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/mac.h,v $
 * $Revision: 1.1.2.10 $ $Locker:  $
 * $Date: 1998/12/08 02:36:33 $ $Author: shuang $ $State: Exp $
 */

#ifndef _MAC_H_
#define _MAC_H_

#include "trace.h"
#include "assert.h"


#define INV_DESCR               0x01000000

__inline__ static tgMacDescr_t *
get_next_mac_tx_descr(tgMacDescr_t *mdp)
{
    return ((mdp == &tdp->tx_mac_ring[TG_NUM_MAC_TX_DESCRS - 1])?
	    tdp->tx_mac_ring:
	    (mdp + 1));
}

__inline__ static tgMacDescr_t *
get_next_mac_rx_descr(tgMacDescr_t *mdp)
{
    return ((mdp == &tdp->rx_mac_ring[TG_NUM_MAC_RX_DESCRS - 1])?
	    tdp->rx_mac_ring:
	    (mdp + 1));
}

__inline__ static U32
deq_mac_tx(tgMacDescr_t *mdp)
{
    tgMacDescr_t *cons;                                         

    cons = trp->local_mem_conf.mac_tx_descr_consumer;
    if (cons == trp->local_mem_conf.mac_tx_descr_producer) {
        return (ERROR);
    }

    NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_SEND), "txDqMac", 0x85000,
	mdp->w0,
	mdp->w1,
	cons,
	0);

    *mdp = *cons; /* struct copy */
    trp->local_mem_conf.mac_tx_descr_consumer = cons + 1;
    return (OK);
}

__inline__ static void
enq_mac_tx(tgMacDescr_t *mdp)
{
    tgMacDescr_t *next;

    next = trp->local_mem_conf.mac_tx_descr_next;
    ASSERT(next);

    NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_SEND), "txQMac", 0x85100,
	mdp->w0,
	mdp->w1,
	next,
	0);    

    *next = *mdp; /* struct copy */
    trp->local_mem_conf.mac_tx_descr_producer = next + 1;
    return;
}

__inline__ static void
enq_mac_tx_cksum(tgMacDescr_t *mdp)
{
    tgMacDescr_t *next;

    next = trp->local_mem_conf.mac_tx_descr_next;
    ASSERT(next);

    NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_SEND), "txQMac", 0x85200,
	mdp->w0,
	mdp->w1,
	next,
	0);    

    *next = *mdp; /* struct copy */
    trp->local_mem_conf.mac_tx_descr_producer = next + 1;
    nicfp->tx_q_mac_pending_cnt--;
    return;
}


#endif /* _MAC_H_ */
