/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: nic.h $
 * Revision nic_api_12_4_latest_shuang/2 1999/12/09 19:51:57 shuang
 * 	work around Tg2 MAC6 bug only if get rx buf full attention.
 * 
 * Revision api12_3_ted/2 1999/08/12 21:06:47 ted
 * 	Assemblerize header files
 * 
 * Revision sgi_api_12_3_latest_shuang/3 1999/07/14 18:58:51 shuang
 * 	work around MAC6 Tigon2 bug, fix rx buffer full cleanup
 * 
 * Revision nic_api_12_3_latest_shuang/2 1999/05/20 23:41:37 shuang
 * 	minimal rx buf is 128Kb, rx high water mark is half of rx buf
 * 
 * Revision 1.1.2.29  1999/01/16  02:55:41  shuang
 * 	use of mini ring and jumbo ring is optional
 * 	[1999/01/16  02:53:08  shuang]
 *
 * Revision 1.1.2.28  1998/12/21  00:33:13  shuang
 * 	fix - return last send BD
 * 	[1998/12/21  00:09:39  shuang]
 * 
 * Revision 1.1.2.27  1998/12/10  02:55:35  shuang
 * 	Prevent from starving tx side. Fix stats counters.
 * 	[1998/12/10  02:54:35  shuang]
 * 
 * Revision 1.1.2.26  1998/12/08  02:36:37  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:28:11  shuang]
 * 
 * Revision 1.1.2.25  1998/10/14  01:14:28  shuang
 * 	support tx buffer ratio
 * 	[1998/10/14  01:06:59  shuang]
 * 
 * Revision 1.1.2.24  1998/09/30  18:50:09  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:40:19  shuang]
 * 
 * Revision 1.1.2.23  1998/09/17  20:58:06  shuang
 * 	fix enq mac tx hang problem on checksum mode
 * 	[1998/09/17  20:54:23  shuang]
 * 
 * Revision 1.1.2.22  1998/08/27  17:33:11  shuang
 * 	fix defect1370- flow control
 * 	[1998/08/27  17:31:52  shuang]
 * 
 * Revision 1.1.2.21  1998/08/03  16:14:28  hayes
 * 	complete firmware update for NT
 * 	[1998/08/03  16:11:19  hayes]
 * 
 * 	checkin so merge w/ fc can happen
 * 	[1998/07/29  23:15:29  hayes]
 * 
 * 	host based send ring now works off RCB flag
 * 	[1998/07/29  18:31:00  hayes]
 * 
 * 	host based send ring works
 * 	[1998/07/28  03:57:21  hayes]
 * 
 * Revision 1.1.2.20  1998/07/29  22:31:06  shuang
 * 	flow control works
 * 	[1998/07/29  22:29:07  shuang]
 * 
 * 	add feature - flow control support
 * 	[1998/07/29  21:59:48  shuang]
 * 
 * Revision 1.1.2.19  1998/07/29  01:45:41  shuang
 * 	work around hang problem of dma channal between SBUS and X-Chip
 * 	[1998/07/29  01:44:36  shuang]
 * 
 * Revision 1.1.2.18  1998/07/29  00:07:25  shuang
 * 	add feature - send coallescing BD update only
 * 	[1998/07/29  00:02:40  shuang]
 * 
 * Revision 1.1.2.17  1998/06/23  01:29:40  shuang
 * 	added dynamic nic mem support.
 * 	[1998/06/23  01:21:47  shuang]
 * 
 * Revision 1.1.2.16  1998/04/25  03:07:38  shuang
 * 	moved IP_SANITY debug flag info cksum.h
 * 	[1998/04/25  03:01:04  shuang]
 * 
 * Revision 1.1.2.15  1998/04/16  03:50:45  shuang
 * 	flag TX_FRAG_FRAMES_IN_ORDER defaults to 1 now
 * 	[1998/04/16  03:50:02  shuang]
 * 
 * Revision 1.1.2.14  1998/04/15  17:39:12  ted
 * 	Fix refresh stats command
 * 	[1998/04/15  17:31:15  ted]
 * 
 * Revision 1.1.2.13  1998/04/11  05:15:13  shuang
 * 	move fast timer to cpu B, add receive coalesed ticks
 * 	[1998/04/11  04:58:30  shuang]
 * 
 * Revision 1.1.2.12  1998/04/01  22:21:25  hayes
 * 	cleanup structures
 * 	[1998/04/01  01:00:49  hayes]
 * 
 * 	checkpoint
 * 	[1998/03/31  19:56:55  hayes]
 * 
 * 	checkpoint- dma completion avoidance works
 * 	[1998/03/30  07:00:31  hayes]
 * 
 * 	rd DMA event avoidance works
 * 	[1998/03/29  03:12:31  hayes]
 * 
 * 	intermediate checkin
 * 	[1998/03/29  00:18:07  hayes]
 * 
 * Revision 1.1.2.11  1998/04/01  07:32:23  shuang
 * 	add vars mac_descr_w1, frame_len_diff, partial_recv_count...
 * 	[1998/04/01  07:19:54  shuang]
 * 
 * Revision 1.1.2.10  1998/04/01  00:35:02  skapur
 * 	Adding Vlan Tag Insertion/Deletion support
 * 	[1998/04/01  00:33:05  skapur]
 * 
 * Revision 1.1.2.9  1998/03/26  01:34:29  shuang
 * 	cleanup
 * 	[1998/03/26  01:25:23  shuang]
 * 
 * Revision 1.1.2.8  1998/02/24  17:41:48  shuang
 * 	     Modify multicast structures
 * 	     [1998/02/24  17:34:21  shuang]
 * 
 * Revision 1.1.2.7  1998/02/19  01:29:57  suresh
 * 	     add bool for software DMA assist and fix up TDP
 * 	     [1998/02/19  01:22:55  suresh]
 * 
 * 	     intermediate checksum offload state
 * 	     [1998/02/15  01:50:43  suresh]
 * 
 * Revision 1.1.2.6  1998/02/11  19:11:22  ted
 * 	     Support for functionalization of q_dma_to_(nic,host), and removal of unnecessary variables
 * 	     [1998/02/11  18:46:30  ted]
 * 
 * 	     intermediate checkin for merge
 * 	     [1998/02/10  00:28:31  ted]
 * 
 * Revision 1.1.2.5  1998/02/05  20:22:21  ted
 * 	     add support for dontFragJumbos flag
 * 	     [1998/02/05  20:17:49  ted]
 * 
 * Revision 1.1.2.4  1998/02/04  23:04:49  ted
 * 	     Move the DMA reference entry by entry when transferring receive BDs to the NIC, PR 813
 * 	     [1998/02/04  23:04:08  ted]
 * 
 * Revision 1.1.2.3  1998/01/29  18:40:22  ted
 * 	     Add multicast mode support and move some things into the fast nic structure
 * 	     [1998/01/29  18:28:59  ted]
 * 
 * Revision 1.1.2.2  1998/01/29  02:32:57  skapur
 * 	     Check in for Rev6 checksum support
 * 	     [1998/01/29  02:27:13  skapur]
 * 
 * Revision 1.1.2.1  1998/01/15  00:03:06  ted
 * 	     fix up ODE weirdness
 * 	     [1998/01/15  00:02:52  ted]
 * 
 * 	     remove REPLICATION_STUFF and rearrange directories
 * 	     [1998/01/14  23:15:00  ted]
 * 
 * Revision 1.1.2.6  1998/01/14  19:56:03  skapur
 * 	     Merged with changes from 1.1.2.5
 * 	     [1998/01/14  19:55:46  skapur]
 * 
 * 	     Checksum off-load support for Tigon2. This is an intermediate checkin; more work needs to be done
 * 	     [1998/01/14  19:26:40  skapur]
 * 
 * Revision 1.1.2.5  1998/01/04  02:42:30  hayes
 * 	     remove TRACE flag :-)
 * 	     [1998/01/04  02:42:14  hayes]
 * 
 * Revision 1.1.2.4  1998/01/04  02:24:13  hayes
 * 	     disassociate active LEd from link bringup
 * 	     [1998/01/04  02:23:06  hayes]
 * 
 * Revision 1.1.2.3  1997/12/29  19:02:08  hayes
 * 	     remove replicate stuff
 * 	     [1997/12/29  18:59:41  hayes]
 * 
 * Revision 1.1.2.2  1997/12/02  02:53:43  hayes
 * 	     fix TRACE=0 compile bug
 * 	     [1997/12/02  02:53:29  hayes]
 * 
 * Revision 1.1.2.1  1997/11/19  19:23:15  ted
 * 	     Support Tigon and Tigon2 in the same driver
 * 	     [1997/11/19  19:19:38  ted]
 * 
 * Revision 1.21.7.24  1997/11/19  00:13:43  ted
 * 	     merged from release 2.0
 * 	     [1997/11/19  00:10:42  ted]
 * 
 * Revision 1.21.7.23  1997/11/14  19:16:33  hayes
 * 	     first checkin after mongo merge...
 * 	     [1997/11/14  19:11:38  hayes]
 * 
 * 	     checkin after stats restructuring
 * 	     [1997/11/12  19:01:33  hayes]
 * 
 * 	     intermediate checkin
 * 	     [1997/11/12  01:30:43  hayes]
 * 
 * Revision 1.21.7.22  1997/11/11  19:05:59  hayes
 * 	     restructuring for scratchpad use
 * 	     [1997/11/11  19:04:24  hayes]
 * 
 * Revision 1.21.7.21  1997/11/10  00:27:29  hayes
 * 	     force alignment of event ring to fix Tigon2 DMA bug
 * 	     [1997/11/10  00:22:28  hayes]
 * 
 * Revision 1.21.7.20  1997/10/21  01:35:34  skapur
 * 	     Merge of release 2.0 and ge_latest branch
 * 	     [1997/10/21  01:31:08  skapur]
 * 
 * Revision 1.21.7.19  1997/09/16  21:46:30  ted
 * 	     Change from MIB-II V1 (RFC 1213) to MIB-II V2 (RFC 1573)
 * 	     [1997/09/16  21:42:05  ted]
 * 
 * Revision 1.21.7.18  1997/09/16  02:47:05  skapur
 * 	     Merged with changes from 1.21.7.17
 * 	     [1997/09/16  02:46:56  skapur]
 * 
 * 	     Took out ifdefed code; removed variables which are no more needed in NIC structure; misc. clean up
 * 	     [1997/09/16  02:45:24  skapur]
 * 
 * Revision 1.21.7.17  1997/09/11  18:05:45  ted
 * 	     Mixing vlans, jumbos, and redundancy works, plus small fixes from release 1.1 and better NIC tracing
 * 	     [1997/09/11  17:59:44  ted]
 * 
 * Revision 1.21.7.16  1997/09/05  21:01:15  hayes
 * 	     Add WZERRO() macro
 * 	     Remove DMA_TO_HOST() macro
 * 	     [1997/09/05  21:00:54  hayes]
 * 
 * Revision 1.21.7.15  1997/08/23  17:40:07  skapur
 * 	     Solaris jumbogram support
 * 	     [1997/08/23  17:37:58  skapur]
 * 
 * Revision 1.21.7.14  1997/07/29  03:41:01  ted
 * 	     Add OP1 LED support
 * 	     [1997/07/29  03:40:08  ted]
 * 
 * Revision 1.21.7.13  1997/07/25  22:59:06  ted
 * 	     Merge fixes from ge_0_22 and add link negotiation support
 * 	     [1997/07/25  22:32:14  ted]
 * 
 * Revision 1.21.21.1  1997/07/11  02:25:45  ted
 * 	     add link negotiation support
 * 	     [1997/07/11  02:11:21  ted]
 * 
 * Revision 1.21.7.12  1997/05/21  03:35:44  ted
 * 	     New data structure for bug 189 fix.
 * 	     New define for new link configuration.
 * 	     Change unsigned to signed to make comparison easier.
 * 	     [1997/05/21  03:14:09  ted]
 * 
 * Revision 1.21.7.11  1997/04/25  18:48:49  ted
 * 	     Add new variables to keep track of modified recv bd dmaing
 * 	     [1997/04/25  18:36:04  ted]
 * 
 * Revision 1.21.7.10  1997/04/22  18:04:04  alfan
 * 	     Fixed Send Events; If any of the send completion events are missed, they will be re-tired until it makes it
 * 	     [1997/04/22  18:02:40  alfan]
 * 
 * Revision 1.21.7.9  1997/04/19  04:04:36  alfan
 * 	     Fixed Stats timer count problem
 * 	     [1997/04/19  04:01:21  alfan]
 * 
 * Revision 1.21.7.8  1997/04/11  23:36:22  ted
 * 	     Added macros to Turn ON/OFF LEDs + Remove TRACES + PROFILES
 * 	     [1997/04/11  23:19:17  ted]
 * 
 * Revision 1.21.7.7  1997/03/28  02:47:05  skapur
 * 	     Added REPLICATION_STUFF here; Took out most of unwanted elements out of nic structure
 * 	     [1997/03/28  02:05:33  skapur]
 * 
 * Revision 1.21.7.6  1997/03/24  08:34:24  ted
 * 	     New event ring handling, producer lives in host memory, misc. other more modest changes
 * 	     [1997/03/24  08:27:58  ted]
 * 
 * Revision 1.21.7.5  1997/03/21  04:15:37  skapur
 * 	     Performance enhancement for Recv and Send; lots changes; almost complete overhaul of the entire firmware
 * 	     [1997/03/21  04:08:17  skapur]
 * 
 * Revision 1.21.7.4  1997/03/17  08:47:45  skapur
 * 	     integration changes...
 * 	     [1997/03/16  03:01:48  skapur]
 * 
 * Revision 1.21.7.3  1997/02/17  00:07:59  ted
 * 	     Working with Rev 1, Chip Express boards (funky internal loopback sync algorith)
 * 	     [1997/02/16  23:44:19  ted]
 * 
 * Revision 1.21.7.2  1997/02/08  05:01:24  skapur
 * 	     Took out simulator flag; For real HW Fun
 * 	     [1997/02/08  05:01:07  skapur]
 * 
 * Revision 1.21.7.1  1997/02/07  23:20:02  skapur
 * 	     replacing TGP/NIC/TDP  with reg variables
 * 	     [1997/02/07  23:19:24  skapur]
 * 
 * Revision 1.21.2.4  1996/11/29  17:43:20  hayes
 * 	     Remove NDP and related structures.
 * 	     Add #define NO_HOST for improved simulation abilities.
 * 	     [1996/11/29  17:39:17  hayes]
 * 
 * Revision 1.21.2.3  1996/11/15  04:24:11  hayes
 * 	     convert to new header file structure.
 * 	     [1996/11/15  04:12:15  hayes]
 * 
 * Revision 1.21.2.2  1996/11/05  21:54:25  hayes
 * 	     Final pre-ODE merge
 * 	     [1996/11/05  21:51:55  hayes]
 * 
 * Revision 1.21.2.1  1996/11/01  18:25:02  ted
 * 	     change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	     [1996/10/29  22:06:24  ted]
 * 
 * $EndLog$
 */
/*
 * FILE nic.h
 *
 * COPYRIGHT (c) Essential Communication Corp. 1995
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/nic.h,v $
 * $Revision: 1.1.2.29 $ $Locker:  $
 * $Date: 1999/01/16 02:55:41 $ $Author: shuang $ $State: Exp $
 */

#ifndef _NIC_H_
#define _NIC_H_


/* build definitions - should be 1 */
#define TX_FRAG_FRAMES_IN_ORDER 1 /* fragged frames are xmit'ed in order */

/* build flags for misc benchmark - defaults to 0 */
#define	USE_U32_STAT_CNT	0 /* use 32 bit stat counter instead of 64 */
#define	RESET_COAL_TICK_CNT	0 /* reset coal tick counter on dma coal also */

/* special internal firmware trace on working around Tigon2 chip bug MAC6 */
#define TRACE_TYPE_V6_MAC6_BUG	(0x08000000)

/* left only for some common modules for switch and nic */
#undef	TRP
#undef	TDP
#undef	NIC
#undef	TSP
#define TRP	trp
#define	TDP	tdp
#define NIC	nicp
#define tsp	((tg_shmem_t *)(0))
#define	TSP	tsp

/* error code */
#undef  OK
#undef  ERROR
#define OK	(1)
#define ERROR	(0)

/* TG_CPU_A and TG_CPU_B only defined for TIGON_REV5 in tg.h */
#if !defined(TG_CPU_A)
#define TG_CPU_A        0
#define TG_CPU_B        1
#endif /* !defined(TG_CPU_A) */

/* software events that fw2 registers */
#define TG_FW_EVENT_DO_INTERRUPT        (TG_FW_EVENT_SW12)
#define TG_FW_EVENT_DO_RX_PROD          (TG_FW_EVENT_SW11)
#define TG_FW_EVENT_DO_TX_CONS          (TG_FW_EVENT_SW10)
#define TG_FW_EVENT_RECV_MINI_BD        (TG_FW_EVENT_SW7)
#define TG_FW_EVENT_RECV_BD             (TG_FW_EVENT_SW6)
#define TG_FW_EVENT_RECV_JUMBO_BD       (TG_FW_EVENT_SW5)
#define TG_FW_EVENT_UPDATE_LEDS         (TG_FW_EVENT_SW4)
#define TG_FW_EVENT_SEND_BD             (TG_FW_EVENT_SW3)
#define TG_FW_EVENT_SEND_DATA           (TG_FW_EVENT_SW2)

#define TG_FW_EVENT_NUM_DO_INTERRUPT    (TG_FW_EVENT_NUM_SW12)
#define TG_FW_EVENT_NUM_DO_RX_PROD      (TG_FW_EVENT_NUM_SW11)
#define TG_FW_EVENT_NUM_DO_TX_CONS      (TG_FW_EVENT_NUM_SW10)
#define TG_FW_EVENT_NUM_RECV_MINI_BD    (TG_FW_EVENT_NUM_SW7)
#define TG_FW_EVENT_NUM_RECV_BD         (TG_FW_EVENT_NUM_SW6)
#define TG_FW_EVENT_NUM_RECV_JUMBO_BD   (TG_FW_EVENT_NUM_SW5)
#define TG_FW_EVENT_NUM_UPDATE_LEDS     (TG_FW_EVENT_NUM_SW4)
#define TG_FW_EVENT_NUM_SEND_BD         (TG_FW_EVENT_NUM_SW3)
#define TG_FW_EVENT_NUM_SEND_DATA       (TG_FW_EVENT_NUM_SW2)

/* Ether type with Vlan Tag support is 0x8100 per 802.1q */
#define ETHER_TYPE_WITH_VLAN_TAG    0x8100
#define INSERT_VLAN_TAG_FLAG        0x80000000

/* nic memory size */
#define NIC_MEM_256K	0x040000
#define	NIC_MEM_512K	0x080000
#define	NIC_MEM_1M	0x100000
#define	NIC_MEM_2M	0x200000
#define	NIC_MEM_4M	0x400000
#define	NIC_MEM_8M	0x800000

/* length of common frame header
 * = 14 (eth hdr) + 4 (vlan tag) + 60 (ip hdr) + 20 (tcp hdr) +
 *   3 (LLC hdr) + 5 (SNAP hdr) = 8 (wrap)
 * = 114
 */
#define	TX_BUF_RED_ZONE_SIZE	(128) /* power of 2 */

/* constants that depend on the size of nic memory */
#define	MIN_MAC_BUF_SIZE (64 * 1024) /* must be multiple of 4KB */
#define	MIN_TX_MAC_BUF	 (MIN_MAC_BUF_SIZE)
#define	MIN_RX_MAC_BUF	 (2 * MIN_MAC_BUF_SIZE)
#define	TX_BUF_SIZE_1M	 (1024 * 64)
#define	RX_BUF_SIZE_1M	 (1024 * (512 - 64))
#define RX_BUF_HI_1M	 (1024 * 200)
#define RX_BUF_HI_512K	 (1024 * 200)
#define RX_BUF_HI_256K	 (1024 * 40)
#define RX_MD_XOFF       ((15 - 7) >> 3) /* 15 descriptors */
#define RX_MD_XON        ((31 - 7) >> 3) /* 31 descriptors */
#define RX_BUF_LO_1M	 ((((1024 * 9)  / 128) - 7) >> 3) /*  9KB */
#define RX_BUF_LO_512K	 ((((1024 * 9)  / 128) - 7) >> 3) /*  9KB */
#define RX_BUF_LO_256K	 ((((1024 * 9)  / 128) - 7) >> 3) /*  9KB */
#define RX_BUF_XOFF_1M	 ((((1024 * 23) / 128) - 7) >> 3) /* 23KB */
#define RX_BUF_XOFF_512K ((((1024 * 23) / 128) - 7) >> 3) /* 23KB */
#define RX_BUF_XOFF_256K ((((1024 * 23) / 128) - 7) >> 3) /* 23KB */
#define RX_BUF_XON_1M	 ((((1024 * 32) / 128) - 7) >> 3) /* 32KB */
#define RX_BUF_XON_512K  ((((1024 * 32) / 128) - 7) >> 3) /* 32KB */
#define RX_BUF_XON_256K  ((((1024 * 32) / 128) - 7) >> 3) /* 32KB */

#if defined(_IN_ASM_)
/* dma_ref_ring structure */
#define dma_ref_ring_type     0
#define dma_ref_ring_count    4

/* typedef struct nic_fast { */
    /* identify myself */
#define nicfp_cpu_id                    0

    /* host state */
#define nicfp_host_state                4
#define TG_HOST_STATE_STACK_UP          0x0001
#define TG_HOST_STATE_LINK_RDY_ON       0x0002

    /* dma states */
#define nicfp_dma_to_host_data_state    8
#define nicfp_dma_to_host_bd_state      12
#define nicfp_dma_to_nic_data_state     16
#define nicfp_dma_to_nic_bd_state       20

    /* event ring */
#define nicfp_event_prod_index          24

    /* send ring */
#define nicfp_send_ring_ptr             28
#define nicfp_send_prod_index           32
#define nicfp_send_prod_index_qd        36
#define nicfp_send_prod_index_avail     40
#define nicfp_send_cons_index           44
#define nicfp_send_cons_index_qd        48
#define nicfp_send_coal_bds             52
#define nicfp_send_coal_index           56
#define nicfp_cmd_event                 60
#define nicfp_send_ring_size            64

    /* recv ring */
#define nicfp_recv_return_prod_index    68
#define nicfp_recv_prod_index           72
#define nicfp_recv_prod_index_qd        76
#define nicfp_recv_prod_index_avail     80
#define nicfp_recv_cons_index           84
#define nicfp_recv_jumbo_prod_index     88
#define nicfp_recv_jumbo_prod_index_qd  92
#define nicfp_recv_jumbo_prod_index_avail 96
#define nicfp_recv_jumbo_cons_index    100
#define nicfp_recv_mini_prod_index     104
#define nicfp_recv_mini_prod_index_qd  108
#define nicfp_recv_mini_prod_index_avail 112
#define nicfp_recv_mini_cons_index     116
#define nicfp_recv_coal_bds            120
#define nicfp_recv_coal_index          124
#define nicfp_recv_std_buf_size        128
#define nicfp_recv_mini_buf_size       132
#define nicfp_ret_ring_size_mask       136

    /* transmit */
#define nicfp_tx_q_mac_pending_cnt     140
#define nicfp_tx_buf_space             144
#define nicfp_tx_buf_prod              148
#define nicfp_tx_buf_next              152
#define nicfp_tx_frame_len             156
#define nicfp_tx_first_frame_start     160
#define nicfp_tx_first_frame_index     164
#define nicfp_tx_first_frame_eth_hlen  168
#define nicfp_tx_insert_vlan_tag       172
#define nicfp_tx_vlan_tag              176
#define nicfp_tx_queue_depth           180

    /* receive */
#define nicfp_rx_frame_start           184
#define nicfp_rx_partial_count         188
#define nicfp_rx_frame_len             192
#define nicfp_rx_frame_cur             196
#define nicfp_rx_md_w1                 200
#define nicfp_rx_frame_len_diff        204

    /* led */
#define nicfp_data_led_timer           208
#define nicfp_disabled_led_timer       212
#define nicfp_led_tick_count           216
#define nicfp_led_ticks                220

    /* disassociate active phy LED from link bringup state machines  */
#define nicfp_active_phy_ticks         224
#define nicfp_active_phy_ticks_count   228
#define nicfp_active_phy_display       232

    /* interrupt and dma completion avoidance */
#define nicfp_rd_dma_ref_prod          236
#define nicfp_rd_dma_ref_cons          240
#define nicfp_wr_dma_ref_prod          244
#define nicfp_wr_dma_ref_cons          248
#define nicfp_wr_dma_free_cnt          252

#define RD_DMA_MAX_DELTA	(TG_DMA_ASSIST_DESCRS/4)
#define WR_DMA_MAX_DELTA	(TG_DMA_ASSIST_DESCRS/4)

    /* rx packet filtering */
#define nicfp_mc_free_list             256
#define nicfp_mc_table_depth           260

    /* pointer to runtime functions */
#define nicfp_q_dma_to_nic_stub        264
#define nicfp_q_dma_to_host_stub       268
#define nicfp_cksum16_stub             272
#define nicfp_peek_mac_rx_stub         276
#define nicfp_deq_mac_rx_stub          280

    /* sbus-xchip lockup work around */
#define nicfp_prev_rddp_cons           284
#define nicfp_prev_wddp_cons           288
#define nicfp_prev_raddr               292
#define nicfp_prev_waddr               296

    /* dynamic nic memory */
#define nicfp_tx_buf                   300
#define nicfp_rx_buf                   304
#define nicfp_tx_buf_red               308
#define nicfp_tx_buf_size              312
#define nicfp_rx_buf_size              316
#define nicfp_rx_buf_attn_hi           320
#define nicfp_rx_buf_attn_lo           324
#define nicfp_rx_buf_xon               328
#define nicfp_rx_buf_xoff              332

    /* host pointers */
#define nicfp_event_producer_ptr_hi    336
#define nicfp_event_producer_ptr_lo    340
#define nicfp_recv_return_producer_ptr_hi 344
#define nicfp_recv_return_producer_ptr_lo 348
#define nicfp_send_consumer_ptr_hi     352
#define nicfp_send_consumer_ptr_lo     356
#define nicfp_event_ring_ptr_hi        360
#define nicfp_event_ring_ptr_lo        364
#define nicfp_send_ring_ptr_hi         368
#define nicfp_send_ring_ptr_lo         372
#define nicfp_recv_ring_ptr_hi         376
#define nicfp_recv_ring_ptr_lo         380
#define nicfp_recv_jumbo_ring_ptr_hi   384
#define nicfp_recv_jumbo_ring_ptr_lo   388
#define nicfp_recv_mini_ring_ptr_hi    392
#define nicfp_recv_mini_ring_ptr_lo    396
#define nicfp_recv_ret_ring_ptr_hi     400
#define nicfp_recv_ret_ring_ptr_lo     404
#define nicfp_stats2_ptr_hi            408
#define nicfp_stats2_ptr_lo            412

    /* stats */
#define nicfp_stats_tick_count         416
#define nicfp_led_ifHCInOctets         420
#define nicfp_led_ifHCOutOctets        424
  
    /*
     * MAC stats, taken from RFC 1643, ethernet-like MIB
     */
#define stats_dot3StatsAlignmentErrors 432
#define stats_dot3StatsFCSErrors       436
#define stats_dot3StatsSingleCollisionFrames 440
#define stats_dot3StatsMultipleCollisionFrames 444
#define stats_dot3StatsSQETestErrors   448
#define stats_dot3StatsDeferredTransmissions 452
#define stats_dot3StatsLateCollisions  456
#define stats_dot3StatsExcessiveCollisions 460
#define stats_dot3StatsInternalMacTransmitErrors 464
#define stats_dot3StatsCarrierSenseErrors 468
#define stats_dot3StatsFrameTooLongs      472
#define stats_dot3StatsInternalMacReceiveErrors 476
    /*
     * interface stats, taken from RFC 1213, MIB-II, interfaces group
     */
#define stats_ifIndex                  480
#define stats_ifType                   484
#define stats_ifMtu                    488
#define stats_ifSpeed                  492
#define stats_ifAdminStatus            496
#define IF_ADMIN_STATUS_UP	1
#define IF_ADMIN_STATUS_DOWN	2
#define IF_ADMIN_STATUS_TESTING	3

#define stats_ifOperStatus             500
#define IF_OPER_STATUS_UP	1
#define IF_OPER_STATUS_DOWN	2
#define IF_OPER_STATUS_TESTING	3
#define IF_OPER_STATUS_UNKNOWN	4
#define IF_OPER_STATUS_DORMANT	5

#define stats_ifLastChange             504
#define stats_ifInDiscards             508
#define stats_ifInErrors               512
#define stats_ifInUnknownProtos        516
#define stats_ifOutDiscards            520
#define stats_ifOutErrors              524
#define stats_ifOutQLen                528
#define stats_ifPhysAddress_hi         532
#define stats_ifPhysAddress_lo         536
#define stats_ifDescr                  540

    /*
     * more interface stats, taken from RFC 1573, MIB-IIupdate, 
     * interfaces group
     */
#define stats_ifHCInOctets_hi          576
#define stats_ifHCInOctets_lo          580
#define stats_ifHCInUcastPkts_hi       584
#define stats_ifHCInUcastPkts_lo       588
#define stats_ifHCInMulticastPkts_hi   592
#define stats_ifHCInMulticastPkts_lo   596
#define stats_ifHCInBroadcastPkts_hi   600
#define stats_ifHCInBroadcastPkts_lo   604
#define stats_ifHCOutOctets_hi         608
#define stats_ifHCOutOctets_lo         612
#define stats_ifHCOutUcastPkts_hi      616
#define stats_ifHCOutUcastPkts_lo      620
#define stats_ifHCOutMulticastPkts_hi  624
#define stats_ifHCOutMulticastPkts_lo  628
#define stats_ifHCOutBroadcastPkts_hi  632
#define stats_ifHCOutBroadcastPkts_lo  636
#define stats_ifLinkUpDownTrapEnable   640
#define stats_ifHighSpeed              644
#define stats_ifPromiscuousMode        648
#define stats_ifConnectorPresent       652
    /* 
     * Host Commands
     */
#define stats_nicCmdsHostState         656
#define stats_nicCmdsFDRFiltering      660
#define stats_nicCmdsSetRecvProdIndex  664
#define stats_nicCmdsUpdateGencommStats 668
#define stats_nicCmdsResetJumboRing    672
#define stats_nicCmdsAddMCastAddr      676
#define stats_nicCmdsDelMCastAddr      680
#define stats_nicCmdsSetPromiscMode    684
#define stats_nicCmdsLinkNegotiate     688
#define stats_nicCmdsSetMACAddr        692
#define stats_nicCmdsClearProfile      696
#define stats_nicCmdsSetMulticastMode  700
#define stats_nicCmdsClearStats        704
#define stats_nicCmdsSetRecvJumboProdIndex 708
#define stats_nicCmdsSetRecvMiniProdIndex 712
#define stats_nicCmdsRefreshStats      716
#define stats_nicCmdsUnknown           720

    /* 
     * NIC Events
     */
#define stats_nicEventsNICFirmwareOperational 724
#define stats_nicEventsStatsUpdated    728
#define stats_nicEventsLinkStateChanged 732
#define stats_nicEventsError           736
#define stats_nicEventsMCastListUpdated 740
#define stats_nicEventsResetJumboRing  744

    /*
     * Ring manipulation
     */
#define stats_nicRingSetSendProdIndex  748
#define stats_nicRingSetSendConsIndex  752
#define stats_nicRingSetRecvReturnProdIndex 756

    /* 
     * Interrupts
     */
#define stats_nicInterrupts            760
#define stats_nicAvoidedInterrupts     764

    /* 
     * BD Coalessing Thresholds
     */
#define stats_nicEventThresholdHit     768
#define stats_nicSendThresholdHit      772
#define stats_nicRecvThresholdHit      776

    /* 
     * DMA Attentions
     */
#define stats_nicDmaRdOverrun          780
#define stats_nicDmaRdUnderrun         784
#define stats_nicDmaWrOverrun          788
#define stats_nicDmaWrUnderrun         792
#define stats_nicDmaWrMasterAborts     796
#define stats_nicDmaRdMasterAborts     800

    /* 
     * NIC Resources
     */
#define stats_nicDmaWriteRingFull      804
#define stats_nicDmaReadRingFull       808
#define stats_nicEventRingFull         812
#define stats_nicEventProducerRingFull 816
#define stats_nicTxMacDescrRingFull    820
#define stats_nicOutOfTxBufSpaceFrameRetry 824
#define stats_nicNoMoreWrDMADescriptors 828
#define stats_nicNoMoreRxBDs           832
#define stats_nicNoSpaceInReturnRing   836
#define stats_nicSendBDs               840
#define stats_nicRecvBDs               844
#define stats_nicJumboRecvBDs          848
#define stats_nicMiniRecvBDs           852
#define stats_nicTotalRecvBDs          856
#define stats_nicTotalSendBDs          860
#define stats_nicJumboSpillOver        864
#define stats_nicSbusHangCleared       868
#define stats_nicEnqEventDelayed       872

    /*
     * Stats from MAC rx completion
     */
#define stats_nicMacRxLateColls        876
#define stats_nicMacRxLinkLostDuringPkt 880
#define stats_nicMacRxPhyDecodeErr     884
#define stats_nicMacRxMacAbort         888
#define stats_nicMacRxTruncNoResources 892
    /*
     * Stats from the mac_stats area
     */
#define stats_nicMacRxDropUla          896
#define stats_nicMacRxDropMcast        900
#define stats_nicMacRxFlowControl      904
#define stats_nicMacRxDropSpace        908
#define stats_nicMacRxColls            912
    
    /*
     * MAC RX Attentions
     */
#define stats_nicMacRxTotalAttns       916
#define stats_nicMacRxLinkAttns        920
#define stats_nicMacRxSyncAttns        924
#define stats_nicMacRxConfigAttns      928
#define stats_nicMacReset              932
#define stats_nicMacRxBufDescrAttns    936
#define stats_nicMacRxBufAttns         940
#define stats_nicMacRxZeroFrameCleanup 944
#define stats_nicMacRxOneFrameCleanup  948
#define stats_nicMacRxMultipleFrameCleanup 952
#define stats_nicMacRxTimerCleanup     956
#define stats_nicMacRxDmaCleanup       960

    /*
     * Stats from the mac_stats area
     */
#define stats_nicMacTxCollisionHistogram 964
    /*
     * MAC TX Attentions
     */
#define stats_nicMacTxTotalAttns       1024
	     
    /*
     * NIC Profile
     */
#define stats_nicProfile               1028

    /*
     * pad out to 1024 bytes...
     */
#define stats_res                      1156

    /* bools */
#define nicfp_led_bits                 1456
#define nicfp_rx_buf_space_monitor     1457
#define nicfp_send_cons_index_to_host_need_intr 1458
#define nicfp_tx_new_frame             1459
#define nicfp_tx_ip_frag               1460
#define nicfp_rx_frame_split           1461
#define nicfp_refresh_stats_active     1462
#define nicfp_stats_to_host_pending    1463

/* } nic_fast_t; */

/* typedef struct nic { */

    /* my mac address */
#define nicp_mac_addr                  0

    /* bds for dma gen_info from host */
#define nicp_gen_info_bd               8

    /* gen_info from host */
#define nicp_gen_info                  40

    /* link state machine */
#define nicp_bringup_link_state        1208	  /* 0x4b8 */

    /* event ring */
#define nicp_event_ring                1216	  /* 0x4c0 */

    /* recv ring */
#define nicp_ext_recv_jumbo_ring       3296	  /* 0xce0 */
#define nicp_recv_jumbo_ring           19648	  /* 0x4cc0 */
#define nicp_recv_mini_ring            27840	  /* 0x6cc0 */
#define nicp_recv_ring                 60608	  /* 0xecc0 */

    /* interrupt and dma completion avoidabce */
#define DMA_REF_RING_ENTRIES	(TG_DMA_ASSIST_DESCRS + 1)

#define nicp_rd_dma_ref_ring           11456      /* plus 0x10000 (0x2cc0) */
#define nicp_wr_dma_ref_ring           11976      /* plus 0x10000 (0x2ec8) */
#define nicp_wr_dma_ref_ring_end       12488      /* plus 0x10000 (0x30c8) */

    /* multicast */
#define nicp_mc_table                  12496	  /* plus 0x10000 (0x30d0) */
#define nicp_mc_bucket                 13520	  /* plus 0x10000 (0x34d0) */
#define nicp_mc_filter_saved           14552	  /* plus 0x10000 (0x38d8) */
#define MCAST_HASH_ENTRY_MASK   0x1f
#define MCAST_HASH_GROUP_MASK   0x60
#define MCAST_HASH_GROUP_SHIFT  5
#define MCAST_MAX_HASH_GROUP    3       /* max hash group number */

    /* tx ip fragments */
#define nicp_tx_queue                  14568	  /* plus 0x10000 (0x38e8) */

    /* shared by both CPUs, do NOT move to the nicfp structure */
#define nicp_send_coal_tick_count      16616	  /* plus 0x10000 (0x40e8) */
#define nicp_recv_coal_tick_count      16620	  /* plus 0x10000 (0x40ec) */
#define nicp_send_cons_index_to_host_pending 16624 /* plus 0x10000 (0x40f0) */
#define nicp_send_cons_index_to_host_pending_no_intr 16625 /* plus 0x10000 */
#define nicp_recv_ret_prod_index_to_host_pendingg 16626 /* plus 0x10000  */
#define nicp_interrupt_host_pending    16627	  /* plus 0x10000 */
#define nicp_sw_dma_assist             16628	  /* plus 0x10000 */

/* } nic_t; */

/* declare global registers */
#define REG_ZERO         $0		/* always 0 */
#define RETURN_CODE      $2		/* return code */
#define REG_EVENT_MASK   $20		/* Events allowed to trigger */
#define	REG_GLOBAL_FLAGS $22		/* configuration flags */
#define REG_NICP         $23		/* rings and stuff */
#define REG_NICFP        $26		/* will in int scratchpad */
#define REG_TDP          $27		/* tigon data area */
#define REG_TRP          $28		/* All hardware registers */
#define REG_LINK         $31		/* link register */

/* parm registers for subroutines */
#define parm1		$4
#define parm2		$5
#define parm3		$6
#define parm4		$7

/* bit definitions of global_flags in cpu register */
#define GF_RCB_TX_TCP_UDP_CKSUM         (1 <<  0)
#define GF_RCB_TX_IP_CKSUM              (1 <<  1)
#define GF_RCB_TX_NO_PSEUDO_HDR_CKSUM   (1 <<  2)
#define GF_RCB_TX_VLAN_ASSIST           (1 <<  3)
#define GF_RCB_TX_COAL_UPDATE_ONLY      (1 <<  4)
#define GF_RCB_TX_HOST_RING             (1 <<  5)
#define GF_RCB_TX_IEEE_SNAP_CKSUM	(1 <<  6)
#define GF_RCB_RX_TCP_UDP_CKSUM         (1 <<  7)
#define GF_RCB_RX_IP_CKSUM              (1 <<  8)
#define GF_RCB_RX_NO_PSEUDO_HDR_CKSUM   (1 <<  9)
#define GF_RCB_RX_VLAN_ASSIST           (1 << 10)
#define GF_RCB_RX_IEEE_SNAP_CKSUM	(1 << 11)
#define GF_RCB_USE_EXT_RECV_BD		(1 << 12)
#define GF_RCB_JUMBO_RING_DISABLED	(1 << 13)
#define GF_RCB_MINI_RING_DISABLED	(1 << 14)
#define GF_CMD_JUMBO_FRAMES_ENABLED     (1 << 15)
#define GF_CMD_MINI_FRAMES_ENABLED      (1 << 16)
#define GF_CMD_MCAST_MODE     		(1 << 17)
#define GF_CMD_PROMISC_MODE    		(1 << 18)
#define GF_MOD_DONT_FRAG_JUMBOS         (1 << 19)
#define GF_MOD_DONT_RUPT_SENDS          (1 << 20)
#define GF_MOD_DONT_RUPT_RECVS          (1 << 21)
#define GF_MOD_DONT_RUPT_EVENTS         (1 << 22)
#define GF_MOD_ALLOW_BAD_FRAMES		(1 << 23)
#define GF_MOD_INCLUDE_CRC		(1 << 24)
#define GF_MOD_SBUS    			(1 << 25)
#define GF_MOD_USE_OP1_LEDS		(1 << 26)
#define GF_VAR_MAC_RX_COMP_SUSP		(1 << 27)

#else /* defined(_IN_ASM_) */

#if (DYNAMIC_NIC_MEM)
#define	NIC_RX_BUF_SIZE	(nicfp->rx_buf_size)
#define	NIC_RX_BUF_END	(TG_DATA_END)
#define	NIC_RX_BUF	(nicfp->rx_buf)
#define	NIC_RX_BUF_HI	(nicfp->rx_buf_attn_hi)
#define	NIC_RX_BUF_LO	(nicfp->rx_buf_attn_lo)
#define	NIC_RX_BUF_XON	(nicfp->rx_buf_xon)
#define	NIC_RX_BUF_XOFF	(nicfp->rx_buf_xoff)
#define	NIC_TX_BUF_SIZE	(nicfp->tx_buf_size)
#define	NIC_TX_BUF_END	NIC_RX_BUF
#define	NIC_TX_BUF_RED	(nicfp->tx_buf_red)
#define	NIC_TX_BUF	(nicfp->tx_buf)
#else (DYNAMIC_NIC_MEM)
#define	NIC_RX_BUF_SIZE	TG_RX_BUF_SIZE
#define	NIC_RX_BUF_END	(TG_DATA_END)
#define	NIC_RX_BUF	(NIC_RX_BUF_END - NIC_RX_BUF_SIZE)
#define	NIC_RX_BUF_HI	RX_BUF_HI_1M
#define	NIC_RX_BUF_LO	RX_BUF_LO_1M
#define	NIC_RX_BUF_XON	RX_BUF_XON_1M
#define	NIC_RX_BUF_XOFF	RX_BUF_XOFF_1M
#define	NIC_TX_BUF_SIZE	TG_TX_BUF_SIZE
#define	NIC_TX_BUF_END	NIC_RX_BUF
#define	NIC_TX_BUF_RED	(NIC_RX_BUF - TX_BUF_RED_ZONE_SIZE)
#define	NIC_TX_BUF	(NIC_TX_BUF_END - NIC_TX_BUF_SIZE)
#endif /* (DYNAMIC_NIC_MEM) */

/* mac buffer macros */
#define WRAP_BUF(b, bend, bsize) \
        if ((U32)(b) >= (U32)(bend)) { (U32)(b) -= (U32)(bsize); }
#define TX_WRAP_BUF(buf)	WRAP_BUF(buf, NIC_TX_BUF_END, NIC_TX_BUF_SIZE)
#define RX_WRAP_BUF(buf)	WRAP_BUF(buf, NIC_RX_BUF_END, NIC_RX_BUF_SIZE)
#define ROUNDUP(x, n)		(((x) + (n) - 1) & ~((n) - 1))
#define ROUNDDN(x, n)		((x) & ~((n) - 1))

typedef struct dma_ref_ring {
    U32 type;
    U32 count;
} dma_ref_ring_t;

typedef struct nic_fast {

    /* identify myself */
    U32 cpu_id;

    /* host state */
    U32 host_state;
#define TG_HOST_STATE_STACK_UP          0x0001
#define TG_HOST_STATE_LINK_RDY_ON       0x0002

    /* dma states */
    U32 dma_to_host_data_state;
    U32 dma_to_host_bd_state;
    U32 dma_to_nic_data_state;
    U32 dma_to_nic_bd_state;

    /* event ring */
    U32 event_prod_index;

    /* send ring */
    struct send_bd *send_ring;		/* send ring from host */
    U32 send_prod_index;		/* update after mbox command */
    U32 send_prod_index_queued; 	/* update after enqueue send bds */
    U32 send_prod_index_avail;		/* update after done dma send bds */
    U32 send_cons_index;		/* update after done dma send data */
    U32 send_cons_index_queued; 	/* update after enqueue send data */
    U32 send_coal_bds;
    S32 send_coal_index;
    U32 send_cmd_event;
    U32 send_ring_size;

    /* recv ring */
    U32 recv_return_prod_index;
    U32 recv_prod_index;		/* update after mbox command */
    U32 recv_prod_index_queued;		/* update after enqueue recv bds */
    U32 recv_prod_index_avail;		/* update after done dma recv bds */
    U32 recv_cons_index;
    U32 recv_jumbo_prod_index;		/* update after mbox command */
    U32 recv_jumbo_prod_index_queued;	/* update after enqueue recv bds */
    U32 recv_jumbo_prod_index_avail;	/* update after done dma recv bds */
    U32 recv_jumbo_cons_index;
    U32 recv_mini_prod_index;		/* update after mbox command */
    U32 recv_mini_prod_index_queued;	/* update after enqueue recv bds */
    U32 recv_mini_prod_index_avail;	/* update after done dma recv bds */
    U32 recv_mini_cons_index;
    U32 recv_coal_bds;
    U32 recv_coal_index;
    U32 recv_std_buf_size;
    U32 recv_mini_buf_size;
    U32 recv_ret_ring_size_mask;	/* size of recv return ring minus 1 */

    /* transmit */
    U32 tx_q_mac_pending_cnt;
    S32 tx_buf_space;
    U32 tx_buf_prod;
    U32 tx_buf_next;
    U32 tx_frame_len;
    U32 tx_first_frame_start;
    U32 tx_first_frame_index; /* debug only */
    U32 tx_first_frame_eth_hlen;
    U32 tx_insert_vlan_tag;
    U32 tx_vlan_tag;
    U32 tx_queue_depth;

    /* receive */
    U32 rx_frame_start;
    U32 rx_partial_count;
    U32 rx_frame_len;
    U32 rx_frame_cur;
    U32 rx_md_w1;
    U32 rx_frame_len_diff;

    /* led */
    S32 data_led_timer;
    S32 disabled_led_timer;
    U32 led_tick_count;
    U32 led_ticks;

    /* disassociate active phy LED from link bringup state machines  */
    S32 active_phy_ticks;
    S32 active_phy_tick_count;
    S32 active_phy_display;

    /* interrupt and dma completion avoidance */
    dma_ref_ring_t *rd_dma_ref_prod;
    dma_ref_ring_t *rd_dma_ref_cons;
    dma_ref_ring_t *wr_dma_ref_prod;
    dma_ref_ring_t *wr_dma_ref_cons;
    S32 wr_dma_free_cnt;

#define RD_DMA_MAX_DELTA	(TG_DMA_ASSIST_DESCRS/4)
#define WR_DMA_MAX_DELTA	(TG_DMA_ASSIST_DESCRS/4)

    /* rx packet filtering */
    U32 mc_free_list;
    S32 mc_table_depth;

    /* pointer to runtime functions */
    U32  (*q_dma_to_nic_stub)(tg_hostaddr_t, U32, U32, U32, U32, U32);
    U32  (*q_dma_to_host_stub)(tg_hostaddr_t, U32, U32, U32, U32, U32);
    U16  (*cksum16_stub)(U16 *, U32, U32, U32);
    tgMacDescr_t *(*peek_mac_rx_stub)(void);
    void (*deq_mac_rx_stub)(void);

    /* MAC6 bug work-around */
    tgMacDescr_t *md_prod_mac6_bug;
    tgMacDescr_t *(*peek_mac_rx_relocated)(void);
    tgMacDescr_t *(*peek_mac_rx_wa_relocated)(void);

    /* sbus-xchip lockup work around */
    struct tg_dma_descr *prev_rddp_cons;
    struct tg_dma_descr *prev_wddp_cons;
    U32 prev_raddr;
    U32 prev_waddr;

#if (DYNAMIC_NIC_MEM)
    /* dynamic nic memory */
    U32 tx_buf;		/* tx buf starting address */
    U32 rx_buf;		/* rx buf starting address */
    U32 tx_buf_red;	/* tx buf red zone */
    U32 tx_buf_size;
    U32 rx_buf_size;
    U32 rx_buf_attn_hi;	/* high water mark */
    U32 rx_buf_attn_lo; /*  low water mark */
    U32 rx_buf_xon;	/* high water mark */
    U32 rx_buf_xoff;	/*  low water mark */
#endif /* (DYNAMIC_NIC_MEM) */

    /* host pointers */
    U64 align_0[0]; /* force double word alignment */
    tg_hostaddr_t event_producer_ptr;
    tg_hostaddr_t recv_return_producer_ptr;
    tg_hostaddr_t send_consumer_ptr;
    tg_hostaddr_t event_ring_ptr;
    tg_hostaddr_t send_ring_ptr;
    tg_hostaddr_t recv_ring_ptr;
    tg_hostaddr_t recv_jumbo_ring_ptr;
    tg_hostaddr_t recv_mini_ring_ptr;
    tg_hostaddr_t recv_ret_ring_ptr;
    tg_hostaddr_t stats2_ptr;

    /* stats */
    U32 stats_tick_count;
    U32 led_ifHCInOctets;
    U32 led_ifHCOutOctets;
#if (USE_U32_STAT_CNT)
    U32 u64_stat_ticks;
    U32 ifHCInOctets;
    U32 ifHCOutOctets;
    U32 ifHCOutPkts[4];
#define UCAST_PKT_INDEX	((BD_FLAG_UCAST_PKT >> BD_FLAG_PKT_SHIFT) & 3)
#define MCAST_PKT_INDEX	((BD_FLAG_MCAST_PKT >> BD_FLAG_PKT_SHIFT) & 3)
#define BCAST_PKT_INDEX	((BD_FLAG_BCAST_PKT >> BD_FLAG_PKT_SHIFT) & 3)
#endif /* (USE_U32_STAT_CNT) */
    struct tg_stats stats;

    /* bools */
    U8 led_bits;
    bool send_cons_index_to_host_need_intr;
    bool tx_new_frame;
    bool tx_ip_frag;
    bool rx_frame_split;
    bool refresh_stats_active;
    bool stats_to_host_pending;

} nic_fast_t;

typedef struct nic {

    /* my mac address */
    struct macaddr mac_addr;

    /* bds for dma gen_info from host */
    struct recv_bd gen_info_bd;

    /* gen_info from host */
    struct tg_gen_info gen_info;

    /* link state machine */
    U32 bringup_link_state;

    /* event ring */
    U64 align_0[0]; /* force double word alignment */
    struct tg_event event_ring[EVENT_RING_ENTRIES];

    /* recv ring */
    struct ext_recv_bd ext_recv_jumbo_ring[RECV_JUMBO_RING_ENTRIES];
    struct recv_bd     recv_jumbo_ring[RECV_JUMBO_RING_ENTRIES];
    struct recv_bd     recv_mini_ring[RECV_MINI_RING_ENTRIES];
    struct recv_bd     recv_ring[RECV_STANDARD_RING_ENTRIES];

    /* interrupt and dma completion avoidabce */
#define DMA_REF_RING_ENTRIES	(TG_DMA_ASSIST_DESCRS + 1)
    dma_ref_ring_t rd_dma_ref_ring[DMA_REF_RING_ENTRIES];
    dma_ref_ring_t wr_dma_ref_ring[DMA_REF_RING_ENTRIES];

    /* multicast */
    struct macaddr mc_table[MAX_MCAST_ENTRIES];
    struct macaddr mc_bucket[MAX_MCAST_ENTRIES + 1];
    U32 mc_filter_saved[4];  /* state of the hardware to restore later */
#define MCAST_HASH_ENTRY_MASK   0x1f
#define MCAST_HASH_GROUP_MASK   0x60
#define MCAST_HASH_GROUP_SHIFT  5
#define MCAST_MAX_HASH_GROUP    3       /* max hash group number */
#define MCAST_FILTER(hash) \
            (MCAST_MAX_HASH_GROUP - \
            ((hash & MCAST_HASH_GROUP_MASK) >> MCAST_HASH_GROUP_SHIFT))

    /* tx ip fragments */
    tgMacDescr_t tx_queue[TG_NUM_MAC_TX_DESCRS];

#if (FW_PROFILE)
    U32 profile_last;
#endif /* (FW_PROFILE) */

    /* shared by both CPUs, do NOT move to the nicfp structure */
    VOLATILE U32 send_coal_tick_count;
    VOLATILE U32 recv_coal_tick_count;
    VOLATILE bool send_cons_index_to_host_pending;
    VOLATILE bool send_cons_index_to_host_pending_no_intr;
    VOLATILE bool recv_ret_prod_index_to_host_pending;
    VOLATILE bool interrupt_host_pending;
    bool sw_dma_assist;

} nic_t;

/* declare global registers */
register U32 event_mask    __asm__ ("$20");	/* Events allowed to trigger */
register U32 global_flags  __asm__ ("$22");	/* configuration flags */
register nic_t *nicp       __asm__ ("$23");	/* rings and stuff */
register nic_fast_t *nicfp __asm__ ("$26");	/* will in int scratchpad */
register tg_data_t *tdp    __asm__ ("$27");	/* tigon data area */
register tg_regs_t *trp    __asm__ ("$28");	/* All hardware registers */

/* bit definitions of global_flags in cpu register */
#define GF_RCB_TX_TCP_UDP_CKSUM         (1 <<  0)
#define GF_RCB_TX_IP_CKSUM              (1 <<  1)
#define GF_RCB_TX_NO_PSEUDO_HDR_CKSUM   (1 <<  2)
#define GF_RCB_TX_VLAN_ASSIST           (1 <<  3)
#define GF_RCB_TX_COAL_UPDATE_ONLY      (1 <<  4)
#define GF_RCB_TX_HOST_RING             (1 <<  5)
#define GF_RCB_TX_IEEE_SNAP_CKSUM	(1 <<  6)
#define GF_RCB_RX_TCP_UDP_CKSUM         (1 <<  7)
#define GF_RCB_RX_IP_CKSUM              (1 <<  8)
#define GF_RCB_RX_NO_PSEUDO_HDR_CKSUM   (1 <<  9)
#define GF_RCB_RX_VLAN_ASSIST           (1 << 10)
#define GF_RCB_RX_IEEE_SNAP_CKSUM	(1 << 11)
#define GF_RCB_USE_EXT_RECV_BD		(1 << 12)
#define GF_RCB_JUMBO_RING_DISABLED	(1 << 13)
#define GF_RCB_MINI_RING_DISABLED	(1 << 14)
#define GF_CMD_JUMBO_FRAMES_ENABLED     (1 << 15)
#define GF_CMD_MINI_FRAMES_ENABLED      (1 << 16)
#define GF_CMD_MCAST_MODE     		(1 << 17)
#define GF_CMD_PROMISC_MODE    		(1 << 18)
#define GF_MOD_DONT_FRAG_JUMBOS         (1 << 19)
#define GF_MOD_DONT_RUPT_SENDS          (1 << 20)
#define GF_MOD_DONT_RUPT_RECVS          (1 << 21)
#define GF_MOD_DONT_RUPT_EVENTS         (1 << 22)
#define GF_MOD_ALLOW_BAD_FRAMES		(1 << 23)
#define GF_MOD_INCLUDE_CRC		(1 << 24)
#define GF_MOD_SBUS    			(1 << 25)
#define GF_MOD_USE_OP1_LEDS		(1 << 26)
#define GF_VAR_MAC_RX_COMP_SUSP		(1 << 27)

__inline__ static void
set_global_flags(U32 flags)
{
    global_flags |= flags;
    return;
}

__inline__ static void
clear_global_flags(U32 flags)
{
    global_flags &= ~flags;
    return;
}

__inline__ static U32
is_global_flags(U32 flags)
{
    return (global_flags & flags);
}

/*
 * memory functions
 *
 * These functions do not handle unaligned addresses or odd lengths, because
 * our processor work in word quantities only.  At some point this needs
 * to be made smarter to use doubles where appropriate.
 */

#define WZERO(d, nwords)        wzero((U32 *)d, (U32)nwords)
#define WCOPY(s, d, nwords)     wcopy((U32 *)s, (U32 *)d, (U32)nwords)

/*
 * Copies Wlen words starting at Wsrc to Wdest.
 * Wsrc and Wdest must be word aligned
 * Wlen must be the number of WORDS in block (sizeof(foo)>>2).
 */
__inline__ static void
wcopy(U32 *Wsrc, U32 *Wdest, U32 Wlen)
{
    register U32 *src = Wsrc;
    register U32 *dst = Wdest;
    U32 *dst_end = dst + Wlen;

    while (dst < dst_end) {
	*dst++ = *src++;
    }
    return;
}

/*
 * Zeroes Wlen words starting at Wptr.
 * Wptr must be word aligned
 * Wlen must be the number of WORDS in block (sizeof(foo)>>2).
 */
__inline__ static void
wzero(U32 *Wptr, U32 Wlen)
{
    register U32 *ptr = Wptr;
    U32 *ptr_end = ptr + Wlen;

    while (ptr < ptr_end) {
	*ptr++ = 0;
    }
    return;
}
#endif /* defined(_IN_ASM_) */

#endif /* _NIC_H_ */
