/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: recv.c $
 * Revision nic_api_12_4_latest_shuang/11 2000/02/12 18:50:05 shuang
 * 	fix incorrect stats nicRecvBDs and nicJumboRecvBDs.
 * 	honor TG_CMD_CODE_STACK_UP and TG_CMD_CODE_STACK_DOWN commands.
 * 
 * Revision nic_api_12_4_latest_shuang/8 1999/12/09 19:51:57 shuang
 * 	work around Tg2 MAC6 bug only if get rx buf full attention.
 * 
 * Revision nic_api_12_4_latest_shuang/6 1999/11/10 17:41:33 shuang
 * 	restore deq_mac_rx asm; undo h_recv_bd_ready asm
 * 
 * Revision nic_api_12_4_latest_shuang/4 1999/08/27 02:14:14 shuang
 * 	undo deq_mac_rx asm
 * 
 * Revision nic_api_12_4_latest_shuang/2 1999/08/27 01:36:23 shuang
 * 	jumbo frame length will be 0x8870
 * 
 * Revision api12_3_ted/3 1999/08/12 21:07:40 ted
 * 	Add assembler routines
 * 
 * Revision sgi_api_12_3_latest_shuang/3 1999/07/14 18:58:51 shuang
 * 	work around MAC6 Tigon2 bug, fix rx buffer full cleanup
 * 
 * Revision nic_api_12_3_latest_shuang/2 1999/03/29 23:50:41 shuang
 * 	fixed incorrect recv tcp/udp cksum using extended recv BD
 * 
 * Revision 1.1.2.53  1999/01/16  02:55:43  shuang
 * 	use of mini ring and jumbo ring is optional
 * 	[1999/01/16  02:53:12  shuang]
 *
 * Revision 1.1.2.52  1999/01/11  18:57:24  shuang
 * 	the ipProtocol and ipFragOffset fields on rx buf may wrap
 * 	if fw is in 802.3 AND vlan mode.
 * 	[1999/01/11  18:56:43  shuang]
 * 
 * Revision 1.1.2.51  1998/12/21  20:39:47  shuang
 * 	fix wr dma free count
 * 	[1998/12/21  20:39:26  shuang]
 * 
 * Revision 1.1.2.50  1998/12/21  00:33:15  shuang
 * 	fix - return last send BD
 * 	[1998/12/21  00:18:31  shuang]
 * 
 * Revision 1.1.2.49  1998/12/12  00:35:53  shuang
 * 	add debug code to check recv ring
 * 	[1998/12/12  00:32:56  shuang]
 * 
 * Revision 1.1.2.48  1998/12/10  17:06:16  shuang
 * 	revise comments
 * 	[1998/12/10  17:05:34  shuang]
 * 
 * Revision 1.1.2.47  1998/12/10  02:55:37  shuang
 * 	Prevent from starving tx side. Fix stats counters.
 * 	[1998/12/10  02:54:41  shuang]
 * 
 * Revision 1.1.2.46  1998/12/08  02:36:41  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:28:19  shuang]
 * 
 * Revision 1.1.2.45  1998/11/19  01:14:21  shuang
 * 	     tcp/udp cksum ok for extra padding frames on rx split jumbo
 * 	     [1998/11/19  01:13:35  shuang]
 * 
 * Revision 1.1.2.44  1998/11/18  16:59:07  shuang
 * 	     fix tcp/udp frames that have extra padding on cksum mode
 * 	     [1998/11/18  16:58:31  shuang]
 * 
 * Revision 1.1.2.43  1998/11/17  19:10:34  shuang
 * 	     remove faulty assertion
 * 	     [1998/11/17  19:10:10  shuang]
 * 
 * Revision 1.1.2.42  1998/11/16  22:41:34  shuang
 * 	     remove faulty assertion
 * 	     [1998/11/16  22:40:51  shuang]
 * 
 * Revision 1.1.2.41  1998/11/11  00:55:26  shuang
 * 	     handle tcp/udp frames that have extra padding on cksum mode
 * 	     [1998/11/11  00:54:21  shuang]
 * 
 * Revision 1.1.2.40  1998/11/09  22:17:35  shuang
 * 	     dont set clear cksum bit when dma data for ext recv bd
 * 	     [1998/11/09  22:16:07  shuang]
 * 
 * Revision 1.1.2.39  1998/11/06  23:58:53  shuang
 * 	     dont return whole ext recv bd, return only common portion
 * 	     [1998/11/06  23:58:20  shuang]
 * 
 * Revision 1.1.2.38  1998/09/30  18:50:13  shuang
 * 	     change for new 12.3 API
 * 	     [1998/09/30  18:40:27  shuang]
 * 
 * Revision 1.1.2.37  1998/08/27  23:22:59  shuang
 * 	     add new mcast and mbox cmds, intr masks, obsolete recv ret cons idx
 * 	     [1998/08/27  23:20:50  shuang]
 * 
 * Revision 1.1.2.36  1998/08/03  16:13:30  hayes
 * 	     play with rcs...  :-(
 * 	     [1998/08/03  16:12:52  hayes]
 * 
 * 	     complete firmware update for NT
 * 	     [1998/08/03  16:11:23  hayes]
 * 
 * Revision 1.1.2.35  1998/07/29  00:07:27  shuang
 * 	     add feature - send coallescing BD update only
 * 	     [1998/07/29  00:02:44  shuang]
 * 
 * Revision 1.1.2.34  1998/07/21  21:37:47  shuang
 * 	     set BD_FLAG_IP_CKSUM and BD_FLAG_TCP_UDP_CKSUM flags also after done
 * 	     Rx cksum calculation.
 * 	     [1998/07/21  21:36:18  shuang]
 * 
 * Revision 1.1.2.33  1998/06/23  01:29:43  shuang
 * 	     added dynamic nic mem support.
 * 	     [1998/06/23  01:21:57  shuang]
 * 
 * Revision 1.1.2.32  1998/06/11  23:30:24  shuang
 * 	          Changed switch-case statements that generate jump tables to if-else.
 * 	          [1998/06/11  23:27:37  shuang]
 * 
 * Revision 1.1.2.31  1998/05/22  03:05:14  ted
 * 	          Fix double increment of ifInDiscards on frameTooLong
 * 	          [1998/05/22  03:04:30  ted]
 * 
 * Revision 1.1.2.30  1998/05/15  20:14:32  shuang
 * 	          add pass ip id to host for cksum debugging.
 * 	          [1998/05/15  20:10:31  shuang]
 * 
 * Revision 1.1.2.29  1998/05/09  02:09:55  shuang
 * 	          Correct check for receive delayed stats in cksum mode.
 * 	          Fix recv frame less than 60 bytes hw problem with vlan tag in cksum mode.
 * 	          [1998/05/09  02:07:45  shuang]
 * 
 * Revision 1.1.2.28  1998/04/30  20:01:55  ted
 * 	          Correct check for receive delayed stats
 * 	          [1998/04/30  20:01:39  ted]
 * 
 * Revision 1.1.2.27  1998/04/25  03:07:40  shuang
 * 	          removed IP_FRAG_SUPPORTdef, moved crc_cksum func into cksum.h
 * 	          [1998/04/25  03:04:28  shuang]
 * 
 * Revision 1.1.2.26  1998/04/24  00:00:42  hayes
 * 	          added const to rcsid definitions to eliminate compiler warnings
 * 	          [1998/04/23  23:58:57  hayes]
 * 
 * Revision 1.1.2.25  1998/04/17  00:58:58  shuang
 * 	          Fix cksum offload on receive
 * 	          [1998/04/17  00:58:14  shuang]
 * 
 * Revision 1.1.2.24  1998/04/15  21:51:56  ted
 * 	          Fix up nicSendThresholdHit and nicRecvThresholdHit stats
 * 	          [1998/04/15  21:51:36  ted]
 * 
 * Revision 1.1.2.23  1998/04/11  05:15:17  shuang
 * 	          move fast timer to cpu B, add receive coalesed ticks
 * 	          [1998/04/11  04:58:41  shuang]
 * 
 * Revision 1.1.2.22  1998/04/06  17:38:30  hayes
 * 	          code generation optimizations
 * 	          [1998/04/06  17:38:02  hayes]
 * 
 * Revision 1.1.2.21  1998/04/01  22:21:27  hayes
 * 	          merge cleanup
 * 	          [1998/04/01  22:18:58  hayes]
 * 
 * 	          change ref ring prod/cons to head/tail
 * 	          [1998/04/01  01:01:38  hayes]
 * 
 * 	          checkpoint
 * 	          [1998/03/31  19:56:58  hayes]
 * 
 * 	          checkpoint- dma completion avoidance works
 * 	          [1998/03/30  07:00:33  hayes]
 * 
 * 	          wr DMA completion avoidance works
 * 	          [1998/03/29  04:20:47  hayes]
 * 
 * 	          intermediate checkin
 * 	          [1998/03/29  00:18:10  hayes]
 * 
 * Revision 1.1.2.20  1998/04/01  07:32:26  shuang
 * 	          add features: pass (CRC, bad, partial) frame, timer tick to 5us...
 * 	          [1998/04/01  07:27:25  shuang]
 * 
 * Revision 1.1.2.19  1998/04/01  00:35:00  skapur
 * 	          Adding Vlan Tag Insertion/Deletion support
 * 	          [1998/04/01  00:32:50  skapur]
 * 
 * Revision 1.1.2.18  1998/03/26  01:34:30  shuang
 * 	          cleanup
 * 	          [1998/03/26  01:25:30  shuang]
 * 
 * Revision 1.1.2.17  1998/03/08  02:07:32  ted
 * 	               Fix up usage because of changed sendbd structure (for VLAN assist)
 * 	               [1998/03/08  02:05:52  ted]
 * 
 * Revision 1.1.2.16  1998/03/04  22:28:36  hayes
 * 	               Merged with changes from 1.1.2.15
 * 	               [1998/03/04  22:28:23  hayes]
 * 
 * 	               enable scratchpad usage, uncluding stats refreesh
 * 	               [1998/03/04  22:19:49  hayes]
 * 
 * Revision 1.1.2.15  1998/03/03  22:23:37  ted
 * 	               Implement faster dual homing code
 * 	               [1998/03/03  22:22:40  ted]
 * 
 * 	               Remove extra traces
 * 	               [1998/02/28  03:20:54  ted]
 * 
 * Revision 1.1.2.14  1998/02/24  23:55:50  shuang
 * 	               Create mcast.h mcast.c for multicast functions
 * 	               [1998/02/24  23:51:39  shuang]
 * 
 * Revision 1.1.2.13  1998/02/24  17:41:51  shuang
 * 	               Modulize multicast functions
 * 	               [1998/02/24  17:35:40  shuang]
 * 
 * Revision 1.1.2.12  1998/02/19  01:30:01  suresh
 * 	               Merged with changes from 1.1.2.11
 * 	               [1998/02/19  01:29:22  suresh]
 * 
 * 	               Checksum offload for Rev 6 support
 * 	               [1998/02/19  01:24:12  suresh]
 * 
 * 	               intermediate checksum offload state
 * 	               [1998/02/15  01:50:41  suresh]
 * 
 * Revision 1.1.2.11  1998/02/12  04:37:56  ted
 * 	               Filtering is NOT discarding, remove broken gotos
 * 	               [1998/02/12  04:37:27  ted]
 * 
 * Revision 1.1.2.10  1998/02/11  19:11:23  ted
 * 	               First try at v5/v6 checksum offload
 * 	               [1998/02/11  18:47:57  ted]
 * 
 * 	               intermediate checkin for merge
 * 	               [1998/02/10  00:28:35  ted]
 * 
 * Revision 1.1.2.9  1998/02/09  17:18:07  hayes
 * 	               convert Q_DMA_XXX macros to functions
 * 	               [1998/02/09  17:17:01  hayes]
 * 
 * Revision 1.1.2.8  1998/02/05  20:22:22  ted
 * 	               Add support for dontFragJumbos flag; small clean ups; fix stats, was using wrong structure; fix split jumbo frame over standard bds
 * 	               [1998/02/05  20:20:33  ted]
 * 
 * Revision 1.1.2.7  1998/02/04  23:04:50  ted
 * 	               Move the DMA reference entry by entry when transferring receive BDs to the NIC, PR 813
 * 	               [1998/02/04  23:04:12  ted]
 * 
 * Revision 1.1.2.6  1998/01/29  19:37:18  ted
 * 	               Fix broken syntax
 * 	               [1998/01/29  19:23:36  ted]
 * 
 * Revision 1.1.2.5  1998/01/29  18:40:24  ted
 * 	               Add multicast mode support and move some things into the fast nic structure
 * 	               [1998/01/29  18:29:04  ted]
 * 
 * Revision 1.1.2.4  1998/01/29  02:32:58  skapur
 * 	               Check in for Rev6 checksum support
 * 	               [1998/01/29  02:27:17  skapur]
 * 
 * 	               merging for Tigon2 rev6 Checksum off load support
 * 	               [1998/01/28  23:15:42  skapur]
 * 
 * 	               To merge with John's code
 * 	               [1998/01/20  19:23:24  skapur]
 * 
 * Revision 1.1.2.3  1998/01/21  21:34:22  hayes
 * 	               fix invalid descriptor bug
 * 	               [1998/01/21  21:28:34  hayes]
 * 
 * 	               cleanup mac rx reference
 * 	               [1998/01/21  01:58:35  hayes]
 * 
 * 	               fix peek_mac_rx_v5() to not over increment reference.
 * 	               [1998/01/20  21:44:11  hayes]
 * 
 * Revision 1.1.2.2  1998/01/20  04:39:23  hayes
 * 	               add tigon Rev6 support
 * 	               [1998/01/20  04:38:03  hayes]
 * 
 * Revision 1.1.2.1  1998/01/15  00:03:07  ted
 * 	               fix up ODE weirdness
 * 	               [1998/01/15  00:02:54  ted]
 * 
 * 	               remove REPLICATION_STUFF, add fix for not allowing TX during busy RX for checksum offload code, remove checksum offload statistics generation in software
 * 	               [1998/01/14  23:15:52  ted]
 * 
 * Revision 1.1.2.7  1998/01/14  20:00:23  skapur
 * 	               Merged with changes from 1.1.2.6
 * 	               [1998/01/14  20:00:10  skapur]
 * 
 * 	               Checksum off-load support for Tigon2. This is an intermediate checkin; more work needs to be done
 * 	               [1998/01/14  19:27:06  skapur]
 * 
 * Revision 1.1.2.6  1998/01/09  03:15:05  ted
 * 	               Clean up tracing slightly
 * 	               [1998/01/09  00:31:56  ted]
 * 
 * Revision 1.1.2.5  1998/01/07  04:23:31  ted
 * 	               Turn off receive processing when out of BDs and turn it back on when new BDs are available
 * 	               [1998/01/07  04:21:14  ted]
 * 
 * Revision 1.1.2.4  1997/12/15  20:02:55  ted
 * 	               Remove TIGON_REV4 support and fix multicast filtering with promiscuous mode bug
 * 	               [1997/12/15  19:35:56  ted]
 * 
 * Revision 1.1.2.3  1997/11/25  22:07:11  ted
 * 	               Fix up statistics
 * 	               [1997/11/25  22:00:08  ted]
 * 
 * Revision 1.1.2.2  1997/11/21  04:48:19  hayes
 * 	               release for NIC 3.1 pre-Beta
 * 	               [1997/11/21  04:46:00  hayes]
 * 
 * 	               intermediate checkin
 * 	               [1997/11/20  01:28:37  hayes]
 * 
 * Revision 1.24.4.29  1997/11/14  19:16:36  hayes
 * 	               first checkin after mongo merge...
 * 	               [1997/11/14  19:11:42  hayes]
 * 
 * 	               checkin after stats restructuring
 * 	               [1997/11/12  19:01:39  hayes]
 * 
 * 	               intermediate checkin
 * 	               [1997/11/12  01:30:47  hayes]
 * 
 * Revision 1.24.4.28  1997/11/11  19:06:01  hayes
 * 	               restructuring for scratchpad use
 * 	               [1997/11/11  19:04:27  hayes]
 * 
 * Revision 1.24.4.27  1997/11/10  00:27:30  hayes
 * 	               fix peek_mac_rx()
 * 	               [1997/11/10  00:26:48  hayes]
 * 
 * Revision 1.24.4.26  1997/11/03  18:42:27  hayes
 * 	               cleanup rev4/5 ifdefs
 * 	               [1997/11/03  18:41:16  hayes]
 * 
 * Revision 1.24.4.25  1997/10/29  03:44:38  hayes
 * 	               add rx buf cleanup
 * 	               [1997/10/29  03:42:33  hayes]
 * 
 * Revision 1.24.4.24  1997/10/28  01:51:20  hayes
 * 	               intermediate checkin - cleanup MAC RX descriptor processing
 * 	               [1997/10/27  22:43:52  hayes]
 * 
 * 	               intermediate checkin - good rx descr fix 2
 * 	               [1997/10/26  22:41:34  hayes]
 * 
 * 	               intermediate MAC RX descriptor workaround checkin
 * 	               [1997/10/25  23:58:51  hayes]
 * 
 * Revision 1.24.4.23  1997/10/21  22:55:59  hayes
 * 	               more REV5 fixes
 * 	               [1997/10/21  22:55:08  hayes]
 * 
 * Revision 1.24.4.22  1997/10/21  01:35:35  skapur
 * 	               Merge of release 2.0 and ge_latest branch
 * 	               [1997/10/21  01:31:13  skapur]
 * 
 * Revision 1.24.4.21  1997/09/16  21:46:33  ted
 * 	               Change from MIB-II V1 (RFC 1213) to MIB-II V2 (RFC 1573)
 * 	               [1997/09/16  21:42:12  ted]
 * 
 * Revision 1.24.4.20  1997/09/16  03:35:25  skapur
 * 	               Merged with changes from 1.24.4.19
 * 	               [1997/09/16  03:35:08  skapur]
 * 
 * 	               Added stats for Jumbo Frames; Added code to discard the frames that are greater that 9K; Took out some of ifdef'ed code; Traces are overhauled; Misc. cleanup
 * 	               [1997/09/16  03:25:27  skapur]
 * 
 * Revision 1.24.4.19  1997/09/11  18:05:48  ted
 * 	               Mixing vlans, jumbos, and redundancy works, plus small fixes from release 1.1 and better NIC tracing
 * 	               [1997/09/11  17:59:50  ted]
 * 
 * Revision 1.24.4.18  1997/08/23  17:40:09  skapur
 * 	               Solaris jumbogram support
 * 	               [1997/08/23  17:38:06  skapur]
 * 
 * Revision 1.24.4.17  1997/08/13  18:46:08  ted
 * 	               Fix IP_SANITY checking for frameOffset stuff.
 * 	               Add nic_addr into recv bd for better debugability
 * 	               [1997/08/13  18:42:53  ted]
 * 
 * Revision 1.24.24.2  1997/08/03  20:03:29  ted
 * 	               Fix sign bug in h_recv_bd_ready
 * 	               [1997/08/03  20:02:00  ted]
 * 
 * Revision 1.24.24.1  1997/08/01  00:08:22  ted
 * 	               Fix DMA wrap bug causing bad packets and checksum errors
 * 	               [1997/07/31  23:54:26  ted]
 * 
 * Revision 1.24.4.15  1997/07/25  22:59:08  ted
 * 	               Architectural performance change
 * 	               [1997/07/24  21:45:15  ted]
 * 
 * Revision 1.24.4.14  1997/07/03  23:56:46  skapur
 * 	               Fixed unicast/Non-unicast frame count problem created becase of recv buffer allignment fix put in earlier
 * 	               [1997/07/03  23:56:02  skapur]
 * 
 * Revision 1.24.4.13  1997/06/17  00:36:34  ted
 * 	               Always do receives into double-word aligned areas by backing up and
 * 	               writing extra data when necessary.
 * 	               [1997/06/11  00:04:34  ted]
 * 
 * Revision 1.24.4.12  1997/05/21  22:59:51  ted
 * 	               Minor statistics tweaks
 * 	               [1997/05/21  22:59:13  ted]
 * 
 * Revision 1.24.4.11  1997/05/21  03:35:45  ted
 * 	               Fix possible bug having to do with adjusting recv mac buffer descriptor
 * 	               pointer.
 * 	               Fix IP_SANITY code to check every mismatching case.
 * 	               [1997/05/21  03:16:15  ted]
 * 
 * Revision 1.24.4.10  1997/05/03  04:58:46  skapur
 * 	               Cleaned up stats code
 * 	               [1997/05/03  04:58:12  skapur]
 * 
 * Revision 1.24.4.9  1997/04/26  02:02:43  skapur
 * 	               Fixed a bug which was shutting down the DMA RD completion events by mistake
 * 	               [1997/04/26  02:02:29  skapur]
 * 
 * Revision 1.24.4.8  1997/04/25  18:48:50  ted
 * 	               Remove old unused code.
 * 	               Add code for new recv bd mechanism
 * 	               [1997/04/25  18:37:02  ted]
 * 
 * Revision 1.24.4.7  1997/04/11  23:36:56  ted
 * 	               Misc. Cleanup; Major Perf. related improvements; SBUS HW Hacks to workaround HW Problems
 * 	               [1997/04/11  23:21:17  ted]
 * 
 * Revision 1.24.4.6  1997/03/28  02:47:07  skapur
 * 	               Fixed Replication code; moved some of the bits which were or'ed in the macros into the appropriate initialization variables in fwmain
 * 	               [1997/03/28  02:23:41  skapur]
 * 
 * Revision 1.24.4.5  1997/03/24  08:34:25  ted
 * 	               New event ring handling, producer lives in host memory, misc. other more modest changes
 * 	               [1997/03/24  08:28:02  ted]
 * 
 * Revision 1.24.4.4  1997/03/21  04:15:39  skapur
 * 	               Performance enhancement for Recv and Send; lots changes; almost complete overhaul of the entire firmware
 * 	               [1997/03/21  04:08:25  skapur]
 * 
 * Revision 1.24.4.3  1997/03/17  08:47:46  skapur
 * 	               changes for early sun release
 * 	               [1997/03/17  08:28:51  skapur]
 * 
 * 	               snapshot before adding second recv ring
 * 	               [1997/03/17  00:29:09  skapur]
 * 
 * 	               remove TG_REV == 0 conditional code
 * 	               [1997/03/16  21:06:58  skapur]
 * 
 * 	               remove old RR related conditionals
 * 	               [1997/03/16  21:05:42  skapur]
 * 
 * Revision 1.24.4.2  1997/02/23  00:23:39  ted
 * 	               Addded more Error handling code/better stats counting
 * 	               [1997/02/23  00:14:53  ted]
 * 
 * Revision 1.24.4.1  1997/02/17  00:08:00  ted
 * 	               Working with Rev 1, Chip Express boards (funky internal loopback sync algorith)
 * 	               [1997/02/16  23:44:22  ted]
 * 
 * Revision 1.24.2.6  1996/12/05  20:07:06  hayes
 * 	               Remove TG_REV 0 specific code.
 * 	               Fix missing ';' in non loopback case.
 * 	               Fix false positive MAC error detection in h_mac_rx_comp().
 * 	               [1996/12/05  20:05:46  hayes]
 * 
 * Revision 1.24.2.5  1996/11/20  21:58:13  skapur
 * 	               Updated to include Receieve Checksome code
 * 	               [1996/11/20  21:57:35  skapur]
 * 
 * Revision 1.24.2.4  1996/11/20  20:04:26  ted
 * 	               Redo include file structuring.
 * 	               [1996/11/20  19:52:00  ted]
 * 
 * Revision 1.24.2.3  1996/11/15  04:24:17  hayes
 * 	               convert to new header file structure.
 * 	               [1996/11/15  04:12:05  hayes]
 * 
 * Revision 1.24.2.2  1996/11/05  21:54:28  hayes
 * 	               Final pre-ODE merge
 * 	               [1996/11/05  21:52:12  hayes]
 * 
 * Revision 1.24.2.1  1996/11/01  18:25:05  ted
 * 	               change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	               [1996/10/29  22:06:34  ted]
 * 
 * $EndLog$
 */
/*
 * FILE recv.c
 *
 * COPYRIGHT (c) Essential Communication Corp. 1995
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/recv.c,v $
 * $Revision: 1.1.2.53 $ $Locker:  $
 * $Date: 1999/01/16 02:55:43 $ $Author: shuang $ $State: Exp $
 */

#ifndef __lint
static const char rcsid[] = "$Header: /projects/rcs/sw/ge/./nic/fw2/common/recv.c,v 1.1.2.53 1999/01/16 02:55:43 shuang Exp $";

#endif /* __lint */

#include "alt_def.h"
#include "nic_conf.h"
#include "nic_api.h"
#include "tg.h"

#include "nic.h"
#include "proto.h"
#include "trace.h"
#include "dma.h"
#include "ring.h"
#include "mac.h"
#include "assert.h"
#include "profile.h"
#include "mcast.h"
#include "cksum.h"


/* debug flags */
#define RX_CKSUM_DEBUG  0

/* type of recv ring */
#define RECV_RING_BIT_MASK      7
#define NO_RECV_RING            0
#define STD_RECV_RING           1
#define MINI_RECV_RING          2
#define JUMBO_RECV_RING         3
#define EXT_JUMBO_RECV_RING     4

/* flags passed via dma index field in cksum mode */
#define FRAME_SPLIT_FLAG        (0x08000000)
#define RING_FLAG_SHIFT         (28)
#define STD_BD_FLAG             (STD_RECV_RING       << RING_FLAG_SHIFT)
#define MINI_BD_FLAG            (MINI_RECV_RING      << RING_FLAG_SHIFT)
#define JUMBO_BD_FLAG           (JUMBO_RECV_RING     << RING_FLAG_SHIFT)
#define EXT_JUMBO_BD_FLAG       (EXT_JUMBO_RECV_RING << RING_FLAG_SHIFT)
#define ring_type_2_flag(type)  ((type) << RING_FLAG_SHIFT)

/* how many host address or length fields in an extend BD */
#define N_EXTS          4

/* globals */
U32  h_mac_rx_comp_stub;
U32  h_recv_bd_ready_stub;
U32  h_recv_jumbo_bd_ready_stub;
U32  h_recv_mini_bd_ready_stub;
void (*h_dma_wr_assist_stub)(void);
void (*deq_mac_rx_stub)(void);
tgMacDescr_t *(*peek_mac_rx_stub)(void);


__inline__ static U16
extract_vlan_tag(U32 frame_start)
{
    U16 vtag;

    vtag = *(U16 *)(frame_start + ETH_HLEN);
    *(U32 *)(frame_start + 12) = *(U32 *)(frame_start + 8);
    *(U32 *)(frame_start + 8)  = *(U32 *)(frame_start + 4);
    *(U32 *)(frame_start + 4)  = *(U32 *)(frame_start);
    return vtag;
}

__inline__ static void
restore_vlan_tag(U32 frame_start, U16 vtag)
{
    *(U32 *)(frame_start)      = *(U32 *)(frame_start + 4); 
    *(U32 *)(frame_start + 4)  = *(U32 *)(frame_start + 8);
    *(U32 *)(frame_start + 8)  = *(U32 *)(frame_start + 12);
    *(U16 *)(frame_start + 12) = ETHER_TYPE_WITH_VLAN_TAG;
    *(U16 *)(frame_start + 14) = vtag;
}

static void
update_mac_rx_error_slow(U32 errors)
{
    nicfp->stats.ifInErrors++;

    if (errors & TG_MAC_RX_W1_BAD_CRC)
        nicfp->stats.dot3StatsFCSErrors++;

    if (errors & TG_MAC_RX_W1_COLL_DETECT)
        nicfp->stats.nicMacRxLateColls++;

    if (errors & TG_MAC_RX_W1_LINK_LOST_DURING_PKT)
        nicfp->stats.nicMacRxLinkLostDuringPkt++;

    if (errors & TG_MAC_RX_W1_PHY_DECODE_ERR)
        nicfp->stats.nicMacRxPhyDecodeErr++;

    if (errors & TG_MAC_RX_W1_ODD_NIBBLES_RCVD_MII)
        nicfp->stats.dot3StatsAlignmentErrors++;

    if (errors & TG_MAC_RX_W1_MAC_ABORT)
        nicfp->stats.nicMacRxMacAbort++;
    #if 0
    if (errors & TG_MAC_RX_W1_LEN_LT_64)
        nicfp->stats.nicMacRxLenLt64++;
    #endif /* 0 */
    return;
}

__inline__ static bool
is_mac_rx_error(tgMacDescr_t *hd)
{
    U32 errors;

    hd->w1 &= ~TG_MAC_RX_W1_LEN_LT_64; /* XXX */
    errors = (hd->w1 & TG_MAC_RX_W1_STATUS_MASK);
    if (!errors)
        return (FALSE);
    
    NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_RECV), "rxFramEr", 0x70100,
        hd->w0,
        hd->w1,
        trp->local_mem_conf.rx_buf_producer,
        trp->local_mem_conf.rx_buf_consumer);

    /* do proper accounting on the error type */
    if (errors & TG_MAC_RX_W1_TRUNC_NO_RESOURCES) {
        nicfp->stats.nicMacRxTruncNoResources++;
    } else {
        update_mac_rx_error_slow(errors);
    }
    return (TRUE);
}

tgMacDescr_t *
peek_mac_rx(void)
{
    tgMacDescr_t *mdp_cons = trp->local_mem_conf.mac_rx_descr_consumer;
    tgMacDescr_t *mdp_prod = trp->local_mem_conf.mac_rx_descr_producer;

    (U32)mdp_prod &= ~0x7;
    if (mdp_cons == mdp_prod) { /* empty */
        NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_RECV), "rxPkMacF", 0x70150,
            mdp_prod,
            mdp_cons,
            trp->local_mem_conf.mac_rx_descr_ref,
            mdp_cons - tdp->rx_mac_ring);

        return ((tgMacDescr_t *)0);
    }

    NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_RECV), "rxPkMac", 0x70160,
        mdp_cons->w0,
        mdp_cons->w1,
        mdp_prod,
        mdp_cons);

    return (mdp_cons);
}

/* null routine to assist with scratchpad setup */
void peek_mac_rx_end(void) {}

/*
 * work around version of peek_mac_rx() for the Tigon2 MAC6 bug 
 * WARNING - consult me before do any change
 * there are critical timing related code to bypass the Tigon2 MAC6 bug
 */
tgMacDescr_t *
peek_mac_rx_wa(void)
{
    U32 bp, bp_next, space;
    tgMacDescr_t *mdp_prod, *mdp_prod_sv, *mdp_cons, *mdp_cons_next;

    /* WARNING: critical timing related code to bypass the Tigon2 MAC6 bug */
    bp = trp->local_mem_conf.rx_buf_producer;
    mdp_prod_sv = trp->local_mem_conf.mac_rx_descr_producer;

    (U32)mdp_prod = ROUNDDN((U32)mdp_prod_sv, 8);
    mdp_cons = trp->local_mem_conf.mac_rx_descr_consumer;
    if (mdp_cons == mdp_prod) { /* empty */

        NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_RECV), "rxWaMacF", 0x70200,
            mdp_prod,
            mdp_cons,
            trp->local_mem_conf.mac_rx_descr_ref,
            mdp_cons - tdp->rx_mac_ring);

        return ((tgMacDescr_t *)0);
    }

    /* the following checks are to bypass the Tigon2 MAC6 bug */

    mdp_cons_next = get_next_mac_rx_descr(mdp_cons);
    if (mdp_cons_next == mdp_prod) { /* last one to consume */
	if ((U32)mdp_prod_sv & 4) /* new incoming frame pending */
	    return ((tgMacDescr_t *)0); /* retry later */
	else /* no new incoming frame */
	    bp_next = bp; /* should only use the saved one */
    } else {
	bp_next = mdp_cons_next->w0;
    }

    /* back to normal if reached the last mac rx descriptor to work around */
    if (mdp_cons == nicfp->md_prod_mac6_bug) {
	nicfp->peek_mac_rx_stub = nicfp->peek_mac_rx_relocated;

	NIC_TRACE(TRACE_TYPE_V6_MAC6_BUG, "ckMac6z", 0x70230,
	    0,
	    0,
	    0,
	    nicfp->md_prod_mac6_bug);
    }

    space = bp_next - mdp_cons->w0;
    if (space > NIC_RX_BUF_SIZE) /* wrap */
	space += NIC_RX_BUF_SIZE;

    if (space >= ((mdp_cons->w1 & TG_MAC_RX_W1_LEN_MASK) + 80)) {

	NIC_UTRACE("rMacChk0", mdp_cons,
	    mdp_cons->w0,
	    mdp_cons->w1,
	    bp_next,
	    space);

	NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_RECV | TRACE_TYPE_V6_MAC6_BUG),
		"rMacChk1", 0x70250,
	    trp->local_mem_conf.rx_buf_producer,
	    trp->local_mem_conf.rx_buf_consumer,
	    trp->local_mem_conf.mac_rx_descr_producer,
	    trp->local_mem_conf.mac_rx_descr_consumer);

	NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_RECV | TRACE_TYPE_V6_MAC6_BUG),
		"rMacChk2", bp,
	    mdp_prod,
	    mdp_prod_sv,
	    mdp_cons_next->w0,
	    mdp_cons_next->w1);

	{
	#if (TRACE && TRACE_ALL)
	U32 cur = mdp_cons->w0;
	#endif /* (TRACE && TRACE_ALL) */

	NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_RECV | TRACE_TYPE_V6_MAC6_BUG),
		"rMacChk3", cur,
	    *(U32 *)cur,
	    *((U32 *)cur + 1),
	    *((U32 *)cur + 2),
	    *((U32 *)cur + 3));
	}

	/* this frame may be bad - discard it */
	nicfp->stats.ifInDiscards++;
	trp->local_mem_conf.mac_rx_descr_ref = mdp_cons_next;
	trp->local_mem_conf.mac_rx_descr_consumer = mdp_cons_next;

	return ((tgMacDescr_t *)0);
    }

    NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_RECV), "rxWaMac", 0x70300,
	mdp_cons->w0,
	bp_next,
	space,
	mdp_cons);

    return (mdp_cons);
}

/* null routine to assist with scratchpad setup */
void peek_mac_rx_wa_end(void) {}

#if !USE_ASM
void
deq_mac_rx(void)
{
    tgMacDescr_t *mdp_cons = trp->local_mem_conf.mac_rx_descr_consumer;

    NIC_TRACE((TRACE_TYPE_MAC | TRACE_TYPE_RECV), "rxDqMac", 0x70400,
        trp->local_mem_conf.mac_rx_descr_producer,
        mdp_cons,
        trp->local_mem_conf.mac_rx_descr_ref,
        mdp_cons - tdp->rx_mac_ring);

    mdp_cons = get_next_mac_rx_descr(mdp_cons);
    trp->local_mem_conf.mac_rx_descr_ref = mdp_cons;
    trp->local_mem_conf.mac_rx_descr_consumer = mdp_cons;

    return;
}

/* null routine to assist with scratchpad setup */
void deq_mac_rx_end(void) {}
#endif /* !USE_ASM */

void
h_mac_rx_comp_nohost(void)
{
    U32 mdp_prod;
    U32 buf_prod;
    U32 tmp;

    NIC_TRACE(TRACE_TYPE_RECV, "#rNohst", 0x70500,
        nicfp->stats.ifHCInOctets,
        nicfp->stats.ifHCInUcastPkts,
        0,
        0);

    /* save both producers so they don't change on us */
    mdp_prod = ROUNDDN((U32)trp->local_mem_conf.mac_rx_descr_producer, 8);
    buf_prod = trp->local_mem_conf.rx_buf_producer;

    /* calculate the number of rx descriptors that have completed
     * we're not really looking at the frames, so just account them all
     * to the unicast counter
     */
    tmp = mdp_prod - (U32)trp->local_mem_conf.mac_rx_descr_ref;
    if (tmp >= (TG_NUM_MAC_RX_DESCRS * sizeof(tgMacDescr_t)))
         tmp += (TG_NUM_MAC_RX_DESCRS * sizeof(tgMacDescr_t));
    tmp /= sizeof(tgMacDescr_t);
    nicfp->stats.ifHCInUcastPkts += tmp;

    /* calculate the amount of buffer space that has been consumed */
    tmp = buf_prod - trp->local_mem_conf.rx_buf_consumer;
    if (tmp >= NIC_RX_BUF_SIZE)
        tmp += NIC_RX_BUF_SIZE;
    nicfp->stats.ifHCInOctets += tmp;

    /* update our consumer and ref so we can proceed */
    trp->local_mem_conf.rx_buf_consumer = buf_prod;
    trp->local_mem_conf.mac_rx_descr_consumer = (tgMacDescr_t *)mdp_prod;
    trp->local_mem_conf.mac_rx_descr_ref = (tgMacDescr_t *)mdp_prod;

    RETURN(TG_FW_EVENT_NUM_MAC_RX_COMP);
}

void
h_mac_rx_comp(void)
{
    tgMacDescr_t hd, *mdp;
    tg_hostaddr_t *ext_host_addr_p[N_EXTS];
    ext_recv_bd_t *ebdp;
    recv_bd_t *bdp;
    U32 *ring_index_ptr;
    U32 ring_index_next;
    U32 ring_index;
    U32 ring_type;
    U32 cur;
    U32 max_len, frame_len, frame_len_diff;
    U32 md_w1;
    U32 rc;
    U32 i;
    U16 vlan_tag;
    U16 vlan_bit;
    U16 bd_len;
    U16 bd_flags, tmp_bd_flags;
    U16 ext_bd_len[N_EXTS];
    bool ext_data_dma;

    ext_data_dma = FALSE;
    ebdp = 0;
    ring_type = STD_RECV_RING;
    vlan_bit = 0;
    vlan_tag = 0; /* eliminate compiler warning */

    if (nicfp->rx_frame_split) {
        cur            = nicfp->rx_frame_cur;
        frame_len      = nicfp->rx_frame_len;
        frame_len_diff = nicfp->rx_frame_len_diff;
        md_w1          = nicfp->rx_md_w1;

        NIC_TRACE(TRACE_TYPE_RECV, "#rxSplit", 0x70600,
            cur,
            frame_len,
            frame_len_diff,
            md_w1);

    } else {

        NIC_TRACE(TRACE_TYPE_RECV, "#rxNewDt", 0x70700,
            trp->local_mem_conf.mac_rx_descr_producer,
            trp->local_mem_conf.mac_rx_descr_consumer,
            trp->local_mem_conf.rx_buf_producer,
            trp->local_mem_conf.rx_buf_consumer);

	if (!(mdp = (*nicfp->peek_mac_rx_stub)())) {
            RETURN(TG_FW_EVENT_NUM_MAC_RX_COMP);
        }

	hd = *mdp; /* struct copy */
        cur = hd.w0;
        md_w1 = hd.w1;
        frame_len = (md_w1 & TG_MAC_RX_W1_LEN_MASK) - 4; /* exclude crc */

        /* reject multicast frame that not appear on our search list */
        if (!is_global_flags(GF_CMD_PROMISC_MODE | GF_CMD_MCAST_MODE) &&
            (*(U8 *)cur & 0x01) &&
            !((*(U32 *)cur == 0xffffffff) && (*(U16 *)(cur + 4) == 0xffff))) {

            if (!mcast_rx_filter((U8 *)cur)) { /* not on list */
                (*nicfp->deq_mac_rx_stub)();
                RETURN(TG_FW_EVENT_NUM_MAC_RX_COMP);
            }
        }

        rc = is_mac_rx_error(&hd);
        if (frame_len > tsp->gen_com.ifMTU) { /* host specified MTU */
            nicfp->stats.dot3StatsFrameTooLongs++;
            md_w1 |= BD_ERR_GIANT_FRAME_RCVD;
            rc = 1; /* error */

            NIC_UTRACE("rxFrm2Lg", 0x70800,
                hd.w0,
                hd.w1,
                trp->local_mem_conf.mac_rx_descr_producer,
                trp->local_mem_conf.mac_rx_descr_consumer);
        }
        if (rc && !is_global_flags(GF_MOD_ALLOW_BAD_FRAMES)) {
            if (md_w1 &
                (BD_ERR_GIANT_FRAME_RCVD | TG_MAC_RX_W1_TRUNC_NO_RESOURCES)) {
                nicfp->stats.ifInDiscards++;
            }
#if (USE_U32_STAT_CNT)
            nicfp->ifHCInOctets += frame_len;
#else /* (USE_U32_STAT_CNT) */
            nicfp->stats.ifHCInOctets += frame_len;
#endif /* (USE_U32_STAT_CNT) */
            (*nicfp->deq_mac_rx_stub)();
            RETURN(TG_FW_EVENT_NUM_MAC_RX_COMP);
        }

        /* extract vlan tag */
        if ((is_global_flags(GF_RCB_RX_VLAN_ASSIST)) &&
            (((eth_t *)cur)->ethFrameType == ETHER_TYPE_WITH_VLAN_TAG)) {

            vlan_bit = BD_FLAG_VLAN_TAG;
            vlan_tag = extract_vlan_tag(cur);
            cur += 4;
            frame_len -= 4;

            NIC_TRACE(TRACE_TYPE_RECV, "rxVTag", 0x70900,
                vlan_tag,
                cur,
                frame_len,
                0);
        }

        NIC_TRACE(TRACE_TYPE_RECV, "rxFrame0", 0x71000,
            *(U32 *)cur,
            *((U32 *)cur + 1),
            *((U32 *)cur + 2),
            *((U32 *)cur + 3));

        NIC_TRACE(TRACE_TYPE_RECV, "rxFrame1", 0x71100,
            *((U32 *)cur + 4),
            *((U32 *)cur + 5),
            *((U32 *)cur + 6),
            *((U32 *)cur + 7));

        /* include crc if requested by host */
        if (is_global_flags(GF_MOD_INCLUDE_CRC))
            frame_len += 4;

        /* dma partial amount of data if requested by host */
        if (nicfp->rx_partial_count && (frame_len > nicfp->rx_partial_count)) {
            frame_len_diff = frame_len - nicfp->rx_partial_count;
            frame_len = nicfp->rx_partial_count;
        } else {
            frame_len_diff = 0;
        }

        /* 1. use jumbo BD if std BD len < frame len
         *         std  BD if min BD len < frame len <= std BD len
         *         mini BD if              frame len <= min BD len
         * 2. will try to use std BD if run out of jumbo BD or mini BD
         * 3. use of jumbo or mini BDs should not split frames
         */
        if (frame_len > nicfp->recv_std_buf_size) {
            if (is_global_flags(GF_CMD_JUMBO_FRAMES_ENABLED)) {
                ring_type = (is_global_flags(GF_RCB_USE_EXT_RECV_BD))?
                            EXT_JUMBO_RECV_RING:
                            JUMBO_RECV_RING;
            }
        } else if (frame_len <= nicfp->recv_mini_buf_size) {
            if (is_global_flags(GF_CMD_MINI_FRAMES_ENABLED)) {
                ring_type = MINI_RECV_RING;
            }
        }
    }

    /* move data to host */
    while (frame_len) {
        /* note the priority */
        if (ring_type == STD_RECV_RING) {
            check_std_ring:

            if (nicfp->recv_cons_index !=
                nicfp->recv_prod_index_avail) { /* std BDs available */

                ring_index_ptr = &nicfp->recv_cons_index;
                ring_index = nicfp->recv_cons_index;
                ring_index_next = 
                    (ring_index + 1) & (RECV_STANDARD_RING_ENTRIES - 1);
                bdp = &nicp->recv_ring[ring_index];

                NIC_TRACE(TRACE_TYPE_RECV, "rxStd", 0x71200,
                    nicfp->recv_cons_index,
                    nicfp->recv_prod_index_avail,
                    nicfp->recv_return_prod_index,
                    frame_len);

            } else { /* no std BDs */
                NIC_UTRACE("rxNoStBd", 0x71300,
                    nicfp->recv_cons_index,
                    nicfp->recv_prod_index_avail,
                    nicfp->recv_return_prod_index,
                    frame_len);

                nicfp->stats.nicNoMoreRxBDs++;
                event_mask &= ~TG_FW_EVENT_MAC_RX_COMP;
                break;
            }

        } else if (ring_type == MINI_RECV_RING) {
            if (nicfp->recv_mini_cons_index !=
                nicfp->recv_mini_prod_index_avail) { /* have mini BDs */

                ring_index_ptr = &nicfp->recv_mini_cons_index;
                ring_index = nicfp->recv_mini_cons_index;
                ring_index_next =
                    (ring_index + 1) & (RECV_MINI_RING_ENTRIES - 1);
                bdp = &nicp->recv_mini_ring[ring_index];

                NIC_TRACE(TRACE_TYPE_RECV, "rxMini", 0x71400,
                    nicfp->recv_mini_cons_index,
                    nicfp->recv_mini_prod_index_avail,
                    nicfp->recv_return_prod_index,
                    frame_len);

            } else { /* no mini BDs */
                NIC_UTRACE("rxNoMiBd", 0x71500,
                    nicfp->recv_mini_cons_index,
                    nicfp->recv_mini_prod_index_avail,
                    nicfp->recv_return_prod_index,
                    frame_len);

                ring_type = STD_RECV_RING; /* try std recv ring */
                goto check_std_ring;
            }

        } else { /* jumbo ring */
            if (nicfp->recv_jumbo_cons_index !=
                nicfp->recv_jumbo_prod_index_avail) { /* have jumbo BDs */

                ring_index_ptr = &nicfp->recv_jumbo_cons_index;
                ring_index = nicfp->recv_jumbo_cons_index;
                ring_index_next =
                    (ring_index + 1) & (RECV_JUMBO_RING_ENTRIES - 1);
                if (ring_type == EXT_JUMBO_RECV_RING) {
                    ebdp = &nicp->ext_recv_jumbo_ring[ring_index];
                    bdp = (recv_bd_t *)&ebdp->bd1;
                    if (ebdp->BD_len < frame_len) {
                        ext_data_dma = TRUE;
                    }

                    NIC_TRACE(TRACE_TYPE_RECV, "rxExJumb", 0x71600,
                        nicfp->recv_jumbo_cons_index,
                        nicfp->recv_jumbo_prod_index_avail,
                        nicfp->recv_return_prod_index,
                        frame_len);
                } else {
                    bdp = &nicp->recv_jumbo_ring[ring_index];

                    NIC_TRACE(TRACE_TYPE_RECV, "rxJumbo", 0x71700,
                        nicfp->recv_jumbo_cons_index,
                        nicfp->recv_jumbo_prod_index_avail,
                        nicfp->recv_return_prod_index,
                        frame_len);
                }

            } else { /* no jumbo BDs */
                NIC_UTRACE("rxNoJmBd", 0x71800,
                    nicfp->recv_jumbo_cons_index,
                    nicfp->recv_jumbo_prod_index_avail,
                    nicfp->recv_return_prod_index,
                    frame_len);

                if (is_global_flags(GF_MOD_DONT_FRAG_JUMBOS)) {
                    nicfp->stats.nicNoMoreRxBDs++;
                    event_mask &= ~TG_FW_EVENT_MAC_RX_COMP;
                    break;
                } else {
                    nicfp->stats.nicJumboSpillOver++;
                    ring_type = STD_RECV_RING; /* try std recv ring */
                    goto check_std_ring;
                }
            }
        }

	#if 0 /* debug */
	{
	    /* make sure upper host addr not corrupted */
	    if (*(U32 *)bdp != 0) {
		NIC_UTRACE("?rxBd", bdp,
		    *(U32 *)bdp,
		    *((U32 *)bdp + 1),
		    *((U32 *)bdp + 2),
		    *((U32 *)bdp + 3));
		ASSERT(0);
	    }
	}
	#endif /* debug */

        /* setup nic addr */
        bdp->BD_nic_addr = cur;

        if (ext_data_dma) {
            S32 len;
            U32 n_exts;

            /* for looping */
            ext_host_addr_p[0] = &ebdp->BD_host_addr;
            ext_host_addr_p[1] = &ebdp->BD_host_addr_1;
            ext_host_addr_p[2] = &ebdp->BD_host_addr_2;
            ext_host_addr_p[3] = &ebdp->BD_host_addr_3;
            ext_bd_len[0] = ebdp->BD_len;
            ext_bd_len[1] = ebdp->BD_len_1;
            ext_bd_len[2] = ebdp->BD_len_2;
            ext_bd_len[3] = ebdp->BD_len_3;

            NIC_TRACE(TRACE_TYPE_RECV, "rxExtBd0", 0x71900,
                *(U32 *)ebdp,           /* host addr 1 */
                *((U32 *)ebdp + 1),     /* host addr 1 */
                *((U32 *)ebdp + 2),     /* host addr 2 */
                *((U32 *)ebdp + 3));    /* host addr 2 */

            NIC_TRACE(TRACE_TYPE_RECV, "rxExtBd1", 0x72000,
                *((U32 *)ebdp + 4),     /* host addr 3 */
                *((U32 *)ebdp + 5),     /* host addr 3 */
                *((U32 *)ebdp + 6),     /* len 1 & 2 */
                *((U32 *)ebdp + 7));    /* len 3 & reserved */
                
            NIC_TRACE(TRACE_TYPE_RECV, "rxExtBd2", 0x72100,
                *((U32 *)ebdp + 8),     /* host addr 0 */ 
                *((U32 *)ebdp + 9),     /* host addr 0 */
                *((U32 *)ebdp + 10),    /* index & len 0 */
                *((U32 *)ebdp + 11));   /* type & flags */
                
            NIC_TRACE(TRACE_TYPE_RECV, "rxExtBd3", 0x72200,
                *((U32 *)ebdp + 12),    /* ip cksum & tcp/udp cksum */
                *((U32 *)ebdp + 13),    /* error flags & vlan tag */
                *((U32 *)ebdp + 14),    /* nic addr */
                *((U32 *)ebdp + 15));   /* opaque */
                
            /* setup dma length */
            len = frame_len;
            n_exts = 0;
            while (n_exts < N_EXTS) {
                if ((len -= ext_bd_len[n_exts++]) <= 0) {
                    ext_bd_len[n_exts - 1] += len; /* dma len for the last */
                    break;
                }
            }
            ASSERT(len <= 0); /* inform driver */

            /* check free dma descriptors for buf data + buf descriptor */
            if (!can_q_dma_to_host(n_exts + 1)) {
		set_global_flags(GF_VAR_MAC_RX_COMP_SUSP);
		event_mask &= ~TG_FW_EVENT_MAC_RX_COMP;

                NIC_UTRACE("rxCkDmaF", 0x72250,
                    trp->host_dma_assist.write_chan_producer,
                    trp->host_dma_assist.write_chan_consumer,
                    trp->host_dma_assist.write_chan_ref,
                    n_exts + 1);
                break;
            }

            for (i = 0; i < n_exts; i++) {

                NIC_TRACE(TRACE_TYPE_RECV, "rxQDmaEx", 0x72300,
                    i,
                    ext_bd_len[i],
                    cur,
                    *ext_host_addr_p[i]);

                if (!(*nicfp->q_dma_to_host_stub)
                        (*ext_host_addr_p[i],
                        cur,
                        ext_bd_len[i],
                        TIGON_TYPE_RECV_DATA,
                        ring_index, /* dummy */
                        nicfp->dma_to_host_data_state)) {

                    NIC_UTRACE("rxQDmExF", 0x72400,
                        i,
                        ext_bd_len[i],
                        cur,
                        *ext_host_addr_p[i]);
                    ASSERT(0); /* should not happen */
                    break;
                }

                cur += ext_bd_len[i];
                RX_WRAP_BUF(cur);
            }

            /* setup to dma BD */
            max_len = frame_len;
            bd_len = ext_bd_len[0];

        } else {

            /* check free dma descriptors for buf data + buf descriptor */
            if (!can_q_dma_to_host(2)) {
		set_global_flags(GF_VAR_MAC_RX_COMP_SUSP);
		event_mask &= ~TG_FW_EVENT_MAC_RX_COMP;

                NIC_UTRACE("rxCkDmaF", 0x72450,
                    trp->host_dma_assist.write_chan_producer,
                    trp->host_dma_assist.write_chan_consumer,
                    trp->host_dma_assist.write_chan_ref,
                    2);
                break;
            }

            bd_len = bdp->BD_len;
            max_len = (frame_len <= bd_len)? frame_len: bd_len;

            NIC_TRACE(TRACE_TYPE_RECV, "rxBd0", 0x72500,
                *(U32 *)bdp,            /* host addr */
                *((U32 *)bdp + 1),      /* host addr */
                *((U32 *)bdp + 2),      /* index & len */
                *((U32 *)bdp + 3));     /* type & flags */

            NIC_TRACE(TRACE_TYPE_RECV, "rxBd1", 0x72600,
                *((U32 *)bdp + 4),      /* ip cksum & tcp cksum */
                *((U32 *)bdp + 5),      /* error flags & vlan tag */
                *((U32 *)bdp + 6),      /* nic addr */
                *((U32 *)bdp + 7));     /* opaque */

            NIC_TRACE(TRACE_TYPE_RECV, "rxLen", 0x72700,
                cur,
                max_len,
                frame_len,
                bd_len);

            /* dma data to host */
            if (!(*nicfp->q_dma_to_host_stub)
                    (bdp->BD_host_addr,
                    cur,
                    max_len,
                    TIGON_TYPE_RECV_DATA,
                    ring_index, /* dummy */
                    nicfp->dma_to_host_data_state)) {

                NIC_UTRACE("rxQDmaF", 0x72800,
                    bdp->BD_host_addr,
                    bdp->BD_nic_addr,
                    max_len,
                    ring_index);
                ASSERT(0); /* should not happen */
                break;
            }
        }

        tmp_bd_flags = bd_flags = bdp->BD_flags;

        /* report vlan info in the first BD */
        if (vlan_bit && !nicfp->rx_frame_split) {
            tmp_bd_flags    |= vlan_bit;
            bdp->BD_vlan_tag = vlan_tag;
        }

        /* report frame error on all BDs */
        if (md_w1 & TG_MAC_RX_W1_STATUS_MASK) {
            tmp_bd_flags       |= BD_FLAG_FRAME_HAS_ERROR;
            bdp->BD_error_flags = (md_w1 >> BD_ERR_SHIFT);
        }

        /* dma BD to host */
        if (max_len == frame_len) {

            /* indicate the last BD */
            bdp->BD_flags = tmp_bd_flags | BD_FLAG_END;

            /* report real length NOT partial length in the last BD */
            bdp->BD_len = frame_len + frame_len_diff;

            NIC_TRACE(TRACE_TYPE_RECV, "rxQDmBdL", 0x72900,
                nicfp->recv_return_prod_index,
                *((U32 *)bdp + 3),	/* type & flags */
                *((U32 *)bdp + 5),	/* error flags & vlan tag */
		ring_index);

            if (!(*nicfp->q_dma_to_host_stub)
                    (nicfp->recv_ret_ring_ptr +
                      (nicfp->recv_return_prod_index * sizeof(recv_bd_t)),
                    (U32)bdp,
                    sizeof(recv_bd_t),
                    TIGON_TYPE_RECV_BD_LAST,
                    nicfp->recv_return_prod_index, /* dummy */
                    nicfp->dma_to_host_bd_state)) {

                /* restore BD flags and length */
                bdp->BD_flags = bd_flags;
                bdp->BD_len   = bd_len;

                NIC_UTRACE("rxQDLBdF", 0x73000,
                    nicfp->recv_return_prod_index,
                    trp->host_dma_assist.write_chan_producer,
                    trp->host_dma_assist.write_chan_consumer,
                    ring_index);
                ASSERT(0); /* should not happen */
                break;
            }

            nicfp->rx_frame_split = FALSE;
            frame_len = 0;
            nicfp->stats.nicTotalRecvBDs--;
            if (ring_type == STD_RECV_RING)
                nicfp->stats.nicRecvBDs--;
            else if (ring_type == MINI_RECV_RING)
                nicfp->stats.nicMiniRecvBDs--;
            else /*if ((ring_type == <EXT_>JUMBO_RECV_RING)*/
                nicfp->stats.nicJumboRecvBDs--;

            *ring_index_ptr = ring_index_next;
            nicfp->recv_return_prod_index =
                (nicfp->recv_return_prod_index + 1) &
		nicfp->recv_ret_ring_size_mask;

            if (++nicfp->recv_coal_bds >= 
                    tsp->gen_com.tuneParms.recv_max_coalesced_bds) {
                recv_return_prod_index_to_host();
                nicfp->stats.nicRecvThresholdHit++;
            }

        } else { /* dma intermediate <std BD> to the host */
            bdp->BD_flags = tmp_bd_flags;
            bdp->BD_len = max_len;

            NIC_TRACE(TRACE_TYPE_RECV, "rxQDmaBd", 0x73100,
                0,
                0,
                nicfp->recv_return_prod_index,
                ring_index);

            if (!(*nicfp->q_dma_to_host_stub)
                    (nicfp->recv_ret_ring_ptr +
                      (nicfp->recv_return_prod_index * sizeof(struct recv_bd)),
                    (U32)bdp,
                    sizeof(struct recv_bd),
                    TIGON_TYPE_RECV_BD_LAST,
                    nicfp->recv_return_prod_index, /* dummy */
                    nicfp->dma_to_host_bd_state)) {

                /* restore BD flags and length */
                bdp->BD_len   = bd_len;
                bdp->BD_flags = bd_flags;

                NIC_UTRACE("rxQDmBdF", 0x73200,
                    nicfp->recv_return_prod_index,
                    trp->host_dma_assist.write_chan_producer,
                    trp->host_dma_assist.write_chan_consumer,
                    ring_index);
                ASSERT(0); /* should not happen */
                break;
            }

            nicfp->rx_frame_split = TRUE;
            frame_len -= max_len;
            cur += max_len;
            RX_WRAP_BUF(cur);

            nicfp->stats.nicTotalRecvBDs--;
            if (ring_type == STD_RECV_RING)
                nicfp->stats.nicRecvBDs--;
            else if (ring_type != MINI_RECV_RING)
                nicfp->stats.nicJumboRecvBDs--;
            else /*if ((ring_type == MINI_RECV_RING)*/
                nicfp->stats.nicMiniRecvBDs--;

            *ring_index_ptr = ring_index_next;
            nicfp->recv_return_prod_index =
                (nicfp->recv_return_prod_index + 1) &
		nicfp->recv_ret_ring_size_mask;

            nicfp->recv_coal_bds++;
        }
    }

    if (frame_len) { /* run out of resources */
        if (nicfp->rx_frame_split) { /* std BD only */

            /* save info to continue later */
            nicfp->rx_frame_cur         = cur;
            nicfp->rx_frame_len         = frame_len;
            nicfp->rx_md_w1             = md_w1;
            nicfp->rx_frame_len_diff    = frame_len_diff;

        } else if (vlan_bit) {
            restore_vlan_tag(hd.w0, vlan_tag);
        }

	/* increase <rx no resource> stats */
	nicfp->stats.nicNoMoreWrDMADescriptors++;

        NIC_TRACE(TRACE_TYPE_RECV, "rxNoRes", 0x73300,
            ring_type,
            nicfp->recv_return_prod_index,
            0,
            0);

    } else {
#if (USE_U32_STAT_CNT)
        nicfp->ifHCInOctets += ((md_w1 & TG_MAC_RX_W1_LEN_MASK) - 4);
#else /* (USE_U32_STAT_CNT) */
        nicfp->stats.ifHCInOctets += ((md_w1 & TG_MAC_RX_W1_LEN_MASK) - 4);
#endif /* (USE_U32_STAT_CNT) */
        (*nicfp->deq_mac_rx_stub)();
    }

    RETURN(TG_FW_EVENT_NUM_MAC_RX_COMP);
}

/* null routine to assist with scratchpad setup */
void h_mac_rx_comp_end(void) {}

void
h_mac_rx_comp_cksum(void)
{
    tgMacDescr_t hd, *mdp;
    tg_hostaddr_t *ext_host_addr_p[N_EXTS];
    ext_recv_bd_t *ebdp;
    recv_bd_t *bdp;
    U32 *ring_index_ptr;
    U32 ring_index_next;
    U32 ring_index;
    U32 ring_type;
    U32 cur;
    U32 max_len, frame_len, len_wo_padding;
    U32 md_w1;
    U32 i;
    U16 vlan_bit;
    U16 vlan_tag;
    U16 bd_flags;
    U16 bd_len;
    U16 ext_bd_len[N_EXTS];
    bool frame_err;
    bool ext_data_dma;

    ext_data_dma = FALSE;
    ebdp = 0;
    ring_type = STD_RECV_RING;
    vlan_bit = 0;
    vlan_tag = 0; /* eliminate compiler warning */

    if (nicfp->rx_frame_split) {
        frame_len = nicfp->rx_frame_len;
        cur       = nicfp->rx_frame_cur;
        md_w1     = nicfp->rx_md_w1;

        NIC_TRACE(TRACE_TYPE_RECV, "#rxSplit", 0x73400,
            cur,
            frame_len,
            md_w1,
            0);
    } else {

        NIC_TRACE(TRACE_TYPE_RECV, "#rxNewDt", 0x73500,
            trp->local_mem_conf.mac_rx_descr_producer,
            trp->local_mem_conf.mac_rx_descr_consumer,
            trp->local_mem_conf.rx_buf_producer,
            trp->local_mem_conf.rx_buf_consumer);

	if (!(mdp = (*nicfp->peek_mac_rx_stub)())) {
            RETURN(TG_FW_EVENT_NUM_MAC_RX_COMP);
        }

	hd = *mdp; /* struct copy */
        cur = hd.w0;
        md_w1 = hd.w1;
        frame_len = (md_w1 & TG_MAC_RX_W1_LEN_MASK) - 4; /* exclude crc */

        /* reject multicast frame that not appear on our search list */
        if (!is_global_flags(GF_CMD_PROMISC_MODE | GF_CMD_MCAST_MODE) &&
            (*(U8 *)cur & 0x01) &&
            !((*(U32 *)cur == 0xffffffff) && (*(U16 *)(cur + 4) == 0xffff))) {

            if (!mcast_rx_filter((U8 *)cur)) { /* not on list */
                (*nicfp->deq_mac_rx_stub)();
                RETURN(TG_FW_EVENT_NUM_MAC_RX_COMP);
            }
        }

        frame_err = is_mac_rx_error(&hd);
        if (frame_len > tsp->gen_com.ifMTU) { /* Host Specified MTU */
            nicfp->stats.dot3StatsFrameTooLongs++;
            md_w1 |= BD_ERR_GIANT_FRAME_RCVD;
            frame_err = TRUE; /* error */

            NIC_UTRACE("rxFrm2Lg", 0x73600,
                hd.w0,
                hd.w1,
                trp->local_mem_conf.mac_rx_descr_producer,
                trp->local_mem_conf.mac_rx_descr_consumer);
        }
        if (frame_err && !is_global_flags(GF_MOD_ALLOW_BAD_FRAMES)) {
            if (md_w1 &
                (BD_ERR_GIANT_FRAME_RCVD | TG_MAC_RX_W1_TRUNC_NO_RESOURCES)) {
                nicfp->stats.ifInDiscards++;
            }
#if (USE_U32_STAT_CNT)
            nicfp->ifHCInOctets += frame_len;
#else /* (USE_U32_STAT_CNT) */
            nicfp->stats.ifHCInOctets += frame_len;
#endif /* (USE_U32_STAT_CNT) */
            (*nicfp->deq_mac_rx_stub)();
            RETURN(TG_FW_EVENT_NUM_MAC_RX_COMP);
        }

        /* extract vlan tag */
        if (is_global_flags(GF_RCB_RX_VLAN_ASSIST) &&
            (((eth_t *)cur)->ethFrameType == ETHER_TYPE_WITH_VLAN_TAG)) {

            vlan_bit = BD_FLAG_VLAN_TAG;
            vlan_tag = extract_vlan_tag(cur);
            cur += 4;
            frame_len -= 4;

            NIC_TRACE(TRACE_TYPE_RECV, "rxVTag", 0x73700,
                vlan_tag,
                cur,
                frame_len,
                0);
        }

        NIC_TRACE(TRACE_TYPE_RECV, "rxFrame0", 0x73800,
            *(U32 *)cur,
            *((U32 *)cur + 1),
            *((U32 *)cur + 2),
            *((U32 *)cur + 3));

        NIC_TRACE(TRACE_TYPE_RECV, "rxFrame1", 0x73900,
            *((U32 *)cur + 4),
            *((U32 *)cur + 5),
            *((U32 *)cur + 6),
            *((U32 *)cur + 7));

        len_wo_padding = frame_len;
        if (!frame_err) {

            /* check for extra padding */
            U32 eth_hlen = 0;
            U32 eth_type = ((eth_t *)cur)->ethFrameType;

	    if ((eth_type <= (ETH_MAX_LEN - ETH_HLEN)) ||
		(eth_type == ETH_JUMBO)) { /* 802.3 frame */
          
                if (is_global_flags(GF_RCB_RX_IEEE_SNAP_CKSUM) &&
                    (((eth_t *)cur)->eth_snap_type == ETH_IP) &&
                    is_llc_snap((U16 *)&((eth_t *)cur)->eth_llc_dsap)) {

                    eth_hlen = ETH_LLC_HLEN;
                }
            } else if (eth_type == ETH_IP) {
                eth_hlen = ETH_HLEN;
            }
            if (eth_hlen) { /* ip packet */
		/*
		 * if in 802.3 vlan mode,
		 * offset of ipProtocol is 35 from rcvd frame
		 *	18 (802.3 mac + vlan) + 8 (802.2 llc + snap) + 9
		 * which may wrap since only first 32 bytes rcvd are contiguous
		 */
		ip_t *ip = (ip_t *)((U32)cur + eth_hlen);
		U8 ip_proto, *cp = &ip->ipProtocol;
		RX_WRAP_BUF(cp);
		ip_proto = *cp;
		if ((ip_proto == IP_PROTO_TCP) || (ip_proto == IP_PROTO_UDP)) {
		    len_wo_padding = ip->ipLen + eth_hlen;
                }
            }
        }

        /* include crc if so requested by host */
        if (is_global_flags(GF_MOD_INCLUDE_CRC)) {

            if (len_wo_padding != frame_len) {
                /* checksum offload may be broken in this case
                 * if the received frame contain non-zero paddings
                 * before the crc field. give a warning.
                 */
                NIC_UTRACE("rxCrcPad", 0x74000,
                    frame_len,
                    len_wo_padding,
                    0,
                    0);
            }
            frame_len += 4; /* include crc */

        } else {

            #if (TRACE || TRACE_ALL)
            if (len_wo_padding != frame_len) {
                NIC_TRACE(TRACE_TYPE_RECV, "rxPadSm", 0x74100,
                    frame_len,
                    len_wo_padding,
                    0,
                    0);
            }
            #endif /* (TRACE || TRACE_ALL) */

            frame_len = len_wo_padding; /* use no padding length to dma */
        }

        /* 1. use jumbo BD if std BD len < frame len
         *         std  BD if min BD len < frame len <= std BD len
         *         mini BD if              frame len <= min BD len
         * 2. will try to use std BD if run out of jumbo BD or mini BD
         * 3. use of jumbo or mini BDs should not split frames
         */
        if (frame_len > nicfp->recv_std_buf_size) {
            if (is_global_flags(GF_CMD_JUMBO_FRAMES_ENABLED)) {
                ring_type = (is_global_flags(GF_RCB_USE_EXT_RECV_BD))?
                            EXT_JUMBO_RECV_RING:
                            JUMBO_RECV_RING;
            }
        } else if (frame_len <= nicfp->recv_mini_buf_size) {
            if (is_global_flags(GF_CMD_MINI_FRAMES_ENABLED)) {
                ring_type = MINI_RECV_RING;
            }
        }
    }

    /* move data back to host */
    while (frame_len) {
        /* note the priority */
        if (ring_type == STD_RECV_RING) {
        check_std_ring:

            if (nicfp->recv_cons_index !=
                nicfp->recv_prod_index_avail) { /* std BDs available */

                ring_index_ptr = &nicfp->recv_cons_index;
                ring_index = nicfp->recv_cons_index;
                ring_index_next = 
                    (ring_index + 1) & (RECV_STANDARD_RING_ENTRIES - 1);
                bdp = &nicp->recv_ring[ring_index];

                NIC_TRACE(TRACE_TYPE_RECV, "rxStd", 0x74200,
                    nicfp->recv_cons_index,
                    nicfp->recv_prod_index_avail,
                    nicfp->recv_return_prod_index,
                    frame_len);

            } else { /* no std BDs */
                NIC_UTRACE("rxNoStBd", 0x74300,
                    nicfp->recv_cons_index,
                    nicfp->recv_prod_index_avail,
                    nicfp->recv_return_prod_index,
                    frame_len);

                nicfp->stats.nicNoMoreRxBDs++;
                event_mask &= ~TG_FW_EVENT_MAC_RX_COMP;
                break;
            }

        } else if (ring_type == MINI_RECV_RING) {
            if (nicfp->recv_mini_cons_index !=
                nicfp->recv_mini_prod_index_avail) { /* have mini BDs */

                ring_index_ptr = &nicfp->recv_mini_cons_index;
                ring_index = nicfp->recv_mini_cons_index;
                ring_index_next =
                    (ring_index + 1) & (RECV_MINI_RING_ENTRIES - 1);
                bdp = &nicp->recv_mini_ring[ring_index];

                NIC_TRACE(TRACE_TYPE_RECV, "rxMini", 0x74400,
                    nicfp->recv_mini_cons_index,
                    nicfp->recv_mini_prod_index_avail,
                    nicfp->recv_return_prod_index,
                    frame_len);

            } else { /* no mini BDs */
                NIC_UTRACE("rxNoMiBd", 0x74500,
                    nicfp->recv_mini_cons_index,
                    nicfp->recv_mini_prod_index_avail,
                    nicfp->recv_return_prod_index,
                    frame_len);

                ring_type = STD_RECV_RING; /* try std recv ring */
                goto check_std_ring;
            }

        } else /*if (ring_type == JUMBO_RECV_RING)*/ {
            if (nicfp->recv_jumbo_cons_index !=
                nicfp->recv_jumbo_prod_index_avail) { /* have jumbo BDs */

                ring_index_ptr = &nicfp->recv_jumbo_cons_index;
                ring_index = nicfp->recv_jumbo_cons_index;
                ring_index_next =
                    (ring_index + 1) & (RECV_JUMBO_RING_ENTRIES - 1);
                if (ring_type == EXT_JUMBO_RECV_RING) {
                    ebdp = &nicp->ext_recv_jumbo_ring[ring_index];
                    bdp = (recv_bd_t *)&ebdp->bd1;
                    if (ebdp->BD_len < frame_len) {
                        ext_data_dma = TRUE;
                    }

                    NIC_TRACE(TRACE_TYPE_RECV, "rxExJumb", 0x74600,
                        nicfp->recv_jumbo_cons_index,
                        nicfp->recv_jumbo_prod_index_avail,
                        nicfp->recv_return_prod_index,
                        frame_len);
                } else {
                    bdp = &nicp->recv_jumbo_ring[ring_index];

                    NIC_TRACE(TRACE_TYPE_RECV, "rxJumbo", 0x74700,
                        nicfp->recv_jumbo_cons_index,
                        nicfp->recv_jumbo_prod_index_avail,
                        nicfp->recv_return_prod_index,
                        frame_len);
                }

            } else { /* no jumbo BDs */
                NIC_UTRACE("rxNoJmBd", 0x74800,
                    nicfp->recv_jumbo_cons_index,
                    nicfp->recv_jumbo_prod_index_avail,
                    nicfp->recv_return_prod_index,
                    frame_len);

                if (is_global_flags(GF_MOD_DONT_FRAG_JUMBOS)) {
                    nicfp->stats.nicNoMoreRxBDs++;
                    event_mask &= ~TG_FW_EVENT_MAC_RX_COMP;
                    break;

                } else {
                    nicfp->stats.nicJumboSpillOver++;
                    ring_type = STD_RECV_RING; /* try std recv ring */
                    goto check_std_ring;
                }
            }
        }

        /* setup nic addr */
        bdp->BD_nic_addr = cur;

        if (ext_data_dma) {
            U32 dma_to_host_data_state_not_clear_cksum;
            U32 n_exts;
            S32 len;

            /* for looping */
            ext_host_addr_p[0] = &ebdp->BD_host_addr;
            ext_host_addr_p[1] = &ebdp->BD_host_addr_1;
            ext_host_addr_p[2] = &ebdp->BD_host_addr_2;
            ext_host_addr_p[3] = &ebdp->BD_host_addr_3;
            ext_bd_len[0] = ebdp->BD_len;
            ext_bd_len[1] = ebdp->BD_len_1;
            ext_bd_len[2] = ebdp->BD_len_2;
            ext_bd_len[3] = ebdp->BD_len_3;

            NIC_TRACE(TRACE_TYPE_RECV, "rxExtBd0", 0x74900,
                *(U32 *)ebdp,           /* host addr 1 */
                *((U32 *)ebdp + 1),     /* host addr 1 */
                *((U32 *)ebdp + 2),     /* host addr 2 */
                *((U32 *)ebdp + 3));    /* host addr 2 */

            NIC_TRACE(TRACE_TYPE_RECV, "rxExtBd1", 0x75000,
                *((U32 *)ebdp + 4),     /* host addr 3 */
                *((U32 *)ebdp + 5),     /* host addr 3 */
                *((U32 *)ebdp + 6),     /* len 1 & 2 */
                *((U32 *)ebdp + 7));    /* len 3 & reserved */
                
            NIC_TRACE(TRACE_TYPE_RECV, "rxExtBd2", 0x75100,
                *((U32 *)ebdp + 8),     /* host addr 0 */ 
                *((U32 *)ebdp + 9),     /* host addr 0 */
                *((U32 *)ebdp + 10),    /* index & len 0 */
                *((U32 *)ebdp + 11));   /* type & flags */
                
            NIC_TRACE(TRACE_TYPE_RECV, "rxExtBd3", 0x75200,
                *((U32 *)ebdp + 12),    /* ip cksum & tcp/udp cksum */
                *((U32 *)ebdp + 13),    /* error flags & vlan tag */
                *((U32 *)ebdp + 14),    /* nic addr */
                *((U32 *)ebdp + 15));   /* opaque */
                
            /* setup dma length */
            len = frame_len;
            n_exts = 0;
            while (n_exts < N_EXTS) {
                if ((len -= ext_bd_len[n_exts++]) <= 0) {
                    ext_bd_len[n_exts - 1] += len; /* dma len for the last */
                    break;
                }
            }
            ASSERT(len <= 0); /* inform driver */

            NIC_TRACE(TRACE_TYPE_RECV, "rxQDma", 0x75300,
                ebdp,
                n_exts,
                frame_len,
                ring_index);

            /* check free dma descriptors for buf data */
            if (!can_q_dma_to_host(n_exts)) {
		set_global_flags(GF_VAR_MAC_RX_COMP_SUSP);
		event_mask &= ~TG_FW_EVENT_MAC_RX_COMP;

                NIC_UTRACE("rxCkDmaF", 0x75300,
                    trp->host_dma_assist.write_chan_producer,
                    trp->host_dma_assist.write_chan_consumer,
                    trp->host_dma_assist.write_chan_ref,
                    n_exts + 1);
                break;
            }

	    /* set clean cksum flag on the first dma only */
            dma_to_host_data_state_not_clear_cksum =
                nicfp->dma_to_host_data_state & ~TG_DMA_STATE_CLEAR_CKSUM;

            for (i = 0; i < n_exts; i++) {

                NIC_TRACE(TRACE_TYPE_RECV, "rxQDmaEx", 0x75400,
                    i,
                    ext_bd_len[i],
                    cur,
                    *ext_host_addr_p[i]);

                if (!(*nicfp->q_dma_to_host_stub)
                        (*ext_host_addr_p[i],
                        cur,
                        ext_bd_len[i],
                        (i == (n_exts - 1))?
                            TIGON_TYPE_RECV_DATA_LAST:
                            TIGON_TYPE_RECV_DATA,
                        ring_index | EXT_JUMBO_BD_FLAG,
                        (i)?
                            dma_to_host_data_state_not_clear_cksum:
                            nicfp->dma_to_host_data_state)) {

                    NIC_UTRACE("rxQDmExF", 0x75500,
                        i,
                        ext_bd_len[i],
                        cur,
                        *ext_host_addr_p[i]);
                    ASSERT(0); /* should not happen */
                    break;
                }

                cur += ext_bd_len[i];
                RX_WRAP_BUF(cur);
            }

            /* setup to dma BD */
            max_len = frame_len;

        } else {
            bd_len = bdp->BD_len;
            max_len = (frame_len <= bd_len)? frame_len: bd_len;

            NIC_TRACE(TRACE_TYPE_RECV, "rxBd0", 0x75600,
                *(U32 *)bdp,            /* host addr */
                *((U32 *)bdp + 1),      /* host addr */
                *((U32 *)bdp + 2),      /* index & len */
                *((U32 *)bdp + 3));     /* type & flags */

            NIC_TRACE(TRACE_TYPE_RECV, "rxBd1", 0x75700,
                *((U32 *)bdp + 4),      /* ip cksum & tcp cksum */
                *((U32 *)bdp + 5),      /* error flags & vlan tag */
                *((U32 *)bdp + 6),      /* nic addr */
                *((U32 *)bdp + 7));     /* opaque */

            NIC_TRACE(TRACE_TYPE_RECV, "rxLen", 0x75800,
                cur,
                max_len,
                frame_len,
                bd_len);

            /* dma data to host */
            if (!(*nicfp->q_dma_to_host_stub)
                    (bdp->BD_host_addr,
                    cur,
                    max_len,
                    TIGON_TYPE_RECV_DATA_LAST,
                    ring_index |
                        ring_type_2_flag(ring_type) |
                        (nicfp->rx_frame_split? FRAME_SPLIT_FLAG: 0),
                    nicfp->dma_to_host_data_state)) {

                NIC_UTRACE("rxQDmaF", 0x75900,
                    bdp->BD_host_addr,
                    bdp->BD_nic_addr,
                    max_len,
                    ring_index);
                break;
            }
        }

        bd_flags = bdp->BD_flags;

        /* report vlan info in the first BD */
        if (vlan_bit && !nicfp->rx_frame_split) {
            bd_flags        |= vlan_bit;
            bdp->BD_vlan_tag = vlan_tag;
        }

        /* report frame error on all BDs */
        if (md_w1 & TG_MAC_RX_W1_STATUS_MASK) {
            bd_flags           |= BD_FLAG_FRAME_HAS_ERROR;
            bdp->BD_error_flags = (md_w1 >> BD_ERR_SHIFT);
        }

        bdp->BD_len = max_len;
        if (max_len == frame_len) {
            bdp->BD_flags = bd_flags | BD_FLAG_END;
            nicfp->rx_frame_split = FALSE;
            frame_len = 0;
        } else {
            bdp->BD_flags = bd_flags;
            nicfp->rx_frame_split = TRUE;
            frame_len -= max_len;
            cur += max_len;
            RX_WRAP_BUF(cur);
        }

        /* This entry has been consumed */
        *ring_index_ptr = ring_index_next;
    }

    if (frame_len) { /* we run out of resources */
        if (nicfp->rx_frame_split) {

            /* save info to continue later */
            nicfp->rx_frame_cur = cur;
            nicfp->rx_frame_len = frame_len;
            nicfp->rx_md_w1     = md_w1;

        } else if (vlan_bit) {
            restore_vlan_tag(hd.w0, vlan_tag);
        }

	/* increase <rx no resource> stats */
	nicfp->stats.nicNoMoreWrDMADescriptors++;

        NIC_TRACE(TRACE_TYPE_RECV, "rxNoRes", 0x76000,
            ring_type,
            nicfp->recv_return_prod_index,
            0,
            0);

    } else {
#if (USE_U32_STAT_CNT)
        nicfp->ifHCInOctets += ((md_w1 & TG_MAC_RX_W1_LEN_MASK) - 4);
#else /* (USE_U32_STAT_CNT) */
        nicfp->stats.ifHCInOctets += ((md_w1 & TG_MAC_RX_W1_LEN_MASK) - 4);
#endif /* (USE_U32_STAT_CNT) */
        (*nicfp->deq_mac_rx_stub)();
    }

    RETURN(TG_FW_EVENT_NUM_MAC_RX_COMP);
}

/* null routine to assist with scratchpad setup */
void h_mac_rx_comp_cksum_end(void) {}

void
h_recv_bd_ready(void)
{
    U32 next_index;
    S32 nentries;
    S32 space;
    S32 to_move;

    if (nicfp->recv_prod_index_queued != nicfp->recv_prod_index) {

        /* how many new entries to move */
        nentries = nicfp->recv_prod_index - nicfp->recv_prod_index_queued;
        if (nentries < 0) {
            nentries += RECV_STANDARD_RING_ENTRIES;
        }

        /* how much space in our recv_ring until wrap */
        space = (nicfp->recv_cons_index <= nicfp->recv_prod_index_queued)?
                  (RECV_STANDARD_RING_ENTRIES - nicfp->recv_prod_index_queued):
                  (nicfp->recv_cons_index - nicfp->recv_prod_index_queued - 1);

        ASSERT(space != 0);

        NIC_TRACE(TRACE_TYPE_RECV_BD, "#rBdRdy", 0x76100,
            nicfp->recv_prod_index,
            nicfp->recv_prod_index_queued,
            nicfp->recv_cons_index,
            (nentries << 16) | space);

        to_move = MIN(space, nentries);

        next_index =
            (nicfp->recv_prod_index_queued + to_move) &
            (RECV_STANDARD_RING_ENTRIES - 1);

        if (!(*nicfp->q_dma_to_nic_stub)
                (nicfp->recv_ring_ptr +
                    (nicfp->recv_prod_index_queued * sizeof(struct recv_bd)),
                (U32)&nicp->recv_ring[nicfp->recv_prod_index_queued],
                sizeof(struct recv_bd) * to_move,
                TIGON_TYPE_RECV_BD,
                next_index,
                nicfp->dma_to_nic_bd_state)) {

            #if (TRACE | TRACE_ALL)
            static U32 fail_cnt = 0;
            #endif /* (TRACE | TRACE_ALL) */

            NIC_TRACE(TRACE_TYPE_RECV_BD, "QrBdF", 0x76200,
                to_move,
                ++fail_cnt,
                0,
                0);
        } else {
            nicfp->recv_prod_index_queued = next_index;

            NIC_TRACE(TRACE_TYPE_RECV_BD, "QrBdOk", 0x76300,
                to_move,
                0,
                0,
                0);
        }
    } else {
        #if (TRACE | TRACE_ALL)
        static U32 fail_cnt = 0;
        #endif /* (TRACE | TRACE_ALL) */

        /* spurious recv bd producer update from driver */
        NIC_TRACE(TRACE_TYPE_RECV_BD, "#rBdRdy?", 0x76400,
            nicfp->recv_prod_index,
            nicfp->recv_prod_index_queued,
            nicfp->recv_cons_index,
            ++fail_cnt);
    }

    if (nicfp->recv_prod_index_queued == nicfp->recv_prod_index) {
        clear_event_reg(TG_FW_EVENT_RECV_BD);
    }

    RETURN(TG_FW_EVENT_NUM_RECV_BD);
}

/* null routine to assist with scratchpad setup */
void h_recv_bd_ready_end(void) {}

void
h_recv_jumbo_bd_ready(void)
{
    U32 next_index;
    S32 nentries;
    S32 space;
    S32 to_move;
    U32 rc;

    if (nicfp->recv_jumbo_prod_index_queued != nicfp->recv_jumbo_prod_index) {

        /* how many new entries to move */
        nentries =
            nicfp->recv_jumbo_prod_index - nicfp->recv_jumbo_prod_index_queued;
        if (nentries < 0) {
            nentries += RECV_JUMBO_RING_ENTRIES;
        }

        /* How much space in our recv_ring until the wrap */
        space =
	    (nicfp->recv_jumbo_cons_index <= 
	     nicfp->recv_jumbo_prod_index_queued) ?
	    (RECV_JUMBO_RING_ENTRIES - nicfp->recv_jumbo_prod_index_queued) :
	    (nicfp->recv_jumbo_cons_index - 
	     nicfp->recv_jumbo_prod_index_queued - 1);

        ASSERT(space != 0);

        NIC_TRACE(TRACE_TYPE_RECV_BD, "#rJBdRdy", 0x76500,
            nicfp->recv_jumbo_prod_index,
            nicfp->recv_jumbo_prod_index_queued,
            nicfp->recv_jumbo_cons_index,
            (nentries << 16) | space);

        to_move = MIN(space, nentries);

        next_index =
            (nicfp->recv_jumbo_prod_index_queued + to_move) &
            (RECV_JUMBO_RING_ENTRIES - 1);

        if (is_global_flags(GF_RCB_USE_EXT_RECV_BD)) {
            rc = (*nicfp->q_dma_to_nic_stub)
                    (nicfp->recv_jumbo_ring_ptr +
                        (nicfp->recv_jumbo_prod_index_queued *
                        sizeof(ext_recv_bd_t)),
                    (U32)&nicp->ext_recv_jumbo_ring
                        [nicfp->recv_jumbo_prod_index_queued],
                    sizeof(ext_recv_bd_t) * to_move,
                    TIGON_TYPE_RECV_JUMBO_BD,
                    next_index,
                    nicfp->dma_to_nic_bd_state);
        } else {
            rc = (*nicfp->q_dma_to_nic_stub)
                    (nicfp->recv_jumbo_ring_ptr +
                        (nicfp->recv_jumbo_prod_index_queued *
                        sizeof(recv_bd_t)),
                    (U32)&nicp->recv_jumbo_ring
                        [nicfp->recv_jumbo_prod_index_queued],
                    sizeof(recv_bd_t) * to_move,
                    TIGON_TYPE_RECV_JUMBO_BD,
                    next_index,
                    nicfp->dma_to_nic_bd_state);
        }

        if (!rc) {
            #if (TRACE | TRACE_ALL)
            static U32 fail_cnt = 0;
            #endif /* (TRACE | TRACE_ALL) */

            NIC_TRACE(TRACE_TYPE_RECV_BD, "QrJBdF", 0x76600,
                to_move,
                ++fail_cnt,
                0,
                0);
        } else {
            nicfp->recv_jumbo_prod_index_queued = next_index;

            NIC_TRACE(TRACE_TYPE_RECV_BD, "QrJBdOk", 0x76700,
                to_move,
                0,
                0,
                0);
        }

    } else {
        #if (TRACE | TRACE_ALL)
        static U32 fail_cnt = 0;
        #endif /* (TRACE | TRACE_ALL) */

        /* spurious recv jumbo bd producer update from driver */
        NIC_TRACE(TRACE_TYPE_RECV_BD, "#rJBdRy?", 0x76800,
            nicfp->recv_jumbo_prod_index,
            nicfp->recv_jumbo_prod_index_queued,
            nicfp->recv_jumbo_cons_index,
            ++fail_cnt);
    }

    if (nicfp->recv_jumbo_prod_index_queued == nicfp->recv_jumbo_prod_index) {
        clear_event_reg(TG_FW_EVENT_RECV_JUMBO_BD);
    }

    RETURN(TG_FW_EVENT_NUM_RECV_JUMBO_BD);
}

/* null routine to assist with scratchpad setup */
void h_recv_jumbo_bd_ready_end(void) {}

void
h_recv_mini_bd_ready(void)
{
    U32 next_index;
    S32 nentries;
    S32 space;
    S32 to_move;

    if (nicfp->recv_mini_prod_index_queued != nicfp->recv_mini_prod_index) {

        /* how many new entries to move */
        nentries =
            nicfp->recv_mini_prod_index - nicfp->recv_mini_prod_index_queued;
        if (nentries < 0) {
            nentries += RECV_MINI_RING_ENTRIES;
        }

        /* How much space in our recv_ring until the wrap */
        space =
          (nicfp->recv_mini_cons_index <= nicfp->recv_mini_prod_index_queued) ?
	    (RECV_MINI_RING_ENTRIES - nicfp->recv_mini_prod_index_queued) :
	    (nicfp->recv_mini_cons_index - 
	     nicfp->recv_mini_prod_index_queued - 1);

        ASSERT(space != 0);

        NIC_TRACE(TRACE_TYPE_RECV_BD, "#rMBdRdy", 0x76900,
            nicfp->recv_mini_prod_index,
            nicfp->recv_mini_prod_index_queued,
            nicfp->recv_mini_cons_index,
            (nentries << 16) | space);

        to_move = MIN(space, nentries);

        next_index =
            (nicfp->recv_mini_prod_index_queued + to_move) &
            (RECV_MINI_RING_ENTRIES - 1);

        if (!(*nicfp->q_dma_to_nic_stub)
                (nicfp->recv_mini_ring_ptr +
                    (nicfp->recv_mini_prod_index_queued *
                    sizeof(struct recv_bd)),
                (U32)&nicp->recv_mini_ring[nicfp->recv_mini_prod_index_queued],
                sizeof(struct recv_bd) * to_move,
                TIGON_TYPE_RECV_MINI_BD,
                next_index,
                nicfp->dma_to_nic_bd_state)) {

            #if (TRACE | TRACE_ALL)
            static U32 fail_cnt = 0;
            #endif /* (TRACE | TRACE_ALL) */

            NIC_TRACE(TRACE_TYPE_RECV_BD, "QrMBdF", 0x77000,
                to_move,
                ++fail_cnt,
                0,
                0);
        } else {
            nicfp->recv_mini_prod_index_queued = next_index;

            NIC_TRACE(TRACE_TYPE_RECV_BD, "QrMBdOk", 0x77100,
                to_move,
                0,
                0,
                0);
        }
    } else {
        #if (TRACE | TRACE_ALL)
        static U32 fail_cnt = 0;
        #endif /* (TRACE | TRACE_ALL) */

        /* spurious recv mini bd producer update from driver */
        NIC_TRACE(TRACE_TYPE_RECV_BD, "#rMBdRy?", 0x77200,
            nicfp->recv_mini_prod_index,
            nicfp->recv_mini_prod_index_queued,
            nicfp->recv_mini_cons_index,
            ++fail_cnt);
    }

    if (nicfp->recv_mini_prod_index_queued == nicfp->recv_mini_prod_index) {
        clear_event_reg(TG_FW_EVENT_RECV_MINI_BD);
    }

    RETURN(TG_FW_EVENT_NUM_RECV_MINI_BD);
}

/* null routine to assist with scratchpad setup */
void h_recv_mini_bd_ready_end(void) {}

void
h_dma_wr_assist(void)
{
    struct tg_dma_descr *ddp;
    dma_ref_ring_t *ref_cons;
    U32 dd_type;

    nicfp->wr_dma_free_cnt++;
    ddp = trp->host_dma_assist.write_chan_ref;

    NIC_TRACE_IF(TRACE_TYPE_DMA, "#wDmaCp", 0x77300,
                 ddp->type, TIGON_TYPE_RX,
        trp->host_dma_assist.write_chan_producer,
        trp->host_dma_assist.write_chan_consumer,
        ddp,
        (ddp - tdp->wr_dma_ring));

    NIC_TRACE_IF(TRACE_TYPE_DMA, "wDmaCpD0", 0x77500,
                 ddp->type, TIGON_TYPE_RX,
        *(U32 *)ddp,            /* host addr */
        *((U32 *)ddp + 1),      /* host addr */
        *((U32 *)ddp + 2),      /* nic addr */
        *((U32 *)ddp + 3));     /* reserved & len */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "wDmaCpD1", 0x77600,
                 ddp->type, TIGON_TYPE_RX,
        *((U32 *)ddp + 4),      /* state */
        *((U32 *)ddp + 5),      /* reserved & cksum addr */
        *((U32 *)ddp + 6),      /* type */
        *((U32 *)ddp + 7));     /* index */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "wDmaAvd", 0x77400,
                 ddp->type, TIGON_TYPE_RX,
        nicfp->wr_dma_ref_prod,
        nicfp->wr_dma_ref_cons,
        nicfp->wr_dma_ref_cons->count, 
        nicfp->wr_dma_ref_cons->type);

    /* set up ref for next event */
    ASSERT(nicfp->wr_dma_ref_cons != nicfp->wr_dma_ref_prod);
    nicfp->wr_dma_ref_cons = ref_cons
			   = get_next_wr_dma_ref(nicfp->wr_dma_ref_cons);
    ref_cons->type = TIGON_TYPE_NULL; /* invalidate entry */

    if (ref_cons == nicfp->wr_dma_ref_prod) {
	/*
	 * no more dma completion to take
	 * bump ref pointer since it is one behind the prod pointer
	 */
	trp->host_dma_assist.write_chan_ref = ddp + 1;

    } else {
	/* move ref pointer to one behind the next producer */
	ref_cons = get_next_wr_dma_ref(nicfp->wr_dma_ref_cons);
	trp->host_dma_assist.write_chan_ref = ddp + ref_cons->count;
	ref_cons->type = TIGON_TYPE_NULL; /* invalidate entry */
    }

    /* WARNING:
     * a jump table is generated by the following switch statement
     * ie, all code segments after the switch statement will still
     * run on the external memory even current function is relocated
     * into scratch pad
     */
    dd_type = ddp->type;
    if (dd_type & WR_DMA_GROUP_1) {

        if (((dd_type & TIGON_TYPE_TX_CONS) &&
                !is_global_flags(GF_MOD_DONT_RUPT_SENDS)) ||
            ((dd_type & TIGON_TYPE_RX_PROD) &&
                !is_global_flags(GF_MOD_DONT_RUPT_RECVS)) ||
            ((dd_type & TIGON_TYPE_EVENT_PROD) &&
                !is_global_flags(GF_MOD_DONT_RUPT_EVENTS))) {

            interrupt_host_w_avoidance();
        }

    } else if (dd_type & (TIGON_TYPE_RECV_BD |
                          TIGON_TYPE_RECV_JUMBO_BD |
                          TIGON_TYPE_RECV_BD_LAST |
                          TIGON_TYPE_RECV_DATA |
                          TIGON_TYPE_RECV_DATA_LAST |
                          TIGON_TYPE_EVENT |
                          TIGON_TYPE_UPDATE_STATS |
                          TIGON_TYPE_TX_CONS_NO_INTR)) {
        ; /* nothing to do */

    } else {
        ASSERT(0);
    }

    NIC_TRACEV_IF(TRACE_TYPE_DMA, "wDmCpOk", 0x77700,
                  ddp->type, TIGON_TYPE_RX,
        trp->host_dma_assist.write_chan_producer,
        trp->host_dma_assist.write_chan_consumer,
        trp->host_dma_assist.write_chan_ref,
        ddp);

    RETURN(TG_FW_EVENT_NUM_DMA_WR_ASST_HI);
}

/* null routine to assist with scratchpad setup */
void h_dma_wr_assist_end(void) {}

void
h_dma_wr_assist_cksum(void)
{
    struct tg_dma_descr *ddp;
    U32 dd_type;

    nicfp->wr_dma_free_cnt++;
    ddp = trp->host_dma_assist.write_chan_ref;

    /* Move the reference at this point forward, so as to make
     * room for the next entry to be enqued on he DMA Ring.
     * If this is not done here, then we may find that we do not
     * have room to enque entry when in-fact there is at least one free 
     * entry on the ring.
     */
    trp->host_dma_assist.write_chan_ref = ddp + 1;

    NIC_TRACE_IF(TRACE_TYPE_DMA, "#wDmaCp", 0x77800,
                 ddp->type, TIGON_TYPE_RX,
        trp->host_dma_assist.write_chan_producer,
        trp->host_dma_assist.write_chan_consumer,
        ddp,
        (ddp - tdp->wr_dma_ring));

    NIC_TRACE_IF(TRACE_TYPE_DMA, "wDmaD0", 0x77900,
                 ddp->type, TIGON_TYPE_RX,
        *(U32 *)ddp,            /* host addr */
        *((U32 *)ddp + 1),      /* host addr */
        *((U32 *)ddp + 2),      /* nic addr */
        *((U32 *)ddp + 3));     /* reserved & len */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "wDmaD1", 0x78000,
                 ddp->type, TIGON_TYPE_RX,
        *((U32 *)ddp + 4),      /* state */
        *((U32 *)ddp + 5),      /* reserved & cksum addr */
        *((U32 *)ddp + 6),      /* type */
        *((U32 *)ddp + 7));     /* index */

    /* NOTE:
     * a jump table will be generated if switch (ddp->type) is used
     * ie, all code segments after the switch statement will still
     * run on the external memory even current function is relocated
     * into scratch pad
     * hence, if...else if...else statements are used here
     * and are ordered from most frequent to least.
     */
    dd_type = ddp->type;
    if (dd_type & WR_DMA_GROUP_1) {

        if (((dd_type & TIGON_TYPE_TX_CONS) &&
                !is_global_flags(GF_MOD_DONT_RUPT_SENDS)) ||
            ((dd_type & TIGON_TYPE_RX_PROD) &&
                !is_global_flags(GF_MOD_DONT_RUPT_RECVS)) ||
            ((dd_type & TIGON_TYPE_EVENT_PROD) &&
                !is_global_flags(GF_MOD_DONT_RUPT_EVENTS))) {

            interrupt_host_w_avoidance();
        }

    } else if (dd_type & TIGON_TYPE_RECV_DATA_LAST) {
        struct ext_recv_bd *ebdp;
        struct recv_bd *bdp;
        U32 dd_index;
        U32 dd_cksum;
        U32 ip_cksum;
        U32 tcpudp_cksum;
        U32 pseudo;
        U32 bd_type;
        U32 bd_flags;
        U32 frame_start;
        U32 frame_split_bit;
        U32 ring_type;
        U32 eth_hlen;
        U16 *buf;
	U8 *cp;
        U16 eth_type;
        U8  ip_proto;

        /* extract frame info from index field */
        dd_index = ddp->index;
        frame_split_bit = dd_index & FRAME_SPLIT_FLAG;
        ring_type = dd_index >> RING_FLAG_SHIFT;
        dd_index &= 0xffff; /* recv std/jumbo cons index */
        dd_cksum = ddp->cksum & 0xffff;

        /* note the priority */
        ebdp = 0;
        if (ring_type == STD_RECV_RING) {
            bdp = &nicp->recv_ring[dd_index];

            NIC_TRACE(TRACE_TYPE_RECV, "rSmStBd0", 0x78100,
                *(U32 *)bdp,            /* host addr */
                *((U32 *)bdp + 1),      /* host addr */
                *((U32 *)bdp + 2),      /* index & len */
                *((U32 *)bdp + 3));     /* type & flags */

            NIC_TRACE(TRACE_TYPE_RECV, "rSmStBd1", 0x78200,
                *((U32 *)bdp + 4),      /* ip cksum & tcp/udp cksum */
                *((U32 *)bdp + 5),      /* error flags and vlan tag */
                *((U32 *)bdp + 6),      /* nic addr */
                *((U32 *)bdp + 7));     /* opague */

        } else if (ring_type == MINI_RECV_RING) {
            bdp = &nicp->recv_mini_ring[dd_index];

            NIC_TRACE(TRACE_TYPE_RECV, "rSmMiBd0", 0x78300,
                *(U32 *)bdp,            /* host addr */
                *((U32 *)bdp + 1),      /* host addr */
                *((U32 *)bdp + 2),      /* index & len */
                *((U32 *)bdp + 3));     /* type & flags */

            NIC_TRACE(TRACE_TYPE_RECV, "rSmMiBd1", 0x78400,
                *((U32 *)bdp + 4),      /* ip cksum & tcp/udp cksum */
                *((U32 *)bdp + 5),      /* error flags and vlan tag */
                *((U32 *)bdp + 6),      /* nic addr */
                *((U32 *)bdp + 7));     /* opague */

        } else if (ring_type == JUMBO_RECV_RING) {
            bdp = &nicp->recv_jumbo_ring[dd_index];

            NIC_TRACE(TRACE_TYPE_RECV, "rSmJuBd0", 0x78500,
                *(U32 *)bdp,            /* host addr */
                *((U32 *)bdp + 1),      /* host addr */
                *((U32 *)bdp + 2),      /* index & len */
                *((U32 *)bdp + 3));     /* type & flags */

            NIC_TRACE(TRACE_TYPE_RECV, "rSmJuBd1", 0x78600,
                *((U32 *)bdp + 4),      /* ip cksum & tcp/udp cksum */
                *((U32 *)bdp + 5),      /* error flags and vlan tag */
                *((U32 *)bdp + 6),      /* nic addr */
                *((U32 *)bdp + 7));     /* opague */

        } else /*if (ring_type == EXT_JUMBO_RECV_RING)*/ {
            ebdp = &nicp->ext_recv_jumbo_ring[dd_index];
            bdp = (struct recv_bd *)&ebdp->bd1;

            NIC_TRACE(TRACE_TYPE_RECV, "rSmExBd0", 0x78700,
                *(U32 *)ebdp,           /* host addr 1 */
                *((U32 *)ebdp + 1),     /* host addr 1 */
                *((U32 *)ebdp + 2),     /* host addr 2 */
                *((U32 *)ebdp + 3));    /* host addr 2 */

            NIC_TRACE(TRACE_TYPE_RECV, "rSmExBd1", 0x78800,
                *((U32 *)ebdp + 4),     /* host addr 3 */
                *((U32 *)ebdp + 5),     /* host addr 3 */
                *((U32 *)ebdp + 6),     /* len 1 & 2 */
                *((U32 *)ebdp + 7));    /* len 3 & reserved */
                
            NIC_TRACE(TRACE_TYPE_RECV, "rSmExBd2", 0x78900,
                *((U32 *)ebdp + 8),     /* host addr 0 */ 
                *((U32 *)ebdp + 9),     /* host addr 0 */
                *((U32 *)ebdp + 10),    /* index & len 0 */
                *((U32 *)ebdp + 11));   /* type & flags */
                
            NIC_TRACE(TRACE_TYPE_RECV, "rSmExBd3", 0x78950,
                *((U32 *)ebdp + 12),    /* ip cksum & tcp/udp cksum */
                *((U32 *)ebdp + 13),    /* error flags & vlan tag */
                *((U32 *)ebdp + 14),    /* nic addr */
                *((U32 *)ebdp + 15));   /* opaque */
        }

        /* clear cksum flags */
        bd_flags = bdp->BD_flags & ~(BD_FLAG_IP_CKSUM | BD_FLAG_TCP_UDP_CKSUM);

	#if defined(PASS_IP_ID) /* for host cksum debug */
        {
            static U16 ip_id = 0xdead;

            frame_start = bdp->BD_nic_addr;
            if (frame_split_bit) {
                bdp->BD_vlan_tag = ip_id;
            } else {
                eth_hlen = 0;
                eth_type = ((eth_t *)frame_start)->ethFrameType;

		if ((eth_type <= (ETH_MAX_LEN - ETH_HLEN)) ||
		    (eth_type == ETH_JUMBO)) { /* 802.3 frame */

                    if (is_global_flags(GF_RCB_RX_IEEE_SNAP_CKSUM) &&
                        (((eth_t *)frame_start)->eth_snap_type == ETH_IP) &&
                        is_llc_snap((U16 *)&((eth_t *)frame_start)->
                                                            eth_llc_dsap)) {
                        eth_hlen = ETH_LLC_HLEN:
                    }
                } else if (eth_type == ETH_IP) {
                    eth_hlen = ETH_HLEN;
                }
                if (eth_hlen) { /* ip packet */
                    ip_id = ((ip_t *)(frame_start + eth_hlen))->ipId;
                    bdp->BD_vlan_tag = ip_id;
                } else {
                    ip_id = 0xdead;
                    bdp->BD_vlan_tag = 0;
                }
            }
            #if 0
            NIC_UTRACE("ipID", 0x79000,
                bdp->BD_vlan_tag,
                frame_split_bit,
                0,
                0);
            #endif /*0*/
        }
	#endif /* defined(PASS_IP_ID) */

        if (bd_flags & BD_FLAG_FRAME_HAS_ERROR) {
            /* skip cksum calculate for an error frame */
            tcpudp_cksum = 0; /* eliminate compiler warning */

            NIC_TRACE(TRACE_TYPE_RECV, "rSmErrBd", 0x79100,
                dd_index,
                bd_flags,
                0,
                0);
        } else if (frame_split_bit) {
            /* just send hw cksum to host for further processing */
            tcpudp_cksum = (U16)dd_cksum;

            NIC_TRACE(TRACE_TYPE_RECV, "rSmSptBd", 0x79200,
                dd_index,
                bd_flags,
                0,
                0);
        } else {
            /* first 32 bytes received are contiguous */
            tcpudp_cksum = 0;
            frame_start = bdp->BD_nic_addr;

            /* to support 802.3 frame */
            eth_hlen = 0;
            eth_type = ((eth_t *)frame_start)->ethFrameType;

	    if ((eth_type <= (ETH_MAX_LEN - ETH_HLEN)) ||
		(eth_type == ETH_JUMBO)) { /* 802.3 frame */

                if (is_global_flags(GF_RCB_RX_IEEE_SNAP_CKSUM) &&
                    (((eth_t *)frame_start)->eth_snap_type == ETH_IP) &&
                   is_llc_snap((U16 *)&((eth_t *)frame_start)->eth_llc_dsap)) {

                    eth_hlen = ETH_LLC_HLEN;

                   NIC_TRACE(TRACE_TYPE_RECV, "rSmLLC0", 0x79300,
                        *(U32 *)frame_start,
                        *((U32 *)frame_start + 1),
                        *((U32 *)frame_start + 2),
                        *((U32 *)frame_start + 3));

                   NIC_TRACE(TRACE_TYPE_RECV, "rSmLLC1", 0x79400,
                        *((U32 *)frame_start + 4),
                        *((U32 *)frame_start + 5),
                        *((U32 *)frame_start + 6),
                        *((U32 *)frame_start + 7));
                }
            } else if (eth_type == ETH_IP) {
                eth_hlen = ETH_HLEN;

                NIC_TRACE(TRACE_TYPE_RECV, "rSmEthr0", 0x79500,
                    *(U32 *)frame_start,
                    *((U32 *)frame_start + 1),
                    *((U32 *)frame_start + 2),
                    *((U32 *)frame_start + 3));

                NIC_TRACE(TRACE_TYPE_RECV, "rSmEthr0", 0x79550,
                    *((U32 *)frame_start + 4),
                    *((U32 *)frame_start + 5),
                    *((U32 *)frame_start + 6),
                    *((U32 *)frame_start + 7));
            }

            /* handle cksum if it is an ip packet */
            if (eth_hlen) { /* ip packet */

                U32 ip_start = frame_start + eth_hlen;
                U32 ip_hlen = IP_HLEN(ip_start);
                U32 tcpudp_start = ip_start + ip_hlen; /* may wrap */

#if (RX_CKSUM_DEBUG)
                {
                    static U32 rx_ip_id_unmatch = 0;
                    static U16 rx_ip_id = 0xdead;

                    if (((ip_t *)ip_start)->ipFragOffset & IP_FRAG_OFS_MASK) {
                        if (((ip_t *)ip_start)->ipId != rx_ip_id) {
                            U32 *w;
                            U16 *h = (U16 *)ip_start;
                            NIC_UTRACE("?rxIpId0", rx_ip_id,
                                (*(h)     << 16) + *(h + 1),
                                (*(h + 2) << 16) + *(h + 3),
                                (*(h + 4) << 16) + *(h + 5),
                                (*(h + 6) << 16) + *(h + 7));
                            w = (U32 *)bdp;
                            NIC_UTRACE("?rxIpId1", dd_index,
                                *w,
                                *(w + 1),
                                *(w + 2),
                                *(w + 3));
                            NIC_UTRACE("?rxIpId2", ++rx_ip_id_unmatch,
                                *(w + 4),
                                *(w + 5),
                                *(w + 6),
                                *(w + 7));
                        }
                    } else {
                        rx_ip_id = ((ip_t *)ip_start)->ipId;
                        {
                            U16 *h = (U16 *)ip_start;
                            NIC_UTRACE("rxIpId_0", rx_ip_id,
                                (*(h)     << 16) + *(h + 1),
                                (*(h + 2) << 16) + *(h + 3),
                                (*(h + 4) << 16) + *(h + 5),
                                (*(h + 6) << 16) + *(h + 7));
                        }
                    }
                }
#endif /* (RX_CKSUM_DEBUG) */

                /* ip cksum */
                if (is_global_flags(GF_RCB_RX_IP_CKSUM)) {

                    /* do quick calculation if no wrap */
                    if (tcpudp_start <= NIC_RX_BUF_END) {

                        ip_cksum = 0;
                        buf = (U16 *)ip_start;
                        do {
                            ip_cksum += *buf++;
                        } while ((U32)buf < tcpudp_start);

                        ip_cksum = (ip_cksum >> 16) + (ip_cksum & 0xffff);
                        ip_cksum = (ip_cksum >> 16) + (ip_cksum & 0xffff);
                    } else {
                        ip_cksum = (*nicfp->cksum16_stub)
                                        ((U16 *)ip_start, (ip_hlen >> 1),
                                         NIC_RX_BUF_END, NIC_RX_BUF_SIZE);
                    }

                    if (ip_cksum == 0)
                        ip_cksum = 0xffff;

                    /* set ip cksum flag */
                    bd_flags |= BD_FLAG_IP_CKSUM;
                } else {
                    /* clear it if ip cksum is not required */
                    ip_cksum = 0;
                }

                /* store ip cksum */
                bdp->BD_ip_cksum = (U16)ip_cksum;

                NIC_TRACE(TRACE_TYPE_RECV, "rSmIpSm", 0x79600, 
                    dd_index,
                    bdp->BD_w4, 
                    dd_cksum,
                    ip_cksum);

                /* tcp or udp cksum */
                if (is_global_flags(GF_RCB_RX_TCP_UDP_CKSUM)) {
		    /*
		     * if in 802.3 vlan mode,
		     * offset of ipProtocol is 35 from rcvd frame
		     *     18 (802.3 mac + vlan) + 8 (802.2 llc + snap) + 9
		     * which may wrap since only first 32 bytes rcvd
		     * are contiguous 
		     */
		    cp = &((ip_t *)ip_start)->ipProtocol;
		    RX_WRAP_BUF(cp);
		    ip_proto = *cp;
		    if ((ip_proto == IP_PROTO_TCP) ||
			(ip_proto == IP_PROTO_UDP)) {

			U32 eth_cksum = 0;

			/* ether header cksum */
			buf = (U16 *)frame_start;
			if (eth_type == ETH_IP) {
			    /* unroll the loop if we know in advance */
			    eth_cksum += *buf++;
			    eth_cksum += *buf++;
			    eth_cksum += *buf++;
			    eth_cksum += *buf++;
			    eth_cksum += *buf++;
			    eth_cksum += *buf++;
			    eth_cksum += *buf++;
			} else {
			    do {
				eth_cksum += *buf++;
			    } while ((U32)buf < ip_start);
			}
			eth_cksum = (eth_cksum >> 16) + (eth_cksum & 0xffff);
			eth_cksum = (eth_cksum >> 16) + (eth_cksum & 0xffff);

			/* pseudo header cksum */
			if (is_global_flags(GF_RCB_RX_NO_PSEUDO_HDR_CKSUM)) {
			    pseudo = 0;

			} else {
			    /*
			     * if in 802.3 vlan mode,
			     * offset of ipFragOffset is 32 from rcvd frame
			     * 18 (802.3 mac + vlan) + 8 (802.2 llc + snap) + 6
			     * which may wrap since only first 32 bytes rcvd
			     * are contiguous 
			     */
			    buf = &((ip_t *)ip_start)->ipFragOffset;
			    RX_WRAP_BUF(buf);
			    if (*buf & IP_FRAG_OFS_MASK) {

				/* use tcp/udp data len for a frag'ed packet */
				pseudo = ((ip_t *)ip_start)->ipLen - ip_hlen;

				NIC_TRACE(TRACE_TYPE_RECV, "rNt1stIp", 0x79700,
				    pseudo,
				    0,
				    0,
				    0);

			    } else {
				/* calculate pseudo hdr for the first frame */
				pseudo = ((ip_t *)ip_start)->ipLen - ip_hlen;
				pseudo += ip_proto;

				buf = (U16 *)&((ip_t *)ip_start)->ipSourceAddr;
				if ((ip_start + IP_MIN_HLEN) <= 
				    NIC_RX_BUF_END) {
				    pseudo += *buf++;
				    pseudo += *buf++;
				    pseudo += *buf++;
				    pseudo += *buf;
				} else {
				    RX_WRAP_BUF(buf);
				    pseudo += *buf++;
				    RX_WRAP_BUF(buf);
				    pseudo += *buf++;
				    RX_WRAP_BUF(buf);
				    pseudo += *buf++;
				    RX_WRAP_BUF(buf);
				    pseudo += *buf;
				}

				NIC_TRACE(TRACE_TYPE_RECV, "r1stIp", 0x79800,
				    pseudo,
				    0,
				    0,
				    0);
			    }
			}

			/* need to substract CRC if it is included */
			if (is_global_flags(GF_MOD_INCLUDE_CRC) &&
			    (bd_flags & BD_FLAG_END)) {

			    /* include CRC for subtraction */
			    eth_cksum +=
				crc_cksum((U8 *)(frame_start + ddp->len - 4));
			    eth_cksum =
				(eth_cksum >> 16) + (eth_cksum & 0xffff);
			}

			/* calculate tcp/udp cksum - dont use ip as it is =0 */
			tcpudp_cksum = dd_cksum - eth_cksum;
			tcpudp_cksum += (tcpudp_cksum >> 16);
			tcpudp_cksum += pseudo;
			tcpudp_cksum += (tcpudp_cksum >> 16);
			tcpudp_cksum = tcpudp_cksum & 0xffff;
			if (tcpudp_cksum == 0) {
			    tcpudp_cksum = 0xffff;
			}

			/* set tcp/udp cksum flag */
			bd_flags |= BD_FLAG_TCP_UDP_CKSUM;

			NIC_TRACE(TRACE_TYPE_RECV, "rSmTcpUd", 0x79900, 
			    eth_cksum,
			    ip_cksum,
			    pseudo,
			    tcpudp_cksum);
		    }
		}
            }
        }

        /* store tcp/udp cksum */
        bdp->BD_tcp_udp_cksum = (U16)tcpudp_cksum;

        /* store BD flags */
        bdp->BD_flags = bd_flags;

        /* dma bd to host */
        bd_type = (bd_flags & BD_FLAG_END)?
                    TIGON_TYPE_RECV_BD_LAST:
                    TIGON_TYPE_RECV_BD;

        NIC_TRACE(TRACE_TYPE_RECV, "rSmQBd", 0x79910, 
            *((U32 *)bdp + 2),  /* index & len */
            *((U32 *)bdp + 3),  /* type & flags */
            *((U32 *)bdp + 4),  /* ip cksum & tcp/udp cksum */
            *((U32 *)bdp + 5)); /* error flags and vlan tag */

        if (!(*nicfp->q_dma_to_host_stub)
                (nicfp->recv_ret_ring_ptr +
                  (nicfp->recv_return_prod_index * sizeof(recv_bd_t)),
                (U32)bdp,
                sizeof(recv_bd_t),
                bd_type,
                nicfp->recv_return_prod_index, /* dummy */
                nicfp->dma_to_host_bd_state)) {

            NIC_UTRACE("rSmQDmaF", 0x79920,
                trp->host_dma_assist.write_chan_consumer,
                trp->host_dma_assist.write_chan_producer,
                trp->host_dma_assist.write_chan_ref,
                dd_index);

            /* MUST NEVER hit this case. If does, we have something
             * wrong and needs investigation.
             * We took a dma completion and therefore, we must have
             * at leaat one entry available
             */
            ASSERT(0);
        }

        /* This entry has been produced */
        nicfp->recv_return_prod_index =
            (nicfp->recv_return_prod_index + 1) &
	    nicfp->recv_ret_ring_size_mask;

        nicfp->stats.nicTotalRecvBDs--;
        if (ring_type == STD_RECV_RING)
            nicfp->stats.nicRecvBDs--;
        else if (ring_type == MINI_RECV_RING)
            nicfp->stats.nicMiniRecvBDs--;
        else /*if ((ring_type == <EXT_>JUMBO_RECV_RING)*/
            nicfp->stats.nicJumboRecvBDs--;

    } else if (dd_type & TIGON_TYPE_RECV_BD) {
        /*
         *  If we are coalescing the Recv BDs, then keep on ticking the 
         *  coalesce count so that it can be appropriately reported. 
         *  Actual reporting of the recv producer index happens only when 
         *  the end of frame is reached.
         */
        nicfp->recv_coal_bds++;

        NIC_TRACE(TRACE_TYPE_RECV_BD, "wDmaRxBd", 0x79930,
            ddp->index,
            ddp->type,
            trp->host_dma_assist.write_chan_consumer,
            trp->host_dma_assist.write_chan_producer);

    } else if (dd_type & TIGON_TYPE_RECV_BD_LAST) {

        if (++nicfp->recv_coal_bds >= 
                tsp->gen_com.tuneParms.recv_max_coalesced_bds) {
            recv_return_prod_index_to_host();
            nicfp->stats.nicRecvThresholdHit++;
        }

        NIC_TRACE(TRACE_TYPE_RECV_BD, "wDmRxBdL", 0x79940,
            ddp->index,
            ddp->type,
            trp->host_dma_assist.write_chan_consumer,
            trp->host_dma_assist.write_chan_producer);
    }

    NIC_TRACEV_IF(TRACE_TYPE_DMA, "wDmCpOk", 0x79950,
                  ddp->type, TIGON_TYPE_RX,
        trp->host_dma_assist.write_chan_producer,
        trp->host_dma_assist.write_chan_consumer,
        trp->host_dma_assist.write_chan_ref,
        ddp);

    RETURN(TG_FW_EVENT_NUM_DMA_WR_ASST_HI);
}

/* null routine to assist with scratchpad setup */
void h_dma_wr_assist_cksum_end(void) {}
