/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: ring.h $
 * Revision nic_api_12_4_latest_shuang/6 2000/02/12 18:50:05 shuang
 * 	fix incorrect stats nicRecvBDs and nicJumboRecvBDs.
 * 	honor TG_CMD_CODE_STACK_UP and TG_CMD_CODE_STACK_DOWN commands.
 * 
 * Revision 1.1.2.16  1998/12/08  02:36:45  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:28:25  shuang]
 *
 * Revision 1.1.2.15  1998/09/30  18:50:17  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:40:33  shuang]
 * 
 * Revision 1.1.2.14  1998/07/29  00:07:30  shuang
 * 	add feature - send coallescing BD update only
 * 	[1998/07/29  00:02:49  shuang]
 * 
 * Revision 1.1.2.13  1998/04/15  21:51:59  ted
 * 	Fix up nicSendThresholdHit and nicRecvThresholdHit stats
 * 	[1998/04/15  21:51:40  ted]
 * 
 * Revision 1.1.2.12  1998/04/14  21:42:27  ted
 * 	Change tracing level of rupttime entry to prevent being swamped
 * 	[1998/04/14  21:21:59  ted]
 * 
 * Revision 1.1.2.11  1998/04/11  05:15:20  shuang
 * 	move fast timer to cpu B, add receive coalesed ticks
 * 	[1998/04/11  04:58:47  shuang]
 * 
 * Revision 1.1.2.10  1998/04/01  22:21:29  hayes
 * 	change INC_XXX to GET_NEXT_XXX
 * 	[1998/04/01  01:02:13  hayes]
 * 
 * 	checkpoint
 * 	[1998/03/31  19:57:00  hayes]
 * 
 * 	checkpoint- dma completion avoidance works
 * 	[1998/03/30  07:00:36  hayes]
 * 
 * 	rd DMA event avoidance works
 * 	[1998/03/29  03:12:32  hayes]
 * 
 * Revision 1.1.2.9  1998/04/01  07:32:30  shuang
 * 	modify must_enq_interrupt_event().
 * 	[1998/04/01  07:22:04  shuang]
 * 
 * Revision 1.1.2.8  1998/03/28  00:53:19  shuang
 * 	No interrupt avoidance on link state change
 * 	[1998/03/28  00:50:25  shuang]
 * 
 * Revision 1.1.2.7  1998/03/26  01:34:32  shuang
 * 	cleanup
 * 	[1998/03/26  01:25:34  shuang]
 * 
 * Revision 1.1.2.6  1998/02/19  01:30:07  suresh
 * 	     Remove hard coded values
 * 	     [1998/02/19  01:24:44  suresh]
 * 
 * Revision 1.1.2.5  1998/02/12  19:10:18  shuang
 * 	     Fix FIFTY_US timer warpping around
 * 	     [1998/02/12  19:03:29  shuang]
 * 
 * Revision 1.1.2.4  1998/02/11  19:11:24  ted
 * 	     Funcion pointer q_dma_to_(host,nic) routines
 * 	     [1998/02/11  18:48:33  ted]
 * 
 * 	     intermediate checkin for merge
 * 	     [1998/02/10  00:28:38  ted]
 * 
 * Revision 1.1.2.3  1998/02/09  17:18:08  hayes
 * 	     convert Q_DMA_XXX macros to functions
 * 	     [1998/02/09  17:17:04  hayes]
 * 
 * Revision 1.1.2.2  1998/01/29  02:32:59  skapur
 * 	     Check in for Rev6 checksum support
 * 	     [1998/01/29  02:27:22  skapur]
 * 
 * 	     merging for tigon2 rev 6 changes
 * 	     [1998/01/28  23:14:08  skapur]
 * 
 * Revision 1.1.2.1  1998/01/14  23:25:58  ted
 * 	     rearrange directories
 * 	     [1998/01/14  23:17:53  ted]
 * 
 * Revision 1.1.2.1  1997/11/19  19:23:21  ted
 * 	     Support Tigon and Tigon2 in the same driver
 * 	     [1997/11/19  19:19:55  ted]
 * 
 * Revision 1.7.7.14  1997/11/14  19:16:42  hayes
 * 	     first checkin after mongo merge...
 * 	     [1997/11/14  19:11:47  hayes]
 * 
 * 	     checkin after stats restructuring
 * 	     [1997/11/12  19:01:43  hayes]
 * 
 * Revision 1.7.7.13  1997/11/11  19:06:04  hayes
 * 	     restructuring for scratchpad use
 * 	     [1997/11/11  19:04:31  hayes]
 * 
 * Revision 1.7.7.12  1997/10/21  01:35:38  skapur
 * 	     Merge of release 2.0 and ge_latest branch
 * 	     [1997/10/21  01:31:18  skapur]
 * 
 * Revision 1.7.7.11  1997/10/02  23:11:33  hayes
 * 	     update SIMULATOR code
 * 	     [1997/10/02  23:11:10  hayes]
 * 
 * Revision 1.7.7.10  1997/09/16  03:54:37  skapur
 * 	     Merged with changes from 1.7.7.9
 * 	     [1997/09/16  03:54:22  skapur]
 * 
 * 	      Removed older version of MUST_ENQ_EVENT
 * 	     [1997/09/16  03:50:19  skapur]
 * 
 * 	     enhanced the tracing, cleaned out the Macros, misc. cleanup
 * 	     [1997/09/16  02:50:42  skapur]
 * 
 * Revision 1.7.7.9  1997/09/11  18:05:52  ted
 * 	     Mixing vlans, jumbos, and redundancy works, plus small fixes from release 1.1 and better NIC tracing
 * 	     [1997/09/11  17:59:58  ted]
 * 
 * Revision 1.7.7.8  1997/08/23  17:40:12  skapur
 * 	     Solaris jumbogram support
 * 	     [1997/08/23  17:38:12  skapur]
 * 
 * Revision 1.7.7.7  1997/07/25  22:59:11  ted
 * 	     Architectural performance change
 * 	     [1997/07/24  21:45:20  ted]
 * 
 * Revision 1.7.7.6  1997/05/15  21:21:04  skapur
 * 	     If not able to Enque an event, because event ring is full, do not interrupt the Host, It may mess it up. So, took out the code which will explicitly interrupt the Host in such a sitiration. Done for both ENQ_EVENT and MUST_ENQ_EVENT macros
 * 	     [1997/05/15  21:02:24  skapur]
 * 
 * Revision 1.7.7.5  1997/04/11  23:36:24  ted
 * 	     Changed size of Event Producer pointer from 32 bit to 64 bit:
 * 	     [1997/04/11  23:24:50  ted]
 * 
 * Revision 1.7.7.4  1997/03/28  02:47:09  skapur
 * 	     Removed obslete macros; added NIC counters to keep track of full rings
 * 	     [1997/03/28  02:09:30  skapur]
 * 
 * Revision 1.7.7.3  1997/03/24  08:34:27  ted
 * 	     New event ring handling, producer lives in host memory, misc. other more modest changes
 * 	     [1997/03/24  08:28:06  ted]
 * 
 * Revision 1.7.7.2  1997/03/21  04:15:41  skapur
 * 	     Performance enhancement for Recv and Send; lots changes; almost complete overhaul of the entire firmware
 * 	     [1997/03/21  04:08:31  skapur]
 * 
 * Revision 1.7.7.1  1997/02/28  20:02:27  ted
 * 	     Remove DMA corruption workaround, fix 3 subtle bugs
 * 	     [1997/02/28  20:01:50  ted]
 * 
 * Revision 1.7.2.4  1996/11/29  17:43:23  hayes
 * 	     Changed some #if (SIMULATOR) to #if (NO_HOST) for better
 * 	     simulation and testing.
 * 	     [1996/11/29  17:39:53  hayes]
 * 
 * Revision 1.7.2.3  1996/11/15  04:24:19  hayes
 * 	     convert to new header file structure.
 * 	     [1996/11/15  04:12:20  hayes]
 * 
 * Revision 1.7.2.2  1996/11/05  21:54:30  hayes
 * 	     Final pre-ODE merge
 * 	     [1996/11/05  21:52:17  hayes]
 * 
 * Revision 1.7.2.1  1996/11/01  18:25:07  ted
 * 	     change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	     [1996/10/29  22:06:38  ted]
 * 
 * $EndLog$
 */
/*
 * FILE ring.h
 *
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/ring.h,v $
 * $Revision: 1.1.2.16 $ $Locker:  $
 * $Date: 1998/12/08 02:36:45 $ $Author: shuang $ $State: Exp $
 */

#ifndef _RING_H_
#define _RING_H_

#include "timer.h"      /* for TG_TICK_XX */
#include "dma.h"


/* timeout for must_enq_event() */
#define ONE_MSEC 1000


__inline__ static U32
get_next_event_index(U32 index)
{
    return ((index == (EVENT_RING_ENTRIES - 1))? 0: (index + 1));
}

__inline__ static U32
enq_event(struct tg_event *ep)
{
    U32 event_prod, event_prod_next;

    event_prod = nicfp->event_prod_index;
    event_prod_next = get_next_event_index(event_prod);

    if (event_prod_next == tsp->gen_com.event_cons_index) {
        nicfp->stats.nicEventRingFull++;

        NIC_UTRACE("evtRngF", 0x90100,
	    ep->TG_EVENT_w0,
	    event_prod,
	    tsp->gen_com.event_cons_index,
	    0);
        return (ERROR);
    }

    /* copy event to ring */
    nicp->event_ring[event_prod] = *ep; /* struct copy */

    /* dma event to the host */
    if (!(*nicfp->q_dma_to_host_stub)
            (nicfp->event_ring_ptr + (event_prod * sizeof(tg_event_t)),
            (U32)&nicp->event_ring[event_prod],
            sizeof(tg_event_t),
            TIGON_TYPE_EVENT,
            event_prod_next, /* dummy */
            nicfp->dma_to_host_bd_state)) {

        NIC_UTRACE("QevtF", 0x90200,
	    ep->TG_EVENT_w0,
	    event_prod,
	    trp->host_dma_assist.write_chan_producer,
	    trp->host_dma_assist.write_chan_consumer);
        return (ERROR);
    }

    /* update producer */
    nicfp->event_prod_index = event_prod_next;

    /* send the new producer to the host */
    if (!q_dma_to_host_index
	    (nicfp->event_producer_ptr,
	    sizeof(U64),
	    TIGON_TYPE_EVENT_PROD,
	    event_prod_next,
	    nicfp->dma_to_host_bd_state)) {

        nicfp->stats.nicEventProducerRingFull++;

        NIC_UTRACE("QevtP_F", 0x90300,
	    ep->TG_EVENT_w0,
	    event_prod,
	    trp->host_dma_assist.write_chan_producer,
	    trp->host_dma_assist.write_chan_consumer);
        return (ERROR);
    }

    NIC_TRACE(TRACE_TYPE_EVENT, "QevtOk", 0x90400,
	ep->TG_EVENT_w0,
	event_prod,
	tsp->gen_com.event_cons_index,
	0);

    return (OK);
}

/*
 * common routine for must_enq_event() and must_enq_interrupt_event()
 */
__inline__ static U32
must_enq_event_2(struct tg_event *ep, bool intr_host)
{
    U32 event_prod, event_prod_next;
    U32 end_time;
    bool successQ;

    event_prod = nicfp->event_prod_index;
    event_prod_next = get_next_event_index(event_prod);

    if (event_prod_next == tsp->gen_com.event_cons_index) {
        nicfp->stats.nicEventRingFull++;

        NIC_UTRACE("MevtRngF", 0x90500,
	    ep->TG_EVENT_w0,
	    event_prod,
	    tsp->gen_com.event_cons_index,
	    0);
        return (ERROR);
    }

    /* copy event in ring */
    nicp->event_ring[event_prod] = *ep; /* struct copy */

    /* dma event to the host */
    end_time = trp->gen_control.timer + ONE_MSEC;
    successQ = FALSE;

    /* computation way is to handle the timer wrapping around */
    while (end_time - trp->gen_control.timer <= ONE_MSEC) {
        if ((*nicfp->q_dma_to_host_stub)
                (nicfp->event_ring_ptr + (event_prod * sizeof(tg_event_t)),
                (U32)&nicp->event_ring[event_prod],
                sizeof(tg_event_t),
		TIGON_TYPE_EVENT,
                event_prod_next, /* dummy */
		nicfp->dma_to_host_bd_state)) {

            successQ = TRUE;
            break;
        }
    }

    if (!successQ) {
	nicfp->stats.nicEnqEventDelayed++;

        NIC_UTRACE("MQevtF", 0x90600,
	    ep->TG_EVENT_w0,
	    event_prod,
	    trp->host_dma_assist.write_chan_producer,
	    trp->host_dma_assist.write_chan_consumer);
        return (ERROR);
    }

    /* update producer */
    nicfp->event_prod_index = event_prod_next;

    /* send the new producer to the host */
    end_time = trp->gen_control.timer + ONE_MSEC;
    successQ = FALSE;

    /* computation way is to handle the timer wrapping around */
    while (end_time - trp->gen_control.timer <= ONE_MSEC) {
        if (q_dma_to_host_index
		(nicfp->event_producer_ptr,
		 sizeof(U64),
		 TIGON_TYPE_EVENT_PROD,
		 event_prod_next,
		 (intr_host)?
		    (nicfp->dma_to_host_bd_state | TG_DMA_STATE_SET_INTERRUPT):
		    nicfp->dma_to_host_bd_state)) {

	    nicfp->stats.nicInterrupts++;
            successQ = TRUE;
            break;
        }
    }

    if (!successQ) {
	nicfp->stats.nicEnqEventDelayed++;

        NIC_UTRACE("MQevP_F", 0x90700,
	    ep->TG_EVENT_w0,
	    event_prod,
	    trp->host_dma_assist.write_chan_producer,
	    trp->host_dma_assist.write_chan_consumer);
        return (ERROR);
    }

    NIC_TRACE(TRACE_TYPE_EVENT, "MQevtOk", 0x90800,
	ep->TG_EVENT_w0,
	event_prod,
	tsp->gen_com.event_cons_index,
	0);

    return (OK);
}

/*
 * enqueue an event 
 * no interrupt is generated when this dma completes
 */
__inline__ static U32
must_enq_event(struct tg_event *ep)
{
    return (must_enq_event_2(ep, FALSE));
}

/*
 * enqueue an event and interrupt host when this dma completes
 */
__inline__ static U32
must_enq_interrupt_event(struct tg_event *ep)
{
    return (must_enq_event_2(ep, TRUE));
}

/*
 * Misc. functions
 */

__inline__ static void
send_cons_index_to_host_intr(void)
{
    if (!q_dma_to_host_index
	    (nicfp->send_consumer_ptr,
	    sizeof(U64),
	    TIGON_TYPE_TX_CONS,
	    nicfp->send_cons_index,
	    nicfp->dma_to_host_bd_state)) {

        nicp->send_cons_index_to_host_pending = TRUE;

        NIC_UTRACE("QsConI_F", 0x90900,
	    nicfp->send_cons_index,
	    trp->host_dma_assist.write_chan_producer,
	    trp->host_dma_assist.write_chan_consumer,
	    trp->host_dma_assist.write_chan_ref);
    } else {
        nicfp->send_coal_index = nicfp->send_cons_index;
        nicfp->stats.nicRingSetSendConsIndex++;
	nicfp->send_cons_index_to_host_need_intr = FALSE;
	nicp->send_cons_index_to_host_pending_no_intr = FALSE;
        nicp->send_cons_index_to_host_pending = FALSE;
        nicfp->send_coal_bds = 0;
#if (RESET_COAL_TICK_CNT)
	nicp->send_coal_tick_count = 0;
#endif (RESET_COAL_TICK_CNT)

	NIC_TRACE(TRACE_TYPE_SEND, "QsConIok", 0x91000,
	    nicfp->send_cons_index,
	    0,
	    0,
	    0);
    }
    return;
}

__inline__ static void
send_cons_index_to_host_no_intr(void)
{
    if (!q_dma_to_host_index
	    (nicfp->send_consumer_ptr,
	    sizeof(U64),
	    TIGON_TYPE_TX_CONS_NO_INTR,
	    nicfp->send_cons_index,
	    nicfp->dma_to_host_bd_state)) {

        nicp->send_cons_index_to_host_pending_no_intr = TRUE;

        NIC_UTRACE("QsConsF", 0x91100,
	    nicfp->send_cons_index,
	    trp->host_dma_assist.write_chan_producer,
	    trp->host_dma_assist.write_chan_consumer,
	    trp->host_dma_assist.write_chan_ref);
    } else {
        nicfp->send_coal_index = nicfp->send_cons_index;
        nicfp->stats.nicRingSetSendConsIndex++;
	nicfp->send_cons_index_to_host_need_intr = TRUE;
	nicp->send_cons_index_to_host_pending_no_intr = FALSE;
        nicp->send_cons_index_to_host_pending = FALSE;
        nicfp->send_coal_bds = 0;
#if (RESET_COAL_TICK_CNT)
	nicp->send_coal_tick_count = 0;
#endif (RESET_COAL_TICK_CNT)

	NIC_TRACE(TRACE_TYPE_SEND, "QsConsOk", 0x91200,
	    nicfp->send_cons_index,
	    0,
	    0,
	    0);
    }
    return;
}

__inline__ static void
send_cons_index_to_host(void)
{
    if (is_global_flags(GF_RCB_TX_COAL_UPDATE_ONLY))
	send_cons_index_to_host_no_intr();
    else
	send_cons_index_to_host_intr();
    return;
}

__inline__ static void
recv_return_prod_index_to_host(void)
{
    if (!q_dma_to_host_index
	    (nicfp->recv_return_producer_ptr,
	    sizeof(U64),
	    TIGON_TYPE_RX_PROD,
	    nicfp->recv_return_prod_index,
	    nicfp->dma_to_host_bd_state)) {

        nicp->recv_ret_prod_index_to_host_pending = TRUE;

        NIC_UTRACE("QrProdF", 0x91300,
	    nicfp->recv_return_prod_index,
	    trp->host_dma_assist.write_chan_producer,
	    trp->host_dma_assist.write_chan_consumer,
	    trp->host_dma_assist.write_chan_ref);
    } else {
        nicfp->recv_coal_index = nicfp->recv_return_prod_index;
        nicfp->stats.nicRingSetRecvReturnProdIndex++;
        nicp->recv_ret_prod_index_to_host_pending = FALSE;
        nicfp->recv_coal_bds = 0;
#if (RESET_COAL_TICK_CNT)
	nicp->recv_coal_tick_count = 0;
#endif (RESET_COAL_TICK_CNT)

	NIC_TRACE(TRACE_TYPE_RECV, "QrProdOk", 0x91400,
	    nicfp->recv_return_prod_index,
	    0,
	    0,
	    0);
    }
    return;
}

__inline__ static void
set_event_handler(U32 *evt_vector_base, U32 event_num, U32 func)
{
    evt_vector_base[(event_num + 1) << 1] =
	(0x08 << 24) | ((0x0fffffff & (func)) >> 2);
    return;
}

__inline__ static bool
is_host_in_interrupt_handler(void)
{
    return ((trp->gen_control.misc_host_control & TG_MHC_RUPT_STATE) ||
            (tsp->mailbox[0].mbox_low == 1) ||
	    (tsp->gen_com.maskRupts))?
	    TRUE: FALSE;
}

__inline__ static void
interrupt_host(void)
{
    trp->gen_control.misc_local_control |= TG_MLC_SET_INTERRUPT;
    nicfp->stats.nicInterrupts++;

    NIC_TRACEV(TRACE_TYPE_EVENT, "intrHost", 0x91500,
	trp->gen_control.timer,
	nicfp->send_cons_index,
	nicfp->recv_return_prod_index,
	nicfp->event_prod_index);
    return;
}

__inline__ static void
interrupt_host_w_avoidance(void)
{
    /* do not interrupt host if it is already processing an interrupt */
    if (is_host_in_interrupt_handler()) {
        nicfp->stats.nicAvoidedInterrupts++;
    } else {
	interrupt_host();
    }
    return;
}

/*
 * lock among processors
 */

__inline__ static U32 
get_cpu_id(void)
{
    return (nicfp->cpu_id);
}

__inline__ static void 
set_cpu_id(U32 id)
{
    nicfp->cpu_id = id;
    return;
}

__inline__ static void
grab_spin_lock(void)
{
    if (get_cpu_id()) {
        do {
            trp->gen_control.semaphore_b = 0;
        } while (! trp->gen_control.semaphore_b);
    } else {
        do {
            trp->gen_control.semaphore_a = 0;
        } while (! trp->gen_control.semaphore_a);
    }
    return;
}

__inline__ static void
free_spin_lock(void)
{
    if (get_cpu_id()) {
        trp->gen_control.semaphore_b = 0;
    } else {
        trp->gen_control.semaphore_a = 0;
    }
    return;
}

__inline__ static void
clear_event_reg(U32 bits)
{
    grab_spin_lock();
    trp->gen_control.event &= ~bits;
    free_spin_lock();
    return;
}

__inline__ static void
set_event_reg(U32 bits)
{
    grab_spin_lock();
    trp->gen_control.event |= bits;
    free_spin_lock();
    return;
}

#endif /* _RING_H_ */
