/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: send.c $
 * Revision nic_api_12_4_latest_shuang/22 2000/03/27 23:19:36 shuang
 * 	fix: Tx hang on jumbo frames checksum offload
 * 
 * Revision nic_api_12_4_latest_shuang/19 2000/02/29 18:18:07 shuang
 * 	fix: BDs related stats should not be refreshed by refresh cmd
 * 
 * Revision nic_api_12_4_latest_shuang/17 2000/02/25 19:28:54 shuang
 * 	fix work around HW MAC HD collision/retransmission for cksum
 * 
 * Revision nic_api_12_4_latest_shuang/15 2000/02/23 00:31:59 shuang
 * 	work around HW MAC HD mode collision/retransmission
 * 
 * Revision nic_api_12_4_latest_shuang/14 2000/02/23 00:07:10 shuang
 * 	work around HW MAC HD mode collision/retransmission
 * 
 * Revision api124_l_thu/7 2000/02/01 19:26:06 thu
 * 	added same fix to h_send_data_read
 * 
 * Revision api124_l_thu/5 2000/01/31 23:45:01 thu
 * 	use per frame buffer management scheme to avoid trash tx buffer when heavy traffic is introduced 
 * 
 * Revision nic_api_12_4_latest_shuang/10 1999/12/11 17:45:04 shuang
 * 	same REFRESH in cksum mode.
 * 
 * Revision nic_api_12_4_latest_shuang/8 1999/12/11 03:11:22 shuang
 * 	fix: need to refresh hw mac stats on the REFRESH command.
 * 
 * Revision nic_api_12_4_latest_shuang/6 1999/10/20 04:34:48 shuang
 * 	fix: mac_descs should be S32 instead of U32
 * 
 * Revision nic_api_12_4_latest_shuang/4 1999/09/08 23:53:30 shuang
 * 	fix insert vlan tag on sending
 * 
 * Revision nic_api_12_4_latest_shuang/2 1999/08/27 01:36:23 shuang
 * 	jumbo frame length will be 0x8870
 * 
 * Revision nic_api_12_3_latest_shuang/2 1999/04/20 23:19:56 shuang
 * 	nicTotalSendBDs is obsolete, update nicSendBDs if using host send ring
 * 
 * Revision 1.1.2.44  1998/12/21  00:33:18  shuang
 * 	fix - return last send BD
 * 	[1998/12/21  00:13:00  shuang]
 *
 * Revision 1.1.2.43  1998/12/12  00:35:56  shuang
 * 	add debug code to check recv ring
 * 	[1998/12/12  00:33:02  shuang]
 * 
 * Revision 1.1.2.42  1998/12/10  02:55:41  shuang
 * 	Prevent from starving tx side. Fix stats counters.
 * 	[1998/12/10  02:54:49  shuang]
 * 
 * Revision 1.1.2.41  1998/12/08  02:36:47  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:28:28  shuang]
 * 
 * Revision 1.1.2.40  1998/11/20  22:39:47  shuang
 * 	remove assertion
 * 	[1998/11/20  22:39:10  shuang]
 * 
 * Revision 1.1.2.39  1998/11/13  04:20:04  shuang
 * 	fix insert vlan tag
 * 	[1998/11/13  04:18:57  shuang]
 * 
 * Revision 1.1.2.38  1998/11/05  00:09:55  shuang
 * 	remove faulty assertion statement
 * 	[1998/11/05  00:09:18  shuang]
 * 
 * Revision 1.1.2.37  1998/10/17  21:49:51  shuang
 * 	remove faulty assertion statement
 * 	[1998/10/17  21:48:26  shuang]
 * 
 * Revision 1.1.2.36  1998/09/30  18:50:19  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:40:37  shuang]
 * 
 * Revision 1.1.2.35  1998/09/17  20:58:10  shuang
 * 	fix enq mac tx hang problem on checksum mode
 * 	[1998/09/17  20:54:32  shuang]
 * 
 * Revision 1.1.2.34  1998/09/02  23:40:40  shuang
 * 	fix host based send ring bug on cksum mode
 * 	[1998/09/02  23:39:17  shuang]
 * 
 * Revision 1.1.2.33  1998/08/27  17:33:13  shuang
 * 	fix defect1370- flow control
 * 	[1998/08/27  17:31:56  shuang]
 * 
 * Revision 1.1.2.32  1998/08/11  01:49:32  shuang
 * 	in cksum mode, clear refresh_stats_active flag after dma'ed stats from host
 * 	[1998/08/11  01:48:30  shuang]
 * 
 * Revision 1.1.2.31  1998/08/07  00:17:05  shuang
 * 	do a dummy dma read forwrite dma hang problem in X-Chip
 * 	[1998/08/06  23:51:22  shuang]
 * 
 * Revision 1.1.2.30  1998/08/03  16:14:32  hayes
 * 	complete firmware update for NT
 * 	[1998/08/03  16:11:26  hayes]
 * 
 * 	checkin so merge w/ fc can happen
 * 	[1998/07/29  23:15:33  hayes]
 * 
 * 	host based send ring now works off RCB flag
 * 	[1998/07/29  18:31:04  hayes]
 * 
 * 	host based send ring works
 * 	[1998/07/28  03:57:27  hayes]
 * 
 * Revision 1.1.2.29  1998/07/29  00:07:32  shuang
 * 	add feature - send coallescing BD update only
 * 	[1998/07/29  00:02:54  shuang]
 * 
 * 	add feature - send coallescing BD now
 * 	[1998/07/25  02:29:10  shuang]
 * 
 * Revision 1.1.2.28  1998/07/24  18:18:33  shuang
 * 	Fix outgoing statistic counters in ip fragments with vlan
 * 	[1998/07/24  18:17:24  shuang]
 * 
 * Revision 1.1.2.27  1998/06/23  01:29:47  shuang
 * 	added dynamic nic mem support.
 * 	[1998/06/23  01:22:02  shuang]
 * 
 * Revision 1.1.2.26  1998/06/11  23:30:30  shuang
 * 	     Skip tcp/udp calculation and give warning if tx packets are not tcp/udp's.
 * 	     Changed switch-case statements that generate jump tables to if-else.
 * 	     [1998/06/11  23:26:39  shuang]
 * 
 * Revision 1.1.2.25  1998/05/09  02:09:53  shuang
 * 	     fix re-calculate tx space bug in cksum mode.
 * 	     fix send 1st ip frag frame with vlan tag bug in cksum mode.
 * 	     [1998/05/09  01:59:53  shuang]
 * 
 * Revision 1.1.2.24  1998/04/30  00:42:58  shuang
 * 	     arranged tx space checking for efficiency
 * 	     [1998/04/30  00:42:13  shuang]
 * 
 * Revision 1.1.2.23  1998/04/25  01:42:27  shuang
 * 	     fix send consumer index problem, add DWord adjust for chking tx space
 * 	     [1998/04/25  01:11:24  shuang]
 * 
 * Revision 1.1.2.22  1998/04/24  00:00:44  hayes
 * 	     added const to rcsid definitions to eliminate compiler warnings
 * 	     [1998/04/23  23:59:00  hayes]
 * 
 * Revision 1.1.2.21  1998/04/18  02:24:37  shuang
 * 	     cksum offload for ip datagram only
 * 	     [1998/04/18  02:24:07  shuang]
 * 
 * Revision 1.1.2.20  1998/04/17  20:20:50  shuang
 * 	     Fix cksum offload on sending (pseudo header)
 * 	     [1998/04/17  20:20:24  shuang]
 * 
 * Revision 1.1.2.19  1998/04/17  06:40:25  shuang
 * 	     fix handle pseudo header on sending
 * 	     [1998/04/17  06:40:01  shuang]
 * 
 * Revision 1.1.2.18  1998/04/16  20:33:21  shuang
 * 	     store tcp/udp cksum to first frame in ip fragment
 * 	     [1998/04/16  20:32:29  shuang]
 * 
 * Revision 1.1.2.17  1998/04/16  03:50:46  shuang
 * 	     flag TX_FRAG_FRAMES_IN_ORDER defaults to 1 now
 * 	     [1998/04/16  03:50:06  shuang]
 * 
 * Revision 1.1.2.16  1998/04/15  21:52:00  ted
 * 	     Fix up nicSendThresholdHit and nicRecvThresholdHit stats
 * 	     [1998/04/15  21:51:43  ted]
 * 
 * Revision 1.1.2.15  1998/04/15  17:39:13  ted
 * 	     Fix refresh stats command
 * 	     [1998/04/15  17:31:18  ted]
 * 
 * Revision 1.1.2.14  1998/04/14  04:58:36  shuang
 * 	     Fix cksum offload on sending.
 * 	     [1998/04/14  04:56:03  shuang]
 * 
 * Revision 1.1.2.13  1998/04/11  05:15:21  shuang
 * 	     move fast timer to cpu B, add receive coalesed ticks
 * 	     [1998/04/11  04:58:50  shuang]
 * 
 * Revision 1.1.2.12  1998/04/06  17:38:32  hayes
 * 	     code generation optimizations
 * 	     [1998/04/06  17:38:00  hayes]
 * 
 * Revision 1.1.2.11  1998/04/01  22:21:30  hayes
 * 	     merge cleanup
 * 	     [1998/04/01  22:19:34  hayes]
 * 
 * 	     change ref ring prod/cons to head/tail
 * 	     [1998/04/01  01:01:49  hayes]
 * 
 * 	     checkpoint
 * 	     [1998/03/31  19:57:02  hayes]
 * 
 * 	     checkpoint- dma completion avoidance works
 * 	     [1998/03/30  07:00:37  hayes]
 * 
 * 	     more cleanup
 * 	     [1998/03/29  04:21:16  hayes]
 * 
 * 	     rd DMA event avoidance works
 * 	     [1998/03/29  03:11:43  hayes]
 * 
 * Revision 1.1.2.10  1998/04/01  07:32:32  shuang
 * 	     add features: pass (CRC, bad, partial) frame, timer tick to 5us...
 * 	     [1998/04/01  07:27:30  shuang]
 * 
 * Revision 1.1.2.9  1998/04/01  00:34:59  skapur
 * 	     Adding Vlan Tag Insertion/Deletion support
 * 	     [1998/04/01  00:32:28  skapur]
 * 
 * Revision 1.1.2.8  1998/03/26  01:34:33  shuang
 * 	     cleanup
 * 	     [1998/03/26  01:25:38  shuang]
 * 
 * Revision 1.1.2.7  1998/03/08  02:07:34  ted
 * 	          Fix up usage because of changed sendbd structure (for VLAN assist)
 * 	          [1998/03/08  02:05:57  ted]
 * 
 * Revision 1.1.2.6  1998/02/21  03:40:28  ted
 * 	          Don't use hardware, it appears to be broken
 * 	          [1998/02/21  03:40:10  ted]
 * 
 * Revision 1.1.2.5  1998/02/19  01:30:09  suresh
 * 	          TDP correction, extra routine to assist scratchpad setup
 * 	          [1998/02/19  01:25:25  suresh]
 * 
 * Revision 1.1.2.4  1998/02/11  19:11:25  ted
 * 	          Function pointer q_dma_to(host,nic) routines
 * 	          [1998/02/11  18:49:04  ted]
 * 
 * Revision 1.1.2.3  1998/02/09  17:18:09  hayes
 * 	          convert Q_DMA_XXX macros to functions
 * 	          [1998/02/09  17:17:06  hayes]
 * 
 * Revision 1.1.2.2  1998/01/29  02:33:00  skapur
 * 	          Check in for Rev6 checksum support
 * 	          [1998/01/29  02:27:26  skapur]
 * 
 * 	          merging for Tigon2 rev6 Checksum off load support
 * 	          [1998/01/28  23:14:59  skapur]
 * 
 * Revision 1.1.2.1  1998/01/14  23:26:00  ted
 * 	          rearrange directories
 * 	          [1998/01/14  23:17:55  ted]
 * 
 * Revision 1.1.2.8  1998/01/14  20:00:25  skapur
 * 	          Merged with changes from 1.1.2.7
 * 	          [1998/01/14  20:00:15  skapur]
 * 
 * 	          Checksum off-load support for Tigon2. This is an intermediate checkin; more work needs to be done
 * 	          [1998/01/14  19:26:55  skapur]
 * 
 * Revision 1.1.2.7  1998/01/09  03:15:06  ted
 * 	          Correct possible holes in multi-BD packets
 * 	          [1998/01/09  01:41:44  ted]
 * 
 * Revision 1.1.2.6  1998/01/05  01:31:48  ted
 * 	          Fix occasional corruption problem (PR 689)
 * 	          [1998/01/05  00:56:35  ted]
 * 
 * Revision 1.1.2.5  1997/12/29  19:02:10  hayes
 * 	          remove replicate stuff
 * 	          [1997/12/29  19:00:46  hayes]
 * 
 * Revision 1.1.2.4  1997/12/27  22:22:02  hayes
 * 	          use more Tigon 2 hardware assist
 * 	          [1997/12/27  22:21:49  hayes]
 * 
 * Revision 1.1.2.3  1997/12/18  19:27:58  ted
 * 	          Fix 0 length send BD bug
 * 	          [1997/12/18  19:27:35  ted]
 * 
 * Revision 1.1.2.2  1997/11/25  22:07:15  ted
 * 	          Fix up statistics
 * 	          [1997/11/25  22:00:10  ted]
 * 
 * Revision 1.1.2.1  1997/11/19  19:23:23  ted
 * 	          Support Tigon and Tigon2 in the same driver
 * 	          [1997/11/19  19:19:58  ted]
 * 
 * Revision 1.29.7.28  1997/11/19  00:13:44  ted
 * 	          merged from release 2.0
 * 	          [1997/11/19  00:10:46  ted]
 * 
 * Revision 1.29.7.27  1997/11/14  19:16:44  hayes
 * 	          first checkin after mongo merge...
 * 	          [1997/11/14  19:11:49  hayes]
 * 
 * 	          checkin after stats restructuring
 * 	          [1997/11/12  19:01:46  hayes]
 * 
 * Revision 1.29.7.26  1997/11/11  19:06:05  hayes
 * 	          restructuring for scratchpad use
 * 	          [1997/11/11  19:04:34  hayes]
 * 
 * Revision 1.29.7.25  1997/10/21  01:35:39  skapur
 * 	          Merge of release 2.0 and ge_latest branch
 * 	          [1997/10/21  01:31:22  skapur]
 * 
 * Revision 1.29.7.24  1997/09/16  21:46:39  ted
 * 	          Change from MIB-II V1 (RFC 1213) to MIB-II V2 (RFC 1573)
 * 	          [1997/09/16  21:42:20  ted]
 * 
 * Revision 1.29.7.23  1997/09/16  03:13:52  skapur
 * 	          Merged with changes from 1.29.7.22
 * 	          [1997/09/16  03:13:40  skapur]
 * 
 * 	          Fixed Taces, Took out code which could cause harm in non-copy data mode; misc. cleanup
 * 	          [1997/09/16  03:08:20  skapur]
 * 
 * Revision 1.29.7.22  1997/09/11  18:05:54  ted
 * 	          Mixing vlans, jumbos, and redundancy works, plus small fixes from release 1.1 and better NIC tracing
 * 	          [1997/09/11  18:00:06  ted]
 * 
 * Revision 1.29.7.21  1997/08/23  17:40:13  skapur
 * 	          Solaris jumbogram support
 * 	          [1997/08/23  17:38:16  skapur]
 * 
 * Revision 1.29.7.20  1997/08/13  18:46:16  ted
 * 	          Fix off by one error when DMA ring is empty
 * 	          [1997/08/13  18:44:00  ted]
 * 
 * Revision 1.29.7.19  1997/07/25  22:59:13  ted
 * 	          Merge fixes from ge_0_22 and add link negotiation support
 * 	          [1997/07/25  22:32:20  ted]
 * 
 * 	          Architectural performance change
 * 	          [1997/07/24  21:45:24  ted]
 * 
 * Revision 1.29.7.18  1997/05/21  03:35:47  ted
 * 	          Fix for bug 189.  The bug was telling the host that the NIC was done
 * 	          with the data buffer as soon as it was queued on the DMA READ ring.
 * 	          The host would then occasionally overwrite this data before the DMA
 * 	          had actually occured creating bad frame data being transmitted.  The
 * 	          fix was to only tell the host the data buffer could be reused after
 * 	          the DMA to the NIC was complete.  This was done by checking in the
 * 	          DMA ring whenever the BD coallescing limit had been reached and sending
 * 	          an event in this case.  Due to possible conflicts on the DMA WRITE
 * 	          ring the fix lapped over into the timer.c code where the event
 * 	          could also be sent from.
 * 	          [1997/05/21  03:20:05  ted]
 * 
 * Revision 1.29.7.17  1997/05/03  04:58:48  skapur
 * 	          Cleaned up stats code
 * 	          [1997/05/03  04:58:16  skapur]
 * 
 * Revision 1.29.7.16  1997/04/25  18:48:53  ted
 * 	          Correct compiler warning
 * 	          [1997/04/25  18:37:23  ted]
 * 
 * Revision 1.29.7.15  1997/04/22  18:04:06  alfan
 * 	          Fixed Send Events; If any of the send completion events are missed, they will be re-tired until it makes it
 * 	          [1997/04/22  18:02:45  alfan]
 * 
 * Revision 1.29.7.14  1997/04/19  04:04:38  alfan
 * 	          Fixed Tx Buf Wrap problem
 * 	          [1997/04/19  04:03:52  alfan]
 * 
 * Revision 1.29.7.13  1997/04/11  23:41:20  ted
 * 	          Merged with changes from 1.29.7.12
 * 	          [1997/04/11  23:41:08  ted]
 * 
 * 	          Misc. Cleanup; Major Perf. related improvements; SBUS HW Hacks to workaround HW Problems
 * 	          [1997/04/11  23:21:09  ted]
 * 
 * Revision 1.29.7.12  1997/04/04  22:38:14  ted
 * 	          Remove two asserts that cause problems on VERY busy systems
 * 	          [1997/04/04  21:59:58  ted]
 * 
 * Revision 1.29.7.11  1997/03/28  02:47:11  skapur
 * 	          Fixed Replication code; moved some of the bits which were or'ed in the macros into the appropriate initialization variables in fwmain
 * 	          [1997/03/28  02:23:31  skapur]
 * 
 * Revision 1.29.7.10  1997/03/24  08:34:29  ted
 * 	          New event ring handling, producer lives in host memory, misc. other more modest changes
 * 	          [1997/03/24  08:28:09  ted]
 * 
 * Revision 1.29.7.9  1997/03/21  04:15:42  skapur
 * 	          Performance enhancement for Recv and Send; lots changes; almost complete overhaul of the entire firmware
 * 	          [1997/03/21  04:08:36  skapur]
 * 
 * Revision 1.29.7.7  1997/02/28  20:02:29  ted
 * 	          Remove DMA corruption workaround, fix 3 subtle bugs
 * 	          [1997/02/28  20:01:53  ted]
 * 
 * Revision 1.29.7.6  1997/02/23  00:23:41  ted
 * 	          Added ASSERTS to catch bad grames; Added h_dma_corruption_hack routine to deal with ON/OFF of DMA Channels, etc.; tx completion shound not be called from now on; LED issue needs to be worked out yet
 * 	          [1997/02/23  00:16:37  ted]
 * 
 * Revision 1.29.7.5  1997/02/17  05:05:51  ted
 * 	          Add back in TG_TX_COMP hardware buf workaround code
 * 	          [1997/02/17  05:05:19  ted]
 * 
 * Revision 1.29.7.4  1997/02/17  00:44:16  skapur
 * 	          If def'ed out the code for hack for SW TX_COMP event
 * 	          [1997/02/17  00:43:51  skapur]
 * 
 * Revision 1.29.7.3  1997/02/17  00:08:03  ted
 * 	          Working with Rev 1, Chip Express boards (funky internal loopback sync algorith)
 * 	          [1997/02/16  23:44:27  ted]
 * 
 * Revision 1.29.7.2  1997/02/11  04:08:08  ted
 * 	          intermediate checkin
 * 	          [1997/02/11  04:01:30  ted]
 * 
 * Revision 1.29.7.1  1997/02/11  02:50:16  ted
 * 	          Workaround for mac tx_comp bug in Rev A tigon:
 * 	          h_mac_tx_comp() is now called from SW0, not tx_comp.
 * 	          It must compare prod, cons and ref and act appropriately
 * 	          to take the place of hardware.
 * 	          [1997/02/11  02:46:51  ted]
 * 
 * Revision 1.29.2.4  1996/11/20  20:04:28  ted
 * 	          Redo include file structuring.
 * 	          [1996/11/20  19:52:15  ted]
 * 
 * Revision 1.29.2.3  1996/11/15  04:24:20  hayes
 * 	          convert to new header file structure.
 * 	          [1996/11/15  04:12:07  hayes]
 * 
 * Revision 1.29.2.2  1996/11/05  21:54:31  hayes
 * 	          Final pre-ODE merge
 * 	          [1996/11/05  21:52:20  hayes]
 * 
 * Revision 1.29.2.1  1996/11/01  18:25:09  ted
 * 	          change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	          [1996/10/29  22:06:41  ted]
 * 
 * $EndLog$
 */
/*
 * FILE send.c
 *
 * COPYRIGHT (c) Essential Communication Corp. 1995
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/send.c,v $
 * $Revision: 1.1.2.44 $ $Locker:  $
 * $Date: 1998/12/21 00:33:18 $ $Author: shuang $ $State: Exp $
 */

#ifndef __lint
static const char rcsid[] = "$Header: /projects/rcs/sw/ge/./nic/fw2/common/send.c,v 1.1.2.44 1998/12/21 00:33:18 shuang Exp $";
#endif /* __lint */

#include "alt_def.h"
#include "nic_conf.h"
#include "nic_api.h"
#include "tg.h"

#include "nic.h"
#include "proto.h"
#include "trace.h"
#include "assert.h"
#include "profile.h"
#include "dma.h"
#include "ring.h"
#include "mac.h"
#include "timer.h"
#include "cksum.h"

/* debug flags */
#define	TX_CKSUM_DEBUG	0

/* pre-reserve size in case of collision/retransmission retry in HD mode
 * the worst case of it is jumbo frame size 9018
 * better change it to be the interface MTU size,
 * ie, nicfp->stats.ifMtu or tsp->gen_com.ifMTU
 */
#define HD_RETRY_RED_ZONE	(9018)

/* globals */
U32 h_send_bd_ready_stub;
U32 h_send_data_ready_stub;
U32 h_dma_rd_assist_stub;


__inline__ static void
update_tx_pkt_stats(U32 bd_flags)
{
#if (USE_U32_STAT_CNT)
    nicfp->ifHCOutPkts[((bd_flags >> BD_FLAG_PKT_SHIFT) & 3)]++;
#else /* (USE_U32_STAT_CNT) */
    switch (bd_flags & BD_FLAG_BCAST_PKT) {
    case BD_FLAG_UCAST_PKT:
	nicfp->stats.ifHCOutUcastPkts++;
	break;
    case BD_FLAG_MCAST_PKT:
	nicfp->stats.ifHCOutMulticastPkts++;
	break;
    case BD_FLAG_BCAST_PKT:
	nicfp->stats.ifHCOutBroadcastPkts++;
	break;
    }
#endif /* (USE_U32_STAT_CNT) */
    return;
}

__inline__ static void
update_tx_pkt_stats_cksum(U32 frame_start)
{
#if (USE_U32_STAT_CNT)
    if (!(*(U8 *)(frame_start) & 0x01)) {
	nicfp->ifHCOutPkts[UCAST_PKT_INDEX]++;
    } else if ((*(U32 *)frame_start == 0xffffffff) &&
	       (*(U16 *)(frame_start + 4) == 0xffff)) {
	nicfp->ifHCOutPkts[BCAST_PKT_INDEX]++;
    } else {
	nicfp->ifHCOutPkts[MCAST_PKT_INDEX]++;
    }
#else /* (USE_U32_STAT_CNT) */
    if (!(*(U8 *)(frame_start) & 0x01)) {
        nicfp->stats.ifHCOutUcastPkts++;
    } else if ((*(U32 *)frame_start == 0xffffffff) &&
	       (*(U16 *)(frame_start + 4) == 0xffff)) {
	nicfp->stats.ifHCOutBroadcastPkts++;
    } else {
	nicfp->stats.ifHCOutMulticastPkts++;
    }
#endif /* (USE_U32_STAT_CNT) */
    return;
}

void
refresh_stats_slow(void)
{
    tg_stats_t *statp;
    U32 tempRxBD;
    U32 tempJumboRxBD;
    U32 tempMiniRxBD;
    U32 tempTotalRxBD;
    U32 tempOperStats;
    U32 tempAdminStats;
    U32 tempSendBDs;
    U32 rx_state, tx_state;

    /* save sw stats that shouldn't be refreshed */
    tempSendBDs = nicfp->stats.nicSendBDs;
    tempRxBD = nicfp->stats.nicRecvBDs;
    tempJumboRxBD = nicfp->stats.nicJumboRecvBDs;
    tempMiniRxBD = nicfp->stats.nicMiniRecvBDs;
    tempTotalRxBD = nicfp->stats.nicTotalRecvBDs;
    tempOperStats = nicfp->stats.ifOperStatus;
    tempAdminStats = nicfp->stats.ifAdminStatus;

    /* refresh sw stats */
    statp = &nicp->gen_info.stats;
    wcopy((U32 *)statp, (U32 *)&nicfp->stats, sizeof(struct tg_stats) >> 2);

    /* restore sw stats */
    nicfp->stats.nicSendBDs = tempSendBDs;
    nicfp->stats.nicRecvBDs = tempRxBD;
    nicfp->stats.nicJumboRecvBDs = tempJumboRxBD;
    nicfp->stats.nicMiniRecvBDs = tempMiniRxBD;
    nicfp->stats.nicTotalRecvBDs = tempTotalRxBD;
    nicfp->stats.ifOperStatus = tempOperStats; 
    nicfp->stats.ifAdminStatus = tempAdminStats; 

    /* save mac state to start to refresh hw stats */
    rx_state = trp->mac_control.mac_rx_state;
    tx_state = trp->mac_control.mac_tx_state;

    /* stop mac after next packet before refresh hw macstats area */
    trp->mac_control.mac_rx_state = 
	(rx_state & ~TG_MAC_RX_ATTN_MASK) |
	TG_MAC_RX_STATE_STOP_NEXT;
    trp->mac_control.mac_tx_state = 
	(tx_state & ~TG_MAC_TX_ATTN_MASK) |
	TG_MAC_TX_STATE_STOP_NEXT;
	
    /* WARNING:
     * we have no way to know when mac gonna stop,
     * just assume it stops right away,
     * and start to refresh the mac_stats hardware area
     * if this is not good, then we need to set a timer variable
     * and do the update after timer expires in the timer routine
     * the disadvantage for a timer wait is no tx and rx allowed
     * during this waiting period.
     */
    tsp->macstats.coll_1	= statp->nicMacTxCollisionHistogram[0];
    tsp->macstats.coll_2	= statp->nicMacTxCollisionHistogram[1];
    tsp->macstats.coll_3	= statp->nicMacTxCollisionHistogram[2];
    tsp->macstats.coll_4	= statp->nicMacTxCollisionHistogram[3];
    tsp->macstats.coll_5	= statp->nicMacTxCollisionHistogram[4];
    tsp->macstats.coll_6	= statp->nicMacTxCollisionHistogram[5];
    tsp->macstats.coll_7	= statp->nicMacTxCollisionHistogram[6];
    tsp->macstats.coll_8	= statp->nicMacTxCollisionHistogram[7];
    tsp->macstats.coll_9	= statp->nicMacTxCollisionHistogram[8];
    tsp->macstats.coll_10	= statp->nicMacTxCollisionHistogram[9];
    tsp->macstats.coll_11	= statp->nicMacTxCollisionHistogram[10];
    tsp->macstats.coll_12	= statp->nicMacTxCollisionHistogram[11];
    tsp->macstats.coll_13	= statp->nicMacTxCollisionHistogram[12];
    tsp->macstats.coll_14	= statp->nicMacTxCollisionHistogram[13];
    tsp->macstats.coll_15	= statp->nicMacTxCollisionHistogram[14];

    tsp->macstats.excess_colls	= statp->dot3StatsExcessiveCollisions;
    tsp->macstats.late_coll	= statp->dot3StatsLateCollisions;
    tsp->macstats.underrun	= statp->dot3StatsInternalMacTransmitErrors;
    tsp->macstats.crs_err	= statp->dot3StatsCarrierSenseErrors;
#if (0) /* XXX */
    tsp->macstats.defers	= statp->dot3StatsDeferredTransmission;
    tsp->macstats.crc_err	= statp->nicMacTxCrcErrs;
#endif /* XXX */

    tsp->macstats.drop_ula	= statp->nicMacRxDropUla;
    tsp->macstats.drop_mc	= statp->nicMacRxDropMcast;
    tsp->macstats.drop_fc	= statp->nicMacRxFlowControl;
    tsp->macstats.drop_space	= statp->nicMacRxDropSpace;
    tsp->macstats.coll		= statp->nicMacRxColls;
    tsp->macstats.kept_uc	= (U32)statp->ifHCInUcastPkts;
    tsp->macstats.kept_bc	= (U32)statp->ifHCInBroadcastPkts;
    tsp->macstats.kept_mc	= (U32)statp->ifHCInMulticastPkts;

    /* restart mac */
    if (!(rx_state & TG_MAC_RX_STATE_STOP_NEXT)) {
	trp->mac_control.mac_rx_state &=
	    ~(TG_MAC_RX_ATTN_MASK | TG_MAC_RX_STATE_STOP_NEXT);
    }
    if (!(tx_state & TG_MAC_TX_STATE_STOP_NEXT)) {
	trp->mac_control.mac_tx_state &=
	    ~(TG_MAC_TX_ATTN_MASK | TG_MAC_TX_STATE_STOP_NEXT);
    }

#if (USE_U32_STAT_CNT)
    /* reset */
    nicfp->ifHCInOctets			= 0;
    nicfp->ifHCOutOctets		= 0;
    nicfp->ifHCOutPkts[UCAST_PKT_INDEX]	= 0;
    nicfp->ifHCOutPkts[MCAST_PKT_INDEX]	= 0;
    nicfp->ifHCOutPkts[BCAST_PKT_INDEX]	= 0;
#endif /* (USE_U32_STAT_CNT) */

    nicfp->refresh_stats_active = FALSE;
    nicfp->stats.nicCmdsRefreshStats++;

    NIC_TRACE(TRACE_TYPE_STATS, "st2nicCp", 0x63500,
	nicfp->stats.nicCmdsRefreshStats,
	0,
	0,
	0);

    return;
}

void
h_send_bd_ready(void)
{
    S32 next_index;
    S32 nentries;
    S32 to_move;
    S32 space;

    if (nicfp->send_prod_index_queued != nicfp->send_prod_index) {

	/* how many new entries to move */
	nentries = nicfp->send_prod_index - nicfp->send_prod_index_queued;
	if (nentries < 0) { /* wrap */
	    nentries += nicfp->send_ring_size;
	}

	/* how much space in our recv_ring until wrap */
	space =
	    (nicfp->send_cons_index_queued <= nicfp->send_prod_index_queued)?
	    (nicfp->send_ring_size - nicfp->send_prod_index_queued):
	    (nicfp->send_cons_index_queued - nicfp->send_prod_index_queued - 1);

	NIC_TRACE(TRACE_TYPE_SEND, "#sBdRdy", 0x60100,
	    nicfp->send_prod_index,
	    nicfp->send_prod_index_queued,
	    nicfp->send_cons_index_queued,
	    (nentries << 16) | space);

	/* Space must never be 0; if it zero, Driver is fooling me */
	ASSERT(space != 0);

	to_move = MIN(space, nentries);

	next_index =
	    (nicfp->send_prod_index_queued + to_move) &
	    (nicfp->send_ring_size - 1);

	if (!(*nicfp->q_dma_to_nic_stub)
		(nicfp->send_ring_ptr +
		    (nicfp->send_prod_index_queued * sizeof(struct send_bd)),
		(U32)&nicfp->send_ring[nicfp->send_prod_index_queued],
		sizeof(struct send_bd) * to_move,
		TIGON_TYPE_SEND_BD,
		next_index,
		nicfp->dma_to_nic_bd_state)) {

	    #if (TRACE | TRACE_ALL)
	    static U32 fail_cnt = 0;
	    #endif /* (TRACE | TRACE_ALL) */

	    NIC_TRACE(TRACE_TYPE_SEND, "QsBdF", 0x60200,
		to_move,
		++fail_cnt,
		0,
		0);
	} else {
	    nicfp->send_prod_index_queued = next_index;

	    NIC_TRACE(TRACE_TYPE_SEND, "QsBdOk", 0x60300,
		to_move,
		0,
		0,
		0);
	}
    } else {
	#if (TRACE | TRACE_ALL)
	static U32 fail_cnt = 0;
	#endif /* (TRACE | TRACE_ALL) */

	/* spurious send bd producer update from driver */
	NIC_TRACE(TRACE_TYPE_SEND, "#sBdRdy?", 0x60400,
	    nicfp->send_prod_index,
	    nicfp->send_prod_index_queued,
	    nicfp->send_cons_index_queued,
	    ++fail_cnt);
    }

    if (nicfp->send_prod_index_queued == nicfp->send_prod_index) {
	clear_event_reg(TG_FW_EVENT_SEND_BD);
    }

    RETURN(TG_FW_EVENT_NUM_SEND_BD);
}

/* null routine to assist with scratchpad setup */
void h_send_bd_ready_end(void) {}

void
h_send_data_ready(void)
{
    struct send_bd *bdp;
    struct tg_mac_descr hd;
    tg_hostaddr_t host_addr;
    U32 prod_index;
    U16 frame_len;
    U16 bd_flags;

    /* see which event we hit when we receive an updated send producer */
    prod_index = is_global_flags(GF_RCB_TX_HOST_RING)?
		    nicfp->send_prod_index_avail:
		    nicfp->send_prod_index;

    if (nicfp->send_cons_index_queued != prod_index) {

	bdp = &nicfp->send_ring[nicfp->send_cons_index_queued];
	host_addr = bdp->host_addr;
	frame_len = bdp->sendbd_length;
	bd_flags = bdp->sendbd_flags;

	NIC_TRACE(TRACE_TYPE_SEND, "#sDtRdy0", 0x60500,
	    *(U32 *)bdp,	/* host addr */
	    *((U32 *)bdp + 1),	/* host addr */
	    *((U32 *)bdp + 2),	/* len & flags */
	    *((U32 *)bdp + 3));	/* reserved & vlan tag */

	NIC_TRACE(TRACE_TYPE_SEND, "sDtBdIdx", 0x60600,
	    prod_index,
	    trp->mac_control.mac_tx_state,
	    trp->local_mem_conf.tx_buf_producer,
	    trp->local_mem_conf.tx_buf_consumer);

	/* check tx buffer space */
	if (nicfp->tx_buf_space < frame_len) {

  	    /* re-calculate tx buffer space
	     * HD_RETRY_RED_ZONE in case of retransmission retry in HD mode
	     * 8 for double words boundary
	     */
  	    nicfp->tx_buf_space =
  		trp->local_mem_conf.tx_buf_consumer -
		nicfp->tx_buf_next -
		(HD_RETRY_RED_ZONE + 8);

  	    if (nicfp->tx_buf_space <= -(HD_RETRY_RED_ZONE + 8))
  		nicfp->tx_buf_space += NIC_TX_BUF_SIZE;
  
	    /* check tx buffer space again */
	    if (nicfp->tx_buf_space < frame_len) {
		nicfp->stats.nicOutOfTxBufSpaceFrameRetry++;

		NIC_TRACE(TRACE_TYPE_SEND, "txBufFul", 0x60700,
		    nicfp->tx_buf_space,
		    frame_len,
		    trp->local_mem_conf.mac_tx_descr_consumer,
		    nicfp->tx_buf_next);
		RETURN(TG_FW_EVENT_NUM_SEND_DATA);
	    }
	}

	/* check tx mac descriptor space */
	if (!trp->local_mem_conf.mac_tx_descr_next) {
	    nicfp->stats.nicTxMacDescrRingFull++;                       

	    NIC_TRACE(TRACE_TYPE_SEND, "txMacD_F", 0x60800,
		      trp->local_mem_conf.mac_tx_descr_producer,
		      trp->local_mem_conf.mac_tx_descr_consumer,
		      nicfp->stats.nicTxMacDescrRingFull,
		      0);
	    RETURN(TG_FW_EVENT_NUM_SEND_DATA);
	}

	/* save vlan tag if necessary */
	if (nicfp->tx_new_frame) {
	    if (is_global_flags(GF_RCB_TX_VLAN_ASSIST) &&
		(bd_flags & BD_FLAG_VLAN_TAG)) {

		nicfp->tx_vlan_tag = (bdp->sendbd_vlan_tag << 16);
		nicfp->tx_insert_vlan_tag = TG_MAC_TX_W0_ADD_TAG;

		NIC_TRACE(TRACE_TYPE_SEND, "sDtVTag", 0x60900,
		    bdp->sendbd_vlan_tag,
		    0,
		    0,
		    0);
	    } else {
		nicfp->tx_insert_vlan_tag = 0;
	    }
	}

	if (bd_flags & BD_FLAG_END) {

	    /* start dma data to nic */
	    if (!(*nicfp->q_dma_to_nic_stub)
		    (host_addr,
		    nicfp->tx_buf_next,
		    frame_len,
		    (bd_flags & BD_FLAG_COAL_NOW)?
			TIGON_TYPE_SEND_DATA_LAST_COAL:
			TIGON_TYPE_SEND_DATA_LAST,
		    nicfp->send_cons_index_queued,
		    nicfp->dma_to_nic_data_state |
			TG_DMA_STATE_UPDATE_TX_BUF_PROD)) {

		#if (TRACE | TRACE_ALL)
		static U32 fail_cnt = 0;
		#endif /* (TRACE | TRACE_ALL) */

		NIC_TRACE(TRACE_TYPE_SEND, "QsDtL_F", 0x61000,
		    nicfp->send_cons_index_queued,
		    ++fail_cnt,
		    0,
		    0);
		RETURN(TG_FW_EVENT_NUM_SEND_DATA);
	    }

	    NIC_TRACE(TRACE_TYPE_SEND, "QsDtL_Ok", 0x61100,
		nicfp->send_cons_index_queued,
		host_addr,
		nicfp->tx_buf_next,
		frame_len);

	    /* maintain our running frame length */
	    nicfp->tx_frame_len += frame_len;

	    /* setup frame starting address in BD */
	    bdp->sendbd_nic_addr = nicfp->tx_buf_prod;

	    /* queue tx descriptor for the mac */
	    if (nicfp->tx_insert_vlan_tag) {
		hd.w0 = nicfp->tx_buf_prod | TG_MAC_TX_W0_ADD_TAG;
		hd.w1 = nicfp->tx_frame_len | nicfp->tx_vlan_tag;
	    } else {
		hd.w0 = nicfp->tx_buf_prod;
		hd.w1 = nicfp->tx_frame_len;
	    }
	    enq_mac_tx(&hd);
#if (USE_U32_STAT_CNT)
	    nicfp->ifHCOutOctets += nicfp->tx_frame_len;
#else /* (USE_U32_STAT_CNT) */
	    nicfp->stats.ifHCOutOctets += nicfp->tx_frame_len;
#endif /* (USE_U32_STAT_CNT) */
	    update_tx_pkt_stats(bd_flags);

	    /* adjust tx info */
	    nicfp->tx_new_frame = TRUE;
	    nicfp->tx_buf_space -= ROUNDUP(frame_len, 8);

#if 0 /* new hardware mechanism is broken */
	    trp->local_mem_conf.tx_buf_next = nicfp->tx_frame_len;
	    trp->local_mem_conf.tx_buf_next = 0;
	    nicfp->tx_buf_prod = trp->local_mem_conf.tx_buf_next;
	    ASSERT(nicfp->tx_buf_prod);
#else /* 0 */
	    nicfp->tx_buf_prod = ROUNDUP((nicfp->tx_buf_next + frame_len), 8);
	    TX_WRAP_BUF(nicfp->tx_buf_prod);
#endif /* 0 */

	    nicfp->tx_buf_next = nicfp->tx_buf_prod;
	    nicfp->tx_frame_len = 0;

	} else if (frame_len) {

	    /* start dma data to nic */
	    if (!(*nicfp->q_dma_to_nic_stub)
		    (host_addr,
		    nicfp->tx_buf_next,
		    frame_len,
		    (bd_flags & BD_FLAG_COAL_NOW)?
			TIGON_TYPE_SEND_DATA_COAL:
			TIGON_TYPE_SEND_DATA,
		    nicfp->send_cons_index_queued,
		    nicfp->dma_to_nic_data_state)) {

		#if (TRACE | TRACE_ALL)
		static U32 fail_cnt = 0;
		#endif /* (TRACE | TRACE_ALL) */

		NIC_TRACE(TRACE_TYPE_SEND, "QsDtF", 0x61200,
		    nicfp->send_cons_index_queued,
		    ++fail_cnt,
		    0,
		    0);
		RETURN(TG_FW_EVENT_NUM_SEND_DATA);
	    }

	    NIC_TRACE(TRACE_TYPE_SEND, "QsDtOk", 0x61300,
		nicfp->send_cons_index_queued,
		host_addr,
		nicfp->tx_buf_next,
		frame_len);

	    /* adjust tx info */
	    nicfp->tx_new_frame = FALSE;
	    nicfp->tx_frame_len += frame_len;
	    nicfp->tx_buf_space -= frame_len;
	    nicfp->tx_buf_next += frame_len;
	    TX_WRAP_BUF(nicfp->tx_buf_next);
	}

	/* send bd consumed */
	if (is_global_flags(GF_RCB_TX_HOST_RING))
	    nicfp->stats.nicSendBDs--;

	/* update consumer index */
	nicfp->send_cons_index_queued =
	    (nicfp->send_cons_index_queued + 1) & (nicfp->send_ring_size - 1);
    } else {
	#if (TRACE | TRACE_ALL)
	static U32 fail_cnt = 0;
	#endif /* (TRACE | TRACE_ALL) */

	/* spurious send bd producer update from driver */
	NIC_TRACE(TRACE_TYPE_SEND, "#sDtRdy?", 0x61400,
	    nicfp->send_prod_index,
	    nicfp->send_prod_index_queued,
	    nicfp->send_cons_index_queued,
	    ++fail_cnt);
    }

    if (nicfp->send_cons_index_queued == prod_index) {
        clear_event_reg(TG_FW_EVENT_SEND_DATA);
    }

    RETURN(TG_FW_EVENT_NUM_SEND_DATA);
}

/* null routine to assist in setting up scratchpad */
void h_send_data_ready_end(void) {}

void
h_send_data_ready_cksum(void)
{
    struct tg_mac_descr *md_prod, *md_cons;
    tg_hostaddr_t host_addr;
    struct send_bd *bdp;
    U32 dma_state;
    U32 prod_index;
    S32 mac_descs;
    U32 insert_vlan;
    U16 frame_len;
    U16 bd_flags;
    bool coal_now;

    /* see which event we hit when we receive an updated send producer */
    prod_index = is_global_flags(GF_RCB_TX_HOST_RING)?
		    nicfp->send_prod_index_avail:
		    nicfp->send_prod_index;

    if (nicfp->send_cons_index_queued != prod_index) {

	bdp = &nicfp->send_ring[nicfp->send_cons_index_queued];
	host_addr = bdp->host_addr;
	frame_len = bdp->sendbd_length;
	bd_flags = bdp->sendbd_flags;

	NIC_TRACE(TRACE_TYPE_SEND, "#sDtRdy0", 0x61500,
	    *(U32 *)bdp,        /* host addr */
	    *((U32 *)bdp + 1),  /* host addr */
	    *((U32 *)bdp + 2),  /* len & flags */
	    *((U32 *)bdp + 3)); /* reserved & vlan tag */


	NIC_TRACE(TRACE_TYPE_SEND, "sDtBdIdx", 0x61600,
	    prod_index,
	    trp->mac_control.mac_tx_state,
	    trp->local_mem_conf.tx_buf_producer,
	    trp->local_mem_conf.tx_buf_consumer);

	/* check tx buffer space */
	if (nicfp->tx_buf_space < frame_len) {

  	    /* re-calculate tx buffer space
	     * HD_RETRY_RED_ZONE in case of retransmission retry in HD mode
	     * TX_BUF_RED_ZONE_SIZE in case of tx buf wrap on protocol header
	     */
  	    nicfp->tx_buf_space =
  		trp->local_mem_conf.tx_buf_consumer -
  		nicfp->tx_buf_next -
		(HD_RETRY_RED_ZONE + TX_BUF_RED_ZONE_SIZE);
  
  	    if (nicfp->tx_buf_space <=
		    -(HD_RETRY_RED_ZONE + TX_BUF_RED_ZONE_SIZE))
  		nicfp->tx_buf_space += NIC_TX_BUF_SIZE;

	    /* check tx buffer space again */
	    if (nicfp->tx_buf_space < frame_len) {
		nicfp->stats.nicOutOfTxBufSpaceFrameRetry++;

		NIC_TRACE(TRACE_TYPE_SEND, "txBufFul", 0x61700,
                    nicfp->tx_buf_space,
		    frame_len,
                    nicfp->stats.nicOutOfTxBufSpaceFrameRetry,
		    0);
		RETURN(TG_FW_EVENT_NUM_SEND_DATA);
	    }
	}

	/* check tx mac descriptor space */
	if (!(md_prod = trp->local_mem_conf.mac_tx_descr_next)) {
	    nicfp->stats.nicTxMacDescrRingFull++;

	    NIC_TRACE(TRACE_TYPE_SEND, "txMacD_F", 0x61800,
		 trp->local_mem_conf.mac_tx_descr_producer,
		 trp->local_mem_conf.mac_tx_descr_consumer,
		 nicfp->stats.nicTxMacDescrRingFull,
		 0);
	    RETURN(TG_FW_EVENT_NUM_SEND_DATA);
	}
	(U32)md_cons =
	    ROUNDDN((U32)trp->local_mem_conf.mac_tx_descr_consumer, 8);
	mac_descs = md_cons - md_prod; /* free count + 1 */
	if (mac_descs <= 0)
	    mac_descs += TG_NUM_MAC_TX_DESCRS;
	if (mac_descs <= nicfp->tx_q_mac_pending_cnt) {
	    nicfp->stats.nicTxMacDescrRingFull++;

	    NIC_TRACE(TRACE_TYPE_SEND, "txMacD_F", 0x61900,
		 trp->local_mem_conf.mac_tx_descr_producer,
		 trp->local_mem_conf.mac_tx_descr_consumer,
		 nicfp->stats.nicTxMacDescrRingFull,
		 (mac_descs << 16) | nicfp->tx_q_mac_pending_cnt);
	    RETURN(TG_FW_EVENT_NUM_SEND_DATA);
	}

	coal_now = (bd_flags & BD_FLAG_COAL_NOW)? TRUE: FALSE;

	if (nicfp->tx_new_frame) {
	    /* set the clear cksum bit on sending a new frame */
	    dma_state = nicfp->dma_to_nic_data_state | TG_DMA_STATE_CLEAR_CKSUM;

	    /* save vlan tag if necessary */
	    if (is_global_flags(GF_RCB_TX_VLAN_ASSIST) &&
		(bd_flags & BD_FLAG_VLAN_TAG)) {

		nicfp->tx_vlan_tag = (bdp->sendbd_vlan_tag << 16);
		nicfp->tx_insert_vlan_tag = TG_MAC_TX_W0_ADD_TAG;

		NIC_TRACE(TRACE_TYPE_SEND, "sDtVTag", 0x62000,
		    bdp->sendbd_vlan_tag,
		    0,
		    0,
		    0);
	    } else {
		nicfp->tx_insert_vlan_tag = 0;
	    }
	} else {
	    dma_state = nicfp->dma_to_nic_data_state;
	}

	if (bd_flags & BD_FLAG_END) {

	    /* start dma data to nic */
	    insert_vlan = nicfp->tx_insert_vlan_tag;
	    if (!(*nicfp->q_dma_to_nic_stub)
		    (host_addr,
		    nicfp->tx_buf_next,
		    frame_len,
		    (insert_vlan)?
			(coal_now)? TIGON_TYPE_SEND_DATA_LAST_VLAN_COAL:
				    TIGON_TYPE_SEND_DATA_LAST_VLAN:
			(coal_now)? TIGON_TYPE_SEND_DATA_LAST_COAL:
				    TIGON_TYPE_SEND_DATA_LAST,
		    (insert_vlan)?
			(nicfp->send_cons_index_queued | nicfp->tx_vlan_tag):
			nicfp->send_cons_index_queued,
		    dma_state | TG_DMA_STATE_UPDATE_TX_BUF_PROD)) {

		#if (TRACE | TRACE_ALL)
		static U32 fail_cnt = 0;
		#endif /* (TRACE | TRACE_ALL) */

		NIC_TRACE(TRACE_TYPE_SEND, "QsDtL_F ", 0x62100,
		    nicfp->send_cons_index_queued,
		    ++fail_cnt,
		    0,
		    0);
		RETURN(TG_FW_EVENT_NUM_SEND_DATA);
	    }

	    NIC_TRACE(TRACE_TYPE_SEND, "QsDtL_Ok", 0x62200,
		nicfp->send_cons_index_queued,
		host_addr,
		nicfp->tx_buf_next,
		frame_len);

	    /* increase enq mac tx pending counter */
	    nicfp->tx_q_mac_pending_cnt++;

	    /* setup frame starting address in BD */
	    bdp->sendbd_nic_addr = nicfp->tx_buf_prod;

	    /* adjust tx info */
	    nicfp->tx_new_frame = TRUE;
	    nicfp->tx_buf_space -= ROUNDUP(frame_len, 8);

#if 0 /* new hardware mechanism is broken */
	    trp->local_mem_conf.tx_buf_next = nicfp->tx_frame_len;
	    trp->local_mem_conf.tx_buf_next = 0;
	    nicfp->tx_buf_prod = trp->local_mem_conf.tx_buf_next;
	    ASSERT(nicfp->tx_buf_prod);
#else /* 0 */
	    nicfp->tx_buf_prod = ROUNDUP((nicfp->tx_buf_next + frame_len), 8);
	    TX_WRAP_BUF(nicfp->tx_buf_prod);
#endif /* 0 */
	    /* make sure beginning N bytes of a tx frame are contigous */
	    if (nicfp->tx_buf_prod > NIC_TX_BUF_RED) {
		nicfp->tx_buf_space -= (NIC_TX_BUF_END - nicfp->tx_buf_prod);
		nicfp->tx_buf_prod = NIC_TX_BUF;
	    }
	    nicfp->tx_buf_next = nicfp->tx_buf_prod;

	} else if (frame_len) {

	    /* start dma data to nic */
	    if (!(*nicfp->q_dma_to_nic_stub)
		    (host_addr,
		    nicfp->tx_buf_next,
		    frame_len,
		    (coal_now)?
			TIGON_TYPE_SEND_DATA_COAL:
			TIGON_TYPE_SEND_DATA,
		    nicfp->send_cons_index_queued,
		    dma_state)) {

		#if (TRACE | TRACE_ALL)
		static U32 fail_cnt = 0;
		#endif /* (TRACE | TRACE_ALL) */

		NIC_TRACE(TRACE_TYPE_SEND, "QsDtF", 0x62300,
		    nicfp->send_cons_index_queued,
		    ++fail_cnt,
		    0,
		    0);
		RETURN(TG_FW_EVENT_NUM_SEND_DATA);
	    }

	    NIC_TRACE(TRACE_TYPE_SEND, "QsDtOk", 0x62400,
		nicfp->send_cons_index_queued,
		host_addr,
		nicfp->tx_buf_next,
		frame_len);

	    /* adjust tx info */
	    nicfp->tx_new_frame = FALSE;
	    nicfp->tx_buf_space -= frame_len;
	    nicfp->tx_buf_next += frame_len;
	    TX_WRAP_BUF(nicfp->tx_buf_next);
	}

	/* update consumer index */
	nicfp->send_cons_index_queued =
	    (nicfp->send_cons_index_queued + 1) & (nicfp->send_ring_size - 1);
    } else {
	#if (TRACE | TRACE_ALL)
	static U32 fail_cnt = 0;
	#endif /* (TRACE | TRACE_ALL) */

	/* spurious send bd producer update from driver */
	NIC_TRACE(TRACE_TYPE_SEND, "#sDtRdy?", 0x62500,
	    nicfp->send_prod_index,
	    nicfp->send_prod_index_queued,
	    nicfp->send_cons_index_queued,
	    ++fail_cnt);
    }

    if (nicfp->send_cons_index_queued == prod_index) {
        clear_event_reg(TG_FW_EVENT_SEND_DATA);
    }

    RETURN(TG_FW_EVENT_NUM_SEND_DATA);
}

/* null routine to assist in setting up scratchpad */
void h_send_data_ready_cksum_end(void) {}

void
h_dma_rd_assist(void)
{
    struct tg_dma_descr *ddp;
    dma_ref_ring_t *ref_cons;
    U32 dd_type;
    S32 nbds;
    U32 ref_count;

    ddp = trp->host_dma_assist.read_chan_ref;

    NIC_TRACE_IF(TRACE_TYPE_DMA, "#rDmaCp", 0x62600,
		 ddp->type, TIGON_TYPE_TX,
	trp->host_dma_assist.read_chan_producer,
	trp->host_dma_assist.read_chan_consumer,
	ddp,
	(ddp - tdp->rd_dma_ring));

    NIC_TRACE_IF(TRACE_TYPE_DMA, "rDmaCpD0", 0x62800,
		 ddp->type, TIGON_TYPE_TX,
	*(U32 *)ddp,		/* host addr */
	*((U32 *)ddp + 1),	/* host addr */
	*((U32 *)ddp + 2),	/* nic addr */
	*((U32 *)ddp + 3));	/* reserved & len */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "rDmaCpD1", 0x62900,
		 ddp->type, TIGON_TYPE_TX,
	*((U32 *)ddp + 4),	/* state */
	*((U32 *)ddp + 5),	/* reserved & cksum addr */
	*((U32 *)ddp + 6),	/* type */
	*((U32 *)ddp + 7));	/* index */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "rDmaAvd", 0x62700,
		 ddp->type, TIGON_TYPE_TX,
	nicfp->rd_dma_ref_prod,
	nicfp->rd_dma_ref_cons, 
	nicfp->rd_dma_ref_cons->count,
	nicfp->rd_dma_ref_cons->type);

    /* set up ref for next event */
    ASSERT(nicfp->rd_dma_ref_cons != nicfp->rd_dma_ref_prod);
    nicfp->rd_dma_ref_cons = ref_cons
			   = get_next_rd_dma_ref(nicfp->rd_dma_ref_cons);
    ref_count = ref_cons->count;
    ref_cons->type = TIGON_TYPE_NULL; /* invalidate entry */

    if (ref_cons == nicfp->rd_dma_ref_prod) {
	/*
	 * no more dma completion to take
	 * bump ref pointer since it is one behind the prod pointer
	 */
	trp->host_dma_assist.read_chan_ref = ddp + 1;

    } else {
	/* move ref pointer to one behind the next producer */
	ref_cons = get_next_rd_dma_ref(nicfp->rd_dma_ref_cons);
	trp->host_dma_assist.read_chan_ref = ddp + ref_cons->count;
	ref_cons->type = TIGON_TYPE_NULL; /* invalidate entry */
    }

    /* NOTE:
     * a jump table will be generated if switch (ddp->type) is used
     * ie, all code segments after the switch statement will still
     * run on the external memory even current function is relocated
     * into scratch pad
     * hence, if...else if...else statements are used here
     * and are ordered from most frequent to least.
     */
    dd_type = ddp->type;
    if (dd_type & (TIGON_TYPE_SEND_DATA |
		   TIGON_TYPE_SEND_DATA_LAST |
		   TIGON_TYPE_SEND_DATA_COAL |
		   TIGON_TYPE_SEND_DATA_LAST_COAL)) {

	#if 0 /* debug */
        {
	    /* verify send data count */
            S32 bds = (ddp->index + 1) - nicfp->send_cons_index;
            if (bds < 0)
                bds += nicfp->send_ring_size;
            ASSERT(bds == ref_cons->count);
        }
	#endif /* debug */

	/* update send info */
        nicfp->send_cons_index = (ddp->index + 1) & (nicfp->send_ring_size - 1);
        nicfp->send_coal_bds += ref_count;

	if ((dd_type & (TIGON_TYPE_SEND_DATA_COAL |
			TIGON_TYPE_SEND_DATA_LAST_COAL)) ||
	    (nicfp->send_coal_bds >=
			tsp->gen_com.tuneParms.send_max_coalesced_bds)) {

            send_cons_index_to_host();
            nicfp->stats.nicSendThresholdHit++;
        }

	NIC_TRACE(TRACE_TYPE_SEND, "sDtCp", 0x63000,
	      nicfp->send_cons_index,
	      nicfp->send_coal_bds,
	      ref_count,
	      ddp->index);

    } else if (dd_type & TIGON_TYPE_SEND_BD) {

	U32 dd_index = ddp->index;

	nbds = dd_index - nicfp->send_prod_index_avail;
	if (nbds < 0)
	    nbds += nicfp->send_ring_size;

	nicfp->send_prod_index_avail = dd_index;
	nicfp->stats.nicSendBDs += nbds;
	#if 0
	nicfp->stats.nicTotalSendBDs += nbds; /* obsolete */
	#endif

	set_event_reg(TG_FW_EVENT_SEND_DATA);

	NIC_TRACE(TRACE_TYPE_SEND, "sBdCp", 0x63100,
	      nbds, 
	      nicfp->stats.nicSendBDs,
	      nicfp->send_prod_index_avail,
	      nicfp->send_cons_index_queued);

    } else if (dd_type & (TIGON_TYPE_RECV_BD |
			  TIGON_TYPE_RECV_MINI_BD |
			  TIGON_TYPE_RECV_JUMBO_BD)) {

	/* note the priority */
        if (dd_type & TIGON_TYPE_RECV_BD) {
	    U32 dd_index = ddp->index;
	    nbds = dd_index - nicfp->recv_prod_index_avail;
	    if (nbds < 0)
		nbds += RECV_STANDARD_RING_ENTRIES;

	    #if 0 /* debug */
	    {
		/* make sure upper host addr not corrupted */
		recv_bd_t *bdp = &nicp->recv_ring[nicfp->recv_prod_index_avail];
		while (nbds--) {
		    if (*(U32 *)bdp != 0) {
			NIC_UTRACE("?rxStBd", bdp,
			    *(U32 *)bdp,
			    *((U32 *)bdp + 1),
			    *((U32 *)bdp + 2),
			    *((U32 *)bdp + 3));
			ASSERT(0);
		    }
		    bdp =
			(bdp == &nicp->recv_ring[RECV_STANDARD_RING_ENTRIES-1])?
			nicp->recv_ring:
			bdp++;
		}
	    }
	    #endif /* debug */

            nicfp->recv_prod_index_avail = dd_index; /* update */
            nicfp->stats.nicRecvBDs += nbds;
            nicfp->stats.nicTotalRecvBDs += nbds;

            NIC_TRACE(TRACE_TYPE_RECV_BD, "rStdBdCp", 0x63200,
		nbds, 
		nicfp->stats.nicRecvBDs,
		nicfp->recv_prod_index_avail,
		nicfp->recv_cons_index);

        } else if (dd_type & TIGON_TYPE_RECV_MINI_BD) {
	    U32 dd_index = ddp->index;
	    nbds = dd_index - nicfp->recv_mini_prod_index_avail;
	    if (nbds < 0)
		nbds += RECV_MINI_RING_ENTRIES;

	    #if 0 /* debug */
	    {
		/* make sure upper host addr not corrupted */
		recv_bd_t *bdp = &nicp->recv_mini_ring
				    [nicfp->recv_mini_prod_index_avail];
		while (nbds--) {
		    if (*(U32 *)bdp != 0) {
			NIC_UTRACE("?rxMiBd", bdp,
			    *(U32 *)bdp,
			    *((U32 *)bdp + 1),
			    *((U32 *)bdp + 2),
			    *((U32 *)bdp + 3));
			ASSERT(0);
		    }
		    bdp =
		       (bdp == &nicp->recv_mini_ring[RECV_MINI_RING_ENTRIES-1])?
			nicp->recv_mini_ring:
			bdp++;
		}
	    }
	    #endif /* debug */

            nicfp->recv_mini_prod_index_avail = dd_index; /* update */
            nicfp->stats.nicMiniRecvBDs += nbds;
            nicfp->stats.nicTotalRecvBDs += nbds;

            NIC_TRACE(TRACE_TYPE_RECV_BD, "rMinBdCp", 0x63300,
		nbds, 
		nicfp->stats.nicMiniRecvBDs,
		nicfp->recv_mini_prod_index_avail,
		nicfp->recv_mini_cons_index);

        } else /*if (dd_type & TIGON_TYPE_RECV_JUMBO_BD)*/ {
            /*
	     * if jumbo frames enabled isn't set then we assume we
             * got an overlap with the reset jumbo ring command
             */
            if (is_global_flags(GF_CMD_JUMBO_FRAMES_ENABLED)) {
		U32 dd_index = ddp->index;
		nbds = dd_index - nicfp->recv_jumbo_prod_index_avail;
		if (nbds < 0)
		    nbds += RECV_JUMBO_RING_ENTRIES;

		#if 0 /* debug */
		{
		    /* make sure upper host addr not corrupted */
		    recv_bd_t *bdp;
		    if (is_global_flags(GF_RCB_USE_EXT_RECV_BD)) {
			bdp = (recv_bd_t *)&nicp->ext_recv_jumbo_ring
				    [nicfp->recv_jumbo_prod_index_avail].bd1;
			while (nbds--) {
			    if (*(U32 *)bdp != 0) {
				NIC_UTRACE("?rxEJuBd", bdp,
				    *(U32 *)bdp,
				    *((U32 *)bdp + 1),
				    *((U32 *)bdp + 2),
				    *((U32 *)bdp + 3));
				ASSERT(0);
			    }
			    bdp = 
				(bdp == (recv_bd_t *)&nicp->ext_recv_jumbo_ring
					    [RECV_JUMBO_RING_ENTRIES-1].bd1)?
				(recv_bd_t *)&nicp->ext_recv_jumbo_ring[0].bd1:
				(bdp + 2); /* note */
			}
		    } else {
			bdp = &nicp->recv_jumbo_ring
				[nicfp->recv_jumbo_prod_index_avail];
			while (nbds--) {
			    if (*(U32 *)bdp != 0) {
				NIC_UTRACE("?rxJuBd", bdp,
				    *(U32 *)bdp,
				    *((U32 *)bdp + 1),
				    *((U32 *)bdp + 2),
				    *((U32 *)bdp + 3));
				ASSERT(0);
			    }
			    bdp = (bdp == &nicp->recv_jumbo_ring
					    [RECV_JUMBO_RING_ENTRIES-1])?
				    nicp->recv_jumbo_ring:
				    bdp++;
			}
		    }
		}
		#endif /* debug */

                nicfp->recv_jumbo_prod_index_avail = dd_index; /* update */
                nicfp->stats.nicJumboRecvBDs += nbds;
                nicfp->stats.nicTotalRecvBDs += nbds;

	     NIC_TRACE(TRACE_TYPE_RECV_BD, "rJumBdCp", 0x63400,
		    nbds,
		    nicfp->stats.nicJumboRecvBDs,
		    nicfp->recv_jumbo_prod_index_avail,
		    nicfp->recv_jumbo_cons_index);
            }
        }

	if (!is_global_flags(GF_VAR_MAC_RX_COMP_SUSP)) {
	    /* make sure we're receiving packets now */
	    event_mask |= TG_FW_EVENT_MAC_RX_COMP;
	}

    } else if (dd_type & TIGON_TYPE_REFRESH_STATS) {

	refresh_stats_slow();

    } else if (dd_type & TIGON_TYPE_SEND_DUMMY_DMA) {
	; /* nothing to do */
    } else {
        ASSERT(0);
    }

    NIC_TRACEV_IF(TRACE_TYPE_DMA, "rDmCpOk", 0x63600,
		  ddp->type, TIGON_TYPE_TX,
	trp->host_dma_assist.read_chan_producer,
	trp->host_dma_assist.read_chan_consumer,
	trp->host_dma_assist.read_chan_ref,
	ddp);

    RETURN(TG_FW_EVENT_NUM_DMA_RD_ASST_HI);
}

/* null routine to assist with scratchpad setup */
void h_dma_rd_assist_end(void) {}

void
h_dma_rd_assist_cksum(void)
{
    struct tg_dma_descr *ddp;
    struct tg_mac_descr hd, md;
    U32 dd_type;
    U32 vlan_tag;
    S32 nbds;
    bool insert_vtag;

    ddp = trp->host_dma_assist.read_chan_ref;
    ASSERT(ddp != trp->host_dma_assist.read_chan_consumer);
    trp->host_dma_assist.read_chan_ref = ddp + 1;
    insert_vtag = FALSE;
    vlan_tag = 0; /* eliminate compiler warning */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "#rDmaCp", 0x63700,
		 ddp->type, TIGON_TYPE_TX,
	trp->host_dma_assist.read_chan_producer,
	trp->host_dma_assist.read_chan_consumer,
	ddp,
	(ddp - tdp->rd_dma_ring));

    NIC_TRACE_IF(TRACE_TYPE_DMA, "rDmaCpD0", 0x63800,
		 ddp->type, TIGON_TYPE_TX,
	*(U32 *)ddp,		/* host addr */
	*((U32 *)ddp + 1),	/* host addr */
	*((U32 *)ddp + 2),	/* nic addr */
	*((U32 *)ddp + 3));	/* reserved & len */

    NIC_TRACE_IF(TRACE_TYPE_DMA, "rDmaCpD1", 0x63900,
		 ddp->type, TIGON_TYPE_TX,
	*((U32 *)ddp + 4),	/* state */
	*((U32 *)ddp + 5),	/* reserved & cksum addr */
	*((U32 *)ddp + 6),	/* type */
	*((U32 *)ddp + 7));	/* index */

    /* NOTE:
     * a jump table will be generated if switch (ddp->type) is used
     * ie, all code segments after the switch statement will still
     * run on the external memory even current function is relocated
     * into scratch pad
     * hence, if...else if...else statements are used here
     * and are ordered from most frequent to least.
     */
    dd_type = ddp->type;
    if (dd_type & (TIGON_TYPE_SEND_DATA_LAST |
		   TIGON_TYPE_SEND_DATA_LAST_COAL)) {
	send_bd_t *bdp;
	U32 tcpudp_cksum;
	U32 sum;
	U32 pseudo;
	U16 *buf;
	U32 bd_flags;
	U32 frame_start;
	U32 ip_start;
	U32 tcpudp_start;
	U32 dd_index;
	U32 hlen;
	U32 eth_hlen;
	U16 dd_cksum;
	U16 eth_type;
	bool tx_first_frame;

    send_data_last:
	dd_cksum = (U16)ddp->cksum; /* tigon 2 only */
	dd_index = ddp->index;
	bdp = &nicfp->send_ring[dd_index];
	bd_flags = bdp->sendbd_flags;
	frame_start = bdp->sendbd_nic_addr;

	ASSERT(frame_start <= NIC_TX_BUF_RED);

	NIC_TRACE(TRACE_TYPE_SEND, "sSmBd0", 0x64000,
	    *(U32 *)bdp,	/* host addr */
	    *((U32 *)bdp + 1),	/* host addr */
	    *((U32 *)bdp + 2),	/* len & flags */
	    *((U32 *)bdp + 3));	/* reserved & vlan tag */

	/* to support 802.3 frame */
	eth_hlen = 0;
	eth_type = ((eth_t *)frame_start)->ethFrameType;

	if ((eth_type <= (ETH_MAX_LEN - ETH_HLEN)) ||
	    (eth_type == ETH_JUMBO)) { /* 802.3 frame */

	    if (is_global_flags(GF_RCB_TX_IEEE_SNAP_CKSUM) &&
		(((eth_t *)frame_start)->eth_snap_type == ETH_IP) &&
		is_llc_snap((U16 *)&((eth_t *)frame_start)->eth_llc_dsap)) {

		eth_hlen = ETH_LLC_HLEN;

		NIC_TRACE(TRACE_TYPE_SEND, "sSmLLC0", 0x64100,
		    *(U32 *)frame_start,
		    *((U32 *)frame_start + 1),
		    *((U32 *)frame_start + 2),
		    *((U32 *)frame_start + 3));

		NIC_TRACE(TRACE_TYPE_SEND, "sSmLLC1", 0x64200,
		    *((U32 *)frame_start + 4),
		    *((U32 *)frame_start + 5),
		    *((U32 *)frame_start + 6),
		    *((U32 *)frame_start + 7));
	    }
	} else if (eth_type == ETH_IP) {
	    eth_hlen = ETH_HLEN;

	    NIC_TRACE(TRACE_TYPE_SEND, "sSmEthr0", 0x64300,
		*(U32 *)frame_start,
		*((U32 *)frame_start + 1),
		*((U32 *)frame_start + 2),
		*((U32 *)frame_start + 3));

	    NIC_TRACE(TRACE_TYPE_SEND, "sSmEthr1", 0x64400,
		*((U32 *)frame_start + 4),
		*((U32 *)frame_start + 5),
		*((U32 *)frame_start + 6),
		*((U32 *)frame_start + 7));
	}

	#if (TX_CKSUM_DEBUG)
	if (eth_hlen) { /* ip packet */
	    static U32 tx_ip_id_unmatch = 0;
	    static U16 tx_ip_id = 0xdead;
	    U32 *w;
	    U16 *h;

	    ip_start = frame_start + eth_hlen;

	    if (!nicfp->tx_ip_frag) {
		tx_ip_id = ((ip_t *)ip_start)->ipId;

		h = (U16 *)ip_start;
		NIC_UTRACE("txIpId_0", tx_ip_id,
		    (*(h)     << 16) + *(h + 1),
		    (*(h + 2) << 16) + *(h + 3),
		    (*(h + 4) << 16) + *(h + 5),
		    (*(h + 6) << 16) + *(h + 7));
	    } else {
		if (((ip_t *)ip_start)->ipId != tx_ip_id) {
		    h = (U16 *)ip_start;
		    NIC_UTRACE("?txIpId0", tx_ip_id,
			(*(h)     << 16) + *(h + 1),
			(*(h + 2) << 16) + *(h + 3),
			(*(h + 4) << 16) + *(h + 5),
			(*(h + 6) << 16) + *(h + 7));

		    w = (U32 *)bdp;
		    NIC_UTRACE("?txIpId1", dd_index,
			*w,
			*(w + 1),
			*(w + 2),
			*(w + 3));
		    NIC_UTRACE("?txIpId2", ++tx_ip_id_unmatch,
			*(w + 4),
			*(w + 5),
			*(w + 6),
			*(w + 7));
		}
		if (!(bd_flags & (BD_FLAG_IP_FRAG | BD_FLAG_IP_FRAG_END))) {
		    w = (U32 *)bdp;
		    NIC_UTRACE("?txBadBd", dd_index,
			*w,
			*(w + 1),
			*(w + 2),
			*(w + 3));
		    /* ASSERT(0); */
		}
	    }
	}
	#endif /* (TX_CKSUM_DEBUG) */

	/* save first frame info */
	if (!nicfp->tx_ip_frag) {
	    tx_first_frame = TRUE;
	    nicfp->tx_first_frame_eth_hlen = eth_hlen;
	    nicfp->tx_first_frame_start = frame_start;
	    nicfp->tx_first_frame_index = dd_index; /* debug only */

	    if (bd_flags & (BD_FLAG_IP_FRAG | BD_FLAG_IP_FRAG_END)) {
		nicfp->tx_ip_frag = TRUE;
	    }
	} else {
	    tx_first_frame = FALSE;
	}

	/* do cksum for ip packets only */
	if (eth_hlen) { /* ip packet */

	    ip_start = frame_start + eth_hlen;
	    hlen = IP_HLEN(ip_start);
	    tcpudp_start = ip_start + hlen;
	    sum = 0;

	    /* ip cksum */
	    if (is_global_flags(GF_RCB_TX_IP_CKSUM) &&
		(bd_flags & BD_FLAG_IP_CKSUM)) {

		/* calculate ip cksum */
		buf = (U16 *)ip_start;
		do {
		    sum += *buf++;
		} while ((U32)buf < tcpudp_start);
		sum = (sum >> 16) + (sum & 0xffff);
		sum = (sum >> 16) + (sum & 0xffff);

		/* store into frame */
		*(U16 *)&((ip_t *)ip_start)->ipCheckSum = (U16)~sum;

		NIC_TRACE(TRACE_TYPE_SEND, "sSmIpSm", 0x64500,
		    dd_index,
		    (~sum) & 0xffff,
		    0,
		    0);
	    }

	    /* tcp or udp cksum */
	    if (is_global_flags(GF_RCB_TX_TCP_UDP_CKSUM) &&
		(bd_flags & BD_FLAG_TCP_UDP_CKSUM) &&
		nicfp->tx_first_frame_eth_hlen) {

                /* Mark fisrt frame start as nicfp->tx_first_frame_start.
                 * This is where the final cksum will be stored. 
		 * Calculate the Pseudo Header for this frame as well.
		 * For the subsequent frames, simply calcualte the IP
		 * header Checksum and data Checksum.
		 * Keep on accumulating the data checksum.
		 * Do not transmit these frames, put them into a 
		 * queue instead.
		 * Once the final fragment is received,
		 * then enque all frames for MAC Transmission.
		 */

		U8 ip_protocol;

		/* do tcp/udp cksum for tcp/udp packets only */
		ip_protocol = 
		 ((ip_t *)(nicfp->tx_first_frame_start +
			   nicfp->tx_first_frame_eth_hlen))->ipProtocol;

		if ((ip_protocol != IP_PROTO_TCP) &&
		    (ip_protocol != IP_PROTO_UDP)) {

		    /* should not happen, bypass cksum calculation */
		    NIC_UTRACE("isntTcpU", 0x64600,
			dd_index,
			ip_protocol,
			nicfp->tx_first_frame_index,
			nicfp->tx_first_frame_start);
		} else {
		    /* pseudo header */
		    if (is_global_flags(GF_RCB_TX_NO_PSEUDO_HDR_CKSUM)) {
			pseudo = 0;

		    } else if (!tx_first_frame) {

			/* NOT the first frame, just need to 
			 * add up tcp/udp data length of current frame 
			 */
			pseudo = ((ip_t *)ip_start)->ipLen - hlen;

			NIC_TRACE(TRACE_TYPE_SEND, "sNt1stIp", 0x64700,
			    pseudo,
			    0,
			    0,
			    0);
		    } else {
			/* calculate pseudo hdr cksum for the first frame */
			buf = (U16 *)&((ip_t *)ip_start)->ipSourceAddr;
			pseudo = *buf++;
			pseudo += *buf++;
			pseudo += *buf++;
			pseudo += *buf;
			pseudo += ((ip_t *)ip_start)->ipProtocol;
			pseudo += ((ip_t *)ip_start)->ipLen - hlen;

			NIC_TRACE(TRACE_TYPE_SEND, "s1stIp", 0x64800,
			    pseudo,
			    0,
			    0,
			    0);
		    }

		    /* add ip cksum with ether header cksum */
		    buf = (U16 *)frame_start;
		    do {
			sum += *buf++;
		    } while ((U32)buf < ip_start);
		    sum = (sum >> 16) + (sum & 0xffff);
		    sum = (sum >> 16) + (sum & 0xffff);

		    /* tcp/udp cksum is frame + pseudo - (ether hdr + ip hdr) */
		    tcpudp_cksum = dd_cksum + pseudo;
		    tcpudp_cksum =
			(tcpudp_cksum >> 16) + (tcpudp_cksum & 0xffff);
		    tcpudp_cksum =
			(tcpudp_cksum >> 16) + (tcpudp_cksum & 0xffff);
		    tcpudp_cksum -= sum;
		    tcpudp_cksum += (tcpudp_cksum >> 16);
		    tcpudp_cksum &= 0xffff;
		    if (tcpudp_cksum == 0)
			tcpudp_cksum = 0xffff;

		 NIC_TRACE(TRACE_TYPE_SEND, "sSmTcpU0", 0x64900,
			dd_index,
			dd_cksum,
			sum,
			pseudo);

		    /* calculate where to store cksum */
		    if (!tx_first_frame) {
			ip_start = nicfp->tx_first_frame_start +
				   nicfp->tx_first_frame_eth_hlen;
			hlen = IP_HLEN(ip_start);
		    }
		    buf = (ip_protocol == IP_PROTO_TCP)?
			    (U16 *)&((tcp_t *)(ip_start + hlen))->tcpCheckSum:
			    (U16 *)&((udp_t *)(ip_start + hlen))->udpCheckSum;

		    /* accumulate cksum of ip fragments */
		    if (!tx_first_frame) {
			tcpudp_cksum += *buf;
			tcpudp_cksum =
			    (tcpudp_cksum >> 16) + (tcpudp_cksum & 0xffff);
		    }

		    /* invert cksum at the end */
		    if (!nicfp->tx_ip_frag ||
			(bd_flags & BD_FLAG_IP_FRAG_END)) {

			tcpudp_cksum = ~tcpudp_cksum;
		    }

		    /* store cksum */
		    *buf = (U16)tcpudp_cksum;

		 NIC_TRACE(TRACE_TYPE_SEND, "sSmTcpU1", 0x65000,
			dd_index,
			pseudo,
			buf,
			*buf);
		}
	    }
	}

	/* maintain our running frame length */
	nicfp->tx_frame_len += ddp->len;

	/* build mac descriptor to fire off mac */
	if (insert_vtag) {
	    hd.w0 = frame_start | TG_MAC_TX_W0_ADD_TAG;
	    hd.w1 = nicfp->tx_frame_len | vlan_tag;
	} else {
	    hd.w0 = frame_start;
	    hd.w1 = nicfp->tx_frame_len;
	}

#if (TX_FRAG_FRAMES_IN_ORDER)
	/* NOTE: This segment of the code deals with the situation
	 * when the Fragmented frames need to transmitted on the
	 * wire in the Serial order, i.e. the First Fragment
	 * first, second frament second, and so on.
	 *
	 * CAUTION: If they need to be transmitted in the reverse
	 * order, to minimize the latency, i.e. Hold the first
	 * first fragment and transmit rest of the fragments
	 * immediately, then the #else part of this code segment
	 * should be compiled in.
	 *
	 * IMPORTANT: The second part has not been FULLY debugged.
	 * There are good chances that it may not work.
	 * So some additional testing may be needed.
	 */

	NIC_TRACE(TRACE_TYPE_SEND, "sSm2Mac", 0x65100,
	    nicfp->tx_first_frame_index,
	    nicfp->tx_frame_len,
	    bd_flags,
	    0);

	if (bd_flags & BD_FLAG_IP_FRAG) {
	    ASSERT(nicfp->tx_queue_depth < TG_NUM_MAC_TX_DESCRS);

	    /* enqueue to a hold queue */
	    nicp->tx_queue[nicfp->tx_queue_depth++] = hd; /* struct copy */

	    NIC_TRACE(TRACE_TYPE_SEND, "sSm2HldQ", 0x65200,
		hd.w0,
		hd.w1,
		bd_flags,
		nicfp->tx_queue_depth);

	} else if (bd_flags & BD_FLAG_IP_FRAG_END) {
	    U32 i;

	    ASSERT(nicfp->tx_queue_depth < TG_NUM_MAC_TX_DESCRS);

	    /* enqueue to a hold queue */
	    nicp->tx_queue[nicfp->tx_queue_depth++] = hd; /* struct copy */

	    /* fire off to mac */
	    for (i = 0; i < nicfp->tx_queue_depth; i++) {

		md = nicp->tx_queue[i]; /* struct copy */
		NIC_TRACE(TRACE_TYPE_SEND, "sSmQMacM", 0x65300,
		    md.w0,
		    md.w1,
		    bd_flags,
		    i);

		enq_mac_tx_cksum(&md);
#if (USE_U32_STAT_CNT)
		nicfp->ifHCOutOctets += md.w1 & TG_MAC_TX_W1_LEN_MASK;
#else /* (USE_U32_STAT_CNT) */
		nicfp->stats.ifHCOutOctets += md.w1 & TG_MAC_TX_W1_LEN_MASK;
#endif /* (USE_U32_STAT_CNT) */
		update_tx_pkt_stats_cksum(md.w0 & TG_MAC_TX_W0_ADDR_MASK);
	    }

	    nicfp->tx_ip_frag = FALSE;
	    nicfp->tx_queue_depth = 0;

	} else { /* no ip fragments */

	    NIC_TRACE(TRACE_TYPE_SEND, "sSmQMac", 0x65400,
		hd.w0,
		hd.w1,
		bd_flags,
		nicfp->tx_first_frame_index);

	    enq_mac_tx_cksum(&hd);
#if (USE_U32_STAT_CNT)
	    nicfp->ifHCOutOctets += nicfp->tx_frame_len;
#else /* (USE_U32_STAT_CNT) */
	    nicfp->stats.ifHCOutOctets += nicfp->tx_frame_len;
#endif /* (USE_U32_STAT_CNT) */
	    update_tx_pkt_stats_cksum(frame_start);
	}
#else   /* (TX_FRAG_FRAMES_IN_ORDER) */
	/*
	 * note that if the host ever hands us a frame that
	 * does not have BD_FLAG_IP_FRAG or BD_FLAG_IP_FRAG_END
	 * set AND we are in the midst of processing a frag 
	 * chain i.e. nicfp->tx_ip_frag is set the NIC will
	 * get out of whack.
	 */

	/* Xmit all frags except the first frag. We hang on to the
	 * first frag to stick the checksums in and xmit it at
	 * the very end i.e. out-of-order.  this was done to
	 * avoid the latency introduced by queueing up all the
	 * frags until the nic got the end-frag and then sending
	 * the whole chain out - suresh
	 */
	NIC_TRACE(TRACE_TYPE_SEND, "sSm2Mac", 0x65500,
	    nicfp->tx_first_frame_index,
	    nicfp->tx_frame_len,
	    bd_flags,
	    0);

	if (!nicfp->tx_ip_frag ||
	    (!tx_first_frame && (bd_flags & BD_FLAG_IP_FRAG))) {

	    NIC_TRACE(TRACE_TYPE_SEND, "sSmQMac", 0x65600,
		hd.w0,
		hd.w1,
		bd_flags,
		nicfp->tx_first_frame_index);

	    enq_mac_tx_cksum(&hd);
#if (USE_U32_STAT_CNT)
	    nicfp->ifHCOutOctets += nicfp->tx_frame_len;
#else /* (USE_U32_STAT_CNT) */
	    nicfp->stats.ifHCOutOctets += nicfp->tx_frame_len;
#endif /* (USE_U32_STAT_CNT) */
	    update_tx_pkt_stats_cksum(frame_start);

	} else if (tx_first_frame && (bd_flags & BD_FLAG_IP_FRAG)) {

	    /* This is the very first frag of the chain.        
	     * just stash the len of the first frag for now
	     * we will need it later when we send this at 
	     * the very end
	     */
	    nicp->tx_queue[0] = hd; /* struct copy */

	    NIC_TRACE(TRACE_TYPE_SEND, "sSm2HldQ", 0x65700,
		hd.w0,
		hd.w1,
		bd_flags,
		0);

	} else { /* BD_FLAG_IP_FRAG_END */

	    /* Now send the Last Frag and follow that with the
	     * very first frag that should now have the computed
	     * ip and udp headers
	     */
	    NIC_TRACE(TRACE_TYPE_SEND, "sSmQMacL", 0x65800,
		hd.w0,
		hd.w1,
		bd_flags,
		nicfp->tx_first_frame_index);

	    enq_mac_tx_cksum(&hd);
#if (USE_U32_STAT_CNT)
	    nicfp->ifHCOutOctets += nicfp->tx_frame_len;
#else /* (USE_U32_STAT_CNT) */
	    nicfp->stats.ifHCOutOctets += nicfp->tx_frame_len;
#endif /* (USE_U32_STAT_CNT) */
	    update_tx_pkt_stats_cksum(frame_start);

	    /* send out the first frag of this chain */
	    md = nicp->tx_queue[0]; /* struct copy */

	    NIC_TRACE(TRACE_TYPE_SEND, "sSmQMacB", 0x65900,
		md.w0,
		md.w1,
		bd_flags,
		nicfp->tx_first_frame_index);

	    enq_mac_tx_cksum(&md);
#if (USE_U32_STAT_CNT)
	    nicfp->ifHCOutOctets += (md.w1 & TG_MAC_TX_W1_LEN_MASK);
#else /* (USE_U32_STAT_CNT) */
	    nicfp->stats.ifHCOutOctets += (md.w1 & TG_MAC_TX_W1_LEN_MASK);
#endif /* (USE_U32_STAT_CNT) */
	    update_tx_pkt_stats_cksum(nicfp->tx_first_frame_start);
	    nicfp->tx_ip_frag = FALSE;
	}
#endif  /* (TX_FRAG_FRAMES_IN_ORDER) */

	/* zero for a new frame */
	nicfp->tx_frame_len = 0;

	/* send bd consumed */
	if (is_global_flags(GF_RCB_TX_HOST_RING))
	    nicfp->stats.nicSendBDs--;

	/* update consumer index */
	nicfp->send_cons_index =
	    ((ddp->index + 1) & (nicfp->send_ring_size - 1));

	if ((dd_type & (TIGON_TYPE_SEND_DATA_LAST_COAL |
			TIGON_TYPE_SEND_DATA_LAST_VLAN_COAL)) ||
	    (++nicfp->send_coal_bds >=
			tsp->gen_com.tuneParms.send_max_coalesced_bds)) {

	    send_cons_index_to_host();
	    nicfp->stats.nicSendThresholdHit++;
	}

    } else if (dd_type & (TIGON_TYPE_SEND_DATA |
			  TIGON_TYPE_SEND_DATA_COAL)) {

	/* maintain our running frame length */
	nicfp->tx_frame_len += ddp->len;

	/* send bd consumed */
	if (is_global_flags(GF_RCB_TX_HOST_RING))
	    nicfp->stats.nicSendBDs--;

	/* update consumer index */
	nicfp->send_cons_index = 
	    ((ddp->index + 1) & (nicfp->send_ring_size - 1));

	if ((dd_type & TIGON_TYPE_SEND_DATA_COAL) ||
	    (++nicfp->send_coal_bds >=
		tsp->gen_com.tuneParms.send_max_coalesced_bds)) {

	    send_cons_index_to_host();
	    nicfp->stats.nicSendThresholdHit++;
	}

	NIC_TRACE(TRACE_TYPE_SEND, "sDtCp", 0x66000,
	      nicfp->send_cons_index,
	      nicfp->send_coal_bds,
	      0,
	      0);

    } else if (dd_type & TIGON_TYPE_SEND_BD) {

	U32 dd_index = ddp->index;

	nbds = dd_index - nicfp->send_prod_index_avail;
	if (nbds < 0)
	    nbds += nicfp->send_ring_size;

	nicfp->send_prod_index_avail = dd_index;
	nicfp->stats.nicSendBDs += nbds;
	#if 0
	nicfp->stats.nicTotalSendBDs += nbds; /* obsolete */
	#endif

	set_event_reg(TG_FW_EVENT_SEND_DATA);

	NIC_TRACE(TRACE_TYPE_SEND, "sBdCp", 0x66100,
	      nbds, 
	      nicfp->stats.nicSendBDs,
	      nicfp->send_prod_index_avail,
	      nicfp->send_cons_index_queued);

    } else if (dd_type & (TIGON_TYPE_RECV_BD |
			  TIGON_TYPE_RECV_MINI_BD |
			  TIGON_TYPE_RECV_JUMBO_BD)) {

	/* note the priority */
        if (dd_type & TIGON_TYPE_RECV_BD) {
	    U32 dd_index = ddp->index;
	    nbds = dd_index - nicfp->recv_prod_index_avail;
	    if (nbds < 0)
		nbds += RECV_STANDARD_RING_ENTRIES;

            nicfp->recv_prod_index_avail = dd_index; /* update */
            nicfp->stats.nicRecvBDs += nbds;
            nicfp->stats.nicTotalRecvBDs += nbds;

            NIC_TRACE(TRACE_TYPE_RECV_BD, "rStdBdCp", 0x66200,
		nbds, 
		nicfp->stats.nicRecvBDs,
		nicfp->recv_prod_index_avail,
		nicfp->recv_cons_index);

        } else if (dd_type & TIGON_TYPE_RECV_MINI_BD) {
	    U32 dd_index = ddp->index;
	    nbds = dd_index - nicfp->recv_mini_prod_index_avail;
	    if (nbds < 0)
		nbds += RECV_MINI_RING_ENTRIES;

            nicfp->recv_mini_prod_index_avail = dd_index; /* update */
            nicfp->stats.nicMiniRecvBDs += nbds;
            nicfp->stats.nicTotalRecvBDs += nbds;

            NIC_TRACE(TRACE_TYPE_RECV_BD, "rMinBdCp", 0x66300,
		nbds, 
		nicfp->stats.nicMiniRecvBDs,
		nicfp->recv_mini_prod_index_avail,
		nicfp->recv_mini_cons_index);

        } else /*if (dd_type & TIGON_TYPE_RECV_JUMBO_BD)*/ {
            /* if jumbo frames enabled isn't set then we assume we
             * got an overlap with the reset jumbo ring command
             */
            if (is_global_flags(GF_CMD_JUMBO_FRAMES_ENABLED)) {
		U32 dd_index = ddp->index;
		nbds = dd_index - nicfp->recv_jumbo_prod_index_avail;
		if (nbds < 0)
		    nbds += RECV_JUMBO_RING_ENTRIES;

                nicfp->recv_jumbo_prod_index_avail = dd_index; /* update */
                nicfp->stats.nicJumboRecvBDs += nbds;
                nicfp->stats.nicTotalRecvBDs += nbds;

	     NIC_TRACE(TRACE_TYPE_RECV_BD, "rJumBdCp", 0x66400,
		    nbds,
		    nicfp->stats.nicJumboRecvBDs,
		    nicfp->recv_jumbo_prod_index_avail,
		    nicfp->recv_jumbo_cons_index);
            }
        }

	if (!is_global_flags(GF_VAR_MAC_RX_COMP_SUSP)) {
	    /* make sure we're receiving packets now */
	    event_mask |= TG_FW_EVENT_MAC_RX_COMP;
	}

    } else if (dd_type & (TIGON_TYPE_SEND_DATA_LAST_VLAN |
			  TIGON_TYPE_SEND_DATA_LAST_VLAN_COAL)) {
	/* extract out vlan information */
	insert_vtag = TRUE;
	vlan_tag = ddp->index & 0xffff0000;
	ddp->index &= 0xffff; /* strip out the tag */
	goto send_data_last;

    } else if (dd_type & TIGON_TYPE_REFRESH_STATS) {

	refresh_stats_slow();

    } else if (dd_type & TIGON_TYPE_SEND_DUMMY_DMA) {
	; /* nothing to do */
    } else {
	ASSERT(0);
    }

    NIC_TRACEV_IF(TRACE_TYPE_DMA, "rDmCpOk", 0x66600,
		  ddp->type, TIGON_TYPE_TX,
	trp->host_dma_assist.read_chan_ref,
	trp->host_dma_assist.read_chan_consumer,
	trp->host_dma_assist.read_chan_producer,
	ddp);

    RETURN(TG_FW_EVENT_NUM_DMA_RD_ASST_HI);
}

/* null routine to assist with scratchpad setup */
void h_dma_rd_assist_cksum_end(void) {}
