/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: timer.c $
 * Revision nic_api_12_4_latest_shuang/2 1999/12/09 19:51:56 shuang
 * 	work around Tg2 MAC6 bug only if get rx buf full attention.
 * 
 * Revision copper_fhwang/5 1999/10/12 19:40:14 fhwang
 * 	to support new led (4 leds)
 * 
 * Revision copper_fhwang/3 1999/08/27 02:01:08 fhwang
 * 	exclude GMII for checking config sync
 * 
 * Revision sgi_api_12_3_latest_shuang/5 1999/08/11 17:35:06 shuang
 * 	in sync action of enable/disable mac rx attn in event mask
 * 
 * Revision sgi_api_12_3_latest_shuang/3 1999/07/14 18:58:50 shuang
 * 	work around MAC6 Tigon2 bug, fix rx buffer full cleanup
 * 
 * Revision 1.1.2.35  1999/01/27  19:09:50  hayes
 * 	incorporate autosense code
 * 	[1999/01/27  19:06:25  hayes]
 *
 * Revision 1.1.32.1  1999/01/20  18:06:17  hayes
 * 	check in autosense link code
 * 	[1999/01/20  18:05:46  hayes]
 * 
 * Revision 1.1.2.34  1998/12/18  02:18:13  shuang
 * 	use original update stats
 * 	[1998/12/18  02:17:39  shuang]
 * 
 * Revision 1.1.2.33  1998/12/16  04:07:03  shuang
 * 	changed update stats schem
 * 	[1998/12/16  04:06:16  shuang]
 * 
 * Revision 1.1.2.32  1998/12/10  02:55:43  shuang
 * 	Prevent from starving tx side. Fix stats counters.
 * 	[1998/12/10  02:54:56  shuang]
 * 
 * Revision 1.1.2.31  1998/12/08  02:36:52  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:28:33  shuang]
 * 
 * Revision 1.1.2.30  1998/10/07  21:00:55  fhwang
 * 	Checking gig_negotiate flag to make sure autonegotiation is on before checking the configuration page's change.
 * 	[1998/10/07  20:59:55  fhwang]
 * 
 * Revision 1.1.2.29  1998/09/30  18:50:25  shuang
 * 	change for new 12.3 API
 * 	[1998/09/30  18:41:38  shuang]
 * 
 * Revision 1.1.2.28  1998/08/20  20:06:09  hayes
 * 	propogate config detection fix to nic code
 * 	[1998/08/20  20:05:59  hayes]
 * 
 * Revision 1.1.2.27  1998/08/11  18:22:04  shuang
 * 	bug fix - incoming packets hw stats (32 counter) overflow handling
 * 	[1998/08/11  18:18:49  shuang]
 * 
 * Revision 1.1.2.26  1998/08/07  00:17:07  shuang
 * 	do a dummy dma read forwrite dma hang problem in X-Chip
 * 	[1998/08/06  23:51:28  shuang]
 * 
 * Revision 1.1.2.25  1998/08/03  17:24:40  shuang
 * 	use TG_CFG_MODE_SBUS flag instead of TG_CFG_MODE_1_DMA_ACTIVE
 * 	[1998/08/03  17:23:15  shuang]
 * 
 * Revision 1.1.2.24  1998/07/29  01:45:44  shuang
 * 	work around hang problem of dma channal between SBUS and X-Chip
 * 	[1998/07/29  01:44:44  shuang]
 * 
 * Revision 1.1.2.23  1998/07/29  00:07:35  shuang
 * 	add feature - send coallescing BD update only
 * 	[1998/07/29  00:02:58  shuang]
 * 
 * Revision 1.1.2.22  1998/07/23  00:26:08  shuang
 * 	adjust led(6) flash rate on stackUP, linkDN state.
 * 	stop data led(6) flashing on idle state.
 * 	[1998/07/23  00:25:07  shuang]
 * 
 * Revision 1.1.2.21  1998/05/18  22:02:13  ted
 * 	Fix port disabled LED timer
 * 	[1998/05/18  22:02:03  ted]
 * 
 * Revision 1.1.2.20  1998/05/18  20:09:23  ted
 * 	Merged with changes from 1.1.2.19
 * 	[1998/05/18  20:09:08  ted]
 * 
 * 	Support link up notification for 10/100 PHY
 * 	[1998/05/18  20:01:12  ted]
 * 
 * Revision 1.1.2.19  1998/05/15  20:14:35  shuang
 * 	stop mac receiver for a while if run out of rx buffer
 * 	[1998/05/15  20:13:19  shuang]
 * 
 * Revision 1.1.2.18  1998/04/24  00:00:45  hayes
 * 	added const to rcsid definitions to eliminate compiler warnings
 * 	[1998/04/23  23:59:01  hayes]
 * 
 * Revision 1.1.2.17  1998/04/17  01:15:02  ted
 * 	Fix stuck tick counts on idle systems
 * 	[1998/04/17  01:14:52  ted]
 * 
 * Revision 1.1.2.16  1998/04/15  17:39:15  ted
 * 	Fix refresh stats command
 * 	[1998/04/15  17:31:21  ted]
 * 
 * Revision 1.1.2.15  1998/04/11  05:15:22  shuang
 * 	move fast timer to cpu B, add receive coalesed ticks
 * 	[1998/04/11  04:58:54  shuang]
 * 
 * Revision 1.1.2.14  1998/04/01  22:21:31  hayes
 * 	clear timer interrupt when done...
 * 	[1998/04/01  22:20:10  hayes]
 * 
 * 	checkpoint
 * 	[1998/03/31  19:57:03  hayes]
 * 
 * 	checkpoint- dma completion avoidance works
 * 	[1998/03/30  07:00:39  hayes]
 * 
 * 	rd DMA event avoidance works
 * 	[1998/03/29  03:11:41  hayes]
 * 
 * 	intermediate checkin
 * 	[1998/03/29  00:18:12  hayes]
 * 
 * Revision 1.1.2.13  1998/04/01  07:32:35  shuang
 * 	add features: pass (CRC, bad, partial) frame, timer tick to 5us...
 * 	[1998/04/01  07:27:35  shuang]
 * 
 * Revision 1.1.2.12  1998/03/28  00:53:20  shuang
 * 	No interrupt avoidance on link state change
 * 	[1998/03/28  00:50:30  shuang]
 * 
 * Revision 1.1.2.11  1998/03/26  01:34:34  shuang
 * 	cleanup
 * 	[1998/03/26  01:25:42  shuang]
 * 
 * Revision 1.1.2.9  1998/03/04  22:28:38  hayes
 * 	     enable scratchpad usage, uncluding stats refreesh
 * 	     [1998/03/04  22:19:52  hayes]
 * 
 * Revision 1.1.2.8  1998/02/23  01:22:35  hayes
 * 	     add support for HP 80 MHz HSC board.
 * 	     [1998/02/23  01:21:26  hayes]
 * 
 * Revision 1.1.2.7  1998/02/19  01:30:11  suresh
 * 	     TDP correction, extra routine to assist scratchpad setup
 * 	     [1998/02/19  01:26:20  suresh]
 * 
 * Revision 1.1.2.6  1998/02/11  19:11:26  ted
 * 	     Function pointer q_dma_to(host,nic) routines
 * 	     [1998/02/11  18:52:59  ted]
 * 
 * Revision 1.1.2.5  1998/02/09  17:18:09  hayes
 * 	     convert Q_DMA_XXX macros to functions
 * 	     [1998/02/09  17:17:08  hayes]
 * 
 * Revision 1.1.2.4  1998/01/28  21:27:56  hayes
 * 	     add stack bounds checking and redzone support
 * 	     [1998/01/28  21:26:53  hayes]
 * 
 * Revision 1.1.2.3  1998/01/20  21:43:07  ted
 * 	     Fix LED flashes after byte count wrap bug, PR 753
 * 	     [1998/01/20  21:42:45  ted]
 * 
 * Revision 1.1.2.2  1998/01/20  04:39:24  hayes
 * 	     add tigon Rev6 support
 * 	     [1998/01/20  04:38:06  hayes]
 * 
 * Revision 1.1.2.1  1998/01/14  23:26:03  ted
 * 	     rearrange directories
 * 	     [1998/01/14  23:17:59  ted]
 * 
 * Revision 1.1.2.9  1998/01/14  19:56:04  skapur
 * 	     Merged with changes from 1.1.2.8
 * 	     [1998/01/14  19:55:55  skapur]
 * 
 * 	     Checksum off-load support for Tigon2. This is an intermediate checkin; more work needs to be done
 * 	     [1998/01/14  19:26:50  skapur]
 * 
 * Revision 1.1.2.8  1998/01/09  03:15:06  ted
 * 	     Put RX stats into the transferred stats area
 * 	     [1998/01/09  03:11:12  ted]
 * 
 * Revision 1.1.2.7  1998/01/04  02:24:12  hayes
 * 	     disassociate active LEd from link bringup
 * 	     [1998/01/04  02:22:56  hayes]
 * 
 * Revision 1.1.2.6  1997/12/28  00:23:09  hayes
 * 	     cleanup LEDs for both 6 and 2 LED cards
 * 	     [1997/12/28  00:22:56  hayes]
 * 
 * Revision 1.1.2.5  1997/12/12  05:34:39  hayes
 * 	     fix link LEDs for multi-phy NICs
 * 	     [1997/12/12  04:02:11  hayes]
 * 
 * Revision 1.1.2.4  1997/12/09  03:48:20  hayes
 * 	     remove TIMER_DEBUG defn
 * 	     [1997/12/09  03:48:11  hayes]
 * 
 * Revision 1.1.2.3  1997/11/30  00:57:37  hayes
 * 	     LED cleanup.
 * 	     [1997/11/30  00:57:07  hayes]
 * 
 * Revision 1.1.2.2  1997/11/25  22:07:16  ted
 * 	     Fix up statistics
 * 	     [1997/11/25  22:00:03  ted]
 * 
 * Revision 1.1.2.1  1997/11/19  19:23:30  ted
 * 	     Support Tigon and Tigon2 in the same driver
 * 	     [1997/11/19  19:20:08  ted]
 * 
 * Revision 1.21.7.39  1997/11/19  00:13:46  ted
 * 	     merged from release 2.0
 * 	     [1997/11/19  00:10:50  ted]
 * 
 * Revision 1.21.7.38  1997/11/15  18:38:43  hayes
 * 	     add support for 6 LEDs
 * 	     [1997/11/15  18:38:22  hayes]
 * 
 * Revision 1.21.7.37  1997/11/14  19:16:47  hayes
 * 	     first checkin after mongo merge...
 * 	     [1997/11/14  19:11:52  hayes]
 * 
 * 	     checkin after stats restructuring
 * 	     [1997/11/12  19:01:50  hayes]
 * 
 * 	     intermediate checkin
 * 	     [1997/11/12  01:30:50  hayes]
 * 
 * Revision 1.21.7.36  1997/11/11  19:06:07  hayes
 * 	     restructuring for scratchpad use
 * 	     [1997/11/11  19:04:36  hayes]
 * 
 * Revision 1.21.7.35  1997/11/10  00:27:32  hayes
 * 	     change board_state to link_ready
 * 	     [1997/11/10  00:25:47  hayes]
 * 
 * Revision 1.21.7.34  1997/11/03  18:42:32  hayes
 * 	     cleanup recv4/5 ifdefs
 * 	     [1997/11/03  18:42:04  hayes]
 * 
 * Revision 1.21.7.33  1997/10/30  02:21:22  hayes
 * 	     get link negotiation working for Tigon2
 * 	     [1997/10/30  02:18:37  hayes]
 * 
 * Revision 1.21.7.32  1997/10/29  03:44:43  hayes
 * 	     add rx buf cleanup
 * 	     [1997/10/29  03:42:37  hayes]
 * 
 * Revision 1.21.7.31  1997/10/28  01:51:22  hayes
 * 	     update for Tigon2
 * 	     [1997/10/27  23:40:11  hayes]
 * 
 * Revision 1.21.7.30  1997/10/21  18:18:07  hayes
 * 	     fix for REV5
 * 	     [1997/10/21  18:17:44  hayes]
 * 
 * Revision 1.21.7.29  1997/10/21  01:35:40  skapur
 * 	     Merge of release 2.0 and ge_latest branch
 * 	     [1997/10/21  01:31:26  skapur]
 * 
 * Revision 1.21.7.28  1997/10/02  23:11:35  hayes
 * 	     update SIMULATOR code
 * 	     [1997/10/02  23:11:13  hayes]
 * 
 * Revision 1.21.7.27  1997/09/17  03:26:39  hayes
 * 	     Merged with changes from 1.21.7.26
 * 	     [1997/09/17  03:25:50  hayes]
 * 
 * 	     finish preparing for multiple phys
 * 	     [1997/09/16  18:24:12  hayes]
 * 
 * Revision 1.21.7.26  1997/09/16  21:46:45  ted
 * 	     Change from MIB-II V1 (RFC 1213) to MIB-II V2 (RFC 1573)
 * 	     [1997/09/16  21:42:27  ted]
 * 
 * Revision 1.21.7.25  1997/09/16  03:06:57  skapur
 * 	     Merged with changes from 1.21.7.24
 * 	     [1997/09/16  03:06:46  skapur]
 * 
 * 	     Took out ifdef'd code, fixed traces; misc. cleanup of code
 * 	     [1997/09/16  03:02:12  skapur]
 * 
 * Revision 1.21.7.24  1997/09/11  18:05:57  ted
 * 	     Merged with changes from 1.21.7.23
 * 	     [1997/09/11  18:05:06  ted]
 * 
 * 	     Mixing vlans, jumbos, and redundancy works, plus small fixes from release 1.1 and better NIC tracing
 * 	     [1997/09/11  18:00:15  ted]
 * 
 * Revision 1.21.7.23  1997/09/10  03:55:17  hayes
 * 	     begin code prep for multiple phys per mac
 * 	     [1997/09/10  03:52:59  hayes]
 * 
 * Revision 1.21.7.22  1997/08/29  22:12:27  hayes
 * 	     completed link code restructuring
 * 
 * Revision 1.21.36.2  1997/08/29  22:09:32  hayes
 * 	     completed link code restructuring
 * 
 * Revision 1.21.7.21  1997/08/26  21:09:16  hayes
 * 	     link code preparation for SC changes
 * 	     [1997/08/26  21:04:08  hayes]
 * 
 * Revision 1.21.7.20  1997/08/23  17:40:15  skapur
 * 	     Solaris jumbogram support
 * 	     [1997/08/23  17:38:21  skapur]
 * 
 * Revision 1.21.7.19  1997/08/13  18:46:23  ted
 * 	     Fix off by one in recv ring when DMA ring is empty
 * 	     [1997/08/13  18:45:09  ted]
 * 
 * Revision 1.21.7.18  1997/07/29  03:41:03  ted
 * 	     Add OP1 LED support
 * 	     [1997/07/29  03:40:12  ted]
 * 
 * Revision 1.21.7.17  1997/07/27  22:03:30  ted
 * 	     Fix merge bug in link negotiation
 * 	     [1997/07/27  21:59:43  ted]
 * 
 * Revision 1.21.7.16  1997/07/25  22:59:16  ted
 * 	     Merge fixes from ge_0_22 and add link negotiation support
 * 	     [1997/07/25  22:32:26  ted]
 * 
 * 	     Architectural performance change
 * 	     [1997/07/24  21:45:29  ted]
 * 
 * Revision 1.21.7.15  1997/06/04  17:36:47  ted
 * 	     Fix link changes statistic
 * 	     [1997/06/04  17:36:34  ted]
 * 
 * Revision 1.21.7.14  1997/05/21  22:59:53  ted
 * 	     Minor statistics tweaks
 * 	     [1997/05/21  22:59:17  ted]
 * 
 * Revision 1.21.7.13  1997/05/21  03:35:49  ted
 * 	     Bug fix for PR 189 (see comments in send.c).
 * 	     Removal of check for interrupt ticks since now the interrupt
 * 	     algorithm is adaptive.
 * 	     [1997/05/21  03:21:10  ted]
 * 
 * Revision 1.21.7.12  1997/05/07  23:50:25  skapur
 * 	     Some interface related Stats (rfc 1213) needs to be re-initialized after the stats are cleared by the host
 * 	     [1997/05/07  23:48:45  skapur]
 * 
 * Revision 1.21.7.11  1997/05/03  04:58:50  skapur
 * 	     Cleaned up stats code
 * 	     [1997/05/03  04:58:20  skapur]
 * 
 * Revision 1.21.7.10  1997/04/23  22:12:32  alfan
 * 	     If recv completion events are missed; retry them util enqued
 * 	     [1997/04/23  22:12:10  alfan]
 * 
 * Revision 1.21.7.9  1997/04/22  18:04:10  alfan
 * 	     Fixed Send Events; If any of the send completion events are missed, they will be re-tired until it makes it
 * 	     [1997/04/22  18:02:52  alfan]
 * 
 * Revision 1.21.7.8  1997/04/19  04:04:40  alfan
 * 	     Fixed Stats timer count problem
 * 	     [1997/04/19  04:01:17  alfan]
 * 
 * Revision 1.21.7.7  1997/04/15  05:46:38  hayes
 * 	     updated for new gig autonegotiation state machine
 * 	     [1997/04/15  05:44:13  hayes]
 * 
 * Revision 1.21.7.6  1997/04/11  23:41:53  ted
 * 	     Removed TX BD events from being reported by Timer + Mics. cleanup + use macros to turn on/off LEDs
 * 	     [1997/04/11  23:27:03  ted]
 * 
 * Revision 1.21.7.5  1997/03/28  02:47:13  skapur
 * 	     Cleaned out most of the un-necessary code
 * 	     [1997/03/28  02:20:45  skapur]
 * 
 * Revision 1.21.7.4  1997/03/24  08:34:30  ted
 * 	     New event ring handling, producer lives in host memory, misc. other more modest changes
 * 	     [1997/03/24  08:28:14  ted]
 * 
 * Revision 1.21.7.3  1997/03/21  04:15:44  skapur
 * 	     Performance enhancement for Recv and Send; lots changes; almost complete overhaul of the entire firmware
 * 	     [1997/03/21  04:08:41  skapur]
 * 
 * Revision 1.21.7.2  1997/03/17  08:47:50  skapur
 * 	     allow other handlers to run when rx_attn is asserted
 * 	     [1997/03/17  08:30:32  skapur]
 * 
 * 	     integration changes...
 * 	     [1997/03/16  03:01:51  skapur]
 * 
 * Revision 1.21.7.1  1997/02/23  00:23:43  ted
 * 	     Timer pre-scaler is set to 0x3420
 * 	     [1997/02/23  00:17:56  ted]
 * 
 * Revision 1.21.2.4  1996/11/20  20:04:30  ted
 * 	     Redo include file structuring.
 * 	     [1996/11/20  19:52:28  ted]
 * 
 * Revision 1.21.2.3  1996/11/15  04:24:21  hayes
 * 	     convert to new header file structure.
 * 	     [1996/11/15  04:12:09  hayes]
 * 
 * Revision 1.21.2.2  1996/11/05  21:54:34  hayes
 * 	     Final pre-ODE merge
 * 	     [1996/11/05  21:52:27  hayes]
 * 
 * Revision 1.21.2.1  1996/11/01  18:25:11  ted
 * 	     change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	     [1996/10/29  22:06:45  ted]
 * 
 * $EndLog$
 */
/*
 * FILE timer.c
 *
 * COPYRIGHT (c) Essential Communication Corp. 1995
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/timer.c,v $
 * $Revision: 1.1.2.35 $ $Locker:  $
 * $Date: 1999/01/27 19:09:50 $ $Author: hayes $ $State: Exp $
 */

#ifndef __lint
static const char rcsid[] = "$Header: /projects/rcs/sw/ge/./nic/fw2/common/timer.c,v 1.1.2.35 1999/01/27 19:09:50 hayes Exp $";
#endif /* __lint */

#include "alt_def.h"
#include "nic_conf.h"
#include "nic_api.h"
#include "tg.h"

#include "nic.h"
#include "trace.h"
#include "proto.h"
#include "link.h"
#include "link_if.h"
#include "dma.h"
#include "ring.h"
#include "timer.h"

#include "mac.h"
#include "zconf.h"
#include "link_op.h"
#include "profile.h"

#include "led.h"

#include "assert.h"

#ifdef STACK_REDZONE
void check_redzone(void);
#endif /* STACK_REDZONE */

U32 h_timer_stub;
U32 h_update_led_stub;
U32 h_do_update_tx_cons_stub;
U32 h_do_update_rx_prod_stub;
U32 dummy_rd_dma;

#if (USE_U32_STAT_CNT)
void
propagate_u64_stats(void)
{
    nicfp->u64_stat_ticks = 0;

    nicfp->stats.ifHCOutUcastPkts += nicfp->ifHCOutPkts[UCAST_PKT_INDEX];
    nicfp->ifHCOutPkts[UCAST_PKT_INDEX] = 0;

    nicfp->stats.ifHCOutMulticastPkts += nicfp->ifHCOutPkts[MCAST_PKT_INDEX];
    nicfp->ifHCOutPkts[MCAST_PKT_INDEX] = 0;

    nicfp->stats.ifHCOutBroadcastPkts += nicfp->ifHCOutPkts[BCAST_PKT_INDEX];
    nicfp->ifHCOutPkts[BCAST_PKT_INDEX] = 0;

    nicfp->stats.ifHCInOctets += nicfp->ifHCInOctets;
    nicfp->ifHCInOctets = 0;

    nicfp->stats.ifHCOutOctets += nicfp->ifHCOutOctets;
    nicfp->ifHCOutOctets = 0;
}
#endif /* (USE_U32_STAT_CNT) */

void
init_timer(void)
{
    NIC_TRACEV(TRACE_TYPE_TIMER, "initTm_A", 0x50100,
	TG_TICK_SLOW,
	0,
	0,
	0);

    /* init prescaler */
    switch (trp->gen_control.misc_host_control & TG_MHC_TIGON_VERSION_MASK) {
    case TG_MHC_LSI_STANDARD_CELL_VERS_B:
	trp->gen_control.misc_config =
	    (trp->gen_control.misc_config & ~TG_MSC_TIMER_PRESCALER_MASK) |
	    0x55;   /* for 86 Mhz */
	break;

    default:
	ASSERT(0);
    }

    /* init timer values */
    trp->gen_control.timer_ref = trp->gen_control.timer + TG_TICK_SLOW;

    nicfp->stats_tick_count = 0;

    nicfp->active_phy_ticks = TG_TICK_US(500000)/TG_TICK_SLOW;
    nicfp->active_phy_tick_count = 0;
    nicfp->active_phy_display = ATTACHED_PHY_IF_SERDES_EXT;

    nicfp->led_ticks = TG_TICK_US(2000)/TG_TICK_SLOW;
    nicfp->led_tick_count = 0;

    nicfp->disabled_led_timer = 0;
    nicfp->data_led_timer = 0;
    return;
}

void
init_timer_b(void)
{
    NIC_TRACEV(TRACE_TYPE_TIMER, "initTm_B", 0x50200,
	TG_TICK_FAST,
	0,
	0,
	0);

    trp->gen_control.timer_ref_b = trp->gen_control.timer + TG_TICK_FAST;
    return;
}

/*
 *  A timer going off [normally?] indicates the need to interrupt the host,
 *  If BD coalescing is active, this includes generating one or more events
 *  before actually generating the hardware interrupt signal.  In particular,
 *  if there are BDs that the host has not been told about, generate
 *  TG_EVENT_SET_SEND_CONSUMER_INDEX and/or TG_EVENT_SET_RECV_CONSUMER_INDEX
 *  events before generating the interrupt.  There are counters associated
 *  with each ring giving the number of BDs that have been consumed without
 *  telling the host (in the nic structure), and BD coalescing is controlled
 *  by the RCB_FLAG_COALESCE_BDS flag in the send and recv RCBs.
 */

void
h_timer(void)
{
    U32 stat_ticks;

    NIC_TRACE(TRACE_TYPE_TIMER, "#timer", 0x50300,
	0,
	0,
	0,
	0);
	
    /* first restart the timer so we make damn sure it gets done! */
    trp->gen_control.timer_ref = trp->gen_control.timer + TG_TICK_SLOW;

    if (is_global_flags(GF_VAR_MAC_RX_COMP_SUSP)) {
	clear_global_flags(GF_VAR_MAC_RX_COMP_SUSP);
	event_mask |= TG_FW_EVENT_MAC_RX_COMP;
    }

    if (link_ready) {

	/* WARNING: note the seq of handling the event_mask for mac rx attn */

	/* re-enable mac rx attn
	 * 1. it may has been masked out on the rx buffer full
	 * 2. it may has been masked out in the link down notification,
	 *    but not re-enabled in the link up notification
	 */
	event_mask |= TG_FW_EVENT_MAC_RX_ATTN;

        /* restart auto negotiation for auto negotiation attn fix */
        if ((gig_negotiate) &&
	    !(link_sense_mode) &&
	    ((attached_phy & ATTACHED_PHY_IF_MII) != ATTACHED_PHY_IF_MII) &&
	    ((attached_phy & ATTACHED_PHY_IF_GMII) != ATTACHED_PHY_IF_GMII) &&
	    (trp->mac_control.mac_rx_config != zs.saved_rx_config)) {

	    /* note: mac rx attn event mask will be masked out */
            link_down_notify();
        }
    } else {
        bringup_link();
    }

    /* fix potential hang problem of dma channal between SBUS and X-Chip */
    if (is_global_flags(GF_MOD_SBUS)) {
	fix_xchip_hang_on_sbus();
    }

    /* need to update leds every 2ms */
    if (++nicfp->led_tick_count >= nicfp->led_ticks) {
        nicfp->led_tick_count = 0;
        event_mask |= TG_FW_EVENT_UPDATE_LEDS;
    }

    /* update stats timer */
    stat_ticks = tsp->gen_com.tuneParms.stat_ticks;
    if (stat_ticks &&
	((nicfp->stats_tick_count += TG_TICK_SLOW) >= stat_ticks) &&
	!nicfp->refresh_stats_active) {

#if (USE_U32_STAT_CNT)
	/* need to propagate 32-bit stats into 64-bit stats */
	propagate_u64_stats();
#endif /* (USE_U32_STAT_CNT) */
        update_stats();
#ifdef STACK_REDZONE
        check_redzone();
#endif /* STACK_REDZONE */
    }
#if (USE_U32_STAT_CNT)
    else if (++nicfp->u64_stat_ticks >= (TG_TICK_US(30000000)/TG_TICK_SLOW)) {
	/* update 64-bit stats every 30 secs to avoid overflow */
	propagate_u64_stats();
    } 
#endif /* (USE_U32_STAT_CNT) */

    RETURN(TG_FW_EVENT_NUM_TIMER);
}

/* null routine to assist with scratchpad setup */
void h_timer_end(void) {}

void
h_do_update_tx_cons(void)
{
    NIC_TRACEV(TRACE_TYPE_TIMER, "#sCons", 0x50400,
	0,
	0,
	0,
	0);

    /* generate an event only if this index has not been reported */
    if (nicfp->send_cons_index != nicfp->send_coal_index) {
	if (nicp->send_cons_index_to_host_pending)
	    send_cons_index_to_host_intr();
	else
	    send_cons_index_to_host_no_intr();
    } else {
	if (nicfp->send_cons_index_to_host_need_intr) {
	    nicfp->send_cons_index_to_host_need_intr = FALSE;
	    interrupt_host_w_avoidance();
	}
        nicp->send_cons_index_to_host_pending = FALSE;
        nicp->send_cons_index_to_host_pending_no_intr = FALSE;
#if (RESET_COAL_TICK_CNT)
	nicp->send_coal_tick_count = 0;
#endif (RESET_COAL_TICK_CNT)
    }
    clear_event_reg(TG_FW_EVENT_DO_TX_CONS);

    RETURN(TG_FW_EVENT_NUM_DO_TX_CONS);
}

/* null routine to assist with scratchpad setup */
void h_do_update_tx_cons_end(void) {}

void
h_do_update_rx_prod(void)
{
    NIC_TRACEV(TRACE_TYPE_TIMER, "#rProd", 0x50500,
	0,
	0,
	0,
	0);

    /* generate an event only if this index has not already been reported */
    if (nicfp->recv_return_prod_index != nicfp->recv_coal_index) {
        recv_return_prod_index_to_host();
    } else {
        nicp->recv_ret_prod_index_to_host_pending = FALSE;
#if (RESET_COAL_TICK_CNT)
	nicp->recv_coal_tick_count = 0;
#endif (RESET_COAL_TICK_CNT)
    }
    clear_event_reg(TG_FW_EVENT_DO_RX_PROD);

    RETURN(TG_FW_EVENT_NUM_DO_RX_PROD);
}

/* null routine to assist with scratchpad setup */
void h_do_update_rx_prod_end(void) {}

void
h_update_led2(void)
{
    struct tg_event event;
    U32 mlc;

    NIC_TRACEV(TRACE_TYPE_TIMER, "#led2", 0x50600,
	0,
	0,
	0,
	0);

    mlc = trp->gen_control.misc_local_control;

    if (link_ready) { /* Is Link Up */
	if (nicfp->data_led_timer++ > 10) {

	    /* Reset the LED Timer counter */
	    nicfp->data_led_timer = 0;

	    /* tell host if the link has changed from down -> up */
	    if (!(nicfp->host_state & TG_HOST_STATE_LINK_RDY_ON)) {
		nicfp->host_state |= TG_HOST_STATE_LINK_RDY_ON;

		/* send the proper link up event (fast or gig) */
		if (active_phy == ATTACHED_PHY_IF_MII) { /* fast enet */
		    event.TG_EVENT_w0 = 
			(TG_EVENT_LINK_STATE_CHANGED << TG_EVENT_EVENT_SHIFT) |
			(TG_EVENT_CODE_FAST_LINK_UP << TG_EVENT_CODE_SHIFT);
		} else {
		    event.TG_EVENT_w0 = 
			(TG_EVENT_LINK_STATE_CHANGED << TG_EVENT_EVENT_SHIFT) |
			(TG_EVENT_CODE_LINK_UP << TG_EVENT_CODE_SHIFT);
		}
		must_enq_interrupt_event(&event);
		nicfp->stats.nicEventsLinkStateChanged++;
	    }
	    if (nicfp->host_state & TG_HOST_STATE_STACK_UP) {
		if (is_global_flags(GF_MOD_USE_OP1_LEDS)) {
		    mlc |= TG_MLC_LED_DATA;
#if (USE_U32_STAT_CNT)
		    if ((nicfp->ifHCInOctets != nicfp->led_ifHCInOctets) ||
			(nicfp->ifHCOutOctets != nicfp->led_ifHCOutOctets)) {

			nicfp->led_ifHCInOctets  = nicfp->ifHCInOctets;
			nicfp->led_ifHCOutOctets = nicfp->ifHCOutOctets;
			mlc ^= TG_MLC_LED_LINK;
		    } else {
			mlc &= ~TG_MLC_LED_LINK;
		    }
#else /* (USE_U32_STAT_CNT) */
		    if (((U32)nicfp->stats.ifHCInOctets !=
			    nicfp->led_ifHCInOctets) ||
			((U32)nicfp->stats.ifHCOutOctets !=
			    nicfp->led_ifHCOutOctets)) {

			nicfp->led_ifHCInOctets  = nicfp->stats.ifHCInOctets;
			nicfp->led_ifHCOutOctets = nicfp->stats.ifHCOutOctets;
			mlc ^= TG_MLC_LED_LINK;
		    } else {
			mlc &= ~TG_MLC_LED_LINK;
		    }
#endif /* (USE_U32_STAT_CNT) */
		} else {  /* Alteon HW */
		    mlc &= ~TG_MLC_LED_LINK;
#if (USE_U32_STAT_CNT)
		    if ((nicfp->ifHCInOctets != nicfp->led_ifHCInOctets) ||
			(nicfp->ifHCOutOctets != nicfp->led_ifHCOutOctets)) {

			nicfp->led_ifHCInOctets  = nicfp->ifHCInOctets;
			nicfp->led_ifHCOutOctets = nicfp->ifHCOutOctets;
			mlc ^= TG_MLC_LED_DATA;
		    } else {
			mlc |= TG_MLC_LED_DATA;
		    }
#else /* (USE_U32_STAT_CNT) */
		    if (((U32)nicfp->stats.ifHCInOctets != 
			     nicfp->led_ifHCInOctets) ||
			((U32)nicfp->stats.ifHCOutOctets != 
			     nicfp->led_ifHCOutOctets)) {
			nicfp->led_ifHCInOctets  = nicfp->stats.ifHCInOctets;
			nicfp->led_ifHCOutOctets = nicfp->stats.ifHCOutOctets;
			mlc ^= TG_MLC_LED_DATA;
		    } else {
			mlc |= TG_MLC_LED_DATA;
		    }
#endif /* (USE_U32_STAT_CNT) */
		}
	    } else {
		mlc |= TG_MLC_LED_DATA;
		if (nicfp->disabled_led_timer++ > 50) {
		    nicfp->disabled_led_timer = 0;
		    mlc ^= TG_MLC_LED_LINK;
		}
	    }
	}
    } else {
        if (nicfp->host_state & TG_HOST_STATE_STACK_UP) {
	    if (is_global_flags(GF_MOD_USE_OP1_LEDS)) {
                mlc &= ~TG_MLC_LED_LINK;
                mlc |= TG_MLC_LED_DATA;
            } else {  /* Alteon HW */
                mlc |= (TG_MLC_LED_DATA | TG_MLC_LED_LINK);
            }
        } else { /* The Stack is down i.e. IFD */
            mlc |= TG_MLC_LED_DATA;
            if (nicfp->disabled_led_timer++ > 500) {
                nicfp->disabled_led_timer = 0;
                mlc ^= TG_MLC_LED_LINK;
            }
        }
    }

    trp->gen_control.misc_local_control = mlc;

    /* turn off event */
    event_mask &= ~TG_FW_EVENT_UPDATE_LEDS;

    RETURN(TG_FW_EVENT_NUM_UPDATE_LEDS);
}

/* null routine to assist with scratchpad setup */
void h_update_led2_end(void) {}

U8 led_data=0;
U8 led_link=0;

void h_update_led4(void)
{
    struct tg_event event;
    U32 led_patn;

    NIC_TRACEV(TRACE_TYPE_TIMER, "#led4", 0x50a00,
	0,
	0,
	0,
	0);

    if (link_ready) { /* Is Link Up */
	if (nicfp->data_led_timer++ > 10) {

	    /* Reset the LED Timer counter */
	    nicfp->data_led_timer = 0;

	    /* tell host if the link has changed from down -> up */
	    if (!(nicfp->host_state & TG_HOST_STATE_LINK_RDY_ON)) {
		nicfp->host_state |= TG_HOST_STATE_LINK_RDY_ON;

		/* send the proper link up event (fast or gig) */
		if (active_phy == ATTACHED_PHY_IF_MII) { /* fast enet */
		    event.TG_EVENT_w0 = 
			(TG_EVENT_LINK_STATE_CHANGED << TG_EVENT_EVENT_SHIFT) |
			(TG_EVENT_CODE_FAST_LINK_UP << TG_EVENT_CODE_SHIFT);
		} else {
		    event.TG_EVENT_w0 = 
			(TG_EVENT_LINK_STATE_CHANGED << TG_EVENT_EVENT_SHIFT) |
			(TG_EVENT_CODE_LINK_UP << TG_EVENT_CODE_SHIFT);
		}
		must_enq_interrupt_event(&event);
		nicfp->stats.nicEventsLinkStateChanged++;
	    }

	    if (nicfp->host_state & TG_HOST_STATE_STACK_UP) {
		led_link = 1;
#if (USE_U32_STAT_CNT)
		if ((nicfp->ifHCInOctets != nicfp->led_ifHCInOctets) ||
		    (nicfp->ifHCOutOctets != nicfp->led_ifHCOutOctets)) {

		    nicfp->led_ifHCInOctets  = nicfp->ifHCInOctets;
		    nicfp->led_ifHCOutOctets = nicfp->ifHCOutOctets;
		    led_data ^= 1;
		} else {
		    led_data = 0;
		}
#else /* (USE_U32_STAT_CNT) */
		if (((U32)nicfp->stats.ifHCInOctets != 
		     nicfp->led_ifHCInOctets) ||
		    ((U32)nicfp->stats.ifHCOutOctets != 
		     nicfp->led_ifHCOutOctets)) {
		    nicfp->led_ifHCInOctets  = nicfp->stats.ifHCInOctets;
		    nicfp->led_ifHCOutOctets = nicfp->stats.ifHCOutOctets;
		    led_data ^= 1;
		} else {
		    led_data = 0;
		}
#endif /* (USE_U32_STAT_CNT) */
	    } else {
		led_data = 0;
		if (nicfp->disabled_led_timer++ > 50) {
		    nicfp->disabled_led_timer = 0;
		    led_link ^= 1;
		}
	    }
	}
    } else {
        if (nicfp->host_state & TG_HOST_STATE_STACK_UP) {
	    set_led4(LED_ALL_OFF);
	    led_link = 0;
	    led_data = 0;
        } else { /* The Stack is down i.e. IFD */
            led_data = 0;
            if (nicfp->disabled_led_timer++ > 500) {
                nicfp->disabled_led_timer = 0;
                led_link ^= led_link;
            }
        }
    }

    /* determine led setting */
    if (active_phy == ATTACHED_PHY_IF_GMII) {
	if (led_link == 1) {
	    if (led_data == 1)
		led_patn = LED_1000_ON_DT_ON;
	    else
		led_patn = LED_1000_ON_DT_OFF;
	} else {
	    led_patn = LED_ALL_OFF;
	}
    } else {
	if (mac_mgt_state[TIGON_MAC_INDEX].link_state_fast & LNK_100MB) {
	    /* 100 MB */
	    if (led_link == 1) {
		if (led_data == 1)
		    led_patn = LED_100_ON_DT_ON;
		else
		    led_patn = LED_100_ON_DT_OFF;
	    } else {
		led_patn = LED_ALL_OFF;
	    }
	} else { /* 10 MB */
	    if (led_link == 1) {
		if (led_data == 1)
		    led_patn = LED_10_ON_DT_ON;
		else
		    led_patn = LED_10_ON_DT_OFF;
	    } else {
		led_patn = LED_ALL_OFF;
	    }
	}
    }
    /*    
    NIC_UTRACE("#led4", 0x50a01,
	led_link,
	led_data,
	led_patn,
	mac_mgt_state[TIGON_MAC_INDEX].link_state_fast);
	*/
    set_led4(led_patn);

    /* turn off event */
    event_mask &= ~TG_FW_EVENT_UPDATE_LEDS;

    RETURN(TG_FW_EVENT_NUM_UPDATE_LEDS);
}

/* null routine to assist with scratchpad setup */
void h_update_led4_end(void) {}

void
h_update_led6(void)
{
    struct tg_event event;

    NIC_TRACEV(TRACE_TYPE_TIMER, "#led6", 0x50700,
	0,
	0,
	0,
	0);

    if (link_ready) { /* Is Link Up */
	nicfp->active_phy_display = active_phy;
	if (nicfp->data_led_timer++ > 10) {

	    /* Reset the LED Timer counter */
	    nicfp->data_led_timer = 0;

	    /* tell host if the link has changed from down -> up */
	    if (!(nicfp->host_state & TG_HOST_STATE_LINK_RDY_ON)) {
		nicfp->host_state |= TG_HOST_STATE_LINK_RDY_ON;

		/* send the proper link up event (fast or gig) */
		if (active_phy == ATTACHED_PHY_IF_MII) { /* fast enet */
		    event.TG_EVENT_w0 = 
			(TG_EVENT_LINK_STATE_CHANGED << TG_EVENT_EVENT_SHIFT) |
			(TG_EVENT_CODE_FAST_LINK_UP << TG_EVENT_CODE_SHIFT);
		} else {
		    event.TG_EVENT_w0 = 
			(TG_EVENT_LINK_STATE_CHANGED << TG_EVENT_EVENT_SHIFT) |
			(TG_EVENT_CODE_LINK_UP << TG_EVENT_CODE_SHIFT);
		}
		must_enq_interrupt_event(&event);
		nicfp->stats.nicEventsLinkStateChanged++;
	    }
	    if (nicfp->host_state & TG_HOST_STATE_STACK_UP) {
		if (is_global_flags(GF_MOD_USE_OP1_LEDS)) {
		    nicfp->led_bits |= LED_1000_DATA;
#if (USE_U32_STAT_CNT)
		    if ((nicfp->ifHCInOctets != nicfp->led_ifHCInOctets) ||
			(nicfp->ifHCOutOctets != nicfp->led_ifHCOutOctets)) {

			nicfp->led_ifHCInOctets  = nicfp->ifHCInOctets;
			nicfp->led_ifHCOutOctets = nicfp->ifHCOutOctets;
			nicfp->led_bits ^= LED_1000_LINK;
		    } else {
			nicfp->led_bits &= ~LED_1000_LINK;
		    }
#else /* (USE_U32_STAT_CNT) */
		    if (((U32)nicfp->stats.ifHCInOctets != 
			     nicfp->led_ifHCInOctets) ||
			((U32)nicfp->stats.ifHCOutOctets != 
			     nicfp->led_ifHCOutOctets)) {
			nicfp->led_ifHCInOctets  = nicfp->stats.ifHCInOctets;
			nicfp->led_ifHCOutOctets = nicfp->stats.ifHCOutOctets;
			nicfp->led_bits ^= LED_1000_LINK;
		    } else {
			nicfp->led_bits &= ~LED_1000_LINK;
		    }
#endif /* (USE_U32_STAT_CNT) */
		} else {  /* Alteon HW */
		    nicfp->led_bits &= ~LED_1000_LINK;
#if (USE_U32_STAT_CNT)
		    if ((nicfp->ifHCInOctets != nicfp->led_ifHCInOctets) ||
			(nicfp->ifHCOutOctets != nicfp->led_ifHCOutOctets)) {

			nicfp->led_ifHCInOctets  = nicfp->ifHCInOctets;
			nicfp->led_ifHCOutOctets = nicfp->ifHCOutOctets;
			nicfp->led_bits ^= LED_1000_DATA;
		    } else {
			nicfp->led_bits |= LED_1000_DATA;
		    }
#else /* (USE_U32_STAT_CNT) */
		    if (((U32)nicfp->stats.ifHCInOctets != 
			     nicfp->led_ifHCInOctets) ||
			((U32)nicfp->stats.ifHCOutOctets != 
			     nicfp->led_ifHCOutOctets)) {
			nicfp->led_ifHCInOctets  = nicfp->stats.ifHCInOctets;
			nicfp->led_ifHCOutOctets = nicfp->stats.ifHCOutOctets;
			nicfp->led_bits ^= LED_1000_DATA;
		    } else {
			nicfp->led_bits |= LED_1000_DATA;
		    }
#endif /* (USE_U32_STAT_CNT) */
		}
	    } else {
		nicfp->led_bits |= LED_1000_DATA;
		if (nicfp->disabled_led_timer++ > 50) {
		    nicfp->disabled_led_timer = 0;
		    nicfp->led_bits ^= LED_1000_LINK;
		}
	    }
	}
    } else {
        if (++nicfp->active_phy_tick_count >= nicfp->active_phy_ticks) {
            /* reset disabled LED countdown timer */
            nicfp->active_phy_tick_count = 0;
            nicfp->active_phy_display =
                (nicfp->active_phy_display == ATTACHED_PHY_IF_MII)?
		    ATTACHED_PHY_IF_SERDES_EXT:
		    ATTACHED_PHY_IF_MII;
        }
        if (nicfp->host_state & TG_HOST_STATE_STACK_UP) {
	    if (is_global_flags(GF_MOD_USE_OP1_LEDS)) {
                nicfp->led_bits |= LED_1000_LINK;
                nicfp->led_bits &= ~LED_1000_DATA;
            } else {  /* Alteon HW */
                nicfp->led_bits |= LED_1000_LINK | LED_1000_DATA;
            }
        } else { /* The Stack is down i.e. IFD */
            nicfp->led_bits |= LED_1000_DATA;
            if (nicfp->disabled_led_timer++ > 250) {
                nicfp->led_bits ^= LED_1000_LINK;
                nicfp->disabled_led_timer = 0;
            }
        }
    }

    /* always turn on the active bit */
    nicfp->led_bits &= ~LED_1000_ACTIVE;
    if (nicfp->active_phy_display == ATTACHED_PHY_IF_MII) {
        /* shift bits down for 100 mode */
        set_led(nicfp->led_bits >> 4 | 0xf0);
    } else {
        set_led(nicfp->led_bits | 0x0f);
    }

    /* turn off event */
    event_mask &= ~TG_FW_EVENT_UPDATE_LEDS;

    RETURN(TG_FW_EVENT_NUM_UPDATE_LEDS);
}

/* null routine to assist with scratchpad setup */
void h_update_led6_end(void) {}

void
update_stats(void)
{
    struct tg_event event;
    U32 i;

    NIC_TRACE(TRACE_TYPE_STATS, "udpStats", 0x50800,
	0,
	0,
	0,
	0);

    /* copy the stats from our internal scratchpad */
    wcopy((U32 *)&nicfp->stats, (U32 *)&nicp->gen_info.stats,
          sizeof(struct tg_stats)>>2);

    /* Update stats from the mac_stats hardware area */
    nicp->gen_info.stats.dot3StatsExcessiveCollisions = 
        tsp->macstats.excess_colls;
    nicp->gen_info.stats.dot3StatsLateCollisions = tsp->macstats.late_coll;

    nicp->gen_info.stats.dot3StatsSingleCollisionFrames = tsp->macstats.coll_1;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[0]  = tsp->macstats.coll_1;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[1]  = tsp->macstats.coll_2;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[2]  = tsp->macstats.coll_3;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[3]  = tsp->macstats.coll_4;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[4]  = tsp->macstats.coll_5;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[5]  = tsp->macstats.coll_6;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[6]  = tsp->macstats.coll_7;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[7]  = tsp->macstats.coll_8;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[8]  = tsp->macstats.coll_9;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[9]  = tsp->macstats.coll_10;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[10] = tsp->macstats.coll_11;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[11] = tsp->macstats.coll_12;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[12] = tsp->macstats.coll_13;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[13] = tsp->macstats.coll_14;
    nicp->gen_info.stats.nicMacTxCollisionHistogram[14] = tsp->macstats.coll_15;

    /* Multiple collision errors start from macstats.coll_2 to coll_15 */
    nicp->gen_info.stats.dot3StatsMultipleCollisionFrames = 0;
    for (i = 1; i < 15; i++) {
        nicp->gen_info.stats.dot3StatsMultipleCollisionFrames += 
            nicp->gen_info.stats.nicMacTxCollisionHistogram[i];
    }
    nicp->gen_info.stats.dot3StatsInternalMacTransmitErrors = 
        tsp->macstats.underrun;
    nicp->gen_info.stats.dot3StatsCarrierSenseErrors = tsp->macstats.crs_err;
#if (0)
    nicp->gen_info.stats.dot3StatsDeferredTransmission = tsp->macstats.defers;
    nicp->gen_info.stats.nicMacTxCrcErrs = tsp->macstats.crc_err;
#endif /* XXX */
    nicp->gen_info.stats.nicMacRxDropUla = tsp->macstats.drop_ula;
    nicp->gen_info.stats.nicMacRxDropMcast = tsp->macstats.drop_mc;
    nicp->gen_info.stats.nicMacRxFlowControl = tsp->macstats.drop_fc;
    nicp->gen_info.stats.nicMacRxDropSpace = tsp->macstats.drop_space;
    nicp->gen_info.stats.nicMacRxColls = tsp->macstats.coll;

    /* watch overflow and handle it */
    if (tsp->macstats.kept_uc < (U32)nicp->gen_info.stats.ifHCInUcastPkts)
        nicp->gen_info.stats.ifHCInUcastPkts += 0x100000000ULL;
    nicp->gen_info.stats.ifHCInUcastPkts =
        (nicp->gen_info.stats.ifHCInUcastPkts & 0xffffffff00000000ULL) |
        tsp->macstats.kept_uc;

    if (tsp->macstats.kept_bc < (U32)nicp->gen_info.stats.ifHCInBroadcastPkts)
        nicp->gen_info.stats.ifHCInBroadcastPkts += 0x100000000ULL;
    nicp->gen_info.stats.ifHCInBroadcastPkts =
        (nicp->gen_info.stats.ifHCInBroadcastPkts & 0xffffffff00000000ULL) |
        tsp->macstats.kept_bc;

    if (tsp->macstats.kept_mc < (U32)nicp->gen_info.stats.ifHCInMulticastPkts)
        nicp->gen_info.stats.ifHCInMulticastPkts += 0x100000000ULL;
    nicp->gen_info.stats.ifHCInMulticastPkts =
        (nicp->gen_info.stats.ifHCInMulticastPkts & 0xffffffff00000000ULL) |
        tsp->macstats.kept_mc;

    /* dma to the host */
    if ((*nicfp->q_dma_to_host_stub)
	    (tsp->gen_com.gen_info_ptr + (U32)&((struct tg_gen_info *)0)->stats,
	    (U32)&nicp->gen_info.stats,
	    sizeof(struct tg_stats), 
	    TIGON_TYPE_UPDATE_STATS,
	    0,
	    nicfp->dma_to_host_bd_state)) {

	/* if the stats DMA failed, don't reset timer, they will be
	 * retried.  This is necessary because the watchdog function
	 * will trigger if 3 consecutive updates are missed.
	 */
        event.TG_EVENT_w0 = (TG_EVENT_STATS_UPDATED << TG_EVENT_EVENT_SHIFT);
	if ((nicfp->stats_to_host_pending)?
		must_enq_event(&event):
		enq_event(&event)) {

            nicfp->stats_to_host_pending = FALSE;
            nicfp->stats_tick_count = 0;
            nicfp->stats.nicEventsStatsUpdated++; /* internal copy */
            nicp->gen_info.stats.nicEventsStatsUpdated++; /* shared memory */
        } else {
            nicfp->stats_to_host_pending = TRUE;
	}
    } else {
        nicfp->stats_to_host_pending = TRUE;
    }

    return;
}

/* patch around the hanging of DMA channal between SBUS and X-Chip */
void
fix_xchip_hang_on_sbus(void)
{
    struct tg_dma_descr *ddp_cons;

    /* read channel */
    if (trp->host_dma.dma_rd_state & TG_DMA_STATE_ACTIVE) {
	if ((nicfp->prev_rddp_cons !=
	     trp->host_dma_assist.read_chan_consumer) ||
	    (nicfp->prev_raddr != trp->host_dma.dma_rd_loc_addr)) {

	    /* record new ones */
	    nicfp->prev_rddp_cons = trp->host_dma_assist.read_chan_consumer;
	    nicfp->prev_raddr = trp->host_dma.dma_rd_loc_addr;
	} else {
	    if (trp->host_dma.dma_rd_state & TG_DMA_STATE_PROGRESS_MADE) {
		/* clear the progress bit */
		trp->host_dma_assist.assist_state |= TG_DMA_ASSIST_PAUSE;
		trp->host_dma.dma_rd_state        &= ~TG_DMA_STATE_ACTIVE;
		trp->host_dma_assist.assist_state &= ~TG_DMA_ASSIST_PAUSE;
	    } else {
		if ((nicfp->prev_rddp_cons != 
		     trp->host_dma_assist.read_chan_consumer) ||
		    (nicfp->prev_raddr != trp->host_dma.dma_rd_loc_addr)) {

		    /* record new ones */
		    nicfp->prev_rddp_cons =
			trp->host_dma_assist.read_chan_consumer;
		    nicfp->prev_raddr = trp->host_dma.dma_rd_loc_addr;

		    NIC_UTRACE("T-DmaRd2", 0x50900,
			trp->host_dma_assist.assist_state,
			trp->host_dma.dma_rd_state,
			nicfp->prev_rddp_cons,
			nicfp->prev_raddr);
		} else {
		    NIC_UTRACE("T-DmaRd1", 0x51000,
			trp->host_dma_assist.assist_state,
			trp->host_dma.dma_rd_state,
			nicfp->prev_rddp_cons,
			nicfp->prev_raddr);

		    /* reset read dma channel */
		    trp->host_dma_assist.assist_state |= TG_DMA_ASSIST_PAUSE;
		    (volatile)ddp_cons =
			trp->host_dma_assist.read_chan_consumer;
		    trp->host_dma.dma_rd_state |= TG_DMA_STATE_RESET;
		    trp->host_dma_assist.read_chan_consumer = ddp_cons;

		    /* force a write dma */
		    (*nicfp->q_dma_to_host_stub)
			 (tsp->gen_com.gen_info_ptr +
			     (U32)&((struct tg_gen_info *)0)->stats,
			 (U32)&nicp->gen_info.stats,
			 sizeof(struct tg_stats),
			 TIGON_TYPE_UPDATE_STATS,
			 0,
			 nicfp->dma_to_host_bd_state);

		    trp->host_dma_assist.assist_state &= ~TG_DMA_ASSIST_PAUSE;
		    nicp->gen_info.stats.nicSbusHangCleared++;

		    NIC_UTRACE("T-DmaRdB", 0x51100,
			trp->host_dma_assist.assist_state,
			trp->host_dma.dma_rd_state,
			nicfp->prev_rddp_cons,
			nicfp->prev_raddr);
		}
	    }
	}
    }

    /* write channel */
    if (trp->host_dma.dma_wr_state & TG_DMA_STATE_ACTIVE) {
	if ((nicfp->prev_wddp_cons !=
	     trp->host_dma_assist.write_chan_consumer) ||
	    (nicfp->prev_waddr != trp->host_dma.dma_wr_loc_addr)) {

	    /* record new ones */
	    nicfp->prev_wddp_cons = trp->host_dma_assist.write_chan_consumer;
	    nicfp->prev_waddr = trp->host_dma.dma_wr_loc_addr;
	} else {
	    if (trp->host_dma.dma_wr_state & TG_DMA_STATE_PROGRESS_MADE) {
		/* clear the progress bit */
		trp->host_dma_assist.assist_state |= TG_DMA_ASSIST_PAUSE;
		trp->host_dma.dma_wr_state        &= ~TG_DMA_STATE_ACTIVE;
		trp->host_dma_assist.assist_state &= ~TG_DMA_ASSIST_PAUSE;
	    } else {
		if ((nicfp->prev_wddp_cons != 
		     trp->host_dma_assist.write_chan_consumer) ||
		    (nicfp->prev_waddr != trp->host_dma.dma_wr_loc_addr)) {

		    /* record new ones */
		    nicfp->prev_wddp_cons =
			trp->host_dma_assist.write_chan_consumer;
		    nicfp->prev_waddr = trp->host_dma.dma_wr_loc_addr;

		    NIC_UTRACE("T-DmaWr2", 0x51200,
			trp->host_dma_assist.assist_state,
			trp->host_dma.dma_wr_state,
			nicfp->prev_wddp_cons,
			nicfp->prev_waddr);
		} else {
		    NIC_UTRACE("T-DmaWr1", 0x51300,
			trp->host_dma_assist.assist_state,
			trp->host_dma.dma_wr_state,
			nicfp->prev_wddp_cons,
			nicfp->prev_waddr);

		    /* reset write dma channel */
		    trp->host_dma_assist.assist_state |= TG_DMA_ASSIST_PAUSE;
		    (volatile)ddp_cons =
			trp->host_dma_assist.write_chan_consumer;
		    trp->host_dma.dma_wr_state |= TG_DMA_STATE_RESET;
		    trp->host_dma_assist.write_chan_consumer = ddp_cons;

		    /* force a dummy read dma */
		    (*nicfp->q_dma_to_nic_stub)
			 (tsp->gen_com.gen_info_ptr,
			 (U32)&dummy_rd_dma,
			 sizeof(U32),
			 TIGON_TYPE_SEND_DUMMY_DMA,
			 0,
			 nicfp->dma_to_nic_bd_state);

		    trp->host_dma_assist.assist_state &= ~TG_DMA_ASSIST_PAUSE;
		    nicp->gen_info.stats.nicSbusHangCleared++;

		    NIC_UTRACE("T-DmaWrB", 0x51400,
			trp->host_dma_assist.assist_state,
			trp->host_dma.dma_wr_state,
			nicfp->prev_wddp_cons,
			nicfp->prev_waddr);
		}
	    }
	}
    }
}

#ifdef STACK_REDZONE
void
check_redzone(void)
{
    register U32 *wp = &tdp->redzone[0];
    U32 *wend = &tdp->redzone[STACK_REDZONE_SIZE];

    do {
        if (*wp != 0xdeadbabe) {
            NIC_UTRACE("redZone2", 0x51500,
		*wp,
		wp,
		0,
		0);
            ASSERT(0);
        }
    } while (++wp < wend);

    return;
}
#endif /* STACK_REDZONE */

void
h_hf_timer(void)
{
    register U32 event;
    U32 send_coal_ticks;
    U32 recv_coal_ticks;

    /* high frequency timer loop */
    while (1) { 
        if (trp->gen_control.event_b & TG_FW_EVENT_TIMER) {
            /* reset timer */
            trp->gen_control.timer_ref_b = 
                trp->gen_control.timer + TG_TICK_FAST;
     
            event = 0;
     
	    send_coal_ticks = tsp->gen_com.tuneParms.send_coal_ticks;
	    if (send_coal_ticks &&
		    ((nicp->send_coal_tick_count += TG_TICK_FAST) >=
		    send_coal_ticks)) {
		nicp->send_coal_tick_count = 0;
		nicp->send_cons_index_to_host_pending = TRUE;
		event |= TG_FW_EVENT_DO_TX_CONS;
            } else if (nicp->send_cons_index_to_host_pending ||
		       nicp->send_cons_index_to_host_pending_no_intr) {
                event |= TG_FW_EVENT_DO_TX_CONS;
            }

	    recv_coal_ticks = tsp->gen_com.tuneParms.recv_coal_ticks;
            if (recv_coal_ticks &&
		    ((nicp->recv_coal_tick_count += TG_TICK_FAST) >=
		    recv_coal_ticks)) {
		nicp->recv_coal_tick_count = 0;
                event |= TG_FW_EVENT_DO_RX_PROD;
            } else if (nicp->recv_ret_prod_index_to_host_pending) {
                event |= TG_FW_EVENT_DO_RX_PROD;
            }

            if (event)
                set_event_reg(event);
        }
    }
    /* not reached */
}

void h_hf_timer_end(void) {}

