/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 */
/*
 * HISTORY
 * $Log: trace.h $
 * Revision api12_3_ted/2 1999/08/12 21:06:47 ted
 * 	Assemblerize header files
 * 
 * Revision 1.1.2.3  1998/12/08  02:36:54  shuang
 * 	Clean up debug tracing. Use sw dma. Fix dma avoidance.
 * 	[1998/12/08  02:28:38  shuang]
 *
 * Revision 1.1.2.2  1998/04/11  05:15:26  shuang
 * 	move fast timer to cpu B, add receive coalesed ticks
 * 	[1998/04/11  04:59:05  shuang]
 * 
 * Revision 1.1.2.1  1998/01/14  23:26:06  ted
 * 	rearrange directories
 * 	[1998/01/14  23:18:04  ted]
 * 
 * Revision 1.1.2.1  1997/11/19  19:23:39  ted
 * 	Support Tigon and Tigon2 in the same driver
 * 	[1997/11/19  19:20:18  ted]
 * 
 * Revision 1.6.9.1  1997/07/25  22:59:19  ted
 * 	Merge fixes from ge_0_22 and add link negotiation support
 * 	[1997/07/25  22:32:32  ted]
 * 
 * Revision 1.6.5.1  1997/07/16  00:20:25  ted
 * 	UTRACE() is now always on.
 * 	[1997/07/15  00:39:59  ted]
 * 
 * Revision 1.6.3.3  1996/11/15  04:24:25  hayes
 * 	convert to new header file structure.
 * 	[1996/11/15  04:12:22  hayes]
 * 
 * Revision 1.6.3.2  1996/11/05  21:54:36  hayes
 * 	Final pre-ODE merge.
 * 	[1996/11/05  21:53:52  hayes]
 * 
 * Revision 1.6.3.1  1996/11/01  18:25:15  ted
 * 	change rr_ to tg_, fix compiler warnings, add copyright notices
 * 	[1996/10/29  22:06:55  ted]
 * 
 * $EndLog$
 */
/*
 * FILE trace.h
 *
 * COPYRIGHT (c) Essential Communication Corp. 1995
 * $Source: /projects/rcs/sw/ge/./nic/fw2/common/trace.h,v $
 * $Revision: 1.1.2.3 $ $Locker:  $
 * $Date: 1998/12/08 02:36:54 $ $Author: shuang $ $State: Exp $
 */

#ifndef _TRACE_H_
#define _TRACE_H_

#if !defined(_IN_ASM_)
/* various things that may be renamed from system to system */
#define ERROR1
#define ERROR2
#define ERROR3
#define ERROR5

/* ------------------- Tracing --------------------- */
/*
 * Tracing is instrumented at various levels:
 * 	Key events
 * 	Verbose
 * Tracing is compiled into the driver based on these flags
 * 	TRACE_ALL	- verbose
 * 	TRACE	- key events
 * Tracing is run-time controlable (assuming it has been compiled in) 
 * as follows:
 * 	Tracing == 2 - verbose
 * 	Tracing == 1 - key events
 * The variables are set in data.c
 *
 * TRACE, TRACE_ALL, TRACE_ELM_SIZE and TRACE_SIZE are defined in nic.h
 */
#define MAX_STRACE_LINES 4	/* Number of lines in mbuf report */
#define MAX_TRACE_MBUFS  4	/* maximum number of mbufs in traceMC */

#if TRACE && TRACE_ALL
#define NIC_TRACEV(type, a, b, c, d, e, f) { \
    U32 tracing = tsp->gen_com.tuneParms.tracing; \
    if ((tracing & TRACE_LEVEL_2) && (tracing & type)) { \
        trace((U32 *)a, (U32)(b), (U32)(c), (U32)(d), (U32)(e), (U32)(f)); \
    } else { \
	; \
    } \
}
#define NIC_TRACE2V(type, a, b, c, d, e, f, g) { \
    U32 tracing = tsp->gen_com.tuneParms.tracing; \
    if (tracing & TRACE_LEVEL_2) && (tracing & type)) { \
        ltrace((U32 *)a, (U32)(b), (U32)(c), (U32)(d), (U32)(e), \
		(U32)(f), (U32 *)(g)); \
    } else { \
	; \
    } \
}
#define NIC_TRACEV_IF(type, a, b, bd_var, bd_type, c, d, e, f) { \
    U32 tracing = tsp->gen_com.tuneParms.tracing; \
    if (((tracing & TRACE_LEVEL_2) && (tracing & type)) || \
	((tracing & (TRACE_TYPE_SEND|TRACE_TYPE_RECV|TRACE_TYPE_RECV_BD)) && \
	    (bd_var & bd_type))) { \
        trace((U32 *)a, (U32)(b), (U32)(c), (U32)(d), (U32)(e), (U32)(f)); \
    } else { \
	; \
    } \
}
#define NIC_TRACE2V_IF(type, a, b, bd_var, bd_type, c, d, e, f, g) { \
    U32 tracing = tsp->gen_com.tuneParms.tracing; \
    if (((tracing & TRACE_LEVEL_2) && (tracing & type)) || \
	((tracing & (TRACE_TYPE_SEND|TRACE_TYPE_RECV|TRACE_TYPE_RECV_BD)) && \
	    (bd_var & bd_type))) { \
        ltrace((U32 *)a, (U32)(b), (U32)(c), (U32)(d), (U32)(e), \
		(U32)(f), (U32 *)(g)); \
    } else { \
	; \
    } \
}
#else	/* TRACE */
#define NIC_TRACEV(type, a, b, c, d, e, f)
#define NIC_TRACE2V(type, a, b, c, d, e, f, g)
#define NIC_TRACEV_IF(bd_var, bd_type, type, a, b, c, d, e, f)
#define NIC_TRACE2V_IF(bd_var, bd_type, type, a, b, c, d, e, f, g)
#endif	/* TRACE && TRACE_ALL */

#if TRACE
#define NIC_TRACE(type, a, b, c, d, e, f) \
    if (tsp->gen_com.tuneParms.tracing & (type | TRACE_LEVEL_1)) { \
        trace((U32 *)a, (U32)(b), (U32)(c), (U32)(d), (U32)(e), (U32)(f)); \
    } else { \
	; \
    }
#define NIC_TRACE2(type, a, b, c, d, e, f, g) \
    if (tsp->gen_com.tuneParms.tracing & (type | TRACE_LEVEL_1)) { \
        ltrace((U32 *)a, (U32)(b), (U32)(c), (U32)(d), (U32)(e), \
		(U32)(f), (U32 *)(g)); \
    } else { \
	; \
    }
#define NIC_TRACE_IF(type, a, b, bd_var, bd_type, c, d, e, f) { \
    U32 tracing = tsp->gen_com.tuneParms.tracing; \
    if ((tracing & (type | TRACE_LEVEL_1)) || \
	((tracing & (TRACE_TYPE_SEND|TRACE_TYPE_RECV|TRACE_TYPE_RECV_BD)) && \
	    (bd_var & bd_type))) { \
        trace((U32 *)a, (U32)(b), (U32)(c), (U32)(d), (U32)(e), (U32)(f)); \
    } else { \
	; \
    } \
}
#define NIC_TRACE2_IF(type, a, b, bd_var, bd_type, c, d, e, f, g) { \
    U32 tracing = tsp->gen_com.tuneParms.tracing; \
    if ((tracing & (type | TRACE_LEVEL_1)) || \
	((tracing & (TRACE_TYPE_SEND|TRACE_TYPE_RECV|TRACE_TYPE_RECV_BD)) && \
	    (bd_var & bd_type))) { \
        ltrace((U32 *)a, (U32)(b), (U32)(c), (U32)(d), (U32)(e), \
		(U32)(f), (U32 *)(g)); \
    } else { \
	; \
    } \
}
#else	/* TRACE */
#define NIC_TRACE(type, a, b, c, d, e, f)
#define NIC_TRACE2(type, a, b, c, d, e, f, g)
#define NIC_TRACE_IF(bd_var, bd_type, type, a, b, c, d, e, f)
#define NIC_TRACE2_IF(bd_var, bd_type, type, a, b, c, d, e, f, g)
#endif	/* TRACE */

/* urgent trace macros - always on for errors */
#define NIC_UTRACE(a, b, c, d, e, f) \
    trace((U32 *)a, (U32)(b), (U32)(c), (U32)(d), (U32)(e), (U32)(f))
#define NIC_UTRACE2(a, b, c, d, e, f, g) \
    ltrace((U32 *)a, (U32)(b), (U32)(c), (U32)(d), (U32)(e), \
	(U32)(f), (U32 *)(g))

/* Tracing variable */
extern U32 TraceInited;
extern U32 *curtb;
extern U32 *endtb;
extern U32 tbuf[];
#endif /* !defined(_IN_ASM_) */

#endif	/* _TRACE_H_ */
