/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996, 1997 
 * All rights reserved
 * 
 */
/*
 * HISTORY
 * $Log: genfw.c $
 * Revision sb_tibor/2 1999/03/24 04:11:57 tibor
 * 	cleaned up nltoh references
 * 
 * Revision 1.1.2.14  1999/02/22  22:06:34  tibor
 * 	corrected ifdef for baf not bfd, sigh...
 * 	[1999/02/22  22:05:27  tibor]
 *
 * Revision 1.1.2.13  1999/02/19  19:05:27  tibor
 * 	forgot about baf.h
 * 	[1999/02/19  19:05:12  tibor]
 * 
 * Revision 1.1.2.12  1999/02/19  19:01:08  tibor
 * 	ifdef'd out baf for cleaner nic build
 * 	[1999/02/19  19:00:53  tibor]
 * 
 * Revision 1.1.2.11  1999/01/29  22:06:26  tibor
 * 	keep kernelImage.h include out of non switch build
 * 	[1999/01/29  22:06:04  tibor]
 * 
 * Revision 1.1.2.10  1998/12/31  01:03:54  tibor
 * 	fixed for nic world
 * 	[1998/12/31  01:03:25  tibor]
 * 
 * Revision 1.1.2.9  1998/12/22  03:14:19  tibor
 * 	use bdf libs instead of direct elf access
 * 	[1998/12/22  02:14:48  tibor]
 * 
 * Revision 1.1.2.8  1998/09/18  18:34:35  mef
 * 	Changed fopen to explicitly open files in binary mode,
 * 	which is necessary for the cygwin32 environment.
 * 	[1998/09/18  18:31:03  mef]
 * 
 * Revision 1.1.2.7  1998/05/01  02:01:13  tibor
 * 	merged ts and nic versions, added -h option
 * 	[1998/05/01  01:35:24  tibor]
 * 
 * Revision 1.1.2.6  1998/04/07  21:04:17  hayes
 * 	increase TEXT_SIZE to 96K
 * 	[1998/04/07  21:03:18  hayes]
 * 
 * Revision 1.1.2.5  1998/04/01  07:32:40  shuang
 * 	incrase max text size to 84k, max rddata size to 8k
 * 	[1998/04/01  06:39:05  shuang]
 * 
 * Revision 1.1.2.4  1998/03/17  03:05:46  tibor
 * 	moved genfw down a level
 * 	[1998/03/17  03:05:24  tibor]
 * 
 * Revision 1.1.2.3  1998/01/29  02:33:03  skapur
 * 	 Increased the size of RO Data from 2 to 3 K
 * 	[1998/01/29  02:30:55  skapur]
 * 
 * Revision 1.1.2.2  1997/12/17  01:37:27  skapur
 * 	Changed the size of fw from 56K to 64K
 * 	[1997/12/17  01:32:07  skapur]
 * 
 * Revision 1.1.2.1  1997/12/01  23:39:29  hayes
 * 	initial checkin
 * 	[1997/12/01  23:38:25  hayes]
 * 
 * $EndLog$
 */

#include "alt_def.h"
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <bfd.h>
#include <stdlib.h>
#include <netinet/in.h>
#include "CElf.h"
#if defined(BUILD_SWITCH)    
#include "baf.h"
#endif /* defined(BUILD_SWITCH) */
#include "crc.h"
#include "kernelImage.h"
#include "release.h"
#include "parsefile.h"

/* should be matched to the ones in rr/essreg.h */
#define MAX_TEXT_SIZE   (65536 * 32)
#define MAX_RODATA_SIZE (65536 * 32)
#define MAX_DATA_SIZE   (65536 * 32)
#define MAX_BSS_SIZE    (65536 * 32)
#define MAX_SBSS_SIZE   (65536 * 32)

static char usage[] = "%s [-p<prefix>] [-h] <elf_file> <data_file>\n\
\t(all parameters are position dependant)\n\n\
\t -p <prefix> = prefix constants with string, default is alt.\n\
\t -h <inc file> = hard allocated sizes of constants with include file\n\
\t <elf_file>  = source file\n\
\t <data_file> = destination file\n";

static symbolElement_t aSymList[] = {
    {"MAX_TEXT_LEN", MAX_TEXT_SIZE},
    {"MAX_DATA_LEN", MAX_RODATA_SIZE},
    {"MAX_RODATA_LEN", MAX_DATA_SIZE},
    {"MAX_BSS_SIZE", MAX_BSS_SIZE},
    {"MAX_SBSS_SIZE", MAX_SBSS_SIZE},
    {NULL, 0}
};

static char *aSecNames[] = {"", "Text", "Data", "Rodata", "Bss", "Sbss"};

int
main(int argc, char **argv)
{
    FILE *sourceFile;
    elfFileData_t *pElf;
    elfSection_t **pElfSecList;
    elfSection_t *pElfSec;
    char prefix[80] = "alt";
    U32 *bp, *bpEnd;
    char name[80];
    bool bHardAlloc = FALSE;
    char *pHardFile = NULL;
    char *progname = argv[0];
    U32 nTemp;
    int i;
#if defined(BUILD_SWITCH)    
    U32 baf;
    int nLen;
    U32 crc;
#endif /* defined(BUILD_SWITCH) */   

    /* need one param */
    if (argc < 3 || argc > 6) {
	(void) printf(usage, progname);
	exit(1);
    }

    if (argv[1][0] == '-' && argv[1][1] == 'p') {
	strcpy(prefix, &argv[1][2]);
	argv++;			/* shift */
    }
    
    if (argv[1][0] == '-' && argv[1][1] == 'h') {
	bHardAlloc = TRUE;
	argv++;			/* shift */
	pHardFile = argv[1];
	argv++;
    }

    if (!(pElf = new_elfFileData(argv[1])) || read_elfFileData(pElf)) {
	return 1;
    }
    
    if (!(sourceFile = fopen(argv[2], "wb"))) {
	(void) printf("Couldn't open file %s\n", argv[1]);
	(void) printf(usage, progname);
	exit(1);
    }

    if (bHardAlloc && !parseFile(pHardFile, aSymList)) {
	(void) fprintf(stderr,"Error reading header file for MAX_xx_LEN info\n");
	return 1;	
    }

    /* build up the .h file */
    (void) fprintf(sourceFile, "/* Generated by genfw.c */\n");
    
#if defined(BUILD_SWITCH)    
    (void) fprintf(sourceFile, "#include \"kernelImage.h\"\n");
#endif /* defined(BUILD_SWITCH) */
    
    (void) fprintf(sourceFile, "int %sFwReleaseMajor = 0x%x;\n", 
		   prefix, RELEASE_MAJOR);
    (void) fprintf(sourceFile, "int %sFwReleaseMinor = 0x%x;\n", 
		   prefix, RELEASE_MINOR);
    (void) fprintf(sourceFile, "int %sFwReleaseFix = 0x%x;\n", 
		   prefix, RELEASE_FIX);
    (void) fprintf(sourceFile, "U32 %sFwStartAddr = 0x%08x;\n", 
		   prefix, (int)pElf->startAddr);

#if defined(BUILD_SWITCH)     
    baf = BAF_GET_CPUTYPE(ntohl((U32)pElf->startAddr));
#endif /* defined(BUILD_SWITCH) */
    
    for (i=pElf->numSections, pElfSecList=pElf->pSecList; i > 0; i--) {
	pElfSec = *pElfSecList++;

	strcpy(name,aSecNames[pElfSec->nSecType]);
	if (pElfSec->name) {
	    strcat(name,pElfSec->name);
	}

	nTemp = (U32)(U32)pElfSec->addr;
	(void) fprintf(sourceFile, "U32 %sFw%sAddr = 0x%08lx;\n", 
		       prefix, name,
#if defined(BUILD_SWITCH)		       
		       (int)(BAF_IS_SCRPAD(nTemp))
		         ? BAF_FROM_LOCAL(nTemp, baf)
		         :
#endif /* defined(BUILD_SWITCH) */		       
		       nTemp);
	(void) fprintf(sourceFile, "int %sFw%sLen = 0x%x;\n",
		       prefix, name, pElfSec->len);
    }

    /* for readability, put addr/len first then the actual data areas */
    for (i=pElf->numSections, pElfSecList=pElf->pSecList; i > 0; i--) {
	pElfSec = *pElfSecList++;

	strcpy(name,aSecNames[pElfSec->nSecType]);
	if (pElfSec->name) {
	    strcat(name,pElfSec->name);
	}
	
	if (pElfSec->data) {
	    if (bHardAlloc) {
		(void) fprintf(sourceFile, "U32 %sFw%s[(%s/4) + 1] = {\n",
			       prefix, name, aSymList[pElfSec->nSecType-1].pSymbol);
	    } else {
		(void) fprintf(sourceFile, "U32 %sFw%s[(0x%0x/4) + 1] = {\n",
			       prefix, name, pElfSec->len);
	    }

	    bp = (U32 *)pElfSec->data;
	    bpEnd = bp + (pElfSec->len+3)/4;
	
	    for (; bp != bpEnd; bp++) {
		(void) fprintf(sourceFile, "0x%lx, ", (unsigned long)htonl(*bp));
		if (((U32)bp & 15) == 0)
		    (void) fprintf(sourceFile, "\n");
	    }
	    (void) fprintf(sourceFile, "0x0 };\n");
	}
    }

#if defined(BUILD_SWITCH)    
    /* output kernel image sections */
    fprintf(sourceFile, "\nstatic section_def_t %sFwSecList[] = {\n", prefix);
    
    for (i=pElf->numSections, pElfSecList=pElf->pSecList; i > 0; i--) {
	pElfSec = *pElfSecList++;

	strcpy(name,aSecNames[pElfSec->nSecType]);
	if (pElfSec->name) {
	    strcat(name,pElfSec->name);
	}
	
	nLen = pElfSec->len;
	
	nTemp = (U32)(U32)pElfSec->addr;
	
	fprintf(sourceFile, "    {%d, 0x%0x, 0,\n", pElfSec->nSecType,
		pElfSec->bSecFlags);

	if (pElfSec->data) {
	    crc = htonl(swapw(~gen_crc(pElfSec->data, nLen, 0xffffffff)));
	    fprintf(sourceFile, "     (U32 *)%sFw%s, 0x%0x, 0x%08lx, 0, 0, 0x%08lx, 0},\n",
		    prefix, name, pElfSec->len,
		    
		    ((U32)(BAF_IS_SCRPAD(nTemp))
		     ? BAF_FROM_LOCAL(nTemp, baf) 
		     : nTemp),
		    crc);
	} else {
	    fprintf(sourceFile, "     (U32 *)0, 0x%0x, 0x%08lx, 0, 0, 0, 0},\n",
		    pElfSec->len,
		    ((U32)(BAF_IS_SCRPAD(nTemp))
		     ? BAF_FROM_LOCAL(nTemp, baf) : nTemp));
	}
	
    }

    fprintf(sourceFile,"};\n");

    /* output kernel def file */
    fprintf(sourceFile, "\nkernel_def_t %sFwKernelDef = {\n", prefix);
    fprintf(sourceFile, "    0x%08lx, 0, %sFwSecList, %d\n};\n",
	    (U32)pElf->startAddr, prefix, pElf->numSections);
#endif /* defined(BUILD_SWITCH) */
    
    return 0;
}
