/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1998
 * All rights reserved
 * 
 */
/*
 * HISTORY
 * $Log: parsefile.c,v $
 * Revision 1.1.2.4  1998/12/31  01:03:56  tibor
 * 	fixed for nic world
 * 	[1998/12/31  01:03:26  tibor]
 *
 * Revision 1.1.2.3  1998/12/22  03:14:20  tibor
 * 	remove requirement of locating all #defined values
 * 	[1998/12/22  02:17:53  tibor]
 * 
 * Revision 1.1.2.2  1998/06/19  07:55:42  tibor
 * 	new pasing algs
 * 	[1998/06/19  07:45:36  tibor]
 * 
 * Revision 1.1.2.1  1998/05/01  02:01:13  tibor
 * 	initial version, for -h option
 * 	[1998/05/01  01:35:58  tibor]
 * 
 * $EndLog$
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "parsefile.h"
#include "nextToken.h"

/*
 *
 * Function:	parseFile
 *
 * Purpose:	Parse input file and locate #defines matching symbols in
 *		list.  Evaluate #define value and save in list entry. Only
 *		simple <num> oper <num> oper ... type evaluation done. oper
 *		must be +, -, *, /.  Also, the entire statement must be on
 * 		a single line.
 *		The element values are zero'ed and its assumed that a isn't
 *		going to set to zero.  No big deal, except doubly defined
 *		values won't be caught.
 *		Which brings us to the next condition, #ifdefs are ignored, so
 *		if the #define is wrapped, you loose.
 *
 * Parms:	fname	- file name to use for parse
 *		list	- zero entry delimited list of symbols to search/eval
 *
 * Returns:	0	- failure (error printed to stderr)
 *		1	- success 
 *
 */
int
parseFile(char *fName, symbolElement_t *list) {
    FILE *pFile;
    symbolElement_t *pElem, *pElemLimit;
    char line[256];
    char *pLine, *pLast;
    char sToken[80];
    int nLine = 0;
    int retcode = 1;
    int errcode; 

    /* make sure a good file was given */
    if (!(pFile = fopen(fName, "r"))) {
	(void) fprintf(stderr, "Couldn't open file %s\n", fName);
	return 0;
    }

     /* set limit pointer */
     for (pElemLimit = list; pElemLimit->pSymbol; pElemLimit++);

    /* process whole file, looking for #define <symbol> in each line */
    while (!feof(pFile)) {
	nLine++;
	if (!(pLine = fgets(line, sizeof(line), pFile))) {
	    /* on any file error abort.  On EOF, drop out. */
	    if ((errcode = ferror(pFile))) { 
		(void) fprintf(stderr, "Read error in %s, errno - %d\n", fName, errcode);
		retcode = 0;
		break;
	    } else {
		continue;
	    }
	}

	if (nextToken(line,sToken, &pLast) == 1) {
	    /* got a valid line, check if token is #define */
	    if (strcmp(sToken,"#define") == 0) {
		if (nextToken(NULL, sToken, &pLast) == 1) {
		    /* check if symbol is one that we want */
		    for (pElem = list; pElem != pElemLimit; pElem++) {
			if (strcmp(sToken, pElem->pSymbol) == 0) {
			    /* found match, overwrite any existing values */
			    pElem->value = parseValue(pLast, nLine);
			    continue;
			}
		    }
		}
	    }
	}

    }

    (void) fclose(pFile);


    for (pElem = list; pElem != pElemLimit; pElem++) {
	if (pElem->value == 0) {
	    (void) fprintf(stderr, "Symbol %s not found in file\n", pElem->pSymbol);
	    retcode = 0;
	}
    }
    
    return retcode;
}
