/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 *
 */
/*
 * HISTORY
 * $Log: basename.c,v $
 * Revision 1.1.2.2  1998/12/22  03:14:29  tibor
 * 	rename basename so it doesn't conflict with the real one
 * 	[1998/12/22  02:15:53  tibor]
 *
 * Revision 1.1.2.1  1998/06/02  04:57:49  tibor
 * 	fixed header info
 * 	[1998/06/02  02:21:24  tibor]
 * 
 * 	libutils.a interface routines, directory utils basename, dirname
 * 	[1998/06/02  02:08:27  tibor]
 * 
 * $EndLog$
 */
/*
 * Function:	basename_r()
 *
 * Purpose:	get file portion of a file path
 *		Given a pointer to a string containing a typical pathname
 *		(/usr/src/cmd/ls/ls.c for example), returns a pointer to
 *		the last component of the pathname ("ls.c" in this case).
 *
 *		The function is similar to the standard basename(3g) except it
 *		will prune trailing //, which necessitates a second parameter
 *		for a temporary data area. The original version did in place
 *		modification which didn't seem right to me.
 *
 *		NOTE: this code was samelessly taken from bash source.
 *		Since the original file had no copyright, none xfered. The
 *		above copyright shouldn't be here either but ODE forces it.
 *
 * Parms:	str	- original file name
 *		base	- pointer to hold area for file base
 *			  Data area must be preallocated so better set it
 *			  to max size or this routine might overrun.
 *
 * Returns:	pointer to base. NULL string can be returned if /.
 *
 */
#include "dir_utils.h"
#include "string.h"

char *
basename_r(const char *string, char *base)
{
    int slen = strlen(string);
    int iTemp;

    /* Strip trailing slashes */
    while (slen > 0 && string[slen - 1] == '/')
	slen--;

    /* (2) If string consists entirely of slash characters, string is null
       In this case, skip steps (3) through (5). */
    if (slen == 0) {
	*base = '\0';
	return base;
    }

    /* (3) If there are any trailing slash characters in string, they
       shall be removed. */
    iTemp = slen;

    /* (4) If there are any slash characters remaining in string, the prefix
       of string up to an including the last slash character in string
       shall be removed. */
    while (--slen >= 0) {
	if (string[slen] == '/') {
	    break;
	}
    }

    strncpy(base, &string[slen + 1], iTemp - slen);
    base[iTemp - slen] = '\0';
    
    return base;
}
