/*
 * COPYRIGHT NOTICE
 * Copyright (c) Alteon Networks, Inc. 1996
 * All rights reserved
 *
 */
/*
 * HISTORY
 * $Log: dirname.c,v $
 * Revision 1.1.2.1  1998/06/02  04:57:52  tibor
 * 	libutils.a interface routines, directory utils basename, dirname
 * 	[1998/06/02  02:20:51  tibor]
 *
 * $EndLog$
 */
/*
 * Function:	dirname()
 *
 * Purpose:	get directory portion of a file path
 *
 *		NOTE: this code was samelessly taken from bash source.
 *		Since the original file had no copyright, none xfered. The
 *		above copyright shouldn't be here either but ODE forces it.
 *
 * Parms:	str	- original file name
 *		dir	- pointer directory portion, cleaned of extra /.
 *			  Data area must be preallocated so better set it
 *			  to max size or this routine might overrun.
 *
 * Returns:	pointer to dir.  If no directory is parsed, then ./ is
 *		returned.
 *
 */
#include <string.h>
#include "dir_utils.h"

char *
dirname(const char *string, char *dirpath)
{
    char *p = dirpath;
    int slen;

    slen = strlen (string);

    /* Strip trailing slashes */
    while (slen > 0 && string[slen - 1] == '/') {
	slen--;
    }

    /* (2) If string consists entirely of slash characters, string shall be
       set to a single slash character.  In this case, skip steps (3)
       through (8). */
    if (slen == 0) {
	*p++ = '/';
	*p = '\0';
	return dirpath;
    }

    /* (3) If there are any trailing slash characters in string, they
       shall be removed. */

    /* (4) If there are no slash characters remaining in string, string
       shall be set to a single period character.  In this case, skip
       steps (5) through (8).
	 
       (5) If there are any trailing nonslash characters in string,
       they shall be removed. */

    while (--slen >= 0) {
	if (string[slen] == '/') {
	    break;
	}
    }

    if (slen < 0) {
	*p++ = '.';
	*p = '\0';
	return dirpath;
    }

    /* (7) If there are any trailing slash characters in string, they
       shall be removed. */
    while (--slen >= 0) {
	if (string[slen] != '/') {
	    break;
	}
    }
  
    /* (8)	 If the remaining string is empty, string shall be set to a single
       slash character. */
    if (slen < 0) {
	*p++ = '/';
	*p = '\0';
    } else {
	strncpy(dirpath, string, ++slen);
	dirpath[slen] = '\0';
    }
    
    return dirpath;
}
